#include "stInstrumentWindow.h"
#include "stCurveControl.h"
#include "stTrackerView.h"
#include "stTrackerEntry.h"
#include "stMainWindow.h"
#include "song.h"

#include <stdio.h>
#include <Application.h>
#include <Message.h>
#include <Point.h>

stInstrumentWindow::stInstrumentWindow(stMainWindow *main_win,BPoint point,Ins *instrument):
	BWindow(BRect(point,point+BPoint(510,410)),"Sawteeth: Instrument editor",B_TITLED_WINDOW_LOOK,B_NORMAL_WINDOW_FEEL,B_NOT_RESIZABLE)
{
	m_main_win = main_win;
	m_instrument = instrument;
	
	BRect frame(5,5,4 + 300,4 + 128 + 28);
	m_amplitude_envelope = new stCurveControl(frame,instrument->amppoints,new BMessage(41));
	AddChild(m_amplitude_envelope);
	
	int32 time = 0;
	for (int32 i=0;i<m_instrument->amppoints;i++)
	{
		time += m_instrument->amp[i].time;
		m_amplitude_envelope->SetKnobPosition(BPoint((float)time, ((float)m_instrument->amp[i].lev)/2.0),i);
	}

	frame.OffsetBy(0,frame.Height() + 5.0);
	m_filter_envelope = new stCurveControl(frame,m_instrument->filterpoints,new BMessage(42));
	AddChild(m_filter_envelope);

	time = 0;
	for (int32 i=0;i<m_instrument->filterpoints;i++)
	{
		time += m_instrument->filter[i].time;
		m_filter_envelope->SetKnobPosition(BPoint((float)time,(float)m_instrument->filter[i].lev/2.0),i);
	}

	field_type type_list[] = {ST_TYPE_INDEX, ST_TYPE_SEPARATOR,ST_TYPE_NOTE, ST_TYPE_SEPARATOR,ST_TYPE_BOOL,ST_TYPE_SEPARATOR,ST_TYPE_NIBBLE,ST_TYPE_NIBBLE};
	m_tracker_control = new stTrackerControl(BRect(320,20,440,220),20,type_list,sizeof(type_list)>>2,new BMessage(43),B_NOT_RESIZABLE);
	AddChild(m_tracker_control);

	m_zoom = 1;
}

stInstrumentWindow::~stInstrumentWindow()
{
}

bool stInstrumentWindow::QuitRequested()
{
	m_main_win->CloseInsWin(m_instrument);
	return true;
}

void stInstrumentWindow::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
	case 41:
		{
		uint8 action;
		message->FindInt8("action",(int8*)&action);
		switch (action)
		{
		case ST_ADD_KNOB:
			{
			if (m_instrument->amppoints == INSPOINTS) return;
			m_amplitude_envelope->AddKnob();

			BPoint pos = m_amplitude_envelope->KnobPosition(m_instrument->amppoints);
			BPoint prev_pos = m_amplitude_envelope->KnobPosition(m_instrument->amppoints - 1);

			m_instrument->amp[m_instrument->amppoints].time = pos.x - prev_pos.x;
			m_instrument->amp[m_instrument->amppoints].lev = pos.y * 2.0;
			
			m_instrument->amppoints++;
			}break;
		case ST_DELETE_KNOB:
			{
			if (m_instrument->amppoints == 1) return;
			m_amplitude_envelope->DeleteKnob();
			m_instrument->amppoints--;
			}break;
		case ST_MOVE_KNOB:
			{
			int8 index;
			message->FindInt8("knob",&index);

			printf("ST_MOVE_KNOB %d\n",index);

			BPoint pos = m_amplitude_envelope->KnobPosition(index);

			if (index > 0)
			{
				BPoint prev_pos = m_amplitude_envelope->KnobPosition(index - 1);

				m_instrument->amp[index].time = pos.x - prev_pos.x;
				m_instrument->amp[index].lev = pos.y * 2.0;
			}
			else
			{
				m_instrument->amp[index].time = pos.x;
				m_instrument->amp[index].lev = pos.y * 2.0;
			}
			
			printf("time: %d\n",m_instrument->amp[index].time);

			}break;
		}
		}break;
	case 42:
		{
		uint8 action;
		message->FindInt8("action",(int8*)&action);
		switch (action)
		{
		case ST_ADD_KNOB:
			{
			if (m_instrument->filterpoints == INSPOINTS) return;
			m_filter_envelope->AddKnob();

			BPoint pos = m_filter_envelope->KnobPosition(m_instrument->filterpoints);
			BPoint prev_pos = m_filter_envelope->KnobPosition(m_instrument->filterpoints - 1);

			m_instrument->filter[m_instrument->filterpoints].time = pos.x - prev_pos.x;
			m_instrument->filter[m_instrument->filterpoints].lev = pos.y * 2.0;
			
			m_instrument->filterpoints++;
			}break;
		case ST_DELETE_KNOB:
			{
			if (m_instrument->filterpoints == 1) return;
			m_filter_envelope->DeleteKnob();
			m_instrument->filterpoints--;
			}break;
		case ST_MOVE_KNOB:
			{
			int8 index;
			message->FindInt8("knob",&index);

			BPoint pos = m_filter_envelope->KnobPosition(index);

			if (index > 0)
			{
				BPoint prev_pos = m_filter_envelope->KnobPosition(index - 1);

				m_instrument->filter[index].time = pos.x - prev_pos.x;
				m_instrument->filter[index].lev = pos.y * 2.0;
			}
			else
			{
				m_instrument->filter[index].time = pos.x;
				m_instrument->filter[index].lev = pos.y * 2.0;
			}
			}break;

		}
		} break;
	case 43:
		{
		uint8 action;
		message->FindInt8("action",(int8*)&action);
		switch (action)
		{
		case ST_SET_LENGTH:
			{
			stTrackerControl *control;
			BTextControl *text;
			message->FindPointer("control",(void**)&control);
			message->FindPointer("source",(void**)&text);
			control->SetLength(atoi(text->Text()));
			} break;
		}
		} break;
	}
}
