#include <stdio.h>
#include <string.h>

#include <Screen.h>
#include <MenuItem.h>
#include <NodeInfo.h>
#include <TextView.h>
#include <ScrollView.h>
#include <TranslationUtils.h>
#include <StorageKit.h>

#include "song.h"
#include "stMainWindow.h"
#include "stPartWindow.h"

stMainWindow::stMainWindow(BRect frame, const char *file):
	BWindow(frame, "sawteeth Editor", B_DOCUMENT_WINDOW, B_CURRENT_WORKSPACE)
{
	file_name=new BPath();

	playing = false;
	zoom = 1.0;

	SetPulseRate(300000);	// typen som visar var man är i låten

	memset(pw,0,PARTS*sizeof(stPartWindow*));
	memset(iw,0,INS*sizeof(stInstrumentWindow*));
	
	BPath path;
	find_directory(B_USER_DIRECTORY, &path);
	save = new BFilePanel(B_SAVE_PANEL,new BMessenger(this));
	save->SetPanelDirectory(path.Path());


	
// -------------------------

	BRect rect = BRect(0,0,0,0);
	BMenuItem *item;
	BMenu *sub;
	menu = new BMenuBar(rect, "MenuBar");
	sub = new BMenu("File");

	sub->AddItem(item=new BMenuItem("New Song", new BMessage(ST_NEW_SONG),'N'));
	item->SetTarget(be_app);
	sub->AddItem(item=new BMenuItem("Open Song…", new BMessage(ST_OPEN_SONG),'O'));
	item->SetTarget(be_app);
	sub->AddItem(new BMenuItem("Close Song", new BMessage(B_CLOSE_REQUESTED),'W'));
	sub->AddSeparatorItem();
	sub->AddItem(save_item=new BMenuItem("Save Song", new BMessage(ST_SAVE_SONG),'S'));
	sub->AddItem(new BMenuItem("Save Song As…", new BMessage(ST_SAVE_SONG_AS),0));
	sub->AddSeparatorItem();
	sub->AddItem(item=new BMenuItem("About…", new BMessage(B_ABOUT_REQUESTED),'A'));
	item->SetTarget(be_app);
	sub->AddSeparatorItem();
	sub->AddItem(item=new BMenuItem("Quit", new BMessage(B_CLOSE_REQUESTED),'Q'));
	item->SetTarget(be_app);
	menu->AddItem(sub);

	sub=new BMenu("Let The Music");
	play_item = new BMenuItem("Play from Focus", new BMessage(ST_PLAY_START),' ');
	sub->AddItem(play_item);
	sub->AddItem(new BMenuItem("Continue", new BMessage(ST_PLAY_CONT),'P'));
	menu->AddItem(sub);

	sub=new BMenu("Edit");
	sub->AddItem(new BMenuItem("Cut", new BMessage(ST_Cut),'X'));
	sub->AddItem(new BMenuItem("Copy", new BMessage(ST_Copy),'C'));
	sub->AddItem(new BMenuItem("Paste", new BMessage(ST_Paste),'V'));
	menu->AddItem(sub);

	sub=new BMenu("View");
	sub->AddItem(new BMenuItem("Zoom In", new BMessage(ST_ZoomIn),'+'));
	sub->AddItem(new BMenuItem("Zoom Out", new BMessage(ST_ZoomOut),'-'));
	sub->AddItem(new BMenuItem("Zoom Normal", new BMessage(ST_ZoomNormal),0));
	menu->AddItem(sub);

	sub=new BMenu("Channel");
	menu->AddItem(sub);
	
	part_menu=new BMenu("Part");
	part_menu->AddItem(new BMenuItem("New", new BMessage(ST_NEW_PART),0));
	part_menu->AddSeparatorItem();
	menu->AddItem(part_menu);

	instrument_menu=new BMenu("Instrument");
	instrument_menu->AddItem(new BMenuItem("New", new BMessage(ST_NEW_INS),0));
	instrument_menu->AddSeparatorItem();
	menu->AddItem(instrument_menu);

	AddChild(menu);

// --------------------
	if (file)
	{
		file_name->SetTo(file);
		song=new stsp((char *)file_name->Path());
		CurrentFile((char *)file_name->Leaf());
	}
	else
	{
		save_item->SetEnabled(false);
		song=new stsp();
		CurrentFile("new");
	}

	if (song->InitCheck() != JNG_OK){
		init = JNG_ERROR;
		return;
	}else init = JNG_OK;
	
	rect = menu->Bounds();
	float hi = rect.bottom;
	rect = frame;
	rect.OffsetTo(0,0);
	rect.top += hi+ 1;
	rect.right -= B_V_SCROLL_BAR_WIDTH;
	rect.bottom -= B_H_SCROLL_BAR_HEIGHT;

	st_seq_view = new stSeqView(song,rect);
	AddChild(new BScrollView("SawTeethView", st_seq_view, B_FOLLOW_ALL_SIDES, B_WILL_DRAW, true, true));

	sub->AddItem(item = new BMenuItem("New", new BMessage(ST_NEW_CHANNEL),0));
	item->SetTarget(st_seq_view);
	sub->AddItem(item = new BMenuItem("Delete", new BMessage(ST_DELETE_CHANNEL),0));
	item->SetTarget(st_seq_view);

	BuildPartMenu();
	BuildInstrumentMenu();

}

status_t stMainWindow::InitCheck(){	return init;};

stMainWindow::~stMainWindow()
{
	for (int c = 0 ; c < PARTS ; c++){
		if (pw[c]){
			fprintf(stderr,"removing pw[%d], (%p)",c,pw[c]);
			pw[c]->Lock();
			pw[c]->Quit();
			fputs(" ok\n",stderr);
		}
	}

	for (int c = 0 ; c < INS ; c++){
		if (iw[c]){
			fprintf(stderr,"removing iw[%d], (%p)",c,pw[c]);
			iw[c]->Lock();
			iw[c]->Quit();
			fputs(" ok\n",stderr);
		}
	}
	
	song->Stop();
	delete song;
	delete save;
	delete file_name;
}

void stMainWindow::BuildPartMenu()
{
	while (part_menu->CountItems()>2) part_menu->RemoveItem(2);

	Part *part;
	BMessage *message;
	for (uint16 i=0;i<song->PartCount();i++)
	{
		part=song->GetPart(i);
		message=new BMessage(ST_EDIT_PART);
		message->AddInt32("index",i);
		part_menu->AddItem(new BMenuItem( part->name ,message,0));
	}
}

void stMainWindow::BuildInstrumentMenu()
{
	while (instrument_menu->CountItems()>2) instrument_menu->RemoveItem(2);

	Ins *instrument;
	BMessage *message;
	for (uint16 i=1;i<song->InsCount();i++)
	{
		instrument=song->GetIns(i);
		if (instrument->name==NULL) instrument->name=strdup("noname");
		message=new BMessage(ST_EDIT_INS);
		message->AddInt32("index",i);
		instrument_menu->AddItem(new BMenuItem(instrument->name,message,0));
	}
}

void stMainWindow::CurrentFile(char *filename)
{
	char title[B_FILE_NAME_LENGTH+20];
	strcpy(title,"Sawteeth Editor: ");
	strcat(title,filename);
	SetTitle(title);
}

void stMainWindow::MessageReceived(BMessage *message)
{
	uint32 type; 
	int32 count;
//fprintf(stderr,"!!!%c%c%c%c\n",message->what >> 24,(message->what >> 16) & 255,(message->what >> 8) & 255,message->what & 255);
	switch(message->what)
	{
	case B_SIMPLE_DATA:
		message->GetInfo("refs", &type, &count); 
		if ( count>0 && type == B_REF_TYPE )
		{
			DetachCurrentMessage();
			message->what = B_REFS_RECEIVED;
			be_app->PostMessage(message);
		}
		break;

	case ST_PPLAY_START:
		if (playing){
			play_item->SetLabel("Play from Focus"); // fixa namn mm.
			song->Stop();
		}else{
			play_item->SetLabel("Stop"); // fixa namn mm.
			uint16 tmp = message->FindInt16("part");
			fprintf(stderr,"Playing part %d\n",tmp);
			song->SetPartToPlay(tmp);
			song->UsePartPlay(true);
			song->Start();
		}
		playing = ! playing;
		break;

	case ST_PLAY_START:
		if (playing){
			play_item->SetLabel("Play from Focus"); // fixa namn mm.
			song->Stop();
		}else{
			play_item->SetLabel("Stop"); // fixa namn mm.
			int tmp = st_seq_view->GetFocusPosition();
			fprintf(stderr,"Playing from pal(%d)\n",tmp);
			song->SetPos(tmp);
			song->UsePartPlay(false);
			song->Start();
		}
		playing = ! playing;
		break;

	case ST_PLAY_CONT:
		if (!playing){
			play_item->SetLabel("Stop"); // fixa namn mm.
			playing = true;
			song->Start();
		}
		break;

	case ST_SAVE_SONG_AS:
		save->Show();
		break;
	case B_SAVE_REQUESTED:
	{
		const char *filename;
		entry_ref ref;
		message->FindString("name",&filename);
		message->FindRef("directory",&ref);

		file_name->SetTo(&BEntry(&ref));
		file_name->Append(filename);
		save_item->SetEnabled(true);
	}	// OBS. här skall ingen break finnas!!!
	case ST_SAVE_SONG:
		song->Save((char *)file_name->Path());
		CurrentFile((char *)file_name->Leaf());
		break;
	case ST_Cut:
		st_seq_view->Cut();
		break;
	case ST_Copy:
		st_seq_view->Copy();
		break;
	case ST_Paste:
		st_seq_view->Paste();
		break;
	case ST_ZoomIn:
		zoom *= 1.1;
		st_seq_view->SetZoom(zoom);
		break;
	case ST_ZoomOut:
		zoom *= 0.9;
		st_seq_view->SetZoom(zoom);
		break;
	case ST_ZoomNormal:
		zoom = 1.0;
		st_seq_view->SetZoom(zoom);
		break;
	case ST_NEW_PART:{
		int temp = song->NewPart();
		if (0 == temp) { fputs("maxparts-error\n",stderr); return;}
		pw[temp] = new stPartWindow(this,BPoint(60,60), song->GetPart(temp), song);
		pw[temp]->Show();
		BuildPartMenu();
	}break;
	
	case ST_EDIT_PART:{
		int temp = message->FindInt32("index");
		if (pw[temp] == NULL)
			pw[temp] = new stPartWindow(this,BPoint(60,60), song->GetPart(temp), song);
		pw[temp]->Show();
	}break;	

	case ST_NEW_INS:{
		int temp = song->NewIns();
		if (0 == temp) { fputs("maxins-error\n",stderr); return;}
		iw[temp] = new stInstrumentWindow(this,BPoint(60,60), song->GetIns(temp));
		iw[temp]->Show();
		BuildInstrumentMenu();
	}break;
	
	case ST_EDIT_INS:{
		int temp = message->FindInt32("index");
		if (iw[temp] == NULL)
			iw[temp] = new stInstrumentWindow(this,BPoint(60,60), song->GetIns(temp));
		iw[temp]->Show();
	}break;


	case ST_PART_CHANGED : {
		BuildPartMenu();
		st_seq_view->Update();
	}break;

	default:
		BWindow::MessageReceived(message);
	}
}

void stMainWindow::ClosePartWin(Part *part)
{
	pw[part - song->GetPart(0)] = NULL;
}

void stMainWindow::CloseInsWin(Ins *instrument_win)
{
	iw[instrument_win - song->GetIns(0)] = NULL;
}
