// Jonas Norberg

#include <stdio.h> // remove this line
#include "stPartSquare.h"
#include "stChannelView.h"
#include "song.h"

stPartSquare::stPartSquare(Song* song,int8 *transp,uint8 *part,BRect rect):BView(rect,"Blubb", B_FOLLOW_NONE, B_WILL_DRAW )
{
	s = song;
	p = part;
	t =transp;

	be_plain_font->GetHeight(&fh);
//	printf("%f, %f, %f\n", fh.ascent, fh.descent, fh.leading);

//	puts("stPartSquare::stPartSquare()");
	SetFont(be_plain_font);
	
	r = 197;
	g = 197;
	b = 197;

	SetViewColor(r,g,b);
	SetLowColor(r,g,b);
		

	SetZoom(1.0);
	height = rect.bottom - rect.top;
	
	PALs = 1;
	startPAL = 0;

	width = Frame().right - Frame().left;


	focus = false;
	marked = false;
	
}

stPartSquare::~stPartSquare()
{
//	puts("stPartSquare::~stPartSquare()");
}
	
void	stPartSquare::SetZoom(float PixelsPerPAL)
{
	Zoom = PixelsPerPAL;
//	puts("stPartSquare::SetZoom()");
//	printf("%p - MoveTo[%f] ResizeTo[%f] NextShouldBe[%f]\n",this,(PixelsPerPAL * startPAL),(PixelsPerPAL * PALs - 1),(PixelsPerPAL * startPAL)+(PixelsPerPAL * PALs));	
	ResizeTo(	(int)(Zoom * (startPAL + PALs)) - (int)(Zoom * startPAL) - 1, height);
	MoveTo(		(int)(Zoom * startPAL), height - *t + 1);

	width = Frame().right - Frame().left;
	Invalidate();
}

void	stPartSquare::Update()
{
	int tmp = height - *t + 1;
	if ( tmp < 0 ) tmp = 0;
	if ( tmp > (2 * height+1)) tmp = (2 * height +1);

	MoveTo((int)(Zoom * startPAL), tmp);
	Invalidate();	
}

void	stPartSquare::SetStartPAL(uint32 start)
{
	int tmp = height - *t + 1;
	if ( tmp < 0 ) tmp = 0;
	if ( tmp > 2 * height ) tmp = 2 * height;

	startPAL = start;
	MoveTo(		(int)(Zoom * startPAL), tmp);
	Invalidate();
}

uint32	stPartSquare::StartPAL()
{
	return startPAL;
}

void	stPartSquare::SetPALs(uint32 l)
{
	PALs = l;
	ResizeTo(	(int)(Zoom * (startPAL + PALs)) - (int)(Zoom * startPAL) - 1, height);
	width = Frame().right - Frame().left;
	Invalidate();
}

#define DIFF 25
void	stPartSquare::Draw(BRect frame)
{
//	puts("stPartSquare::Draw()");

	MovePenTo(1,height - fh.descent);

	if (marked){
		SetHighColor(0,0,0);
		SetLowColor(80,80,80);
		FillRect(BRect(0,0,width,height),B_MIXED_COLORS);

		SetHighColor(255,255,255);

		char temp[5]={0,0,0,0,0};
		DrawString(s->GetPart(*p)->name);
		
		return;
	}

	if (focus){
		SetHighColor(240,240,240);
		SetLowColor(240,240,240);
		FillRect(BRect(1,1,width-1,height-1));

		SetHighColor(255,255,255);
		StrokeRect(BRect(0,0,width,height));
		SetHighColor(0,0,0);

		char temp[5]={0,0,0,0,0};
		sprintf(temp,"%d",*p);
		DrawString(temp);
		DrawString(":");
		sprintf(temp,"%d",*t);
		DrawString(temp);
		DrawString(":");
		DrawString(s->GetPart(*p)->name);
		
		return;
	}
	

	SetHighColor(0,0,0);
	DrawString(s->GetPart(*p)->name);

	SetHighColor(r+DIFF,g+DIFF,b+DIFF);
	StrokeLine(BPoint(width-1,0), BPoint(0,0));
	StrokeLine(BPoint(0,height-1));

	SetHighColor(r-DIFF,g-DIFF,b-DIFF);
	StrokeLine(BPoint(1,height), BPoint(width,height));
	StrokeLine(BPoint(width,1));

}

void	stPartSquare::MakeFocus(bool focused)
{
	BView::MakeFocus(focused);
	focus = focused;
	((stChannelView *)Parent())->MakeSemiFocus(focused);
	Invalidate();
}

void	stPartSquare::MouseDown(BPoint point)
{
	MakeFocus();
	((stChannelView*)Parent())->FindSquare();
}

void	stPartSquare::KeyDown(const char *bytes, int32 numBytes) 
{
	int32 mod;
//	Window()->CurrentMessage()->PrintToStream();
	Window()->CurrentMessage()->FindInt32("modifiers",&mod);
//	printf("mod == %x\n",mod);
	
	if ( numBytes == 1 ) {
		switch ( bytes[0] ) { 
			case B_RIGHT_ARROW:
				if (mod & 0x1004)
					Window()->PostMessage('ch_r',Parent());
				else
					if (mod & 0x101){
						Window()->PostMessage('mk_r',Parent());
					}else
					Window()->PostMessage('mv_r',Parent());
			break; 
			case B_LEFT_ARROW: 
				if (mod & 0x1004)
					Window()->PostMessage('ch_l',Parent());
				else
					if (mod & 0x101)
						Window()->PostMessage('mk_l',Parent());
					else
						Window()->PostMessage('mv_l',Parent());
			break; 
			case B_UP_ARROW:
				if (mod & 0x1004) Window()->PostMessage('tr_u',Parent());
				else{
					BMessage *mess = new BMessage('mv_u');
					mess->AddInt32("startPAL",startPAL);
					Window()->PostMessage(mess,Parent()->Parent());
				}
			break; 
			case B_DOWN_ARROW: 
				if (mod & 0x1004) Window()->PostMessage('tr_d',Parent());
				else{
					BMessage *mess = new BMessage('mv_d');
					mess->AddInt32("startPAL",startPAL);
					Window()->PostMessage(mess,Parent()->Parent());
				}
			break; 
		} 
	} 
}

void	stPartSquare::Mark(bool mark)
{
	marked = mark;
}

bool	stPartSquare::IsMark()
{
	return marked;
}
