#include <stdio.h>
#include <stdlib.h>

#include "stPartWindow.h"
#include "stCurveControl.h"
#include "stTrackerView.h"
#include "stTrackerEntry.h"
#include "stApp.h"
#include "song.h"

stPartWindow::stPartWindow(stMainWindow *win, BPoint point,Part *part_to_edit, Song *song):
	BWindow(BRect(point,point+BPoint(300,300)),"",B_FLOATING_WINDOW, B_NOT_H_RESIZABLE)
{
	main_win = win;
	part=part_to_edit;
	s = song;

	BMessage *mess = new BMessage(ST_PPLAY_START);
	AddShortcut(' ',B_COMMAND_KEY, mess);


	field_type type_list[]={ST_TYPE_INDEX, ST_TYPE_SEPARATOR,ST_TYPE_SPACE,ST_TYPE_NOTE, ST_TYPE_SPACE,ST_TYPE_BLANK_NIBBLE,ST_TYPE_NIBBLE,ST_TYPE_SPACE,ST_TYPE_BLANK_NIBBLE,ST_TYPE_NIBBLE,ST_TYPE_SPACE};
	tracker_control=new stTrackerControl(BRect(5,45,120,295),20,type_list,sizeof(type_list)>>2,new BMessage(MSG_FROM_TRACKER),B_FOLLOW_TOP_BOTTOM);

	tracker_control->SetLength(part->len);
	for (uint16 i=0;i<part->len;i++)
	{
		tracker_control->SetFieldContent(i,3,part->steps[i].note);

		tracker_control->SetFieldContent(i,5,(part->steps[i].ins & 0xf0) >> 4);
		tracker_control->SetFieldContent(i,6,part->steps[i].ins & 0xf);

		tracker_control->SetFieldContent(i,8,(part->steps[i].eff & 0xf0) >> 4);
		tracker_control->SetFieldContent(i,9,part->steps[i].eff & 0xf);
	}

	float right_border = tracker_control->Frame().right;

	speed_string = new BTextControl(BRect(5,5,right_border,15),"speed_string","blanks/step:",NULL,new BMessage(ST_NEW_PART_SPEED));
	
	char tmp[10];
	sprintf(tmp,"%d",part->sps);
	speed_string->SetText(tmp);
	speed_string->SetDivider(60);
	
	name_string = new BTextControl(BRect(5,25,right_border,35),"name_string","Name:",part->name,new BMessage(ST_NEW_PART_NAME));

	SetTitle(part->name);
	name_string->SetDivider(30);

	ResizeTo(right_border + 5.0,300);

	BRect frame = Frame();
	frame.OffsetTo(0.0,0.0);
	frame.bottom += 1.0;
	frame.right += 1.0;

	BBox *box = new BBox(frame,"background",B_FOLLOW_ALL_SIDES);

	AddChild(box);
	box->AddChild(tracker_control);
	box->AddChild(speed_string);
	box->AddChild(name_string);
}

stPartWindow::~stPartWindow()
{
	main_win->ClosePartWin(part);
}

/*
bool stPartWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}
*/

void stPartWindow::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		case ST_PPLAY_START:
		{
			BMessage *mess = new BMessage(ST_PPLAY_START);
			int16 temp = part - s->GetPart(0);
			mess->AddInt16("part",temp);

			main_win->PostMessage( mess );
		}
		case ST_NEW_PART_NAME:
		{
			part->SetName((char*)name_string->Text());
			SetTitle(part->name);
		}
		case ST_NEW_PART_SPEED:
		{
			part->sps = atoi(speed_string->Text());
			if (part->sps < 1) part->sps = 1;
			char tmp[10];
			sprintf(tmp,"%d",part->sps);
			speed_string->SetText(tmp);
			main_win->PostMessage(ST_PART_CHANGED);
		}
		case MSG_FROM_TRACKER:
		{
			uint8 action;
			message->FindInt8("action",(int8*)&action);
			switch (action)
			{
				case ST_SET_LENGTH:
				{
					stTrackerControl *control;
					BTextControl *text;
					message->FindPointer("control",(void**)&control);
					message->FindPointer("source",(void**)&text);
					control->SetLength(atoi(text->Text()));
					part->len = atoi(text->Text());
					main_win->PostMessage(ST_PART_CHANGED);
				}break;
			}
		}break;
		case ST_ENTRY_CHANGED:
			{
			int8 row,field,val;
			message->FindInt8("field",&field);
			message->FindInt8("row",&row);
			message->FindInt8("value",&val);
			printf("ST_ENTRY_CHANGED (%d,%d)=%d\n",field,row,val);
			switch(field)
			{
				case 3: // note
				{
					part->steps[row].note = val;
				}break;
				case 5:
				{
					part->steps[row].ins &= 0xf;
					part->steps[row].ins |= val << 4;
				 // protection
					if (part->steps[row].ins >= s->InsCount()){
						part->steps[row].ins = s->InsCount()-1;
						tracker_control->SetFieldContent(row,5,(part->steps[row].ins & 0xf0) >> 4);
						tracker_control->SetFieldContent(row,6,part->steps[row].ins & 0xf);
					}
				}break;
				case 6:
				{
					part->steps[row].ins &= 0xf0;
					part->steps[row].ins |= val;
				 // protection
					if (part->steps[row].ins >= s->InsCount()){
						part->steps[row].ins = s->InsCount()-1;
						tracker_control->SetFieldContent(row,5,(part->steps[row].ins & 0xf0) >> 4);
						tracker_control->SetFieldContent(row,6,part->steps[row].ins & 0xf);
					}
				}break;
				case 8:
				{
					part->steps[row].eff &= 0xf;
					part->steps[row].eff |= val << 4;
				}break;
				case 9:
				{
					part->steps[row].eff &= 0xf0;
					part->steps[row].eff |= val;
				}break;
			}
		}
	}
}
