#include <stdio.h>

#include "stSeqView.h"
#include "stMainWindow.h"
#include "song.h"

stSeqView::stSeqView(Song *song, BRect frame) : 
	BView(frame, "Channel",  B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS | B_PULSE_NEEDED )
{
	s = song;
	SetViewColor(168,168,168);
	font_height fh;
	be_plain_font->GetHeight(&fh);
//	printf("%f, %f, %f\n", fh.ascent, fh.descent, fh.leading);
	height = 3 * (fh.ascent + fh.descent);

	ruler = new stChRulerView(BRect(0,0,frame.Width(),5));
	AddChild(ruler);

	for (int cnt=0 ; cnt<song->ChannelCount() ; cnt++){
		ps[cnt] = new stChannelView(s, s->GetChannel(cnt), BRect(0,6+cnt*height,300,6+((cnt+1)*height) -1));
		AddChild(ps[cnt]);
	}
}

void stSeqView::Pulse()
{
	ruler->SetPos(s->GetPos());
}

void stSeqView::FrameResized(float w, float h)
{
	Update(true);
}

void stSeqView::AttachedToWindow(void)
{
	Update();
}

void stSeqView::Update(bool ScrollbarsOnly)
{
	width = 0;
	for (int cnt=0 ; cnt<s->ChannelCount() ; cnt++){
		if (!ScrollbarsOnly) ps[cnt]->Update();
		if (ps[cnt]->Width() > width){
			width = ps[cnt]->Width();
		}
	}

	int totalheight = height * s->ChannelCount();
	int range = totalheight - Frame().Height();
	if (range < 0 ) range = 0;
	BScrollBar *sb;
	if ((sb=ScrollBar(B_VERTICAL))!=NULL)
	{
		sb->SetProportion(Frame().Height()/totalheight);
		sb->SetRange(0,range);
		sb->SetSteps(height,Frame().Height());
	}

	range = width - Frame().Width();

	if (range < 0 ) range = 0;
	if ((sb=ScrollBar(B_HORIZONTAL))!=NULL)
	{
		sb->SetProportion(Frame().Width()/width);
		sb->SetRange(0,range);
		sb->SetSteps(16,Frame().Width());
	}

	if (!ScrollbarsOnly) Invalidate();
}

stSeqView::~stSeqView()
{
}

void stSeqView::MessageReceived(BMessage *message)
{
//puts("stSeqView::MessageReceived");
	int c;
	uint32 startPAL;
	switch ( message->what ) {
		case ST_NEW_CHANNEL :{
			int tmp = s->NewChannel();
			if (tmp > 0){
				ps[tmp] = new stChannelView(s,s->GetChannel(tmp),BRect(0,6+tmp*height,300,6+((tmp+1)*height) -1));
				AddChild(ps[tmp]);
				Update();
			}
		} break;

		case ST_DELETE_CHANNEL :{
			puts("DELETE_CHANNEL (not implemented)");
		} break;
		
		case 'mv_u'://	puts("seqview-mv-up");
			c = 0;
			while ( c < 1+s->ChannelCount() && !ps[c]->IsSemiFocus()) c++;
			if ( c < 1 || c >= s->ChannelCount() ) return;
			if (B_OK != message->FindInt32("startPAL",(int32 *)&startPAL)) return;
			ps[c-1]->MakeFocusPAL(startPAL);
		break; 

		case 'mv_d'://	puts("seqview-mv-down");
			c = 0;
			while ( c < 1+s->ChannelCount() && !ps[c]->IsSemiFocus()) c++;
			if ( (c+1) >= s->ChannelCount()) return;
			if (B_OK != message->FindInt32("startPAL",(int32 *)&startPAL)) return;
			ps[c+1]->MakeFocusPAL(startPAL);
		break;

		default:
			BView::MessageReceived(message);
		break; 
	}

}

void stSeqView::SetZoom(float z)
{
	ruler->SetZoom(z);
	for (int c=0 ; c < s->ChannelCount() ; c++) ps[c]->SetZoom(z);

	Update();
}

/*
void stSeqView::Draw(BRect frame)
{
//	FillRect(frame);
//	puts("stSeqView::Draw()");	
}
*/










int	stSeqView::GetFocusPosition()
{
	int c = 0; while ( c < s->ChannelCount() && !ps[c]->IsSemiFocus()) c++;

	if (c >= s->ChannelCount()) return 0;
	return ps[c]->GetFocusPosition();
	
}



void stSeqView::Cut() // kör copy och tar bort...
{
	int c = 0; while ( c < s->ChannelCount() && !ps[c]->IsSemiFocus()) c++;
	ps[c]->Cut();

	Update();
}

void stSeqView::Copy() // kopiera till clipboard
{
	int c = 0; while ( c < s->ChannelCount() && !ps[c]->IsSemiFocus()) c++;
	ps[c]->Copy();
}

void stSeqView::Paste() // töm clipboard och insert i 
{
	int c = 0; while ( c < s->ChannelCount() && !ps[c]->IsSemiFocus()) c++;
	ps[c]->Paste();
	
	Update();
}
