#include "string.h"
#include "stdlib.h"
#include "song.h"
#include "txt.h"
#include "player.h"

void *CALLOC(int s1, int s2);
void *CALLOC(int s1, int s2)
{
	char *tmp = (char*)malloc(s1 * s2);
	for (int c = 0; c< (s1*s2) ; c++) tmp[c]=0;
	return (void*)tmp;
}


_EXPORT Song::Song(char *filename)
{
	//////////////////////////////////////////////////////// FREQ-TABLES
	#define MUL 1.0594630943593

	N2F = (float *) malloc(22*12*sizeof(float));
	R2F = (float *) malloc(22*12*sizeof(float));

	int count = 0;
	double octbase=1.02197486445547712033;

	for (int oc=0;oc<22;oc++){
		double base=octbase;
		for (int c=0;c<12;c++){
			N2F[count++] = base;
			base *= MUL;
		}
		octbase *= 2;
	}

	count = 0;
	octbase = 1 / MUL;
	
	for (int oc=0;oc<22;oc++){
		double base=octbase;
		for (int c=0;c<12;c++){
			R2F[count++] = base;
			base *= MUL;
		}
		octbase *= 2;
	}
	//////////////////////////////////////////////////////// END OF FREQ-TABLES

	init = JNG_OK;
	
	///////////////////////////////////////	här allokeras elände till sången...

	channelcount = partcount = instrumentcount = breakpcount = 0L;

	chan = 0;
	parts = 0;
	ins = 0;
	breakpoints = 0;
	PALs = 0;

	if (filename == NULL){
#ifdef TARGET_EDITOR
		AllocDefSong();
#else
		init=JNG_ERROR;
#endif
		return;
	};
	
	if (JNG_OK != (init = Load(filename))) return;

	///////////////////////////////////////	partplayer
#ifdef TARGET_EDITOR
		pchan.steps = &pchanstep;
		pchan.len = 1;
		pchanstep.part = 0;
		pchanstep.transp = 0;
		pp = new Player(this,&pchan);
		pbs = pp->Buffer();
#endif
	return;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT status_t Song::Load(char *filename)
{
#ifdef TARGET_EDITOR
	FreeSong();
#endif

	uint32 c, c1, num;
	txt t(filename);
	if (JNG_OK!=t.InitCheck()){
		init=JNG_ERROR;
		return init;
	};
	
	t.IsBin(true); // för att kunna läsa in första fyra
	num = t.NextVal(4);
//fprintf(stderr,"<%c%c%c%c>\n",num>>24,num>>16 & 255,num>>8 & 255,num & 255);
	switch (num){
		case 'SWTD': 		break;
		case 'SWTT': t.IsBin(false);	break;
		default : return init=JNG_ERROR;
	}
	
	spspal = t.NextVal(2);
	if (spspal < 1){
		fputs("spspal < 1\n",stderr);
		return init = JNG_ERROR;
	}

	//////////////////////////////////////////////////////// CHANNELS
fprintf(stderr,"CHANNELS\n");
	channelcount = t.NextVal();		// no. of channels
	if (channelcount < 1){
		fputs("channelcount < 1\n",stderr);
		return init = JNG_ERROR;
	}

#ifdef TARGET_EDITOR
	chan = (Channel*) CALLOC( CHN , sizeof(Channel));	// allocates more space for editor
#else
	chan = (Channel*) CALLOC( channelcount , sizeof(Channel));
#endif

	for (c=0; c < channelcount; c++){
		chan[c].len = t.NextVal();
		if (1 > chan[c].len){
			fprintf(stderr,"chan[%d].len < 1\n",c);
			init=JNG_ERROR;
			return init;
		}
		
		//	seq
#ifdef TARGET_EDITOR
		chan[c].steps = (ChStep*) CALLOC( CHNSTEPS , sizeof(ChStep));
#else
		chan[c].steps = (ChStep*) CALLOC( chan[c].len , sizeof(ChStep));
#endif

		for (c1=0; c1 < chan[c].len; c1++){
			chan[c].steps[c1].part = t.NextVal();
			chan[c].steps[c1].transp = t.NextVal();
		}
	}
	
	//////////////////////////////////////////////////////// PARTS
fprintf(stderr,"PARTS\n");
	partcount = t.NextVal();
	if (1 > partcount){fputs("partcount < 1\n",stderr);init=JNG_ERROR;return init;}

#ifdef TARGET_EDITOR
	parts = (Part*) CALLOC( PARTS , sizeof(Part));	// allocates more space for editor
#else
	parts = (Part*) CALLOC( partcount , sizeof(Part));
#endif

	for (c=0; c < partcount; c++){
		parts[c].sps = t.NextVal();
		if (parts[c].sps < 1){
			fprintf(stderr,"parts[%d].sps < 1\n",c);
			return init = JNG_ERROR;
		}

		parts[c].len = t.NextVal();
		if (parts[c].len < 1){
			fprintf(stderr,"parts[%d].len < 1\n",c);
			return init = JNG_ERROR;
		}

#ifdef TARGET_EDITOR
		parts[c].steps = (Step*) CALLOC( PARTSTEPS , sizeof(Step));
#else
		parts[c].steps = (Step*) CALLOC( parts[c].len , sizeof(Step));
#endif

		for (c1=0; c1 < parts[c].len; c1++){
			parts[c].steps[c1].ins = t.NextVal();
			parts[c].steps[c1].eff = t.NextVal();
			parts[c].steps[c1].note = t.NextVal();
//			fprintf(stderr,"%3d - %3d - %3d\n", parts[c].steps[c1].ins, parts[c].steps[c1].eff, parts[c].steps[c1].note );
		}
	}
	
	
	//////////////////////////////////////////////////////// INSTRUMENTS
fprintf(stderr,"INS\n");

	instrumentcount = 1+t.NextVal();
	if (instrumentcount < 2){
		fprintf(stderr,"instrumentcount < 1\n");
		return init = JNG_ERROR;
	}

#ifdef TARGET_EDITOR
	ins = (Ins*) CALLOC( INS , sizeof(Ins));	// allocates more space for editor
#else
	ins = (Ins*) CALLOC( instrumentcount , sizeof(Ins));
#endif

// allocate dummy instrument
	ins[0].name = strdup("dummy");
	ins[0].filterpoints	= 1;
	ins[0].amppoints	= 1;
	ins[0].filter		= (TimeLev*) CALLOC( 1 , sizeof(TimeLev));
	ins[0].amp			= (TimeLev*) CALLOC( 1 , sizeof(TimeLev));

	ins[0].filtermode = 0;
	ins[0].clipmode = 0;

	ins[0].sps = 30;
	ins[0].len = 1;
	
	ins[0].steps = (InsStep*) CALLOC(1 , sizeof(InsStep));
// end of dummy ins allocation


	for (c = 1; c < instrumentcount; c++){
		
		ins[c].filterpoints = t.NextVal();
		if (ins[c].filterpoints < 1){ fprintf(stderr,"ins[%d].filterpoints < 1\n",c);return init = JNG_ERROR;}

#ifdef TARGET_EDITOR
		ins[c].filter = (TimeLev*) CALLOC( INSPOINTS , sizeof(TimeLev));
#else
		ins[c].filter = (TimeLev*) CALLOC( ins[c].filterpoints , sizeof(TimeLev));
#endif
		for (c1= 0; c1 < ins[c].filterpoints; c1++){
			ins[c].filter[c1].time =	t.NextVal();
			ins[c].filter[c1].lev =		t.NextVal();
		}
		
		ins[c].amppoints = t.NextVal();
		if (ins[c].amppoints < 1){ fprintf(stderr,"ins[%d].amppoints < 1\n",c); return init = JNG_ERROR;}

#ifdef TARGET_EDITOR
		ins[c].amp = (TimeLev*) CALLOC( INSPOINTS , sizeof(TimeLev));
#else
		ins[c].amp = (TimeLev*) CALLOC( ins[c].amppoints , sizeof(TimeLev));
#endif
		for (c1= 0; c1 < ins[c].amppoints; c1++){
			ins[c].amp[c1].time =	t.NextVal();
			ins[c].amp[c1].lev =	t.NextVal();
		}
		
		ins[c].filtermode = t.NextVal();
		ins[c].clipmode = t.NextVal();
		ins[c].vibs = t.NextVal();
		ins[c].vibd = t.NextVal();
		ins[c].pwms = t.NextVal();
		ins[c].pwmd = t.NextVal();
		ins[c].res = t.NextVal();
		ins[c].sps = t.NextVal();
		if (ins[c].sps < 1){ fprintf(stderr,"ins[%d].sps < 1\n",c); return init = JNG_ERROR;}
		ins[c].len = t.NextVal();
		if (ins[c].len < 1){ fprintf(stderr,"ins[%d].len < 1\n",c); return init = JNG_ERROR;}
		
#ifdef TARGET_EDITOR
		ins[c].steps = (InsStep*) CALLOC( INSSTEPS , sizeof(InsStep));
#else
		ins[c].steps = (InsStep*) CALLOC( (ins[c].len & 0x7f) , sizeof(InsStep));
#endif

		for (c1=0; c1 < (ins[c].len & 0x7f); c1++){
			uint8 temp = t.NextVal();
			ins[c].steps[c1].relative = 	(temp & 0x80) >> 7;
			ins[c].steps[c1].wform =		(temp & 0xf);
			ins[c].steps[c1].note = t.NextVal();
			//fprintf(stderr,"%3d - %3d\n", ins[c].steps[c1].mask, ins[c].steps[c1].note );
		}
	}
	
	/////////////////// Load BreakPoints
	bpcount = 0;
	breakpcount = 0;
	breakpcount = t.NextVal();

#ifdef TARGET_EDITOR
		breakpoints = (BreakPoint*) CALLOC( BREAKPOINTS , sizeof(BreakPoint*));
#else
		breakpoints = (BreakPoint*) CALLOC( breakpcount , sizeof(BreakPoint*));
#endif

	for (int c = 0 ; c < breakpcount ; c++){
		breakpoints[c].PAL = t.NextVal(4);
		breakpoints[c].command = t.NextVal(4);
	}
	
	/////////////////// Load Names
	name = strdup(t.NextString());
//	printf("songname = %s\n",name);

	for (int c = 0 ; c < channelcount ; c++){
		chan[c].SetName(t.NextString());
	}

	for (int c = 0 ; c < partcount ; c++){
		parts[c].SetName(t.NextString());
//		printf("part[%d] = %s\n",c,parts[c].name);
	}

	for (int c = 1 ; c < instrumentcount ; c++){
		ins[c].SetName(t.NextString());
//		printf("ins[%d] = %s\n",c,ins[c].name);
	}

	/////////////////// Create Playaz
#ifdef TARGET_EDITOR
	p = (Player **)CALLOC(CHN , sizeof(Player *));
#else
	p = (Player **)CALLOC(channelcount , sizeof(Player *));
#endif
	for (int c = 0; c< channelcount; c++){
		p[c] = new Player(this,chan + c);
	}		
	
	return InitCheck();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT status_t Song::InitCheck(){
// check for big instruments
	for (int c = 0 ; c < partcount ; c++){
		for (int c2 = 0 ; c2 < parts[c].len ; c2++){
			if (parts[c].steps[c2].ins >= instrumentcount){
				fprintf(stderr,"ERROR-Ins-# too big: part #%d - row #%d\n",c,c2);
				return JNG_ERROR;
			}
		}
	}

	return init;
}

_EXPORT void Song::FreeSong()
{
	int c;
	//////////////////////////////////////////////////////// CHANNELS
	for (c=0; c < channelcount; c++)
	{
		delete p[c];
		free( chan[c].name );
		free( chan[c].steps );
	}
	if (chan){
		free( chan );
		free( p );
		chan = 0L;
		channelcount = 0;
	}
	
	//////////////////////////////////////////////////////// PARTS
	for (c=1; c < partcount; c++){
		free( parts[c].name );
		free( parts[c].steps );
	}
	if (parts){
		free( parts );
		parts = 0L;
		partcount = 0;
	}
	
	//////////////////////////////////////////////////////// INSTRUMENTS
	for (c = 0; c < instrumentcount; c++)
	{
		free( ins[c].name );
		free( ins[c].filter );
		free( ins[c].amp );
		free( ins[c].steps );
	}
	if (ins){
		free( ins );
		ins = 0L;
		instrumentcount = 0;
	}
	
	free( breakpoints );
	breakpcount = 0;
}

_EXPORT Song::~Song()
{
	free( N2F );
	free( R2F );
	FreeSong();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT void Song::Play(	float *buff,
					uint32 SampleCount	)
{
// ----------------------
// SoundPart
//fprintf(stderr, "\nSong::Play()\n");

	float *bd = buff;
//	float *bs = p[0]->Buffer();

// skall köra allas buffrar
	while (SampleCount > 0){
//fprintf(stderr, "\t[B]samplecount:%d, smp_left:%d, bs:%p\n\n", SampleCount, smp_left, bs);
		uint32 lim;
		if (SampleCount < smp_left){
			lim = SampleCount;
			SampleCount = 0;
			smp_left -= lim;
		}else{
			lim = smp_left;
			smp_left = 0;
			SampleCount -= lim;
		}
		
		while ( lim ){
			*bd = *bs;
			bd++;
			bs++;
			lim--;
		}
//fprintf(stderr, "\t[A]samplecount:%d, smp_left:%d, bs:%p\n\n", SampleCount, smp_left, bs);
		
		if (smp_left < 1){
			if (! p[0]->NextBuffer() ) memset( p[0]->Buffer(), 0, spspal << 2 );			
			for (int c = 1 ; c < channelcount ; c++){
				if (p[c]->NextBuffer()) MemAdd(p[0]->Buffer(), p[c]->Buffer(), spspal);
			}
			PALs++;
			smp_left = spspal;
			bs = p[0]->Buffer();
		}
		
	}
	
// ----------------------
// BreakPointPart
	if (bpcount < breakpcount)	// kolla att det finns breakpoints kvar
		if (breakpoints[bpcount].PAL < PALs){
			Break(breakpoints[bpcount].command);
			bpcount++;
		}
	
}

inline void Song::MemAdd( float *d, float *s, uint32 count)
{
	while ( count-- ){
		*d += *s;
		d++;
		s++;
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT void Song::Break(uint32 command)
{
	fprintf(stderr,"Song::Break(%lx)\n",command);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT uint32 Song::GetPos()
{
	return PALs;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT void Song::SetPos(uint32 pals)
{
	smp_left = 0;
	PALs = pals;

	// hitta breakpoint
	for (bpcount = 0 ; (bpcount < breakpcount) && breakpoints[bpcount].PAL < PALs; bpcount ++);

	for (int c=0;c<channelcount;c++){
		// set seqcount
		uint32	sum = 0;
		uint8	seqcount = 0, stepcount = 0, palcount = 0, sps = 0, len = 0;
		
		while (sum <= PALs){
			len = parts[chan[c].steps[seqcount].part].len;
			sps = parts[chan[c].steps[seqcount].part].sps;
			sum += sps * len;

			seqcount++;
			if (seqcount >= chan[c].len) seqcount=0;
		}

		if (seqcount > 0)
			seqcount--;
		else
			seqcount = chan[c].len - 1;

		len = parts[chan[c].steps[seqcount].part].len;
		sps = parts[chan[c].steps[seqcount].part].sps;

		sum -= sps * len;

			
		stepcount	= (PALs - sum) / sps;
		palcount	= (PALs - sum) % sps;

		p[c]->JumpPos(seqcount,stepcount,palcount);
//		printf("PALs[%d] == Seq[%d] Step[%d] Rest[%d]\n",PALs,seqcount,stepcount,palcount);
	}
}


void _____________________________________();
void _____________________________________(){};

#ifdef TARGET_EDITOR

_EXPORT void Song::PartPlay( float *buff, uint32 SampleCount )
{
// ----------------------
// SoundPart
//fprintf(stderr, "\nSong::PartPlay()\n");

	float *pbd = buff;

	while (SampleCount > 0){
//fprintf(stderr, "\t[B]samplecount:%d, smp_left:%d, bs:%p\n\n", SampleCount, psmp_left, pbs);
		uint32 lim;
		if (SampleCount < psmp_left){
			lim = SampleCount;
			SampleCount = 0;
			psmp_left -= lim;
		}else{
			lim = psmp_left;
			psmp_left = 0;
			SampleCount -= lim;
		}
		
		while ( lim ){
			*pbd = *pbs;
			pbd++;
			pbs++;
			lim--;
		}
//fprintf(stderr, "\t[A]samplecount:%d, smp_left:%d, bs:%p\n\n", SampleCount, psmp_left, pbs);
		
		if (psmp_left < 1){
			if (! pp->NextBuffer() ) memset( pp->Buffer(), 0, spspal << 2 );
			psmp_left = spspal;
			pbs = pp->Buffer();
		}
		
	}
};

_EXPORT void Song::SetPartToPlay( uint8 part )
{
	pchanstep.part = part;
	pp->JumpPos(0,0,0);
};

_EXPORT void Song::AllocDefSong()
{
	FreeSong();
	ins = (Ins*) CALLOC( INS , sizeof(Ins));
	instrumentcount = 2;

// dummy ins
	ins[0].name = strdup("dummy");
	ins[0].filterpoints	= 1;
	ins[0].amppoints	= 1;
	ins[0].filter		= (TimeLev*) CALLOC( 1 , sizeof(TimeLev));
	ins[0].amp			= (TimeLev*) CALLOC( 1 , sizeof(TimeLev));

	ins[0].sps = 30;
	ins[0].len = 1;
	
	ins[0].steps = (InsStep*) CALLOC(1 , sizeof(InsStep));
// end of dummy ins

	ins[1].filter = (TimeLev*) CALLOC( INSPOINTS , sizeof(TimeLev));
	ins[1].filterpoints = 1;
	ins[1].filter[0].time =	1;
	ins[1].filter[0].lev =	250;

	ins[1].amp = (TimeLev*) CALLOC( INSPOINTS , sizeof(TimeLev));
	ins[1].amppoints = 1;
	ins[1].amp[0].time =	1;
	ins[1].amp[0].lev =	250;

	ins[1].steps = (InsStep*) CALLOC( INSSTEPS , sizeof(InsStep));
	ins[1].steps[0].relative = 1;
	ins[1].steps[0].wform = 3;
	ins[1].steps[0].note = 1;

	ins[1].vibs = 0;
	ins[1].vibd = 0;
	ins[1].pwms = 0;
	ins[1].pwmd = 0;
	ins[1].res = 30;
	ins[1].sps = 12;
	ins[1].len = 1;
	ins[1].SetName("AutoInstrument");
	
	parts = (Part*) CALLOC( PARTS , sizeof(Part));
	partcount = 1;
	parts[0].steps = (Step*) CALLOC( PARTSTEPS , sizeof(Step));
	parts[0].sps = 6;
	parts[0].len = 8;
	parts[0].steps[0].ins = 1;
	parts[0].steps[0].eff = 0;
	parts[0].steps[0].note = 100;
	parts[0].SetName("AutoPart");


	chan = (Channel*) CALLOC( CHN , sizeof(Channel));
	channelcount = 1;
	chan[0].len = 1;
	chan[0].steps = (ChStep*) CALLOC( CHNSTEPS , sizeof(ChStep));
	chan[0].SetName("AutoChannel");

	p = (Player **)CALLOC(CHN , sizeof(Player *));
	p[0] = new Player(this,chan + 0);

	breakpoints = (BreakPoint*) CALLOC( BREAKPOINTS , sizeof(BreakPoint));
	bpcount = 0;
	breakpcount = 0;


	spspal = 882;
	name = strdup("AutoSong");
	return;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT uint32 Song::SimpLen()
{
	// beräkna sångens längd
	int bufc = 0;	// number of buffers
	for (int c = 0 ; c < chan[0].len ; c++){
		Part *tpart = parts + (chan[0].steps[c].part);
		bufc += tpart->sps * tpart->len;			
	}
	return bufc;
}

_EXPORT uint32 Song::Sps()
{
	return spspal;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT void Song::Save(char *filename,bool bin)
{
	txt t(filename,true);
	if (JNG_OK!=t.InitCheck()) return;
	
	int c, c1;
	
	t.IsBin(bin);
	
	if (bin) t.Val('SWTD',4);
	
	t.Comment("SWTT\n// SAWTEETH Generated\n// spspal & len\n");
	t.Val(spspal,2);
	
	// channels
	t.Comment("\n\n\n// CHANNELS -----------\n");
	
	t.Comment("// # of channels\n");
	t.Val(channelcount);
	
	for (c=0; c < channelcount; c++){
		t.Comment("\n// CHANNEL ---------- ");
		t.Comment(c);
		t.Comment(" --- ");
		t.Comment(chan[c].name);
		t.Comment("\n// len\n");
		t.Val(chan[c].len);
		
		//	seq
		t.Comment("// part-transp\n");
		for (c1=0; c1 < chan[c].len; c1++){
			t.Val(chan[c].steps[c1].part);
			t.Val((uint8)chan[c].steps[c1].transp);
		}
	}
	
	//	parts
	t.Comment("\n\n\n// PARTS --------------\n");
	t.Comment("// # of parts\n");
	t.Val(partcount);
	
	for (c=0; c < partcount; c++){
		t.Comment("\n// PART ------------- ");
		t.Comment(c);
		t.Comment(" --- ");
		t.Comment(parts[c].name);
		t.Comment("\n// sps & len\n");
		t.Val(parts[c].sps);
		t.Val(parts[c].len);
		t.Comment("// ins - eff - note\n");
		for (c1=0; c1 < parts[c].len; c1++){
			t.Val(parts[c].steps[c1].ins);
			t.Val(parts[c].steps[c1].eff);
			t.Val(parts[c].steps[c1].note);
		}
	}
	
	// instruments
	//fprintf(stderr,"INS\n");
	t.Comment("\n\n\n// INSTRUMENTS\n");
	t.Comment("// # of ins\n");
	t.Val(instrumentcount-1);
	
	for (c = 1; c < instrumentcount; c++){
		t.Comment("\n// INS -------------- ");
		t.Comment(c);
		t.Comment(" --- ");
		t.Comment(ins[c].name);
		t.Comment("\n// points - fadsr time-lev\n");
		t.Val(ins[c].filterpoints);
		for (c1= 0; c1 < ins[c].filterpoints; c1++){
			t.Val(ins[c].filter[c1].time);
			t.Val(ins[c].filter[c1].lev);
		}
		
		t.Comment("// points - adsr time-lev\n");
		t.Val(ins[c].amppoints);
		for (c1= 0; c1 < ins[c].amppoints; c1++){
			t.Val(ins[c].amp[c1].time);
			t.Val(ins[c].amp[c1].lev);
		}

		t.Comment("// filtermode, clipping\n");
		t.Val(ins[c].filtermode);
		t.Val(ins[c].clipmode);

		t.Comment("// vibs, vibd, pwms, pwmd\n");
		t.Val(ins[c].vibs);
		t.Val(ins[c].vibd);
		t.Val(ins[c].pwms);
		t.Val(ins[c].pwmd);
		t.Comment("// res\n");
		t.Val(ins[c].res);
		t.Comment("// sps\n");
		t.Val(ins[c].sps);
		t.Comment("// len bit 7 means loop\n");
		t.Val(ins[c].len);
		
		t.Comment("// track mask - note\n");
		for (c1=0; c1 < (ins[c].len & 0x7f); c1++){
			uint8 temp;
			temp = 	ins[c].steps[c1].relative << 7;
			temp |=	ins[c].steps[c1].wform;

			t.Val(temp);
			t.Val(ins[c].steps[c1].note);
		}
	}

	t.Comment("\n\n\n// breakpoints, PAL - command\n");
	t.Val(breakpcount);
	for (int c = 0 ; c < breakpcount ; c++){
		t.Val( breakpoints[c].PAL , 4 );
		t.Val( breakpoints[c].command , 4 );
	}

	if (name) t.String(name);

	for (c=0; c < channelcount; c++)
		if (chan[c].name) t.String(chan[c].name);

	for (c=0; c < partcount; c++)
		if (parts[c].name) t.String(parts[c].name);

	for (c = 1; c < instrumentcount; c++)
		if (ins[c].name) t.String(ins[c].name);

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT	uint8 Song::ChannelCount()		{ return channelcount;		}
_EXPORT	uint8 Song::PartCount()			{ return partcount;			}
_EXPORT	uint8 Song::InsCount()			{ return instrumentcount;	}
_EXPORT	uint8 Song::BreakPointCount()	{ return breakpcount;		}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT Channel *Song::GetChannel( uint8 c)
{
	if (c < channelcount)
		return chan + c;
	else
		return NULL;
	
}


_EXPORT uint8 Song::NewChannel()
{
	if (channelcount >= CHN) return 0;

	chan[channelcount].steps = (ChStep*) CALLOC( CHNSTEPS , sizeof(ChStep));
	chan[channelcount].name = strdup( "Unnamed Channel" );
	chan[channelcount].len = 1;

	p[channelcount] = new Player(this,chan + channelcount);
	channelcount++;
	return channelcount-1;		
}

_EXPORT status_t Song::RemoveChannel(uint8 c)
{
	// om det är den sista
	if (channelcount == 1) return JNG_ERROR;

	channelcount--;
	delete p[channelcount];

	free( parts[c].name );
	free( parts[c].steps );

	// om den ligger sist
	if (c == channelcount) return JNG_OK;

	chan[c] = chan[ channelcount ];

	return JNG_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT Part *Song::GetPart( uint8 part )
{
	if (part < partcount)
		return parts + part;
	else
		return parts;
}


_EXPORT uint8 Song::NewPart( Part* copy_me )
{
	if (partcount>= PARTS ) return 0;

	parts[partcount].steps = (Step*) CALLOC( PARTSTEPS, sizeof(Step));
	memset(parts[partcount].steps,0,PARTSTEPS *sizeof(Step));
	parts[partcount].name = strdup("Unnamed Part");
	parts[partcount].sps = 6;
	parts[partcount].len = 8;
	
	if ( copy_me != NULL ){
		parts[partcount].sps = copy_me->sps;
		parts[partcount].len = copy_me->len;
		memcpy(parts[partcount].steps,copy_me->steps,parts[partcount].len *sizeof(Step));
	}

	partcount++;
	return partcount-1;
}


_EXPORT status_t Song::RemovePart(uint8 part)
{
	// om det är den sista
	if (partcount == 1) return JNG_ERROR;

	// check if part is used
	for (int c = 0; c < channelcount; c++){
		for (int c2 = 0; c2 < chan[c].len; c2++){
			if (part == chan[c].steps[c2].part) return JNG_ERROR;
		}
	}
	partcount--;

	// om den ligger i mitten
	free( parts[part].name );
	free( parts[part].steps );

	// om den ligger sist
	if (part == partcount) return JNG_OK;

	parts[part] = parts[ partcount ];

	// replace
	for (int c = 0; c < channelcount; c++)
		for (int c2 = 0; c2 < chan[c].len; c2++)
			if (partcount == chan[c].steps[c2].part)
				chan[c].steps[c2].part = part;

	return JNG_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT Ins *Song::GetIns( uint8 i ){
	if (i < instrumentcount)
		return ins + i;
	else
		return ins;
}


_EXPORT uint8 Song::NewIns( Ins *copy_me ){
	if ( instrumentcount >= INS ) return 0;

	ins[instrumentcount].name = strdup( "New Ins" );
	ins[instrumentcount].steps = (InsStep*) CALLOC( INSSTEPS ,sizeof(InsStep));
	ins[instrumentcount].len = 0;

	ins[instrumentcount].amp = (TimeLev*) CALLOC( INSPOINTS ,sizeof(TimeLev));
	ins[instrumentcount].amppoints = 1;

	ins[instrumentcount].filter = (TimeLev*) CALLOC( INSPOINTS ,sizeof(TimeLev));
	ins[instrumentcount].filterpoints = 1;

	if ( copy_me != NULL ){
		ins[instrumentcount].sps = copy_me->sps;
		ins[instrumentcount].len = copy_me->len;
		memcpy(ins[instrumentcount].steps,copy_me->steps,ins[instrumentcount].len *sizeof(InsStep));
		memcpy(ins[instrumentcount].amp,copy_me->amp,ins[instrumentcount].amppoints *sizeof(TimeLev));
		memcpy(ins[instrumentcount].filter,copy_me->filter,ins[instrumentcount].filterpoints *sizeof(TimeLev));
	}

	instrumentcount++;
	return instrumentcount-1;
}

_EXPORT status_t Song::RemoveIns(uint8 l_ins)
{
	// om det är den sista
	if (instrumentcount == 2) return JNG_ERROR;

	// check if ins is used
	for (int c = 0; c < partcount; c++)
		for (int c2 = 0; c2 < parts[c].len; c2++)
			if (l_ins == parts[c].steps[c2].ins) return JNG_ERROR;

	instrumentcount--;

	// om den ligger i mitten
	free( ins[l_ins].name );
	free( ins[l_ins].steps );
	free( ins[l_ins].amp );
	free( ins[l_ins].filter );

	// om den ligger sist
	if (l_ins == instrumentcount) return JNG_OK;

	ins[l_ins] = ins[ instrumentcount ];

	// replace
	for (int c = 0; c < partcount; c++)
		for (int c2 = 0; c2 < parts[c].len; c2++)
			if (instrumentcount == parts[c].steps[c2].ins)
				parts[c].steps[c2].ins = l_ins;

	return JNG_OK;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
_EXPORT status_t Song::Optimize(){
	// ta bort parts som ej används
	for (int c = partcount-1 ; c >=0 ; c--){
		if (JNG_OK==RemovePart(c)) fprintf(stderr,"p%2d\n",c);
	}

	// ta bort instruments
	for (int c = instrumentcount-1 ; c > 0 ; c--){
		if (JNG_OK==RemoveIns(c)) fprintf(stderr,"i%2d\n",c);
	}
	return JNG_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ______________________________________();
void ______________________________________(){};

#define FIND int c = 0; while ( ( c < breakpcount ) && ( PAL > breakpoints[c].PAL ) ) c++;

_EXPORT uint8 Song::GetBreakPointIndex( uint32 PAL ){
	FIND
	// check if exists
	if ( c >= breakpcount ) return 0;
	if ( PAL != breakpoints[c].PAL ) return 0;

	return c;
}

_EXPORT uint32 Song::GetBreakPointByIndex( uint8 index ){
//	if ( index < breakpcount ) return 0;
	return breakpoints[index].command;
}

_EXPORT void Song::SetBreakPointByIndex( uint8 index, uint32 command ){
//	if ( index < breakpcount ) return;
	breakpoints[index].command = command;
}

_EXPORT uint32 Song::GetBreakPoint( uint32 PAL ){
	FIND

	// check if exists
	if ( c >= breakpcount ) return 0;
	if ( PAL != breakpoints[c].PAL ) return 0;

	return breakpoints[c].command;
}

_EXPORT void Song::NewBreakPoint( uint32 PAL, uint32 command ){
	puts("NewBP");
	FIND

	// check if exists
	if (PAL == breakpoints[c].PAL){ 
		puts("case 0");
		breakpoints[c].command = command;
		return;
	}

	if (c >= breakpcount)
	{
		puts("case 1");
		breakpcount++;
		breakpoints[c].command = command;	
		return;
	}
	
	// flytta efter
	for (int d = breakpcount; d > c ; d--){
		breakpoints[d].PAL = breakpoints[d-1].PAL;
		breakpoints[d].command = breakpoints[d-1].command;
	}

	breakpcount++;

	breakpoints[c].PAL = PAL;
	breakpoints[c].command = command;
}

_EXPORT void Song::RemoveBreakPoint( uint32 PAL ){
	FIND

	// check if exists
	if (c >= breakpcount) return;
	if (PAL != breakpoints[c].PAL) return;

	// flytta efter
	breakpcount--;
	for (;c < breakpcount; c++){
		breakpoints[c].command = breakpoints[c+1].command;
		breakpoints[c].PAL = breakpoints[c+1].PAL;
	}
}


#endif

