#include <stdio.h>
#include "song.h"

void PrintBPMTable(uint32 sps,uint8 speed, uint8 len);
void PrintBPMTable(uint32 sps,uint8 speed, uint8 len)
{
	//				1	2				3					others
	uint8 pri[] = {	1,	4,8,2,16,32,	6,3,12,24,18,28,	5,10,20,14,7,15,9,25,30	};
	
	
	fprintf(stderr,"bpm guess (speed = %d), most likely first.\n",speed);

	for (int c2 = 0 ; c2 < sizeof(pri) ; c2++){
		float temp = (44100.0*60.0)/(float)(speed*pri[c2]*sps);
		if (((len % pri[c2]) == 0) && ((temp < 200) && (temp > 70))){
			fprintf(stderr,"(%d)%.3f\n",pri[c2],temp);
		}
	}
}

void Big32(uint32 n);
void Big32(uint32 n){printf("%c%c%c%c",n>>24,(n>>16)&255,(n>>8)&255,n&255);}
void Big16(uint16 n);
void Big16(uint16 n){printf("%c%c",n>>8,n&255);}

// writes a AIFF header to a mono 44.1kHz soundstream
void PrintHeader(uint32 samplecount);
void PrintHeader(uint32 samplecount)
{
	uint32 bytespersample = 2;
	Big32( 'FORM' );
	Big32( (samplecount * bytespersample) + 46 );
	Big32( 'AIFF' );
	Big32( 'COMM' );

	Big32( 18 );
	Big16( 0x0001);			// channelcount!!
	Big32( samplecount );
	Big16( 8 * bytespersample );		// bitcount!!
	
	Big16( 0x400E );		// z = 400e - 400e ????
	Big16( 0xAC44 );		// rate = ac44 >> z
	Big16( 0x0000 );
	Big16( 0x0000 );

	Big16( 0x0000 );

	Big32( 'SSND' );
	Big32( (samplecount * bytespersample) + 8 );

	Big32( 0x00000000 );
	Big32( 0x00000000 );
}

int main(int argc, char **argv)
{
	if (argc != 2){
		fprintf(stderr,"stPlay filename\nWill output an aiff-file on stdout\n");
		return -1;
	}
	float max = 0, min = 0;

	Song *s = new Song(argv[1]);

	PrintBPMTable(s->Sps(), s->GetPart(0)->sps, s->GetPart(0)->len);

	if (0 == s->InitCheck()){
		int sbufs = s->Sps();
		int bufc = s->SimpLen();
//fprintf(stderr,"sbufs = %d\n",sbufs);
//fprintf(stderr,"bufc = %d\n",bufc);
		float *sbuf = new float[sbufs];
						
		PrintHeader( bufc * sbufs );

		for(int bigc = 0 ; bigc < bufc ; bigc++){
			s->Play(sbuf,sbufs);
			for (int c = 0 ; c< sbufs; c++){

				//max, min and clipping
				if (sbuf[c] > max) max = sbuf[c];if (sbuf[c] < min) min = sbuf[c];
				if (sbuf[c] > 0.999) sbuf[c] = 0.999; else if (sbuf[c] < -0.999) sbuf[c] = -0.999;

				int16 tmp = (int16)(30000.0 * sbuf[c]);
				putchar(tmp >> 8);
				putchar(tmp & 255);
			}
		}

fprintf(stderr,"\nmin = %f, max = %f\n",min,max);
		delete sbuf;
		delete s;
	}
}
