#include <stdio.h>
#include <OS.h>
#include <math.h>

#include "WarPathfinder.h"
#include "MyPathfinder.h"
#include "WarMap.h"
#include "UnitList.h"
#include "WarHeap.h"
#include "WarPath.h"

MyPathfinder::MyPathfinder(WarMap *war_map,UnitList *ul,uint16 *tmp_map):WarPathfinder(war_map,ul,tmp_map)
{
	map=war_map; // copying the WarMap pointer
	ulist=ul;    // copying the unitlist pointer
	dist_map=tmp_map;
	path=new WarPath(400);
}

MyPathfinder::~MyPathfinder()
{
	delete path;
}

void MyPathfinder::NewPosition(uint16 x,uint16 y)
{
	unit_x=x; // updating position
	unit_y=y;
}

void MyPathfinder::NewDestination(uint16 dest_x,uint16 dest_y,const uint8 spds[32])
{
	uint8 i;
	WarHeap* sq; // start queue
	WarHeap* dq; // destination queue
	uint16 x,y;
	uint32 offs,new_offs;
	bool goal_reached=false;
	uint32 data;
	path_found=false;

	timeout=0;

	// removes the old path
	path->Empty();

	// if the destination block is occupied, do nothing
	if (CheckBlock(dest_x,dest_y)) return;

//	fprintf(stderr,"MyPathfinder testing direct line\n");
	// handles the special case is there's nothing in the way
	if (!CheckLine(dest_x,dest_y,unit_x,unit_y))
	{
		DrawLinePath(path,dest_x,dest_y,unit_x,unit_y);
		path_found=true;
		return;
	}

	// clear the distance map
	memset(dist_map,0xff,map->BlockCount()<<1);

	// we add different values to the distance map depending on if we go diagonally
	static add[2]={5,7};

	// construct the two queues (destination-queue and start-queue)
	sq=new WarHeap(700);
	dq=new WarHeap(700);

	// initializing the start-queue
	dist_map[(unit_y<<map->WidthShift())+unit_x]=0;
	sq->Push(unit_x,unit_y,0);

	// initializing the destination-queue
	dist_map[(dest_y<<map->WidthShift())+dest_x]=32768;
	dq->Push(dest_x,dest_y,0);

//	fprintf(stderr,"MyPathfinder starting the queues!\n");

	while (true)
	{
	// ************** Start queue **************
		// if we're out of blocks in the start queue we couldn't reach the destination
//		fprintf(stderr,"sq->Pop()\n");
		if (!sq->Pop(&x,&y)) break;

		// look in all directions
		offs=(y<<map->WidthShift())+x;
//		fprintf(stderr,"offs:%d\n",offs);
		for (i=0;i<8;i++)
		{
//			fprintf(stderr,"calculating the new_offs for angle %d\n",i);
			// Make sure the square is on the map
			if ((x+Cos(i)<0)||(x+Cos(i)>=map->Width())||(y+Sin(i)<0)||(y+Sin(i)>=map->Height())) continue;
			new_offs=((y+Sin(i))<<map->WidthShift())+x+Cos(i);
//			fprintf(stderr,"the new_offs is within the bounds, %d\n",new_offs);
			// See if we have been here before
			if (dist_map[new_offs]==0xffff)
			{
//				fprintf(stderr,"we haven't been here!\n");
				if (!CheckBlock(new_offs))
				{
					dist_map[new_offs]=dist_map[offs]+add[i&1];
//					fprintf(stderr,"sq->Push()\n");
					sq->Push(x+Cos(i),y+Sin(i),(dist_map[new_offs]) + (Dist(dest_x-x-Cos(i),dest_y-y-Sin(i))<<1));
				}
			}
			else
				// if we have been here before, have the destination queue been here?
				if (dist_map[new_offs]&0x8000) {goal_reached=true;break;}
		}

		if (goal_reached) break;

	// ************** Destination queue **************
		// if we're out of blocks in the destination queue there's no connection with the start point
		if (!dq->Pop(&x,&y)) break;

		// look in all directions
		offs=(y<<map->WidthShift())+x;
		for (i=0;i<8;i++)
		{
			// Make sure the square is on the map
			if ((x+Cos(i)<0)||(x+Cos(i)>=map->Width())||(y+Sin(i)<0)||(y+Sin(i)>=map->Height())) continue;
			new_offs=((y+Sin(i))<<map->WidthShift())+x+Cos(i);
			// See if we have been here before
			if (dist_map[new_offs]==0xffff)
			{
				if (!CheckBlock(new_offs))
				{
					dist_map[new_offs]=dist_map[offs]+add[i&1];
					dq->Push(x+Cos(i),y+Sin(i),(0xffff-dist_map[new_offs]) + (Dist(unit_x-x-Cos(i),unit_y-y-Sin(i))<<1));
				}
			}
			else
				// if we have been here before, have the start queue been here?
				if (!dist_map[new_offs]&0x8000) {goal_reached=true;break;}
		}
		if (goal_reached) break;

	}
	delete dq;
	delete sq;

//	fprintf(stderr,"MyPathfinder finished!\n");

	if (goal_reached)
	{
//		fprintf(stderr,"Goal was reached\n");
		WarPath *dest_path=new WarPath(400);
		WarPath *start_path=new WarPath(400);
	// If we reached the goal do the backtrace
		static uint16 min_angle,min_dist;

		dest_path->AddCoord(x,y); // add the current position

		while (x!=dest_x || y!=dest_y)
		{
			min_dist=0xffff;
			min_angle=0;
			for (i=0;i<8;i++)
			{
				if (x+Cos(i)>=0 && x+Cos(i)<map->Width() && y+Sin(i)>=0 && y+Sin(i)<map->Height())
				{
					new_offs=((y+Sin(i))<<map->WidthShift())+x+Cos(i);
					if (dist_map[new_offs]<min_dist && dist_map[new_offs]>32767)
					{
						min_dist=dist_map[new_offs];
						min_angle=i;
					}
				}
			}
			x+=Cos(min_angle);
			y+=Sin(min_angle);
			dest_path->AddCoord(x,y);
		}

		while (dest_path->Coord(&x,&y))
		{
			start_path->AddCoord(x,y);
			dest_path->Remove();
		}

		while (x!=unit_x || y!=unit_y)
		{
			min_dist=65535;
			min_angle=0;
			for (i=0;i<8;i++)
			{
				if (x+Cos(i)>=0 && x+Cos(i)<map->Width() && y+Sin(i)>=0 && y+Sin(i)<map->Height())
				{
					new_offs=((y+Sin(i))<<map->WidthShift())+x+Cos(i);
					if (dist_map[new_offs]<min_dist)
					{
						min_dist=dist_map[new_offs];
						min_angle=i;
					}
				}
			}
			x+=Cos(min_angle);
			y+=Sin(min_angle);
			start_path->AddCoord(x,y);
		}

//		fprintf(stderr,"MyPathfinder direct line pathfinding\n");

		// Direct-line pathfinding (find lines)
		int low_counter=0,hi_counter=start_path->CountCoords()-1;
		path->AddCoord(dest_x,dest_y);
		start_path->Coord(&x,&y,low_counter);
		start_path->Coord(&unit_x,&unit_y,hi_counter);
		while (low_counter!=hi_counter)
		{
			while (CheckLine(x,y,unit_x,unit_y))
			{
				hi_counter-=1;
				start_path->Coord(&unit_x,&unit_y,hi_counter);
			}
			DrawLinePath(path,x,y,unit_x,unit_y);
			low_counter=hi_counter;
			hi_counter=start_path->CountCoords()-1;
			start_path->Coord(&x,&y,low_counter);
			start_path->Coord(&unit_x,&unit_y,hi_counter);
		}
		delete start_path;
		delete dest_path;
		path_found=true;
	}
}

uint8 MyPathfinder::FindPath()
{
	if (!path_found) return PATH_CANNOT_ARRIVE;
	uint16 x,y;
	if (path->Coord(&x,&y))
	{
		while (unit_x==x && unit_y==y)
		{
			path->Remove();
			if (!path->Coord(&x,&y))
				return PATH_ARRIVED;
		}
		if (!CheckBlock(x,y))
		{
			timeout=30; // we wait for 30 cycles
			// calculate direction (unit_x,uinit_y)->(x,y)
			return ATan(x-unit_x,y-unit_y);
		}
		else
		{
			timeout--; // we will not wait forever
			if (timeout==1)
				return PATH_CANNOT_ARRIVE;
			return PATH_WAIT;
		}
	}
	else return PATH_ARRIVED;
}

bool MyPathfinder::CheckBlock(uint16 x,uint16 y)
{
	uint16 block=map->GetBlock((y<<map->WidthShift())+x);
	if (x==unit_x && y==unit_y) return false;
	return (ulist->IsBlockOccupied(x,y));
}

bool MyPathfinder::CheckBlock(uint32 offs)
{
	uint16 block=map->GetBlock(offs);
	if ((unit_y<<map->WidthShift())+unit_x==offs) return false;
	return (ulist->IsBlockOccupied(offs));
}

// Bresenham's Line Algorithm
bool MyPathfinder::CheckLine(uint16 x1,uint16 y1,uint16 x2,uint16 y2)
{
	static int i,deltax,deltay,numpixels,d,dinc1,dinc2,x,xinc1,xinc2,y,yinc1,yinc2;

  // Calculate deltax and deltay for initialisation
	if (x1>x2) deltax=x1-x2; else deltax=x2-x1;
	if (y1>y2) deltay=y1-y2; else deltay=y2-y1;

  // Initialize all vars based on which is the independent variable
  if (deltax>=deltay)
  {
		// x is independent variable
		numpixels=deltax+1;
		d=(deltay<<1)-deltax;
		dinc1=deltay<<1;
		dinc2=(deltay-deltax)<<1;
		xinc1=1;
		xinc2=1;
		yinc1=0;
		yinc2=1;
	}
  else 
	{
		// y is independent variable
		numpixels=deltay+1;
		d=(deltax<<1)-deltay;
		dinc1=deltax<<1;
		dinc2=(deltax-deltay)<<1;
		xinc1=0;
		xinc2=1; 
		yinc1=1;
		yinc2=1; 
	}

	// Make sure x and y move in the right directions
  if (x1>x2)
  {
		xinc1=-xinc1;
		xinc2=-xinc2;
	}
  if (y1>y2)
	{
		yinc1=-yinc1;
		yinc2=-yinc2;
	}

	// Start checking @(x1,y1)
	x=x1;
	y=y1;

	for (i=0;i<numpixels;i++)
	{
		if (CheckBlock(x,y)) return true;
		if (d<0)
		{
			d+=dinc1;
			x+=xinc1;
			y+=yinc1; 
		}
		else
		{
			d+=dinc2;
			x+=xinc2;
			y+=yinc2; 
		}
	}
	return false;
}

// Bresenham's Line Algorithm
void MyPathfinder::DrawLinePath(WarPath *path,uint16 x1,uint16 y1,uint16 x2,uint16 y2)
{
	static int i,deltax,deltay,numpixels,d,dinc1,dinc2,x,xinc1,xinc2,y,yinc1,yinc2;

	// Calculate deltax and deltay for initialisation
	if (x1>x2) deltax=x1-x2; else deltax=x2-x1;
	if (y1>y2) deltay=y1-y2; else deltay=y2-y1;

	// Initialize all vars based on which is the independent variable
	if (deltax>=deltay)
	{
		// x is independent variable
		numpixels=deltax;
		d=(deltay<<1)-deltax;
		dinc1=deltay<<1;
		dinc2=(deltay-deltax)<<1;
		xinc1=1;
		xinc2=1;
		yinc1=0;
		yinc2=1;
	}
	else 
	{
		// y is independent variable
		numpixels=deltay;
		d=(deltax<<1)-deltay;
		dinc1=deltax<<1;
		dinc2=(deltax-deltay)<<1;
		xinc1=0;
		xinc2=1; 
		yinc1=1;
		yinc2=1; 
	}

	// Make sure x and y move in the right directions
	if (x1>x2)
	{
		xinc1=-xinc1;
		xinc2=-xinc2;
	}
	if (y1>y2)
	{
		yinc1=-yinc1;
		yinc2=-yinc2;
	}

	// Start adding @(x1,y1)
	x=x1;
	y=y1;

	// Add the path
	for (i=0;i<numpixels;i++)
	{
		path->AddCoord(x,y);
		if (d<0)
		{
			d+=dinc1;
			x+=xinc1;
			y+=yinc1; 
		}
		else
		{
			d+=dinc2;
			x+=xinc2;
			y+=yinc2; 
		}
	}
}
