#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <OS.h>
#include <math.h>
#include <ctype.h>
#include <Entry.h>
#include <File.h>
#include <getopt.h>
#include "libprof.h"

#include "WarMap.h"
#include "MyPathfinder.h"

int8 sinus[]={1,1,0,-1,-1,-1,0,1};

#define Sin(x) sinus[(x+2)&7]
#define Cos(x) sinus[x&7]

uint16 *tmp_map;
uint16 *dist_map;
WarMap *map;

bigtime_t TimePathfinder(uint16 start_x,uint16 start_y,uint16 dest_x,uint16 dest_y)
{
	MyPathfinder *pathfinder;
	uint32 i;
	uint16 x,y;
	uint8 angle;

	printf("src(%d,%d) dest(%d,%d)\n",start_x,start_y,dest_x,dest_y);

	pathfinder=new MyPathfinder(map,NULL,tmp_map);
	pathfinder->precalced_dist_map=dist_map;
	pathfinder->NewPosition(start_x,start_y,NULL);

	memset(tmp_map,0xff,map->BlockCount()<<1);

	bigtime_t newdest_time=system_time();
	pathfinder->NewDestination(dest_x,dest_y,NULL);
	newdest_time=system_time()-newdest_time;

	for(i=0;i<map->BlockCount();i++)
	{
		tmp_map[i]=(tmp_map[i]!=0xFFFF)?'.':' ';
		if (map->GetBlock(i)==1) tmp_map[i]='#';
	}

	i=0;
	x=start_x;
	y=start_y;

	tmp_map[(y<<map->WidthShift())+x]='S';

	angle=pathfinder->FindPath();
	while (angle!=PATH_ARRIVED)
	{
		x+=Cos(angle);
		y+=Sin(angle);
		tmp_map[(y<<map->WidthShift())+x]='0';
		pathfinder->NewPosition(x,y,NULL);
		angle=pathfinder->FindPath();
		i++;
	}
	tmp_map[(dest_y<<map->WidthShift())+dest_x]='G';

	delete pathfinder;
	return newdest_time;
}

#include "WarQueue.h"

int main(int argc, char **argv)
{
/*
	WarPath *t=new WarPath();
	t->AddCoord(13,13);
	t->AddCoord(11,11);
	t->AddCoord(9,9);
	t->AddCoord(12,12);
	t->AddCoord(10,10);

	uint16 x1,y1,i,num;

	for(i=0;i<10;i++)
	{
		num=rand()>>13;
		t->Coord(&x1,&y1,num);
		printf("Coord(%d)=(%d,%d)\n",num,x1,y1);
	}

	delete t;
	exit(0);
*/
	uint16 start_x,start_y,dest_x,dest_y,x,y;
	uint32 i;
	bool draw_map=true;
	
	if (argc!=4)
	{
		printf("USAGE: sp map_filename mapsize cycles\n\ncycles=number of times to run the pathfinder. The avarage time will be shown afterwards.\n\nmapsizes\n");
		for(i=0;i<10;i++)
		{
			map=new WarMap(i);
			printf("%d  %dx%d\n",i,map->Width(),map->Height());
			delete map;
		}
		return 2;
	}
	map=new WarMap(atoi(argv[2]));

	{
		BEntry *entry=new BEntry(argv[1]);
		BFile *file=new BFile(entry,B_READ_ONLY);
		if (file->InitCheck()==B_OK)
		{
			size_t size=0;
			char buffer;
			size=file->Read(&buffer,1);
			y=0;
			x=0;
			while (size>0)
			{
				while(buffer!='\n' && size>0)
				{
					map->SetBlock((y<<map->WidthShift())+x,(buffer=='#')?1:0);
					size=file->Read(&buffer,1);
					x++;
					if (x>map->Width())
					{
						printf("Mapfile is too large\n");
						exit(1);
					}
				}
				size=file->Read(&buffer,1);
				x=0;
				y++;
			}
		}
		else printf("Couldn't open file\n");
		delete file;
		delete entry;
	}

	uint32 int_time;
	printf("mapsize=(%d,%d)\n",map->Width(),map->Height());

	tmp_map=(uint16*)malloc(map->BlockCount()<<1);

	dist_map=(uint16*)malloc(512*512*2);

	printf("generating distance map\n");
	for(y=0;y<512;y++)
		for(x=0;x<512;x++)
			dist_map[(y<<9)+x]=(uint16)(sqrt(x*x+y*y)*8);

	printf("distmap generated\n");
	bigtime_t tot_time=0,tmp_time,max_time=0,min_time=1000000000;

	x=100;
	y=100;

	srand(system_time());

	uint16 num_cycles=atoi(argv[3]);
	uint32 j;

	for(i=0;i<num_cycles;i++)
	{
		// randomizing start point, makes sure it's not on a occupied block
		start_x=rand()>>(15-map->WidthShift());
		start_y=rand()>>(15-map->HeightShift());
		while (map->GetBlock((start_y<<map->WidthShift())+start_x)==1)
		{
			start_x=rand()>>(15-map->WidthShift());
			start_y=rand()>>(15-map->HeightShift());
		}
		// randomizing destination point, makes sure it's not on a occupied block
		dest_x=rand()>>(15-map->WidthShift());
		dest_y=rand()>>(15-map->HeightShift());
		while (map->GetBlock((dest_y<<map->WidthShift())+dest_x)==1)
		{
			dest_x=rand()>>(15-map->WidthShift());
			dest_y=rand()>>(15-map->HeightShift());
		}

		// running the pathfinder and timing it
		tmp_time=TimePathfinder(start_x,start_y,dest_x,dest_y);
		tot_time+=tmp_time;
		// saves the record Min and Max times
		if (tmp_time>max_time) max_time=tmp_time;
		if (tmp_time<min_time) min_time=tmp_time;
		if (draw_map)
		{
		// draw the map
			printf("+");
			for (x=0;x<map->Width();x++)
			{
				printf("-");
			}
			printf("+\n");
			for (y=0;y<map->Height();y++)
			{
				printf("|");
				for (x=0;x<map->Width();x++)
				{
					printf("%c",tmp_map[(y<<map->WidthShift())+x]);
				}
				printf("|\n");
			}
			printf("+");
			for (x=0;x<map->Width();x++)
			{
				printf("-");
			}
			printf("+\n");
			int_time=tmp_time;
			printf("time consumed: %d | %f seconds\n",int_time,(float)int_time/1000000);
		}
	}

	int_time=max_time;
	printf("max time consumed: %d | %f seconds\n",int_time,(float)max_time/1000000);
	int_time=min_time;
	printf("min time consumed: %d | %f seconds\n",int_time,(float)min_time/1000000);
	int_time=tot_time/num_cycles;
	printf("avareage time consumed: %d | %f seconds\n",int_time,(float)tot_time/num_cycles/1000000);
	free(dist_map);
	free(tmp_map);
	delete map;
	return 0;
}
