#ifndef _GLOBAL_H
#define _GLOBAL_H

#include <OS.h>
#include <string.h>
#include <GraphicsDefs.h>

#include "WarMessage.h"

typedef struct
{
	uint8 alpha;
	uint8 red;
	uint8 green;
	uint8 blue;
} gfx_color;

#define _MEMORY_STATISTICS_

const uint32 WAR_CHANGE_SPACE='cspc';

// flags for the vision map
const uint8 MAPSTATE_VISIBLE=1;		// the block is currently seen
const uint8 MAPSTATE_DISCOVERED=2;	// the block is discovered, but not visible for the moment
const uint8 DARK_BIT=4;

const uint8 BLOCK_SHIFT=7;
const uint8 BLOCK_WIDTH=1<<BLOCK_SHIFT; // the width of one block
const uint16 BLOCK_AND=BLOCK_WIDTH-1;

// senders
const uint16 ID_SERVER=0xffff; // if the sender is ID_SERVER the message comes from the server

const bigtime_t WAR_NOW=0; // is used to timestamp messages that should arrive as fast as possible
const bigtime_t WAR_INFINITE_TIMEOUT=B_INFINITE_TIMEOUT;

const int32 TILE_SPAN = 8; // the number of blocks the width of a tile spans
const int32 TILE_SHIFT = 3; // 1 << TILE_SHIFT = TILE_SPAN

// predefined teams
enum
{
	TEAM_TREES=251,
	TEAM_CIVILIANS,
	TEAM_MONSTERS,
	TEAM_ARTIFACTS,
	TEAM_WRECKAGES
};

// the type of damage
typedef enum
{
	DMG_EXPLOSION=0,
	DMG_PROJECTILE,
	DMG_LASER,
	DMG_ELECTRICITY, // all your electronic equipment should be malfunctioning for a short period of time
	DMG_FIRE, // maybe you should melt?
	DMG_ION,  // all your computer equipment should be malfunctioning for a short period of time. If your unit is a bot this means that it can't see and can't move.
	DMG_MAGIC_FREEZE, // you should not be able to move, or you should move very slow
	DMG_MAGIC_NO_FIRE, // you should not be able to fire any shots
	DMG_MAGIC_NO_VISION // you should not be able to see anything. (unless you're a magician)
} damage_type;

// pre-defined messages
enum def_message
{
	MSG_HIT=0,    // this message indicates that you are hit by some bullet, the data field contains a pointer to a hit_data structure (see below)
	MSG_RESOURCE, // This message is resources, the data field uses the resources_data structure (see below)
	MSG_GIVE_ME_RESOURCE // you should send resources back to the sender (MSG_RESOURCE), the data field uses the resources_data structure (see below)	
};

// the data field for resources
struct resource_data
{
	uint8 resource_type;
	uint16 amount;
};

// the data field for hits
struct hit_data
{
	damage_type type;
	uint8 hit_angle; // NOT IMPLEMENTED YET!!
	uint16 damage;
};

#endif //_GLOBAL_H