#ifndef _WAR_BRAIN_
#define _WAR_BRAIN_

#include "Global.h"
//#include "WarMessenger.h"
//#include "Client2Server.h"
#include "WarMessage.h"
#include "WarBuffer.h"
#include "WarUnit.h"

#include <List.h>
#include <stdlib.h>

class WarMap;
class WarEntityList;
class WarPathfinder;
class WarServer;

struct coordinates
{
	uint8 x,y;
};

// unit_flags
enum
{
	UFLG_FIRE_AT_WILL=1,
	UFLG_RETURN_FIRE=2,
	UFLG_HOLD_FIRE=4,
	UFLG_MANEUVRE=8,
	UFLG_FOLLOW=16,
	UFLG_HOLD_POSITION=32
};

enum
{
	ROTATE_UNIT=0,
	ROTATE_WEAPON=1
};

enum
{
	VALUE_HEALTH=252,
	VALUE_SHEILD,
	VALUE_AMMO,
	VALUE_POWER
};

// used by the Sin() and Cos() functions
const int8 _war_integer_sinus_table_[]={1,1,0,-1,-1,-1,0,1};

class WarBrain
{
public:
//	WarBrain(WarBuffer *buffer);
	WarBrain(WarMap *m,WarEntityList *ul,uint8 unit_team,uint16 unit_id,int32 x,int32 y,WarUnit *your_body,WarServer *server);
	virtual ~WarBrain();

//	virtual void Flatten(WarBuffer *buffer);

	virtual void MessageReceived(WarMessage *message);
	virtual bool Cycle(void *data,float frame_time);

// NON-VIRTUAL functions

	uint16 ID();
	uint16 Team();

	void SetFlags(uint8 flags);
	void RemoveFlags(uint8 flags);

	uint16 GetNumMissions();
	bool GetMission(uint16 index,WarMessage **mission);
	void GetPosition(uint16 *x,uint16 *y);

protected:

// use these functions to control your unit

	void AddMission(WarMessage *message,bool add);
	// if add is true the mission will be added at the end of the list,
	// if not the entire list will be deleted and the mission will be the
	// only mission left


	uint16 LoadSound(char *filename);
	void PlaySound(uint16 sound_id);

	void PostMessage(uint16 x,uint16 y,WarMessage *message,bigtime_t timestamp);
	void PostMessage(uint16 dest_id,WarMessage *message,bigtime_t timestamp);

	void Damage(uint16 x,uint16 y,uint16 damage,damage_type type);
	void Damage(uint16 dest_id,uint16 damage,damage_type type);

	uint16 LoadUnit(char *filename);
	// loads the unit with the given filename. Returns an unit_type_id.
	// use this id when you adds a unit

	void UnloadUnit(char *filename);
	
	uint16 AddUnit(int32 x,int32 y,uint16 type,uint8 team,uint8 angle);
	uint16 AddUnit(int32 x,int32 y,uint16 type,uint8 team);
	uint16 AddUnit(int32 x,int32 y,uint16 type);
	// adds a unit to the world. type is previously returned by LoadUnit()

	void ExecuteFunction(uint8 index,uint32 param1,uint32 param2);
	// runs the WarUnit::ExecuteFunction() on this brain's all bodies

	bool OccupyBlock(uint16 x,uint16 y);
	// If you're flying you can't occupy blocks. This function
	// will always return false in that case. If you're not flying
	// you can occupy any block you want, but it is highly recommended
	// to occupy the block(s) youre standing on. It will not only
	// cause less confusion for the player but it will also help
	// the game be more efficient. When the display is redrawn (or
	// parts of it is) the game will look in the occupy list and
	// assume that you're unit should be redrawn where your occupied
	// block(s) are. Therfore, if you're not flying, you have to occupy
	// at least one block to get drawn.

	void RemoveOccupation(uint16 x,uint16 y);
	
	WarPathfinder *NewPathfinder();
	// allocates a new WarPathfinder for you. Remember that you
	// have to delete it.
	
	virtual void TakeOff();
	// removes the unit from the regular unit list and places it into 
	// the flying unit list. When you're in the flying unit list
	// you always get drawn (you don't have to occupy a block).
	
	virtual bool Land();
	// If you know what the TakeOff() method do this should be rather
	// easy to guess. It removes your unit from the flying unit list
	// and places it in the regular unit list (where you have to occupy
	// blocks). Before you land you should occupy the block you want to
	// land on.

	void MoveUnit(int32 x,int32 y,uint16 speed); // Orders the unit to move, returns true if the unit want the next direction to move.
	// MoveUnit() will tell your body to move to the given coordinates
	// with the given speed. When he's reached the new position he
	// should tell you (the brain) through the data pointer he gets i
	// his cycle loop. This data will then be sent to your
	// (the brain's) cycle loop

	void Rotate(uint8 index,uint16 direction,uint16 speed);

	void SetMaxValue(uint8 index,uint16 amount);
	void SetValue(uint8 index,uint16 amount);
	// sets a value for the given index. There's a few predefined
	// indexes. Values could also be used to trigger events at the body,
	// e.g. when the body has a low health it could begin to burn or something.

	void DisplayMessage(char *message);
	// displays a message to the unit-owner (the human player)

	void RemoveMission(uint16 index);
	// use RemoveMission(0) when your unit has completed
	// or failed the current mission. Short: deletes the
	// current mission and makes the next mission in que
	// (if any) the current mission. This Functions can
	// also be used to Cancel any mission in que, the unit
	// should not delete any of these missions. If your unit
	// don't want to do some things, wait until that mission
	// bocomes the current mission and then delete it.

	int8 Sin(uint8 a);
	int8 Cos(uint8 a);
	uint8 ATan(int8 x,int8 y);

	WarEntityList *ulist; // the unitlist containing every other unit
	WarMap *map; // the map your unit lives on
	uint32 unit_flags; // contains info on how to behave
	uint16 unit_x,unit_y; // the unit's block position
private:
	BList *mission_list;
	uint16 id;
	uint8 team;
	WarServer *war_server;
};

// inline definitions

inline void WarBrain::SetFlags(uint8 flags)
{
	unit_flags|=flags;
}

inline void WarBrain::RemoveFlags(uint8 flags)
{
	unit_flags&=!flags;
}

inline int8 WarBrain::Sin(uint8 a)
{
	return _war_integer_sinus_table_[(a+2)&7];
}

inline int8 WarBrain::Cos(uint8 a)
{
	return _war_integer_sinus_table_[a&7];
}

inline void WarBrain::RemoveMission(uint16 index) // removes the specified mission
{
	void *entry=mission_list->RemoveItem(index);
	if (entry) free(entry);
}

inline uint16 WarBrain::GetNumMissions()
{
	return mission_list->CountItems();
}

inline uint16 WarBrain::ID()
{
	return id;
}

inline uint16 WarBrain::Team()
{
	return team;
}

#endif // _WAR_BRAIN_