#ifndef _WAR_BUFFER_H_
#define _WAR_BUFFER_H_

#include "WarSemaphore.h"

class WarBuffer:public WarSemaphore
{
public:
	WarBuffer(size_t size,const char *name);
	~WarBuffer();

	status_t PushBuffer(WarBuffer *buffer);

	status_t PushUint8(uint8 val);
	status_t PushUint16(uint16 val);
	status_t PushInt16(int16 val);
	status_t PushUint32(uint32 val);
	status_t PushString(char *string);
	status_t PushFloat(float val);

	status_t PopBuffer(WarBuffer *buffer);

	uint8 PopUint8();
	uint16 PopUint16();
	int16 PopInt16();
	uint32 PopUint32();
	status_t PopString(char *buffer,uint16 size);
	float PopFloat();

// returns the amount of data written to the buffer (in bytes)
	size_t DataSize();

// returns the allocated buffer size
	size_t Size();

// emties the buffer
	void Empty();

	size_t LeftToWrite();
	size_t LeftToRead();

	bool EndOfBuffer();
	
	void *Data();
protected:
	uint8 *buffer;
	size_t size,read_offs,write_offs;
};

// inline definitions

inline bool WarBuffer::EndOfBuffer()
{
	return (read_offs>=write_offs);
}

inline size_t WarBuffer::LeftToWrite()
{
	return size-write_offs;
}

inline size_t WarBuffer::LeftToRead()
{
	return write_offs-read_offs;
}

inline size_t WarBuffer::DataSize()
{
	return write_offs;
}

inline size_t WarBuffer::Size()
{
	return size;
}

inline void *WarBuffer::Data()
{
	return (void *)buffer;
}

#endif //_WAR_BUFFER_H_