#ifndef _WAR_ENTITY_
#define _WAR_ENTITY_

#include "Global.h"
#include "WarBuffer.h"
//#include "WarColor.h"
//#include "WarClass.h"
#include "WarBox.h"
#include "WarMatrix.h"

#include <SupportDefs.h>
#include <image.h>
#include <gl.h>

class WarCamera;
class WarMap;
class WarEntityList;

class WarEntity:public WarMatrix
{
public:
	WarEntity(int32 id, float mass, WarVertex position = WarVertex(0,0,0), WarVertex rotation = WarVertex(0,0,0), WarVertex speed = WarVertex(0,0,0), WarVertex direction = WarVertex(0,0,0), WarVertex scale = WarVertex(1,1,1));
	virtual ~WarEntity();

	void Draw(WarCamera *camera);
	virtual void Update(WarCamera *camera);
	virtual void GetBoundingBox(WarBox *box);

	virtual void SetDetailLevel(float detail);
	virtual float DetailLevel();

	double Mass();
	void SetMass(double mass);

	WarVertex Speed();
	void SetSpeed(WarVertex speed);

	WarVertex Rotation();
	void SetRotation(WarVertex rotation);

//	virtual void Flatten(WarBuffer *buffer);
	// can be called anytime if the server
	// wants to make a copy of the body and send it to a client
	// it's VERY important to call the base class' Flatten() method
	// BEFORE you write anything to the buffer

	virtual void Cycle(double frame_time);
	// frame_time is the number of seconds since the function
	// last was called, any movement should depend on this timer.
	// Because a fast computer shouldn't have faster units
	// than a slow one.

// ----------- Runned when a brain-message is received ---------------------
	virtual void MessageReceived(WarBuffer *buffer);
// -------------------------------------------------------------------------

	virtual uint16 MaxValue(uint8 index);
	virtual uint16 Value(uint8 index);

	virtual void Select(bool is); // called to select or deselect the unit
	virtual bool IsSelected(); // returns true if the unit is selected and otherwise false

// NON-VIRTUAL functions

	int32 Id();

	virtual void JoinGroup(uint8 group_id); // is called to join this unit to a group
	virtual void LeaveGroup(); // is called when the unit should leave the group
	virtual uint8 Group(); // returns the group number this unit is joined to, -1 = none

private:
	int32 m_id;

	double m_mass;
	WarVertex m_speed;
	WarVertex m_rotation;
};

// inline definitions

inline int32 WarEntity::Id()
{
	return m_id;
}

inline double WarEntity::Mass()
{
	return m_mass;
}

inline void WarEntity::SetMass(double mass)
{
	m_mass = mass;
}

inline WarVertex WarEntity::Speed()
{
	return m_speed;
}

inline void WarEntity::SetSpeed(WarVertex speed)
{
	m_speed = speed;
}

inline WarVertex WarEntity::Rotation()
{
	return m_rotation;
}

inline void WarEntity::SetRotation(WarVertex rotation)
{
	m_rotation = rotation;
}

inline void WarEntity::Draw(WarCamera *camera)
{
	glPushMatrix();
	glTranslatef(XPosition(),YPosition(),ZPosition());
	glRotatef(ZDirection() * 180.0 / 3.1415927,0.0,0.0,1.0); // direction
	glRotatef(YDirection() * 180.0 / 3.1415927,0.0,1.0,0.0); // up-down
	glRotatef(XDirection() * 180.0 / 3.1415927,1.0,0.0,0.0); // tilt
	glScalef(XScale(),YScale(),ZScale());

	Update(camera);

	glPopMatrix();
}

#endif // _WAR_ENTITY_