#ifndef _WAR_ENTITY_
#define _WAR_ENTITY_

#include "Global.h"
#include "WarObject.h"
#include "WarBuffer.h"
#include "WarColor.h"
#include "WarClass.h"
#include "WarBox.h"

#include <SupportDefs.h>
#include <image.h>
#include <gl.h>

class WarMap;
class WarEntityList;

class WarEntity:public WarObject
{
public:
	WarEntity(WarBuffer *buffer,WarObject *model); // takes the buffer received in the Flatten() call
	WarEntity(float x,float y,float z,uint16 direction,uint8 team_id,WarColor color,WarMap *map,uint16 id,WarObject *model,WarClass *war_class);
	virtual ~WarEntity();

	virtual void Flatten(WarBuffer *buffer);
	// can be called anytime if the server
	// wants to make a copy of the body and send it to a client
	// it's VERY important to call the base class' Flatten() method
	// BEFORE you write anything to the buffer

	virtual void Vision(WarMap *map,uint8 flag);
	// the unit is supposed to set the blocks that the unit sees to
	// flag

	virtual void Cycle(void **data,float frame_time);
	// The pointer
	// to the void-pointer-variable should contain a
	// pointer the a unit specific data structure when
	// the function returns. This structure pointer
	// is then passed to the brain's Cycle function
	// (if run on server). The structure should contain
	// all data the Brain wants to know. E.g The current
	// rotation angle, the direction the gun is pointing
	// in etc.
	// frame_time is the number of seconds since the function
	// last was called, any movement should depend on this timer.
	// Because a fast computer shouldn't have faster units
	// than a slow one.

// ----------- Runned when a brain-message is received ---------------------
	virtual void MessageReceived(WarBuffer *buffer);
// -------------------------------------------------------------------------

	virtual uint16 MaxValue(uint8 index);
	virtual uint16 Value(uint8 index);

	virtual void Select(bool is); // called to select or deselect the unit
	virtual bool IsSelected(); // returns true if the unit is selected and otherwise false

// NON-VIRTUAL functions

	uint8 Team();
	uint16 Id();

	void BlockPos(uint16 *x,uint16 *y);

	void GetBoundingBox(WarBox *box);
	
	virtual void Draw(WarCamera *camera);

	virtual void JoinGroup(uint8 group_id); // is called to join this unit to a group
	virtual void LeaveGroup(); // is called when the unit should leave the group
	virtual uint8 Group(); // returns the group number this unit is joined to, -1 = none

	WarClass *Class();

protected:
	void MoveTo(float x,float y,float z);

	WarClass *war_class;
private:
	uint8 team;
	uint16 unit_id;
	uint16 block_x,block_y;
};

// inline definitions

inline WarClass *WarEntity::Class()
{
	return war_class;
}

inline void WarEntity::GetBoundingBox(WarBox *box)
{
	*box = war_class->BoundingBox();
	box->RotateBy(Direction());
	box->OffsetBy(Position());
}

inline void WarEntity::BlockPos(uint16 *bx,uint16 *by)
{
	*bx=block_x;
	*by=block_y;
}

inline uint8 WarEntity::Team()
{
	return team;
}

inline uint16 WarEntity::Id()
{
	return unit_id;
}

#endif // _WAR_ENTITY_