#ifndef _UNIT_LIST_H_
#define _UNIT_LIST_H_

#include <image.h>
#include <Locker.h>
#include <List.h>
#include <stdio.h>

#include "WarUnit.h"
#include "WarMap.h"
#include "WarList.h"
#include "WarHeap.h"
#include "WarMessage.h"
#include "WarClass.h"
#include "WarSelectionView.h"

class WarMessenger;
class WarBrain;
class WarPathfinder;
class WarEntityList;

struct unit_entry
{
	WarUnit *body;
	WarBrain *brain;
};

struct class_entry
{
	char filename[B_FILE_NAME_LENGTH];
	image_id image;
	WarClass *war_class;
};

struct pathfinder_class_struct
{
	char filename[B_FILE_NAME_LENGTH];
	WarPathfinder *(*new_pathfinder)(WarMap *map,WarEntityList *ulist);
	image_id image;
};

class WarEntityList:public BLocker
{
public:
	WarEntityList(WarMap *map,WarSelectionView *sel_view,BDirectory *unit_dir,bool brains=false);
	~WarEntityList();

	status_t init;
	
//  the unit id_map

//	bool IsBlockOccupied(uint32 offs);
//	bool IsBlockOccupied(uint16 x,uint16 y);

//	bool OccupyBlock(uint16 id,uint16 x,uint16 y);
//	bool OccupyBlock(uint16 id,uint32 offs);
//	void RemoveOccupation(uint16 id,uint16 x,uint16 y);
//	void RemoveOccupation(uint16 id,uint32 offs);

//	uint16 GetId(uint32 offs);

	bool GetNextEntity(WarUnit **body,WarBrain **brain,uint16 *id,uint16 *cookie); // the cookie must be initialized to 0
	bool GetNextFlyingEntity(WarUnit **body,WarBrain **brain,uint16 *id,uint16 *cookie); // the cookie must be initialized to 0
	bool GetNextSelectedEntity(WarUnit **body,WarBrain **brain,uint16 *id,uint16 *cookie); // the cookie must be initialized to 0

	WarUnit *GetBodyByID(uint16 id);
//	WarUnit *GetBodyByOffs(uint32 offs);
	WarBrain *GetBrainByID(uint16 id);
//	WarBrain *GetBrainByOffs(uint32 offs);
	
// the unit_list

	void LoadUsedEntities();
	void LoadUsedFiles();
	void LoadUsedSounds();

//	bool CanEntityBeAdded(uint16 x,uint16 y,uint16 type);
	uint16 AddEntity(float x,float y,float z,uint16 type,uint8 team,uint16 angle);
	uint16 AddEntity(WarBuffer *buffer);
	bool RemoveEntity(uint16 id);

	void TakeOff(uint16 id);
	void Land(uint16 id);

	uint16 IsClassLoaded(char *filename);
	uint16 LoadClass(char *filename);
	bool UnloadClass(uint16 id);

	bool SelectEntity(WarUnit *war_body,uint8 team);
	bool DeselectEntity(WarUnit *war_body,uint8 team);

	void AssignGroup(uint8 team,uint8 group_id);
	void LeaveGroups(uint8 team);
	void SelectGroup(uint8 team,uint8 group_id);
	void DeselectGroup(uint8 team,uint8 group_id);
	void DeselectAll(uint8 team);

	uint16 SelectedEntities(uint8 team); // returns the number of selected entities in the given team

	void UpdateSelection();
private:
	WarList<class_entry*> *class_list; // a list of all loaded add-ons
	WarList<unit_entry*> *unit_list; // a list of every entity
	WarList<uint16> *flying_list; // a list with id-numbers of all flying entities

//	uint32 block_count; // number of blocks in the id_map
//	uint8 width_shift; // the value to shift left if you want to multiply with the map_width
//	uint16 *id_map; // an ID-map, every block is marked with the unit's id (that stands on the block)
	WarMap *war_map; // the map every unit lives on
	WarSelectionView *war_selection_view; // the view that shows the selected units
	BDirectory *unit_dir; // the directory where all the unit add-ons are
	bool brains; // tells th unitlist if it should contain brains as well
};

// inline definitions

inline bool WarEntityList::SelectEntity(WarUnit *entity,uint8 team)
{
	if (entity->Team()==team)
	{
		if (!entity->IsSelected())
		{
//			selected_units[team]++;
			entity->Select(true);
			if (entity->IsSelected())
				war_selection_view->Add(entity);
			return true;
		}
	}
	return false;
}

inline bool WarEntityList::DeselectEntity(WarUnit *entity,uint8 team)
{
	if (entity->Team() == team)
	{
		if (entity->IsSelected())
		{
			entity->Select(false);
			if (!entity->IsSelected())
				war_selection_view->Remove(entity);
			return true;
		}
	}
	return false;
}

inline void WarEntityList::TakeOff(uint16 id)
{
	if (unit_list->ItemAt(id)) flying_list->AddItem(id+1);
}

inline void WarEntityList::Land(uint16 id)
{
	flying_list->RemoveItem(id+1);
}
/*
inline void WarEntityList::RemoveOccupation(uint16 id,uint32 offs)
{
	offs&=block_count;
	if (id_map[offs]==id) id_map[offs]=0xffff;
}

inline void WarEntityList::RemoveOccupation(uint16 id,uint16 x,uint16 y)
{
	uint32 offs=((y<<width_shift)+x)&block_count;
	if (id_map[offs]==id) id_map[offs]=0xffff;
}

inline bool WarEntityList::IsBlockOccupied(uint32 offs)
{
	return (id_map[offs&block_count]!=0xFFFF)?true:false;
}

inline bool WarEntityList::IsBlockOccupied(uint16 x,uint16 y)
{
	return (id_map[((y<<width_shift)+x)&block_count]!=0xFFFF)?true:false;
}
*/
inline WarUnit *WarEntityList::GetBodyByID(uint16 id)
{
	unit_entry *entry=(unit_entry*)unit_list->ItemAt(id);
	if (entry) return entry->body;
	return NULL;
}
/*
inline WarUnit *WarEntityList::GetBodyByOffs(uint32 offs)
{
	if (id_map[offs&block_count]!=0xFFFF) return GetBodyByID(id_map[offs&block_count]);
	else return NULL;
}
*/
inline WarBrain *WarEntityList::GetBrainByID(uint16 id)
{
	unit_entry *entry=(unit_entry*)unit_list->ItemAt(id);
	if (entry) return entry->brain;
	return NULL;
}
/*
inline WarBrain *WarEntityList::GetBrainByOffs(uint32 offs)
{
	if (id_map[offs&block_count]<0xFFFF) return GetBrainByID(id_map[offs&block_count]);
	else return NULL;
}

inline uint16 WarEntityList::GetId(uint32 offs)
{
	return id_map[offs&block_count];
}
*/
inline void WarEntityList::AssignGroup(uint8 team,uint8 group_id)
{
	WarUnit *entity;
	uint16 id,cookie=0;
	while (GetNextSelectedEntity(&entity,NULL,&id,&cookie))
		if (entity->Team() == team) entity->JoinGroup(group_id);
}

inline void WarEntityList::LeaveGroups(uint8 team)
{
	WarUnit *entity;
	uint16 id,cookie=0;
	while (GetNextSelectedEntity(&entity,NULL,&id,&cookie))
		if (entity->Team() == team) entity->LeaveGroup();
}

inline void WarEntityList::SelectGroup(uint8 team,uint8 group_id)
{
	WarUnit *entity;
	uint16 id,cookie=0;
	while (GetNextEntity(&entity,NULL,&id,&cookie))
		if (entity->Group() == group_id)
			SelectEntity(entity,team);
	UpdateSelection();
}

inline void WarEntityList::DeselectGroup(uint8 team,uint8 group_id)
{
	WarUnit *entity;
	uint16 id,cookie=0;
	while (GetNextSelectedEntity(&entity,NULL,&id,&cookie))
		if (entity->Group() == group_id)
			DeselectEntity(entity,team);
	UpdateSelection();
}

inline void WarEntityList::DeselectAll(uint8 team)
{
	WarUnit *entity;
	uint16 id,cookie=0;
	while (GetNextSelectedEntity(&entity,NULL,&id,&cookie))
		if (entity->Team() == team) entity->Select(false);

	war_selection_view->RemoveAll();
	UpdateSelection();
}

inline void WarEntityList::UpdateSelection()
{
	war_selection_view->Regenerate();
}

#endif //_UNIT_LIST_H_