#ifndef _WAR_HEAP_H_
#define _WAR_HEAP_H_

#include <OS.h>
#include <List.h>

class WarHeap
{
public:
	WarHeap(uint16 size);
	~WarHeap();
	void Push(void *ptr,bigtime_t wt);
	void Push(uint16 x,uint16 y,bigtime_t wt);
	bool Pop(void **ptr);
	bool Pop(uint16 *x,uint16 *y);
	bool TopWeight(bigtime_t *wt);
	void EmptyQueue();
private:

	struct entry_data
	{
		void *ptr;
		bigtime_t wt;
	};

	void make_float(uint16 index);
	void make_sink(uint16 index);
	void swap(uint16 index,uint16 index2);

	entry_data *heap;
	uint16 heap_size;
	uint16 element_count;
	int16 tmp_index;
};

inline void WarHeap::swap(uint16 index,uint16 index2)
{
	heap[index2].wt^=heap[index].wt;
	heap[index].wt^=heap[index2].wt;
	heap[index2].wt^=heap[index].wt;

	void *tmp=heap[index].ptr;
	heap[index].ptr=heap[index2].ptr;
	heap[index2].ptr=tmp;
}

inline void WarHeap::make_float(uint16 index)
{
//	printf("WarHeap: make_float(%d)\n",index);
	tmp_index=index>>1;
	while (heap[tmp_index].wt>heap[index].wt && index!=1)
	{
		swap(index,tmp_index);
		index=tmp_index;
		tmp_index=index>>1;
	}
//	printf("WarHeap: floated to %d\n",index);
}

inline void WarHeap::make_sink(uint16 index)
{
	tmp_index=(heap[index<<1].wt<heap[(index<<1)+1].wt)?index<<1:(index<<1)+1;
	while (tmp_index<heap_size && heap[index].wt>heap[tmp_index].wt)
	{
		swap(index,tmp_index);
		index=tmp_index;
		tmp_index=(heap[index<<1].wt<heap[(index<<1)+1].wt)?index<<1:(index<<1)+1;
	}
}

inline bool WarHeap::TopWeight(bigtime_t *wt)
{
	if (element_count==0) return false;
	*wt=heap[1].wt;
	return true;
}

#endif //_WAR_HEAP_H_