#ifndef _WAR_MATRIX_H
#define _WAR_MATRIX_H

#include "WarVertex.h"

#include <gl.h>
#include <GraphicsCard.h>
#include <List.h>
#include <stdio.h>

class WarMatrix
{
public:
	WarMatrix(WarVertex position = WarVertex(0,0,0), WarVertex direction = WarVertex(0,0,0), WarVertex scale = WarVertex(1,1,1));
	virtual ~WarMatrix();

	virtual WarVertex Position(); // returns the position in a vertex
	virtual float XPosition();
	virtual float YPosition();
	virtual float ZPosition();

	void MoveTo(WarVertex camera);
	void MoveXTo(float x);
	void MoveYTo(float y);
	void MoveZTo(float z);

	void MoveBy(WarVertex delta_camera);
	void MoveXBy(float x);
	void MoveYBy(float y);
	void MoveZBy(float z);


	virtual WarVertex Direction(); // returns the x, y and z angle (degrees) in a vertex
	virtual float XDirection();
	virtual float YDirection();
	virtual float ZDirection();

	void RotateTo(float xa,float ya,float za);
	void RotateTo(WarVertex angle);
	void RotateXTo(float xa);
	void RotateYTo(float ya);
	void RotateZTo(float za);

	void RotateBy(float xa,float ya,float za);
	void RotateBy(WarVertex angle);
	void RotateXBy(float xa);
	void RotateYBy(float ya);
	void RotateZBy(float za);
	

	virtual WarVertex Scale(); // returns the x, y and z scale in a vertex
	virtual float XScale();
	virtual float YScale();
	virtual float ZScale();

	void ScaleTo(WarVertex scale_factor);
	void ScaleXTo(float factor);
	void ScaleYTo(float factor);
	void ScaleZTo(float factor);

	void ScaleBy(WarVertex scale_factor);
	void ScaleXBy(float factor);
	void ScaleYBy(float factor);
	void ScaleZBy(float factor);

	bool IsLocked();
	void Lock();
	void Unlock();

protected:

	WarVertex m_position; // the current position
	WarVertex m_direction; // stores the x, y and z angle in a Vertex
	WarVertex m_scale; // stores the scale

private:

	bool m_locked; // is true when the camera is locked, when itäs locket you can't move or rotate it.
};

// inline definitions

inline void WarMatrix::MoveBy(WarVertex pos)
{
	MoveTo(m_position + pos);
}

inline void WarMatrix::MoveXTo(float pos)
{
	if (IsLocked()) return;
	m_position.x = pos;
}

inline void WarMatrix::MoveYTo(float pos)
{
	if (IsLocked()) return;
	m_position.y = pos;
}

inline void WarMatrix::MoveZTo(float pos)
{
	if (IsLocked()) return;
	m_position.z = pos;
}

inline void WarMatrix::MoveXBy(float pos)
{
	MoveXTo(m_position.x + pos);
}

inline void WarMatrix::MoveYBy(float pos)
{
	MoveYTo(m_position.y + pos);
}

inline void WarMatrix::MoveZBy(float pos)
{
	MoveZTo(m_position.z + pos);
}

inline bool WarMatrix::IsLocked()
{
	return m_locked;
}

inline void WarMatrix::Lock()
{
	m_locked = true;
}

inline void WarMatrix::Unlock()
{
	m_locked = false;
}

inline void WarMatrix::ScaleXTo(float factor)
{
	if (IsLocked()) return;
	m_scale.x = factor;
}

inline void WarMatrix::ScaleYTo(float factor)
{
	if (IsLocked()) return;
	m_scale.y = factor;
}

inline void WarMatrix::ScaleZTo(float factor)
{
	if (IsLocked()) return;
	m_scale.z = factor;
}

inline void WarMatrix::ScaleXBy(float factor)
{
	if (IsLocked()) return;
	m_scale.x+= factor;
}

inline void WarMatrix::ScaleYBy(float factor)
{
	if (IsLocked()) return;
	m_scale.y+= factor;
}

inline void WarMatrix::ScaleZBy(float factor)
{
	if (IsLocked()) return;
	m_scale.z+= factor;
}

inline void WarMatrix::ScaleTo(WarVertex scale_factor)
{
	if (IsLocked()) return;
	m_scale = scale_factor;
}

inline void WarMatrix::ScaleBy(WarVertex scale_factor)
{
	if (IsLocked()) return;
	m_scale+= scale_factor;
}

inline void WarMatrix::RotateXTo(float xa)
{
	if (IsLocked()) return;
	m_direction.x = xa;
	if (m_direction.x<-3.1415927/2.0) m_direction.x=-3.1415927/2.0;
	else if (m_direction.x>3.1415927/2.0) m_direction.x=3.1415927/2.0;
}

inline void WarMatrix::RotateYTo(float ya)
{
	if (IsLocked()) return;
	m_direction.y = ya;
	if (m_direction.y<-3.1415927/2) m_direction.y=-3.1415927/2.0;
	else if (m_direction.y>3.1415927/2.0) m_direction.y=3.1415927/2.0;
}

inline void WarMatrix::RotateZTo(float za)
{
	if (IsLocked()) return;
	m_direction.z = za;
}

inline void WarMatrix::RotateBy(WarVertex angle)
{
	RotateTo(m_direction + angle);
}

inline void WarMatrix::RotateXBy(float angle)
{
	RotateXTo(m_direction.x + angle);
}

inline void WarMatrix::RotateYBy(float angle)
{
	RotateYTo(m_direction.y + angle);
}

inline void WarMatrix::RotateZBy(float angle)
{
	RotateZTo(m_direction.z + angle);
}

inline void WarMatrix::RotateBy(float x,float y,float z)
{
	RotateTo(m_direction.x + x,m_direction.y + y,m_direction.z + z);
}

inline void WarMatrix::RotateTo(WarVertex angle)
{
	RotateTo(angle.x,angle.y,angle.z);
}

#endif // #ifndef _WAR_MATRIX_