#ifndef _WAR_MESSAGE_H_
#define _WAR_MESSAGE_H_

#include <OS.h>
#include <List.h>


// __predefined_messages__
//if you get a message from the client
// (e.g. MISSION_MOVE) the sender will be 0xFFFF
const uint8 WAR_MISSION_CANCELED=0xf0;
						// a mission from your mission list
						// has been deleted. The data contains the index
						// for the deleted mission.
const uint8 WAR_NEEDING_BACKUP=0xf1;
						// some unit is under attack and don't think
						// it can handle it. If you're not assigned to
						// do anything, go and help. The coordinates
						// for the attack comes in the coordinates
						// structure, defined below. IMPORTANT! you should
						// only go if the attack less than about 10 blocks
						// away and your MANUEVRE flag is set.
const uint8 WAR_DAMAGE_TAKEN=0xf2;
						// Your unit have taken damage, the data variable is a
						// pointer to a hit_data structure. The structure will
						// only be valid in the MessageReceived() method. If
						// you want to keep any data you should store it in
						// some other variable.

class WarMessage
{
public:
	WarMessage(uint16 id,uint16 parameters);
	~WarMessage();
	uint16 What();
	uint16 CountParameters();
	void SetParameter(uint16 index,uint32 value);
	uint32 Parameter(uint16 index);
	uint16 Sender();
	uint16 sender;
	uint16 what;
	uint16 id;
	uint16 x,y;
private:
	uint16 num_parameters;
	uint32 *data;
};

inline WarMessage::WarMessage(uint16 msg_id,uint16 parameters)
{
	what=msg_id;
	num_parameters=parameters;
	data=new uint32[parameters];
}

inline WarMessage::~WarMessage()
{
	delete []data;
}

inline uint16 WarMessage::Sender()
{
	return sender;
}

inline uint16 WarMessage::What()
{
	return what;
}

inline uint16 WarMessage::CountParameters()
{
	return num_parameters;
}

inline void WarMessage::SetParameter(uint16 index,uint32 value)
{
	data[index%num_parameters]=value;
}

inline uint32 WarMessage::Parameter(uint16 index)
{
	return data[index%num_parameters];
}

#endif //_WAR_MESSAGE_H_