#ifndef _WAR_PATHFINDER_
#define _WAR_PATHFINDER_

#include <OS.h>

class WarMap;
class WarEntityList;

// return codes for FindPath()
const uint8 PATH_WAIT=0xff;
const uint8 PATH_ARRIVED=0xfe;
const uint8 PATH_CANNOT_ARRIVE=0xfd;

const int8 integer_sinus_table[]={1,1,0,-1,-1,-1,0,1};

class WarPathfinder
{
public:
	WarPathfinder(WarMap *map,WarEntityList *ul,uint16 *tmp_map);
	virtual ~WarPathfinder();

	virtual uint8 FindPath();
	// should return which way to go if the unit
	// wants to go the the specified coordinates
	virtual void NewPosition(uint16 unit_x,uint16 unit_y);
	// Updates the unit's location on the map
	// This function is called every time the unit
	// moves from one block to another
	virtual void NewDestination(uint16 dest_x,uint16 dest_y,const uint8 unit_speed[32]);
	uint16 *precalced_dist_map;
protected:
	int8 Sin(uint8 a);
	int8 Cos(uint8 a);
	uint8 ATan(int16 x,int16 y);
	uint16 Dist(int16 x,int16 y);
};

// inline definitions

inline int8 WarPathfinder::Sin(uint8 a)
{
	return integer_sinus_table[(a+2)&7];
}

inline int8 WarPathfinder::Cos(uint8 a)
{
	return integer_sinus_table[a];
}

inline uint16 WarPathfinder::Dist(int16 x,int16 y)
{
	if (y<0) y=-y;
	if (x<0) x=-x;
	return precalced_dist_map[(y<<9)+x];
}

#endif // _WAR_PATHFINDER_