#ifndef _WAR_SEMAPHORE_
#define _WAR_SEMAPHORE_

#include <OS.h>

class WarSemaphore
{
public:
	WarSemaphore(bool locked,const char *name);
	~WarSemaphore();
	bool Lock();
	void Unlock();
	bool LockWithTimeout(bigtime_t time);
	int32 Count();
private:
	sem_id sem;
};

// inline definitions

inline WarSemaphore::WarSemaphore(bool locked,const char *name)
{
	sem=create_sem((locked)?0:1,name);
}

inline WarSemaphore::~WarSemaphore()
{
	delete_sem(sem);
}

inline bool WarSemaphore::Lock()
{
	return (acquire_sem(sem)==B_NO_ERROR);
}

inline void WarSemaphore::Unlock()
{
	release_sem(sem);
}

inline bool WarSemaphore::LockWithTimeout(bigtime_t timeout)
{
	return (acquire_sem_etc(sem,1,B_TIMEOUT,timeout)==B_NO_ERROR);
}

inline int32 WarSemaphore::Count()
{
	int32 count;
	get_sem_count(sem,&count);
	return count;
}

#endif // #ifndef _WAR_SEMAPHORE_