#ifndef _WAR_STACK_H_
#define _WAR_STACK_H_

#include <OS.h>
#include <stdio.h>

#include "WarException.h"

template<class Item> class WarStack
{
public:
	WarStack(int32 size);
	virtual ~WarStack();
	bool Push(Item element);
	Item Pop();
	Item Top();

	Item Pick(int32 index);

	bool Remove(Item element);

	int32 CountElements();
	void Empty();
	bool IsEmpty();

	bool HasItem(Item item);

// is thrown when the stack underruns	
	class WarStackUnderrun: public WarException
	{
		public:
		WarStackUnderrun(char *message):WarException(message)
		{}
	};

private:
	Item *m_stack;
	int32 m_stack_size;
	int32 m_element_count;
};

// inline definitions

template <class Item>
WarStack<Item>::WarStack(int32 size)
{
	m_stack_size = size;
	m_stack = new Item[m_stack_size];
	Empty();

#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"WarStack\n");
	fprintf(stderr,"  stack: %ld\n",m_stack_size * sizeof(Item));
#endif

}

template <class Item>
WarStack<Item>::~WarStack()
{
	delete []m_stack;
}

template <class Item>
inline bool WarStack<Item>::Push(Item element)
{
	if (m_element_count == m_stack_size) return false;

	m_stack[m_element_count] = element;
	m_element_count++;
	return true;
}

template <class Item>
inline bool WarStack<Item>::Remove(Item element)
{
	if (Top() == element)
	{
		m_element_count--;
		return true;
	}

	bool found = false;

	for (int32 i = 0;i < m_element_count - 1;i++)
	{
		if (found || m_stack[i] == element)
		{
			found = true;
			m_stack[i] = m_stack[i+1];
		}
	}
	if (found) m_element_count--;
	return found;
}

template <class Item>
inline bool WarStack<Item>::HasItem(Item element)
{
	for (int32 i = 0;i < m_element_count;i++)
		if (m_stack[i] == element) return true;
	return false;
}

template <class Item>
inline Item WarStack<Item>::Pop()
{
	if (m_element_count == 0) throw new WarStackUnderrun("Stack underrun");

	m_element_count--;
	return m_stack[m_element_count];
}

template <class Item>
inline Item WarStack<Item>::Top()
{
	if (m_element_count == 0) throw new WarStackUnderrun("Stack underrun");

	return m_stack[m_element_count - 1];
}

template <class Item>
inline Item WarStack<Item>::Pick(int32 index)
{
	if (m_element_count - index - 1 < 0) throw new WarStackUnderrun("Stack underrun");

	return m_stack[m_element_count - index - 1];
}

template <class Item>
inline void WarStack<Item>::Empty()
{
	m_element_count = 0;
}

template <class Item>
inline int32 WarStack<Item>::CountElements()
{
	return m_element_count;
}

template <class Item>
inline bool WarStack<Item>::IsEmpty()
{
	return m_element_count == 0;
}

#endif //_WAR_STACK_H_