#ifndef WAR_TILESET_H
#define WAR_TILESET_H

#include <File.h>
#include <gl.h>

#include "WarColor.h"
#include "WarVertex.h"
#include "WarPoint.h"
#include "Global.h"

class WarTileset
{
public:
	WarTileset(BDirectory *dir,char *filename);
	~WarTileset();

	WarColor *TileColor(uint16 index);
	GLuint Tile(uint16 index);
	uint16 TileCount();
	char *Name();
	WarPoint ConvertUV(uint16 tile_index,int32 x,int32 y);
private:
	void BuildConvertionTable();
	
	struct uv_rect
	{
		float u0,v0;
		float u1,v1;
		float u2,v2;
		float u3,v3;
	};

	uint16 num_blocks;
	char name[21];
	GLuint *textures;
	
	WarPoint *convertion_table; // size: TILE_SPAN * TILE_SPAN * 8
	WarPoint *translation_table; // size: num_blocks
};

// inline definitions

inline char *WarTileset::Name()
{
	return name;
}

inline GLuint WarTileset::Tile(uint16 tile_index)
{
	return textures[0];
}

inline WarPoint WarTileset::ConvertUV(uint16 tile_index,int32 x,int32 y)
{
//	tile_index &= 0x1ff;
	return convertion_table[((TILE_SPAN + 1) * (TILE_SPAN + 1))*(tile_index&7) + (y<<(TILE_SHIFT)) + y + x] + translation_table[tile_index>>3];
}

#endif // #ifndef WAR_TILESET_H