#ifndef _WAR_TREE_H_
#define _WAR_TREE_H_

#include <OS.h>
#include <stdio.h>
#include "WarNode.h"

template<class Item> class WarTree
{
public:
	WarTree(int32 size,int32 step);
	virtual ~WarTree();
	uint16 Root();
	uint16 LeftChild(uint16 index);
	uint16 RightChild(uint16 index);
	uint16 Parent(uint16 index);
	Item *Data(uint16 index);
	int32 CountElements();
	
	void Empty();
	bool IsEmpty();
private:
	WarNode<Item> *m_tree;	// the array where the tree lives
	int32 m_tree_size;		// the number of allocated items
	int32 m_element_count;	// the number of items used in the array
	int32 m_lowest_available;	
};

// inline definitions

template <class Item>
WarTree<Item>::WarTree(int32 size,int32 step)
{
	m_tree_size = size;
	m_tree = new WarNode<Item>[m_tree_size];
	Empty();

#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"WarTree\n");
	fprintf(stderr,"  array: %ld\n",m_stack_size * sizeof(WarNode<Item>));
#endif

}

template <class Item>
WarTree<Item>::~WarTree()
{
	delete []m_tree;
}

template <class Item>
inline uint16 WarTree<Item>::LeftChild(uint16 index)
{
	return m_tree[index].m_left_child;
}

template <class Item>
inline uint16 WarTree<Item>::RightChild(uint16 index)
{
	return m_tree[index].m_right_child;
}

template <class Item>
inline Item *WarTree<Item>::Data(uint16 index)
{
	return &m_tree[index].data;
}

template <class Item>
inline void WarTree<Item>::Empty()
{
	m_element_count = 0;
	m_lowest_available = 0;
	m_root = 0;
	
	for (int32 i=0;i<m_tree_size;i++)
		m_tree[i].available = true;
}

template <class Item>
inline bool WarTree<Item>::IsEmpty()
{
	return m_element_count == 0;
}

template <class Item>
inline int32 WarTree<Item>::CountElements()
{
	return m_element_count;
}

#endif //_WAR_TREE_H_