#ifndef _WAR_VERTEX_H_
#define _WAR_VERTEX_H_

#include <OS.h>
#include <gl.h>
#include <math.h>

class WarVertex
{
public:
	WarVertex();
	WarVertex(float x,float y,float z);

	void SetTo(float x,float y,float z);

	float Angle(WarVertex vertex);
	float Abs();
	void SetToAngle(float y_angle,float z_angle);
	void RotateBy(WarVertex vertex);

	GLfloat x,y,z;	// the coordinates

	operator GLfloat*();

	WarVertex operator=(WarVertex vec);
	WarVertex &operator=(WarVertex *vec);
	WarVertex &operator+=(WarVertex vec);
	WarVertex &operator+=(WarVertex *vec);
	WarVertex &operator/=(int32 val);
	WarVertex operator/(int32 val);
	WarVertex operator*(int32 val);
	WarVertex operator*(float val);
	void operator*=(float val);
	WarVertex operator+(WarVertex vec);
	WarVertex operator-(WarVertex vec);
	bool operator==(WarVertex vec);
	bool operator!=(WarVertex vec);
	WarVertex operator-();
};

// inline definitions

inline WarVertex::WarVertex()
{
	x=0.0;
	y=0.0;
	z=0.0;
}

inline WarVertex::WarVertex(float x_,float y_,float z_)
{
	x=x_;
	y=y_;
	z=z_;
}

inline WarVertex::operator GLfloat*()
{
	return (GLfloat*)this;
}

inline WarVertex WarVertex::operator=(WarVertex vec)
{
	x=vec.x;
	y=vec.y;
	z=vec.z;
	return vec;
}

inline WarVertex &WarVertex::operator=(WarVertex *vec)
{
	x=vec->x;
	y=vec->y;
	z=vec->z;
	return *this;
}

inline WarVertex &WarVertex::operator+=(WarVertex vec)
{
	x+=vec.x;
	y+=vec.y;
	z+=vec.z;
	return *this;
}

inline WarVertex &WarVertex::operator+=(WarVertex *vec)
{
	x+=vec->x;
	y+=vec->y;
	z+=vec->z;
	return *this;
}

inline WarVertex WarVertex::operator+(WarVertex vec)
{
	return WarVertex(x+vec.x,y+vec.y,z+vec.z);
}

inline WarVertex WarVertex::operator-(WarVertex vec)
{
	return WarVertex(x-vec.x,y-vec.y,z-vec.z);
}

inline WarVertex WarVertex::operator*(int32 val)
{
	return WarVertex(x*val,y*val,z*val);
}

inline WarVertex WarVertex::operator*(float val)
{
	return WarVertex(x*val,y*val,z*val);
}

inline void WarVertex::operator*=(float val)
{
	x*=val;
	y*=val;
	z*=val;
}

inline bool WarVertex::operator==(WarVertex vec)
{
	return x==vec.x && y==vec.y && z==vec.z;
}

inline bool WarVertex::operator!=(WarVertex vec)
{
	return x!=vec.x || y!=vec.y || z!=vec.z;
}

inline WarVertex &WarVertex::operator/=(int32 val)
{
	x/=val;
	y/=val;
	z/=val;
	return *this;
}

inline WarVertex WarVertex::operator-()
{
	return WarVertex(-x,-y,-z);
}

inline WarVertex WarVertex::operator/(int32 val)
{
	return WarVertex(x/val,y/val,z/val);
}

inline float WarVertex::Angle(WarVertex vertex)
{
	return acos(x*vertex.x + y*vertex.y + z*vertex.z);
}

inline void WarVertex::SetTo(float nx,float ny,float nz)
{
	x=nx;
	y=ny;
	z=nz;
}

inline void WarVertex::SetToAngle(float y_angle,float z_angle)
{
	x = 1.0;
	y = 0.0;
	z = 0.0;
	
	RotateBy(WarVertex(0.0,y_angle,z_angle));
}

inline void WarVertex::RotateBy(WarVertex vertex)
{
	float tx;
	float ty;
	float tz;

	ty = y * cos(vertex.x) - z * sin(vertex.x);
	tz = y * sin(vertex.x) + z * cos(vertex.x);
	y=ty;
	z=tz;

	tx = z * sin(vertex.y) + x * cos(vertex.y);
	tz = z * cos(vertex.y) - x * sin(vertex.y);
	x=tx;
	z=tz;

	tx = x * cos(vertex.z) - y * sin(vertex.z);
	ty = x * sin(vertex.z) + y * cos(vertex.z);
	x=tx;
	y=ty;
}

inline float WarVertex::Abs()
{
	return sqrt(x*x + y*y + z*z );
}

#endif // _WAR_VERTEX_H_
