#ifndef _WAR_DISPLAY_
#define _WAR_DISPLAY_

#include "defs.h"

#include "WarView.h"
#include "WarMap.h"
#include "WarVertex.h"
#include "WarEntityList.h"
#include "WarCamera.h"
#include "WarPolygon.h"
#include "WarTriangulation.h"
#include "WarGLView.h"
#include "WarTileset.h"
#include "WarPlanet.h"

class WarObject;
class WarWeather;
class WarSkydome;

#include <gl.h>

#include <Application.h>
#include <GraphicsCard.h>
#include <List.h>
#include <Point.h>

class WarDisplay: public WarView
{
public:
	WarDisplay(int32 left,int32 top,uint16 width,uint16 height,WarEntity *planet,WarSkydome *skydome,WarCamera *camera,uint8 team);
	virtual ~WarDisplay();

//	virtual void Resize(int32 left,int32 top,uint16 width,uint16 height);

	virtual void MouseDown(WarPoint point,uint8 button);
	virtual void MouseUp(WarPoint point,uint8 button);
	virtual void MouseMoved(WarPoint point);
	virtual void Update();

	virtual void Resize(int32 width,int32 height);
/*
	virtual void KeyDown(char key);
	virtual void KeyUp(char key);
*/

//	void Cycle(float frame_time);

	void BeginMouseLook();
	void EndMouseLook();
	bool MouseLook();

	void BeginMouseSelection();
	void EndMouseSelection();
	bool MouseSelection();

	void BeginMouseScroll();
	void EndMouseScroll();
	bool MouseScroll();

	void SetCamera(WarCamera *new_eyes); // sets the camera to look through
	WarCamera *Camera(); // returns the camera this display is looking through

	void SetDetailLevel(float detail);
	float DetailLevel();

	void SetWireframe(bool is);
	bool IsWireframe();

	int32 PolygonCount();
	int32 FanCount();

protected:
private:
// used when drawing the map
	void render_map(int32 x,int32 y,int32 level/*,bool entire*/);
	void render_tile(int32 x,int32 y,int8 level,uint16 tile_index,bool direction);

	float blended_leaf_height(int32 x,int32 y,uint8 direction,int8 level);
	float blended_middle_height(int32 x,int32 y,bool dir,int8 level);
// –––––––––––––––––––––––

	float abs(float val);

// –––––––––––––––––––––––
	WarCamera *m_safe_camera;
	WarCamera *m_camera;	// the camera attached to this display
// –––––––––––––––––––––––
	// desribes the scene to draw
	
	// this should be a list of objects
	WarEntity *m_entity;
	
	WarSkydome *m_war_skydome;
// –––––––––––––––––––––––
	bool mouse_selection; // is true while dragging a selectionbox
	bool mouse_scroll; // is true while mouse scrolling
	bool mouse_look; // is true while mouselook is active
	WarPoint current_mouse_position; // the current mouse position
	WarPoint mouse_position; // the mouse position in the begining of the current frame (the mouse can move during a frame)
	WarPoint orig_mouse_position; // the mouse position when the selection box was activated
// –––––––––––––––––––––––
	bool wireframe; // is true if the landscape should be drawn in wireframe
// –––––––––––––––––––––––

	uint8 team; // the players team id
};


// inline definitions

inline void WarDisplay::SetWireframe(bool is)
{
	wireframe = is;
}

inline bool WarDisplay::IsWireframe()
{
	return wireframe;
}

inline int32 WarDisplay::PolygonCount()
{
	return 0;
//	return poly_count;
}

inline int32 WarDisplay::FanCount()
{
	return 0;
//	return fan_count;
}

inline void WarDisplay::SetCamera(WarCamera *new_eyes)
{
	m_camera = new_eyes;
}

inline WarCamera *WarDisplay::Camera()
{
	return m_safe_camera;
}

inline void WarDisplay::BeginMouseLook()
{
	m_gl_view->LockCursor(true);
	orig_mouse_position = mouse_position;
	SetMousePosition(WarPoint(Width()>>1,Height()>>1));
	mouse_look = true;
}

inline void WarDisplay::EndMouseLook()
{
	mouse_look = false;
	SetMousePosition(orig_mouse_position);
	m_gl_view->LockCursor(false);
}

inline bool WarDisplay::MouseLook()
{
	return mouse_look;
}

inline void WarDisplay::BeginMouseSelection()
{
	Camera()->Lock();
	orig_mouse_position = current_mouse_position;
	mouse_selection = true;
}

inline void WarDisplay::EndMouseSelection()
{
/*
	WarUnit *entity;
	uint16 id,cookie=0;
	while(war_entity_list->GetNextEntity(&entity,NULL,&id,&cookie))
	{
		WarBox box;
		entity->GetBoundingBox(&box);
		
		BRect rect(
			(min(orig_mouse_position.x,mouse_position.x))/Width(),
			(max(orig_mouse_position.y,mouse_position.y))/Height(),
			(max(orig_mouse_position.x,mouse_position.x))/Width(),
			(min(orig_mouse_position.y,mouse_position.y))/Height());

		if (Camera()->IsBoxVisible(&box,rect))
		{
			if (entity->IsSelected()) war_entity_list->DeselectEntity(entity,team);
			else war_entity_list->SelectEntity(entity,team);
		}
	}
	war_entity_list->UpdateSelection();
*/
	mouse_selection = false;
	Camera()->Unlock();
}

inline bool WarDisplay::MouseSelection()
{
	return mouse_selection;
}

inline void WarDisplay::BeginMouseScroll()
{
	m_gl_view->LockCursor(true);
	orig_mouse_position = mouse_position;
	SetMousePosition(WarPoint(Width()>>1,Height()>>1));
	mouse_scroll = true;
}

inline void WarDisplay::EndMouseScroll()
{
	mouse_scroll = false;
	SetMousePosition(orig_mouse_position);
	m_gl_view->LockCursor(false);
}

inline bool WarDisplay::MouseScroll()
{
	return mouse_scroll;
}

inline float WarDisplay::abs(float val)
{
	return (val<0.0)?-val:val;
}

inline void WarDisplay::SetDetailLevel(float level)
{
	m_entity->SetDetailLevel(level);
}

inline float WarDisplay::DetailLevel()
{
	return m_entity->DetailLevel();
}

#endif // #ifndef _WAR_DISPLAY_