#ifndef _WAR_GL_VIEW_
#define _WAR_GL_VIEW_

#include <opengl/GLView.h>

#include "WarList.h"
#include "WarStack.h"
#include "WarMouseView.h"

class WarView;
class WarCli;
class WarStringInput;
class WarMenu;
class WarDisplay;
class WarSelectionView;
class WarStatView;
class WarTextOutput;

class WarEntityList;
class WarMap;
class WarWeather;
class WarCamera;
class WarSkydome;

class WarGLView : public BGLView
{
public:
	WarGLView( BRect frame,char *name, int32 mode, int32 flags );
	virtual ~WarGLView( void );

	// Override BView methods:
	virtual void AttachedToWindow();
	virtual void DetachedFromWindow();
	virtual void FrameResized( float width, float height );
	virtual void KeyDown( const char *bytes, int32 size );
	virtual void KeyUp( const char *bytes, int32 size );
	virtual void MouseDown(BPoint point);
	virtual void MouseUp(BPoint point);
	virtual void MouseMoved(BPoint point,uint32 transit,const BMessage *message);
	
	// Override BGLView method:
	virtual void ErrorCallback( GLenum err );

	void HideCursor(bool hidden);
	bool IsCursorHidden();

	void LockCursor(bool locked);
	bool IsCursorLocked();

	void AddChild(WarView *view);
	void RemoveChild(WarView *view);

	void Render();
	void Cycle(float frame_time);

	void SetMousePosition(WarPoint position);

	// takes a line, parses it and executes every command
	void Parse(char *line,bool quiet=false,int32 call_stack = 0);
	
	// if the given view is in focus, the focus is taken away from
	// that view, and given back to the view that was focused before
	void LeaveFocus(WarView *focus_view);
	
	// makes the given view the focus view
	void GiveFocus(WarView *view);

	// adds the given view to the remove stack
	void Quit(WarView *view);

private:
	// executes one word (one command)
	int32 execute(char *word,char *command_line,bool quiet=false,int32 call_stack = 0);

	// my own version of atoi, it supports negative numbers and returns
	// false if the string contains anything else than digits.
	bool atoi(int32 *val,char *string);
	bool atof(double *val,char *string);
	
	// a support function to split a line in words, used by the parser
	char *GetWord(char *line,char **cookie);

	WarMenu *ParseMenu(WarPoint position,char *command_line);

	static long main_task(void *data);
	
	int32 m_mouse_buttons; // the mouse buttons currently pressed

	bool m_running; // is true while the main thread is running
	thread_id m_main_thread;

// the top view on this stack is the one who should get the input
// if the stack is empty the WarGLView (this class) receives the input
	WarStack<WarView*> *m_focus_stack;

// every view in this stack is removed in the Cycle() call
	WarStack<WarView*> *m_remove_stack;

//–––––––––– the stacks ––––––––––––––
	WarStack<int32> *m_integer_stack;
	WarStack<double> *m_float_stack;
	WarStack<BString*> *m_string_stack;

	WarList<char*> *m_word_list;

//––––––– the directories ––––––––––––
	BDirectory *m_app_dir; // the applications directory
	BDirectory *m_script_dir; // the script directory
	BDirectory *m_map_dir; // the maps directory
	BDirectory *m_environment_dir; // the environment directory
	BDirectory *m_unit_dir; // the unit directory
	
//–––––––––– the views –––––––––––––––
	WarDisplay *m_war_display;
	WarCli *m_war_cli;
	WarStringInput *m_war_string_input;
	WarTextOutput *m_war_text_output;
	WarSelectionView *m_war_selection_view;
	WarStatView *m_war_statview;
	WarMouseView *m_war_mouse_view;

//––––––––– the fonts ––––––––––––––––
	WarFont *m_bigfont;
	WarFont *m_smallfont;

//––––––––– the cameras ––––––––––––––
	WarCamera *m_current_camera;
	WarCamera *m_camera; 

//–––––––––– the world –––––––––––––––––
	WarWeather *m_war_weather;
	WarSkydome *m_war_skydome;
	WarMap *m_war_map;
	WarEntityList *m_war_entity_list;
//––––––––––––––––––––––––––––––––––––

	uint8 m_team; // the players team

	WarList<WarView*> m_childs;
	BLocker m_child_lock;

	bool m_keyboard_state[130];
	
	float time_accum; // a timer that's used to count the fps
	int32 frames;		// countes the frame during one second

};

// inline definitions

inline void WarGLView::Quit(WarView *view)
{
	m_remove_stack->Push(view);
}

inline void WarGLView::HideCursor(bool hidden)
{
	m_war_mouse_view->HideCursor(hidden);
}

inline bool WarGLView::IsCursorHidden()
{
	return m_war_mouse_view->IsCursorHidden();
}

inline void WarGLView::LockCursor(bool locked)
{
	m_war_mouse_view->LockCursor(locked);
}

inline bool WarGLView::IsCursorLocked()
{
	return m_war_mouse_view->IsCursorLocked();
}

enum word_id
{
// constants
	WORD_FALSE = 0,
	WORD_TRUE,

// output functions
	WORD_PRINT_STRING,
	WORD_TRACE_STACK,
	WORD_POP,
	WORD_EMIT,
	WORD_TYPE,
	WORD_CR,
	WORD_SPACE,
	WORD_SPACES,

// arithmetic operators
	WORD_ADD,
	WORD_SUB,
	WORD_MUL,
	WORD_DIV,
	WORD_MODULO,
	WORD_DIV_MODULO,
	WORD_INC,
	WORD_DEC,
	WORD_MAX,
	WORD_MIN,
	WORD_ABS,
	WORD_NEGATE,
	WORD_LEFT_SHIFT,
	WORD_RIGHT_SHIFT,

// boolean operators
	WORD_EQUALS,
	WORD_LESS,
	WORD_GREATER,
	WORD_NOT_EQUAL,
	WORD_EQUAL_TO_ZERO,
	WORD_GREATER_THAN_ZERO,
	WORD_LESS_THAN_ZERO,
	WORD_AND,
	WORD_OR,
	WORD_NOT,

// bitwise operators
	WORD_BITWISE_AND,
	WORD_BITWISE_OR,
	WORD_BITWISE_XOR,

// string stack manipulation
	WORD_PUSH_STRING,

// integer stack manipulation
	WORD_DROP,
	WORD_2DROP,
	WORD_DUP,
	WORD_NON_ZERO_DUP,
	WORD_2DUP,
	WORD_SWAP,
	WORD_2SWAP,
	WORD_ROTATE,
	WORD_MINUS_ROTATE,
	WORD_NIP,
	WORD_TUCK,
	WORD_OVER,
	WORD_2OVER,
	WORD_PICK,
	WORD_CLEAR,

//
	WORD_DEPTH,
	WORD_WORDS,
	WORD_EXEC,

// control structures
	WORD_IF,
	WORD_ELSE,
	WORD_THEN,
	WORD_CASE,
	WORD_ENDCASE,
	WORD_OF,
	WORD_ENDOF,

// word related
	WORD_CONSTANT,
	WORD_COMPILE,
	WORD_END_COMPILE,
	WORD_FORGET,

// comments
	WORD_START_COMMENT,
	WORD_END_COMMENT,

// ––––––––––––––––––––––––––––––––––––– GAME RELATED –––––––––––––––––––––––––––––––––––––-

	WORD_SET_CURSOR,
	WORD_CURSOR,

	WORD_GAME_MODIFIERS,

	WORD_GAME_CONSOLE_SHOW,
	WORD_GAME_CONSOLE_IS_VISIBLE,

	WORD_GAME_INPUT,

	WORD_GAME_SET_FULLSCREEN_MODE,
	WORD_GAME_FULLSCREEN_MODE,
	WORD_GAME_STATS_SHOW,
	WORD_GAME_STATS_IS_VISIBLE,
	WORD_GAME_SAY,
	WORD_GAME_QUIT,
	WORD_GAME_MENU,
	WORD_GAME_MOUSE_POSITION,

// selection operations
	WORD_GAME_SELECTION_DESELECT_UNITS,
	WORD_GAME_SELECTION_ASSIGN_GROUP,
	WORD_GAME_SELECTION_SELECT_GROUP,
	WORD_GAME_SELECTION_DESELECT_GROUP,
	WORD_GAME_SELECTION_LEAVE_GROUPS,
	WORD_GAME_SELECTION_COUNT_UNITS,
	WORD_GAME_SELECTION_UNIT_ID,

// camera operations
	WORD_GAME_CAMERA_ATTACH,
	WORD_GAME_CAMERA_DETACH,
	WORD_GAME_CAMERA_LOOK_AT_UNIT,
	WORD_GAME_CAMERA_STOP_LOOK_AT_UNIT,
	WORD_GAME_CAMERA_LOCK,
	WORD_GAME_CAMERA_UNLOCK,
	WORD_GAME_CAMERA_ADD_MOVE_SPEED,
	WORD_GAME_CAMERA_ADD_ROTATION_SPEED,
	WORD_GAME_CAMERA_ADD_SCROLL_SPEED,
	WORD_GAME_CAMERA_ROTATE_BY,
	WORD_GAME_CAMERA_MOVE_BY,
	WORD_GAME_CAMERA_SET_FOV,
	WORD_GAME_CAMERA_FOV,
	WORD_GAME_CAMERA_SET_FAR_CLIP,
	WORD_GAME_CAMERA_FAR_CLIP,
	WORD_GAME_CAMERA_MOVE,
	WORD_GAME_CAMERA_LOOK_AT,
	WORD_GAME_CAMERA_POSITION,

// display operations
	WORD_GAME_DISPLAY_SWITCH_CAMERA,

	WORD_GAME_DISPLAY_BEGIN_MOUSE_LOOK,
	WORD_GAME_DISPLAY_END_MOUSE_LOOK,
	WORD_GAME_DISPLAY_MOUSE_LOOK,

	WORD_GAME_DISPLAY_BEGIN_MOUSE_SELECTION,
	WORD_GAME_DISPLAY_END_MOUSE_SELECTION,
	WORD_GAME_DISPLAY_MOUSE_SELECTION,

	WORD_GAME_DISPLAY_BEGIN_MOUSE_SCROLL,
	WORD_GAME_DISPLAY_END_MOUSE_SCROLL,
	WORD_GAME_DISPLAY_MOUSE_SCROLL,

	WORD_GAME_DISPLAY_SET_DETAIL_LEVEL,
	WORD_GAME_DISPLAY_DETAIL_LEVEL,
	WORD_GAME_DISPLAY_SET_WIREFRAME,
	WORD_GAME_DISPLAY_WIREFRAME,
};

static char *words[]=
{
// constants
	"false",	// represents boolean false	( -- 0 )
	"true",	// represents boolean true		( -- -1 )

// output functions
	".\"",	// displays the folowing string
	".s",		// peeks the integer stack without touching it ( -- )
	".",		// pops one entry from the integer stack and displays it ( a -- )
	"emit",	// pops an element from the integer stack and displays it as a character ( a -- )
	"type",	// pops a string from the string stack and displays it
	"cr",		// prints a carrige return
	"space",	// prints a space
	"spaces",	// prints n spaces ( n -- )
	
// arithmetic operators
	"+",		// add	( a b -- a+b )
	"-",		// sub	( a b -- a-b )
	"*",		// multiply	( a b -- a*b )
	"/",		// divide	( a b -- a/b )
	"mod",	// modulo	( a b -- mod(a,b) )
	"/mod",	// returns the integer result part and the rest from the division	( a b -- a/b mod(a,b) )
	"1+",		// increases the last stack entry by 1	( a -- a+1 )
	"1-",		// decreases the last stack entry by 1	( a -- a-1 )
	"max",	// pops two elements from the integer stack and pushes the maximum back to the stack ( a b -- max(a,b) )
	"min",	// pops two elements from the integer stack and pushes the mainimum back to the stack ( a b -- min(a,b))
	"abs",	// returns the absolute value ( a -- abs(a) )
	"negate",	// negate the top element on the stack ( a -- -a )
	"lshift",	// left shift ( a n -- a<<n )
	"rshift",	// right shift ( a n -- a>>n )

// boolean operators
	"=",		// equal to
	"<",		// less than
	">",		// greater than
	"!=",		// not equal to
	"0=",		// equal to zero
	"0>",		// greater than zero
	"0<",		// less than zero
	"and",	// boolean and
	"or",		// boolean or
	"not",	// boolean not

// bitwise operators
	"&", // bitwise AND
	"|", // bitwise OR
	"^", // bitwise XOR

// string stack manipulation
	"\"",		// push everythin between the quotes to the string stack

// integer stack manipulation
	"drop",	// removes the last entry on the stack ( a b -- a )
	"2drop",	// removes the last pair of entries on the stack ( a b c -- a )
	"dup",	// duplicates the last stack entry ( a -- a a )
	"?dup",	// duplivates the last entry if non-zero ( a -- a a | 0 )
	"2dup",	// duplicates the last pair of stack entries ( a b -- a b a b )
	"swap",	// swaps the two last entries on the stack ( a b -- b a )
	"2swap",	// swaps the two last pair of entries on the stack ( a b c d -- c d a b )
	"rot",	// rotates the 3 top stack items to the left ( a b c -- b c a )
	"-rot",	// rotates the 3 top stack items to the right ( a b c -- c a b )
	"nip",	// remove the second item from the stack ( a b -- b )
	"tuck",	// copy top item to third position on stack ( a b -- b a b )
	"over",	// copies the second item to the top of the stack ( a b -- a b a )
	"2over",	// copies the second pair of items to the top of the stack ( a b c d -- a b c d a b )
	"pick",	// copies the Nth element to the first position ( ... v3 v2 v1 v0 N --- ... v3 v2 v1 v0 vN )
	"0sp",	// clears the stack, 'zero Stack pointer ( ... a b -- )



//
	"depth", // returns the depth of the stack, # stack entries
	"words", // lists the available words
	"include",  // executes the given file

// control structures
	"if",	// pops an element from the integer stack, if it is not equal to zero it jumps to it's else (or then)
	"else",	// jumps to then
	"then",
	"case",	// case
	"endcase",
	"of",
	"endof",

// word related
	"constant",	// defines a constnt (value <constant name> -- )
	":", // define a new word
	";",			// end compile definition
	"forget", // undefines a user defined word

// comments
	"(",	// start comment
	")",	// end comment



// ––––––––––––––––––––––––––––––––––––– GAME RELATED –––––––––––––––––––––––––––––––––––––-



// words to control the game
	"set_cursor", // sets the cursor type
	"cursor", // returns the cursor type

	"modifiers", // returns a mask of the currently pressed modifiers

	"console.show", // throws up the console
	"console.is_visible", // returns true or false

	"accept", // lets the user type a string, which is pushed to the stack

	"set_fullscreen_mode", // sets the 0: windowed or 1: fullscreen mode
	"fullscreen_mode", // returns 1 if in fullscreen mode
	"stats.show", // show or hide the stat view
	"stats.is_visible", // is the stats view visible?
	"say", // sends the string from the stack to the server
	"quit", // quits the game
	"menu", // takes the number of menuitems, the menuitems (as strings), returns the menuitem selected
	"mouse_position", // pushes the current mouse position as two integers, (x,y)

// selection operations
	"selection.deselect", // deselects all units
	"selection.assign_group", // assigns every selected unit to the given group
	"selection.select_group", // selects only units that is a part of the given group
	"selection.deselect_group", // selects only units that is a part of the given group
	"selection.leave_group", // every selected unit will leave it's group
	"selection.count_units", // returns the number of selected units
	"selection.unit_id", // returns the unit id of one of the selected units

// camera operations
	"camera.attach", // attaches the camera to the given unit_id
	"camera.detach", // detaches the camera from the unit_id the camera is attached to
	"camera.look_at_unit", // makes the camera look at a specified unit
	"camera.stop_look_at_unit", // makes the camera stop looking at the unit the camera currently is locked on
	"camera.lock", // makes the camera look at a specified unit
	"camera.unlock", // makes the camera stop looking at the unit the camera currently is locked on
	"camera.add_move_speed", // makes the camera move in the given direction in the given speed
	"camera.add_rotation_speed", // makes the camera rotate with the given speed
	"camera.add_scroll_speed", // scrolls the current display the given steps per second
	"camera.rotate_by", // makes the camera rotate
	"camera.move_by", // moves the camera to a relative position
	"camera.set_fov", // takes one parameter, sets the field of vision
	"camera.fov", // pushes the field of vision on the stack
	"camera.set_far_clip", // sets the distance to the far clip plane
	"camera.far_clip",	// retrieves the current distance to the far clip plane
	"camera.move", // takes x,y,z
	"camera.look_at", // takes a vertex (x,y,z) and rotates the camer so that it looks directly at the given point
	"camera.position", // returns the camera's current position

// display operations
	"display.switch_camera", // switches camera

	"display.begin_mouse_look",
	"display.end_mouse_look",
	"display.mouse_look",

	"display.begin_mouse_selection",
	"display.end_mouse_selection",
	"display.mouse_selection",

	"display.begin_mouse_scroll",
	"display.end_mouse_scroll",
	"display.mouse_scroll",

	"display.set_detail_level",
	"display.detail_level",
	"display.set_wireframe",
	"display.wireframe"
};

const int32 num_words=sizeof(words)/4;
const int32 stack_size=1024;

#endif // _WAR_GL_VIEW_
