#ifndef _WAR_INPUT_VIEW_
#define _WAR_INPUT_VIEW_
#include <gl.h>

#include "WarList.h"
#include "WarView.h"
#include "WarObject.h"
#include "WarFont.h"

class WarInputView: public WarView
{
public:
	WarInputView(int32 left,int32 top,int32 width,int32 height,WarFont *font,WarList<char*> *command_list);
	virtual ~WarInputView();

	void GetCommandLine(char *buffer,uint32 size);
	int32 CommandLineLength();
	
	void GetPrompt(char *buffer,uint32 size);

	void SetPrompt(char *string);
	void Reset();

	virtual void Resize(int32 width,int32 height);
	virtual void KeyDown(char key);

	virtual void LostFocus();
	virtual void GotFocus();

	void SetCursorType(uint8 type);
	uint8 CursorType();

	void SetCursorBlink(bool blink);
	bool CursorBlink();

	void SetColor(WarVertex color);
	WarVertex Color();

protected:
	virtual void Update();
private:
	WarVertex m_color;
	WarFont *m_war_font;

	char *m_command_line;	// the line the user is typing to.
	int32 m_command_len; // the length of the command_line, is zero before the user has typed anything
	int32 m_scroll_offset; // the number of caracters that has been scrolled out on the left side

	int32 m_cursor_position; // the position of the cursor in the string

	char m_prompt[30];
	int32 m_prompt_len;

	WarList<char*> *m_word_list;

	char *m_history[256]; // the history list works as a ringbuffer
	uint8 m_history_position; //the position in the history buffer
	uint8 m_history_end; // the end of the list, every command line is appended into the buffer

	uint32 m_speed; // the blink speed, a ,low number gives a faster blinking.
	uint8 m_cursor_type; // 0:block 1:pipe 2:underscore
	
	bool m_blink; // is tru if the cursor should blink
};

inline void WarInputView::SetCursorType(uint8 type)
{
	if (type < 3) m_cursor_type = type;
	else m_cursor_type = 0;
}

inline uint8 WarInputView::CursorType()
{
	return m_cursor_type;
}

inline void WarInputView::SetCursorBlink(bool blink)
{
	m_blink = blink;
}

inline bool WarInputView::CursorBlink()
{
	return m_blink;
}

inline void WarInputView::SetColor(WarVertex color)
{
	m_color = color;
}

inline WarVertex WarInputView::Color()
{
	return m_color;
}

inline int32 WarInputView::CommandLineLength()
{
	return m_command_len;
}

#endif // WAR_INPUT_VIEW_
