#ifndef _WAR_PLANET_
#define _WAR_PLANET_

#include "defs.h"

#include "WarMap.h"
#include "WarVertex.h"
#include "WarEntityList.h"
#include "WarCamera.h"
#include "WarPolygon.h"
#include "WarTriangulation.h"
#include "WarEntity.h"
#include "WarTileset.h"

class WarWeather;

#include <gl.h>

#include <Application.h>
#include <GraphicsCard.h>
#include <List.h>
#include <Point.h>

class WarPlanet: public WarEntity
{
public:
	WarPlanet(WarEntity entity,float radius,WarMap *map,WarEntityList *elist,WarWeather *weather);
	virtual ~WarPlanet();

	virtual void SetDetailLevel(float detail);
	virtual float DetailLevel();

	virtual void Update(WarCamera *camera);

	int32 PolygonCount();
	int32 FanCount();

protected:
private:
// used when drawing the map
	void render_map(int32 x,int32 y,int32 level,WarCamera *camera/*,bool entire*/);
	void render_tile(int32 x,int32 y,int8 level,uint16 tile_index,bool direction);

	float blended_leaf_height(int32 x,int32 y,uint8 direction,int8 level);
	float blended_middle_height(int32 x,int32 y,bool dir,int8 level);
// –––––––––––––––––––––––

	float abs(float val);

// –––––––––––––––––––––––
	// desribes the scene to draw
	WarMap *m_map;
	WarEntityList *m_entity_list;
	WarWeather *m_weather;
	WarTileset *m_tiles;
	float m_radius;
// –––––––––––––––––––––––
	WarBox *tmp_box; // temporary box used while rendering the map

	WarTriangulation *m_triangulation;

	WarVertex vertex_array[18];
	WarPoint texcoord_array[18];
	WarVertex color_array[18];

// –––––––––––––––––––––––
	int32 polygon_count; // is used while counting, can be zero if read from just after it was reset
	int32 poly_count; // the last frame's polygon count
// –––––––––––––––––––––––
	int32 trianglefan_count; // is used while counting, can be zero if read from just after it was reset
	int32 fan_count; // the last frame's fan count
};


// inline definitions

inline int32 WarPlanet::PolygonCount()
{
	return poly_count;
}

inline int32 WarPlanet::FanCount()
{
	return fan_count;
}

// calculate a geomorphed height

inline float WarPlanet::blended_leaf_height(int32 x,int32 y,uint8 dir,int8 level)
{
	static int32 sinus1[]={1,1,-1,-1};
	static int32 cosin1[]={1,-1,-1,1};
	static int32 sinus2[]={0,1,0,-1};
	static int32 cosin2[]={1,0,-1,0};

	return m_map->GetHeight(x + level * cosin2[dir],y + level * sinus2[dir]);

	uint8 blend = m_triangulation->Lookup(x + level * cosin2[dir],y + level * sinus2[dir]);

	if (blend == 255) return m_map->GetHeight(x + level * cosin2[dir],y + level * sinus2[dir]);

	float t = blend / 255.0;
	return t*(m_map->GetHeight(x + level * cosin2[dir],y + level * sinus2[dir])) + 
				(1.0-t)*(m_map->GetHeight(x + level * cosin1[dir],y + level * sinus1[dir]) + m_map->GetHeight(x + level * cosin1[(dir-1)&3],y + level * sinus1[(dir-1)&3])) * 0.5;
}

inline float WarPlanet::blended_middle_height(int32 x,int32 y,bool dir,int8 level)
{
	return m_map->GetHeight(x,y);
/*
	dir is false:
	+-----+
	|    /|
	|   / |
	|  *  |
	| /   |
	|/    |
	+-----+

	dir is true:
	+-----+
	|\    |
	| \   |
	|  *  |
	|   \ |
	|    \|
	+-----+
	
	the * is the position of the returned blended value
*/
	float height = m_map->GetHeight(x,y);
	uint8 blend = m_triangulation->Lookup(x,y);
	if (blend == 255) return height;
	float t = blend / 255.0;
	if (dir) return t*height + (1.0-t) * (m_map->GetHeight(x - level,y + level) + m_map->GetHeight(x + level,y - level)) * 0.5;
	else return t*height + (1.0-t) * (m_map->GetHeight(x + level,y + level) + m_map->GetHeight(x - level,y - level)) * 0.5;
}

inline float WarPlanet::abs(float val)
{
	return (val<0.0)?-val:val;
}

inline void WarPlanet::SetDetailLevel(float level)
{
	m_triangulation->SetDetailLevel(level);
}

inline float WarPlanet::DetailLevel()
{
	return m_triangulation->DetailLevel();
}

#endif // #ifndef _WAR_PLANET_