#ifndef _WAR_TRIANGULATION_
#define _WAR_TRIANGULATION_

#include <Locker.h>
#include <stdio.h>

#include "WarHeapf.h"
#include "WarMap.h"
#include "WarCamera.h"

class WarTriangulation
{
public:
	WarTriangulation(WarMap *map);
	~WarTriangulation();

// ––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––

	uint16 Width(); // returns the size of the map (in 16x16 blocks)
	uint16 Height();

	uint8 Lookup(int32 x,int32 y);

	void Regenerate(WarCamera *war_camera);
	void GenerateStatic();

	float DetailLevel();
	void SetDetailLevel(float level);

private:

	void set(int32 x,int32 y,uint8 val);

	void generate_map_triangulation(int32 x,int32 y,int32 level,WarCamera *camera);
	void generate_tile_triangulation(int32 x,int32 y,int32 level,float blend,WarCamera *camera);
	float subdivide(int32 x,int32, int32 level,WarCamera *camera);


	void generate_map_triangulation(int32 x,int32 y,int32 level);
	void generate_tile_triangulation(int32 x,int32 y,int32 level,float blend);
	float subdivide(int32 x,int32, int32 level);

// ––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
	WarMap *war_map;	// the map

	uint8 *quad_tree; // the quad-tree that describes the map-tesselation
							// size: (width+1)*(height+1) one byte for each map height-sample
	WarHeapf *queue;	// is used while building the triangulation

	float detail_level;

	WarBox *box;	// temporary box used for frustum culling
};

// inline definitions

inline float WarTriangulation::DetailLevel()
{
	return detail_level;
}

inline void WarTriangulation::SetDetailLevel(float level)
{
	if (level < 1.0) detail_level = 1.0;
	else detail_level = level;
}

inline uint8 WarTriangulation::Lookup(int32 x,int32 y)
{
	if(x<0 || y<0 || x>war_map->Width() || y>war_map->Height()) return 255;
	return quad_tree[(y<<war_map->WidthShift()) + y + x];
}

inline uint16 WarTriangulation::Width()
{
	return war_map->Width();
}

inline uint16 WarTriangulation::Height()
{
	return war_map->Height();
}

inline void WarTriangulation::set(int32 x,int32 y,uint8 val)
{
	quad_tree[(y<<war_map->WidthShift()) + y + x] = val;
}

#endif // #ifndef _WAR_TRIANGULATION_