#ifndef _WAR_VIEW_
#define _WAR_VIEW_

#include "WarFont.h"
#include "WarVertex.h"
#include "WarPoint.h"

#include <GraphicsCard.h>
#include <List.h>

class WarView
{
friend class WarGLView;
public:
	WarView(int32 left,int32 top,int32 width,int32 height);
	virtual ~WarView();

	virtual void Resize(int32 width,int32 height);
	void Move(int32 x,int32 y);
	void Move(WarPoint position);

	void Invalidate();

	void Hide(bool hidden);
	bool IsHidden();

	int32 Left();
	int32 Top();
	int32 Right();
	int32 Bottom();
	int32 Width();
	int32 Height();

	void DrawString(float x,float y,WarFont *font,const char *string,WarVertex color = WarVertex(1.0,1.0,1.0));

	virtual void SetMousePosition(WarPoint position);

	// called on mouse events
	virtual void MouseDown(WarPoint point,uint8 button);
	virtual void MouseUp(WarPoint point,uint8 button);
	virtual void MouseMoved(WarPoint point);
	
	// called on key events
	virtual void KeyDown(char key);
	virtual void KeyUp(char key);

	// this method is called when the view loses it's focus
	virtual void LostFocus();
	
	// this method is called when the view gaines it's focus
	virtual void GotFocus();

	// if this view is in focus, take it
	// away from it
	void LeaveFocus();
	
	void Quit(); // removes the view

protected:
	virtual void Update();
	WarGLView *m_gl_view;
private:
	bool m_hidden;

	int32 m_view_left,
			m_view_top,
			m_view_width,
			m_view_height;
};

// inline definitions
inline int32 WarView::Left()
{
	return m_view_left;
}

inline int32 WarView::Top()
{
	return m_view_top;
}

inline int32 WarView::Right()
{
	return m_view_left+m_view_width-1;
}

inline int32 WarView::Bottom()
{
	return m_view_top+m_view_height-1;
}

inline int32 WarView::Width()
{
	return m_view_width;
}

inline int32 WarView::Height()
{
	return m_view_height;
}

inline void WarView::Move(int32 x,int32 y)
{
	m_view_left=x;
	m_view_top=y;
}

inline void WarView::Move(WarPoint position)
{
	m_view_left = (int32)position.x;
	m_view_top = (int32)position.y;
}

inline void WarView::Hide(bool hidden)
{
	m_hidden=hidden;
}

inline bool WarView::IsHidden()
{
	return m_hidden;
}

#endif // #ifndef _WAR_VIEW_