#include "WarBuffer.h"
#include "WarSemaphore.h"

#include <stdio.h>
#include <string.h>

_EXPORT WarBuffer::WarBuffer(size_t buffer_size,const char *name):WarSemaphore(false,name)
{
	size=buffer_size;
	buffer=new uint8[size];
	Empty();
}

_EXPORT WarBuffer::~WarBuffer()
{
	delete []buffer;
}

_EXPORT void WarBuffer::Empty()
{
	write_offs=0;
	read_offs=0;
}

// ******* WRITE *******

_EXPORT status_t WarBuffer::PushBuffer(WarBuffer *buffer)
{
	if (write_offs + buffer->DataSize() > size) return B_ERROR;
	
	PushUint16(buffer->DataSize());
	memcpy(buffer+write_offs,buffer->Data(),buffer->DataSize());
	return B_OK;
}

_EXPORT status_t WarBuffer::PushUint32(uint32 longword)
{
	if (write_offs+sizeof(longword) > size) return B_ERROR;

	memcpy(buffer+write_offs,&longword,sizeof(longword));
	write_offs+=sizeof(longword);
	return B_OK;
}

_EXPORT status_t WarBuffer::PushUint16(uint16 word)
{
	if (write_offs+sizeof(word) > size) return B_ERROR;

	memcpy(buffer+write_offs,&word,sizeof(word));
	write_offs+=sizeof(word);
	return B_OK;
}

_EXPORT status_t WarBuffer::PushInt16(int16 word)
{
	if (write_offs+sizeof(word) > size) return B_ERROR;

	memcpy(buffer+write_offs,&word,sizeof(word));
	write_offs+=sizeof(word);
	return B_OK;
}

_EXPORT status_t WarBuffer::PushString(char *string)
{
	uint16 len=strlen(string);
	if (write_offs+len+1 > size) return B_ERROR;

	strcpy((char*)buffer+write_offs,string);
	write_offs+=len;
	buffer[write_offs++]=0;
	return B_OK;
}

_EXPORT status_t WarBuffer::PushUint8(uint8 byte)
{
	if (write_offs+sizeof(byte) > size) return B_ERROR;

	buffer[write_offs]=byte;
	write_offs+=sizeof(byte);
	return B_OK;
}

_EXPORT status_t WarBuffer::PushFloat(float val)
{
	if (write_offs+sizeof(float) > size) return B_ERROR;

	memcpy(buffer+write_offs,&val,sizeof(float));
	write_offs+=sizeof(float);
	return B_OK;
}

// ******* READ *******

_EXPORT status_t WarBuffer::PopBuffer(WarBuffer *buf)
{
	if (read_offs+sizeof(uint32) > write_offs) return B_ERROR;

	uint16 size = PopUint16();

	buf->Empty();
	if (buf->Size() < size) return B_ERROR;

	read_offs += size;
	memcpy(buf->Data(),buffer + read_offs,size);
	buf->write_offs = size;
	return B_OK;
}

_EXPORT uint32 WarBuffer::PopUint32()
{
	if (read_offs+sizeof(uint32) > write_offs) return 0;

	read_offs+=sizeof(uint32);
	return *((uint32*)buffer+read_offs);
}

_EXPORT uint16 WarBuffer::PopUint16()
{
	if (read_offs+sizeof(int16) > write_offs) return 0;

	read_offs+=sizeof(int16);
	return *((uint32*)buffer+read_offs);
}

_EXPORT int16 WarBuffer::PopInt16()
{
	if (read_offs+sizeof(int16) > write_offs) return 0;

	read_offs+=sizeof(int16);
	return *((int16*)buffer+read_offs);
}

_EXPORT uint8 WarBuffer::PopUint8()
{
	if (read_offs+sizeof(uint8) > write_offs) return 0;

	read_offs+=sizeof(uint8);
	return buffer[read_offs];
}

_EXPORT float WarBuffer::PopFloat()
{
	if (read_offs+sizeof(float) > write_offs) return 0;

	read_offs+=sizeof(float);
	return *((float*)buffer+read_offs);
}

_EXPORT status_t WarBuffer::PopString(char *buffer,uint16 size)
{
	if (size == 0) return B_OK;
	if (write_offs == read_offs) return B_ERROR;

	strncpy(buffer,(char*)buffer+read_offs,size);
	read_offs+=strlen(buffer)+1;
	return B_OK;
}
