#include "WarDisplay.h"
#include "WarGLView.h"
#include "WarMap.h"
#include "WarPolygon.h"
#include "WarBox.h"
#include "WarStack.h"
#include "WarSkydome.h"
#include "defs.h"

#include <TranslationUtils.h>
#include <List.h>
#include <stdlib.h>
#include <math.h>
#include <gl.h>
#include <glu.h>
#include <Autolock.h>

//#define WIREFRAME

_EXPORT WarDisplay::WarDisplay(int32 left,int32 top,uint16 width,uint16 height,WarMap *map,WarEntityList *elist,WarWeather *weather,WarSkydome *skydome,WarCamera *eyes,uint8 team_id):
	WarView(left,top,width,height)
{
	team = team_id;
	war_map = map;
	m_safe_camera = m_camera = eyes;
	war_entity_list=elist;
	war_weather = weather;
	m_war_skydome = skydome;
	tiles = war_map->Tileset();

// initialization
	mouse_look = false;
	mouse_selection = false;
	mouse_scroll = false;
	wireframe = false;

	m_triangulation=new WarTriangulation(war_map);
	m_triangulation->GenerateStatic();

	tmp_box = new WarBox();
	
//	GLfloat LightDiffuse[]=         { 250.0f, 250.0f, 100.0f, 1.0f };
	GLfloat DirectionalDiffuse[]=        { .5f, .5f, .5f, .5f };

	// Define material properties of specular color and degree of
	// shininess.  Since this is only done once in this particular
	// example, it applies to all objects.  Material properties can
	// be set for individual objects, individual faces of the objects,
	// individual vertices of the faces, etc...
/*
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);
	glLightf(GL_LIGHT1, GL_QUADRATIC_ATTENUATION, 2.0);
	glLightf(GL_LIGHT1, GL_LINEAR_ATTENUATION, 50.0);
*/
	glLightfv(GL_LIGHT0, GL_DIFFUSE, DirectionalDiffuse);

	// Set the GL_AMBIENT_AND_DIFFUSE color state variable to be the 
	// one referred to by all following calls to glColor 
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE); 
	glEnable(GL_COLOR_MATERIAL);

	// Create a Directional Light Source 
	glEnable(GL_LIGHT0);
//	glEnable(GL_LIGHT1);
}

_EXPORT WarDisplay::~WarDisplay()
{
	delete m_triangulation;
	delete tmp_box;
}

void WarDisplay::MouseDown(WarPoint point,uint8 buttons)
{
	if (buttons&1) m_gl_view->Parse("+mouse1",true);
	if (buttons&2) m_gl_view->Parse("+mouse2",true);
	if (buttons&4) m_gl_view->Parse("+mouse3",true);
}

void WarDisplay::MouseUp(WarPoint point,uint8 buttons)
{
	if (buttons&1) m_gl_view->Parse("-mouse1",true);
	if (buttons&2) m_gl_view->Parse("-mouse2",true);
	if (buttons&4) m_gl_view->Parse("-mouse3",true);
}

_EXPORT void WarDisplay::MouseMoved(WarPoint point)
{
	if (mouse_look && point != WarPoint(Width()>>1,Height()>>1))
	{
		Camera()->RotateYBy((current_mouse_position.y - point.y)/300.0);
		Camera()->RotateZBy((current_mouse_position.x - point.x)/300.0);

		SetMousePosition(WarPoint(Width()>>1,Height()>>1));
	}
	else if (mouse_scroll && point != WarPoint(Width()>>1,Height()>>1))
	{
		float horiz = -(current_mouse_position.x - point.x)/8.0;
		float vertical = -(current_mouse_position.y - point.y)/8.0;

		Camera()->Scroll(horiz,vertical);

		SetMousePosition(WarPoint(Width()>>1,Height()>>1));
	}
	else current_mouse_position = point;
}
/*
_EXPORT void WarDisplay::KeyDown(char key)
{
}

_EXPORT void WarDisplay::KeyUp(char key)
{
}
*/

_EXPORT void WarDisplay::Resize(int32 width,int32 height)
{
	WarView::Resize(width,height);
	float fov = Camera()->HorizontalFov();
	Camera()->SetFov(fov,atan2(Height(),Width() / tan(fov)));
}

_EXPORT void WarDisplay::Update()
{
	m_safe_camera = m_camera;
	glShadeModel(GL_SMOOTH);

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glFrustum(-tan(Camera()->HorizontalFov()/2.0),tan(Camera()->HorizontalFov()/2.0),-tan(Camera()->VerticalFov()/2.0),tan(Camera()->VerticalFov()/2.0), 1.0,Camera()->FarClip());
//	glFrustum(-tan(Camera()->HorizontalFov()/2.0)*2,tan(Camera()->HorizontalFov()/2.0)*2,-tan(Camera()->VerticalFov()/2.0)*2,tan(Camera()->VerticalFov()/2.0)*2, 1.0,far_z_clip + 1000.0 );
	glMatrixMode( GL_MODELVIEW );

	glLoadIdentity();

	glRotatef(90.0,0.0,0.0,1.0);
	glRotatef(90.0,0.0,1.0,0.0);

	glRotatef(-Camera()->Direction().x * 180.0 / 3.1415927,1.0,0.0,0.0); // tilt
	glRotatef(-Camera()->Direction().y * 180.0 / 3.1415927,0.0,1.0,0.0); // up-down
	glRotatef(-Camera()->Direction().z * 180.0 / 3.1415927,0.0,0.0,1.0); // direction

	m_war_skydome->Draw();

	glTranslatef(-Camera()->Position().x,-Camera()->Position().y,-Camera()->Position().z); // position

// now, we generate the map triangulation
//	m_triangulation->Regenerate(Camera());

// Enable the GL fog
	glEnable(GL_FOG); 
	glFogfv(GL_FOG_COLOR, war_weather->FogColor());
	glFogi(GL_FOG_MODE, GL_LINEAR);
	glFogf(GL_FOG_DENSITY, 1.0);
/*
// draw the sky plane
// ---------------------

	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();
	glFrustum(-tan(Camera()->HorizontalFov()/2.0),tan(Camera()->HorizontalFov()/2.0),-tan(Camera()->VerticalFov()/2.0),tan(Camera()->VerticalFov()/2.0), 1.0,Camera()->FarClip());
	glMatrixMode( GL_MODELVIEW );

	glFogf(GL_FOG_START, 5000);
	glFogf(GL_FOG_END, 10000);
	glHint(GL_FOG_HINT, GL_NICEST);


	glBindTexture(GL_TEXTURE_2D, war_weather->SkyTexture());
	glColor3f(1.0,1.0,1.0);
	glDisable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);

	glBegin(GL_QUADS);
	glTexCoord2f(0.0,0.0);
	glNormal3f(0.0,0.0,1.0);
	glVertex3f(-10000.0,-10000.0,300.0);
	glTexCoord2f(war_map->Width()/50,0.0);
	glNormal3f(0.0,0.0,1.0);
	glVertex3f(war_map->Width() + 10000.0,-10000.0,300.0);
	glTexCoord2f(war_map->Width()/50,war_map->Height()/50);
	glNormal3f(0.0,0.0,1.0);
	glVertex3f(war_map->Width() + 10000.0,war_map->Height() + 10000.0,300.0);
	glTexCoord2f(0.0,war_map->Height()/50);
	glNormal3f(0.0,0.0,1.0);
	glVertex3f(-10000.0,war_map->Height() + 10000.0,300.0);
	glEnd();

// reset the projection matrix

	glMatrixMode( GL_PROJECTION );
	glPopMatrix();
	glMatrixMode( GL_MODELVIEW );

// ---------------------
*/
	glFogf(GL_FOG_START, Camera()->FarClip() * war_weather->FogStart());
	glFogf(GL_FOG_END, Camera()->FarClip() );

	glHint(GL_FOG_HINT, GL_FASTEST);

// render the landscape
// ---------------------
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);

	bool reset = false;
	if (wireframe)
	{
		glPolygonMode(GL_FRONT,GL_LINE);
		glDisable(GL_TEXTURE_2D);
		reset = true;
	}
	polygon_count = 0;
	trianglefan_count = 0;


	// !!!! TEMPOARARY !!!!
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnableClientState(GL_COLOR_ARRAY);

	glVertexPointer(3,GL_FLOAT,0,vertex_array);
	glColorPointer(3,GL_FLOAT,0,color_array);
	glTexCoordPointer(2,GL_FLOAT,0,texcoord_array);
	
	glBindTexture(GL_TEXTURE_2D, tiles->Tile(0));
	// *****

	render_map(war_map->Width()/2,war_map->Height()/2,war_map->Width()/2/*,false*/);
	poly_count = polygon_count;
	fan_count = trianglefan_count;
	if (reset)
	{
		glPolygonMode(GL_FRONT,GL_FILL);
		glEnable(GL_TEXTURE_2D);
	}
// ---------------------


// draw all the flying units
// ---------------------
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	WarUnit *body;
	uint16 id,cookie=0;
	while(war_entity_list->GetNextFlyingEntity(&body,NULL,&id,&cookie))
	{
		// draws the unit, only if it's visible
		body->GetBoundingBox(tmp_box);
		if (Camera()->IsBoxVisible(tmp_box)) body->Draw(Camera());
	}

	glDisable(GL_LIGHTING);
// ---------------------

// disable the GL fog
	glDisable(GL_FOG);


// if the selectionbox is active, then draw it
// ---------------------
	mouse_position = current_mouse_position;

	if (mouse_selection)
	{
		// initialize gl machine
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0.0,Width()+1,0.0,Height()+1,-10.0,1.0);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		// disable depthtest and texturing
		glDisable(GL_DEPTH_TEST);
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_FOG);

		// teh color of the selection box
		glColor3f(1.0,1.0,1.0);
		glDisable(GL_LIGHTING);
		// draw the selection box

		glBegin(GL_LINE_STRIP);
		glVertex2f(orig_mouse_position.x,orig_mouse_position.y);
		glVertex2f(orig_mouse_position.x,mouse_position.y);
		glVertex2f(mouse_position.x,mouse_position.y);
		glVertex2f(mouse_position.x,orig_mouse_position.y);
		glVertex2f(orig_mouse_position.x,orig_mouse_position.y);
		glEnd();
	}
// ---------------------
}

void WarDisplay::render_map(int32 x,int32 y,int32 level/*,bool entire*/)
{
	if (level == TILE_SPAN>>1)
	{
		tmp_box->SetTo(TILE_SPAN,TILE_SPAN,war_map->BoxHeight(x>>TILE_SHIFT,y>>TILE_SHIFT));
		tmp_box->OffsetBy(WarVertex(x,y,war_map->BoxElevation(x>>TILE_SHIFT,y>>TILE_SHIFT)));
//		glDisable(GL_TEXTURE_2D);
//		tmp_box->Draw();
//		glEnable(GL_TEXTURE_2D);
	}
	else
	{
		tmp_box->SetTo(level<<1,level<<1,64.0);
		tmp_box->OffsetBy(WarVertex(x,y,32.0));
	}

	if (/*entire || */Camera()->IsBoxVisible(tmp_box/*,&entire*/))
	{
		WarTileset *tiles=war_map->Tileset();

		if (level == TILE_SPAN>>1)
		{
			uint16 tile_index = war_map->TileIndex(x>>TILE_SHIFT,y>>TILE_SHIFT);
			if (m_triangulation->Lookup(x,y) == 0)
			{

				float tex_offset_x = x & (TILE_SPAN-1);
				float tex_offset_y = y & (TILE_SPAN-1);
/*
				glBegin(GL_TRIANGLE_FAN);

					glColor3fv(*war_weather->Color(war_map->Slope(x + level,y + level)));
					glTexCoord2fv(tiles->ConvertUV(tile_index,tex_offset_x + level_span,tex_offset_y + level_span));
					glVertex3f(x + level,y + level,war_map->GetHeight(x + level,y + level));

					glColor3fv(*war_weather->Color(war_map->Slope(x - level,y + level)));
					glTexCoord2fv(tiles->ConvertUV(tile_index,tex_offset_x - level_span,tex_offset_y + level_span));
					glVertex3f(x - level,y + level,war_map->GetHeight(x - level,y + level));

					glColor3fv(*war_weather->Color(war_map->Slope(x - level,y - level)));
					glTexCoord2fv(tiles->ConvertUV(tile_index,tex_offset_x - level_span,tex_offset_y - level_span));
					glVertex3f(x - level,y - level,war_map->GetHeight(x - level,y - level));

					glColor3fv(*war_weather->Color(war_map->Slope(x + level,y - level)));
					glTexCoord2fv(tiles->ConvertUV(tile_index,tex_offset_x + level_span,tex_offset_y - level_span));
					glVertex3f(x + level,y - level,war_map->GetHeight(x + level,y - level));

				glEnd();
*/

				vertex_array[0].SetTo(x + level,y + level,war_map->GetHeight(x + level,y + level));
				vertex_array[1].SetTo(x - level,y + level,war_map->GetHeight(x - level,y + level));
				vertex_array[2].SetTo(x - level,y - level,war_map->GetHeight(x - level,y - level));
				vertex_array[3].SetTo(x + level,y - level,war_map->GetHeight(x + level,y - level));

				color_array[0] = war_weather->Color(war_map->Slope(x + level,y + level));
				color_array[1] = war_weather->Color(war_map->Slope(x - level,y + level));
				color_array[2] = war_weather->Color(war_map->Slope(x - level,y - level));
				color_array[3] = war_weather->Color(war_map->Slope(x + level,y - level));

				texcoord_array[0] = tiles->ConvertUV(tile_index,(tex_offset_x + level),(tex_offset_y + level));
				texcoord_array[1] = tiles->ConvertUV(tile_index,(tex_offset_x - level),(tex_offset_y + level));
				texcoord_array[2] = tiles->ConvertUV(tile_index,(tex_offset_x - level),(tex_offset_y - level));
				texcoord_array[3] = tiles->ConvertUV(tile_index,(tex_offset_x + level),(tex_offset_y - level));

				glDrawArrays(GL_TRIANGLE_FAN,0,4);

				trianglefan_count++;
				polygon_count += 2;
			}
			else render_tile(x,y,level,tile_index,false);
		}
		else
		{
			int32 half_level = level>>1;
			render_map(x+half_level,y+half_level,half_level/*,entire*/);
			render_map(x+half_level,y-half_level,half_level/*,entire*/);
			render_map(x-half_level,y-half_level,half_level/*,entire*/);
			render_map(x-half_level,y+half_level,half_level/*,entire*/);
		}
	}
}

void WarDisplay::render_tile(int32 x,int32 y,int8 level,uint16 tile_index,bool direction)
{
	int8 half_level = level>>1;
	int8 double_level = level<<1;

	static int32 sinus1[]={1,1,-1,-1};
	static int32 cosin1[]={1,-1,-1,1};
	static int32 sinus2[]={0,1,0,-1};
	static int32 cosin2[]={1,0,-1,0};

	bool children[4];
	if (level <= 1)
	{
		children[0] = false;
		children[1] = false;
		children[2] = false;
		children[3] = false;
	}
	else
	{
		children[0] = m_triangulation->Lookup(x + half_level,y + half_level);
		children[1] = m_triangulation->Lookup(x - half_level,y + half_level);
		children[2] = m_triangulation->Lookup(x - half_level,y - half_level);
		children[3] = m_triangulation->Lookup(x + half_level,y - half_level);
	}

	bool sibblings[4];
	sibblings[0] = m_triangulation->Lookup(x+double_level,y);
	sibblings[1] = m_triangulation->Lookup(x,y+double_level);
	sibblings[2] = m_triangulation->Lookup(x-double_level,y);
	sibblings[3] = m_triangulation->Lookup(x,y-double_level);

	bool draw[4];

	draw[0] = !sibblings[0] || !sibblings[1] || !children[0];
	draw[1] = !sibblings[1] || !sibblings[2] || !children[1];
	draw[2] = !sibblings[2] || !sibblings[3] || !children[2];
	draw[3] = !sibblings[3] || !sibblings[0] || !children[3];

	int32 j;

	bool fan_open = false;
	int32 tex_offset_x = x & (TILE_SPAN-1);
	int32 tex_offset_y = y & (TILE_SPAN-1);

	int32 counter = 0;
	for (int32 i=0;i<4;i++)
	{
		int32 x_add1 = level * cosin1[i];
		int32 y_add1 = level * sinus1[i];
		int32 x_add2 = level * cosin2[i];
		int32 y_add2 = level * sinus2[i];

		if (draw[i])
		{
			if (!fan_open)
			{
//				float height = blended_middle_height(x,y,direction,level);
				float height = war_map->GetHeight(x,y);

				vertex_array[counter].SetTo(x,y,height);
				color_array[counter] = war_weather->Color(war_map->Slope(x,y));
				texcoord_array[counter] = tiles->ConvertUV(tile_index,tex_offset_x,tex_offset_y);
				counter++;
				fan_open = true;
			}

			if (sibblings[i])
			{
//				float height = blended_leaf_height(x,y,i,level);
				float height = war_map->GetHeight(x + x_add2,y + y_add2);

				color_array[counter] = war_weather->Color(war_map->Slope(x + x_add2,y + y_add2));
				vertex_array[counter].SetTo(x + x_add2,y + y_add2,height);
				texcoord_array[counter] = tiles->ConvertUV(tile_index,tex_offset_x + x_add2,tex_offset_y + y_add2);
				counter++;
			}

			color_array[counter] = war_weather->Color(war_map->Slope(x + x_add1,y + y_add1));
			vertex_array[counter].SetTo(x + x_add1,y + y_add1,war_map->GetHeight(x + x_add1,y + y_add1));
			texcoord_array[counter] = tiles->ConvertUV(tile_index,tex_offset_x + x_add1,tex_offset_y + y_add1);
			counter++;

			j=(i+1)&3;
			if (!draw[j])
			{
//				float height = blended_leaf_height(x,y,j,level);
				float height = war_map->GetHeight(x + level * cosin2[j],y + x_add2);

				color_array[counter] = war_weather->Color(war_map->Slope(x + level * cosin2[j],y + x_add2));
				texcoord_array[counter] = tiles->ConvertUV(tile_index,tex_offset_x + (level * cosin2[j]),tex_offset_y + x_add2);
				vertex_array[counter].SetTo(x + level * cosin2[j],y + x_add2,height);
				counter++;

				glDrawArrays(GL_TRIANGLE_FAN,0,counter);
				trianglefan_count++;
				polygon_count += counter - 2;

				fan_open = false;
				counter = 0;
			}
		}
	}

	int32 x_add1 = level * cosin1[0];
	int32 y_add1 = level * sinus1[0];
	int32 x_add2 = level * cosin2[0];
	int32 y_add2 = level * sinus2[0];

	if (fan_open)
	{
		if (sibblings[0])
		{
//			float height = blended_leaf_height(x,y,0,level);
			float height = war_map->GetHeight(x + x_add2,y + y_add2);

			color_array[counter] = war_weather->Color(war_map->Slope(x + x_add2,y + y_add2));
			texcoord_array[counter] = tiles->ConvertUV(tile_index,tex_offset_x + x_add2,tex_offset_y + y_add2);
			vertex_array[counter].SetTo(x + x_add2,y + y_add2,height);
			counter++;
		}
		else
		{
			color_array[counter] = war_weather->Color(war_map->Slope(x + x_add1,y + y_add1));
			texcoord_array[counter] = tiles->ConvertUV(tile_index,tex_offset_x + x_add1,tex_offset_y + y_add1);
			vertex_array[counter].SetTo(x + x_add1,y + y_add1,war_map->GetHeight(x + x_add1,y + y_add1));
			counter++;
		}
		glDrawArrays(GL_TRIANGLE_FAN,0,counter);
		polygon_count += counter - 2;
		trianglefan_count++;
	}

/*
	int32 counter = 0;
	for (int32 i=0;i<4;i++)
	{
		int32 x_add1 = level * cosin1[i];
		int32 y_add1 = level * sinus1[i];
		int32 x_add2 = level * cosin2[i];
		int32 y_add2 = level * sinus2[i];
		float x_add3 = x_add2/(float)TILE_SPAN;
		float y_add3 = y_add2/(float)TILE_SPAN;
		float x_add4 = x_add1/(float)TILE_SPAN;
		float y_add4 = y_add1/(float)TILE_SPAN;

		if (draw[i])
		{
			if (!fan_open)
			{
				float height = blended_middle_height(x,y,direction,level);

				glBegin(GL_TRIANGLE_FAN);

				glColor3fv(war_weather->Color(war_map->Slope(x,y)));
				glTexCoord2fv(tiles->ConvertUV(tile_index,tex_offset_x,tex_offset_y));
				glVertex3f(x,y,height);
				counter++;
				fan_open = true;
			}

			if (sibblings[i])
			{
				float height = blended_leaf_height(x,y,i,level);

				glColor3fv(war_weather->Color(war_map->Slope(x + x_add2,y + y_add2)));
				glTexCoord2fv(tiles->ConvertUV(tile_index,tex_offset_x + x_add3,tex_offset_y + y_add3));
				glVertex3f(x + x_add2,y + y_add2,height);
				counter++;
			}

			glColor3fv(war_weather->Color(war_map->Slope(x + x_add1,y + y_add1)));
			glTexCoord2fv(tiles->ConvertUV(tile_index,tex_offset_x + x_add4,tex_offset_y + y_add4));
			glVertex3f(x + x_add1,y + y_add1,war_map->GetHeight(x + x_add1,y + y_add1));
			counter++;

			j=(i+1)&3;
			if (!draw[j])
			{
				float height = blended_leaf_height(x,y,j,level);

				glColor3fv(war_weather->Color(war_map->Slope(x + level * cosin2[j],y + level * sinus2[j])));
				glTexCoord2fv(tiles->ConvertUV(tile_index,tex_offset_x + (level * cosin2[j])/((float)TILE_SPAN),tex_offset_y + (level * sinus2[j])/(float)TILE_SPAN));
				glVertex3f(x + level * cosin2[j],y + level * sinus2[j],height);
				counter++;

				glEnd();
				trianglefan_count++;
				polygon_count += counter - 2;

				fan_open = false;
				counter = 0;
			}
		}
	}

	int32 x_add1 = level * cosin1[0];
	int32 y_add1 = level * sinus1[0];
	int32 x_add2 = level * cosin2[0];
	int32 y_add2 = level * sinus2[0];
	float x_add3 = x_add2/(float)TILE_SPAN;
	float y_add3 = y_add2/(float)TILE_SPAN;
	float x_add4 = x_add1/(float)TILE_SPAN;
	float y_add4 = y_add1/(float)TILE_SPAN;

	if (fan_open)
	{
		if (sibblings[0])
		{
			float height = blended_leaf_height(x,y,0,level);

			glColor3fv(war_weather->Color(war_map->Slope(x + x_add2,y + y_add2)));
			glTexCoord2fv(tiles->ConvertUV(tile_index,tex_offset_x + x_add3,tex_offset_y + y_add3));
			glVertex3f(x + x_add2,y + y_add2,height);
			counter++;
		}
		else
		{
			glColor3fv(war_weather->Color(war_map->Slope(x + x_add1,y + y_add1)));
			glTexCoord2fv(tiles->ConvertUV(tile_index,tex_offset_x + x_add4,tex_offset_y + y_add4));
			glVertex3f(x + x_add1,y + y_add1,war_map->GetHeight(x + x_add1,y + y_add1));
			counter++;
		}
		glEnd();
		polygon_count += counter - 2;
		trianglefan_count++;
	}
*/

	if (!draw[0]) render_tile(x+half_level,y+half_level,half_level,tile_index,0);
	if (!draw[1]) render_tile(x-half_level,y+half_level,half_level,tile_index,1);
	if (!draw[2]) render_tile(x-half_level,y-half_level,half_level,tile_index,0);
	if (!draw[3]) render_tile(x+half_level,y-half_level,half_level,tile_index,1);
}
