#include "WarDisplay.h"
#include "WarGLView.h"
#include "WarMap.h"
#include "WarPolygon.h"
#include "WarBox.h"
#include "WarStack.h"
#include "WarSkydome.h"
#include "defs.h"

#include <TranslationUtils.h>
#include <List.h>
#include <stdlib.h>
#include <math.h>
#include <gl.h>
#include <glu.h>
#include <Autolock.h>

//#define WIREFRAME

_EXPORT WarDisplay::WarDisplay(int32 left,int32 top,uint16 width,uint16 height,WarEntity *planet,WarSkydome *skydome,WarCamera *eyes,uint8 team_id):
	WarView(left,top,width,height)
{
	m_entity = planet;
	team = team_id;
	m_safe_camera = m_camera = eyes;
	m_war_skydome = skydome;

// initialization
	mouse_look = false;
	mouse_selection = false;
	mouse_scroll = false;
	wireframe = false;

//	GLfloat LightDiffuse[]=         { 250.0f, 250.0f, 100.0f, 1.0f };
	GLfloat DirectionalDiffuse[]=        { .5f, .5f, .5f, .5f };

	// Define material properties of specular color and degree of
	// shininess.  Since this is only done once in this particular
	// example, it applies to all objects.  Material properties can
	// be set for individual objects, individual faces of the objects,
	// individual vertices of the faces, etc...
/*
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);
	glLightf(GL_LIGHT1, GL_QUADRATIC_ATTENUATION, 2.0);
	glLightf(GL_LIGHT1, GL_LINEAR_ATTENUATION, 50.0);
*/
	glLightfv(GL_LIGHT0, GL_DIFFUSE, DirectionalDiffuse);

	// Set the GL_AMBIENT_AND_DIFFUSE color state variable to be the 
	// one referred to by all following calls to glColor 
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE); 
	glEnable(GL_COLOR_MATERIAL);

	// Create a Directional Light Source 
	glEnable(GL_LIGHT0);
//	glEnable(GL_LIGHT1);
}

_EXPORT WarDisplay::~WarDisplay()
{
}

void WarDisplay::MouseDown(WarPoint point,uint8 buttons)
{
	if (buttons&1) m_gl_view->Parse("+mouse1",true);
	if (buttons&2) m_gl_view->Parse("+mouse2",true);
	if (buttons&4) m_gl_view->Parse("+mouse3",true);
}

void WarDisplay::MouseUp(WarPoint point,uint8 buttons)
{
	if (buttons&1) m_gl_view->Parse("-mouse1",true);
	if (buttons&2) m_gl_view->Parse("-mouse2",true);
	if (buttons&4) m_gl_view->Parse("-mouse3",true);
}

_EXPORT void WarDisplay::MouseMoved(WarPoint point)
{
	if (mouse_look && point != WarPoint(Width()>>1,Height()>>1))
	{
		Camera()->RotateYBy((current_mouse_position.y - point.y)/300.0);
		Camera()->RotateZBy((current_mouse_position.x - point.x)/300.0);

		SetMousePosition(WarPoint(Width()>>1,Height()>>1));
	}
	else if (mouse_scroll && point != WarPoint(Width()>>1,Height()>>1))
	{
		float horiz = -(current_mouse_position.x - point.x)/8.0;
		float vertical = -(current_mouse_position.y - point.y)/8.0;

		Camera()->Scroll(horiz,vertical);

		SetMousePosition(WarPoint(Width()>>1,Height()>>1));
	}
	else current_mouse_position = point;
}
/*
_EXPORT void WarDisplay::KeyDown(char key)
{
}

_EXPORT void WarDisplay::KeyUp(char key)
{
}
*/

_EXPORT void WarDisplay::Resize(int32 width,int32 height)
{
	WarView::Resize(width,height);
	float fov = Camera()->HorizontalFov();
	Camera()->SetFov(fov,atan2(Height(),Width() / tan(fov)));
}

_EXPORT void WarDisplay::Update()
{
	m_safe_camera = m_camera;
	glShadeModel(GL_SMOOTH);

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glFrustum(-tan(Camera()->HorizontalFov()/2.0),tan(Camera()->HorizontalFov()/2.0),-tan(Camera()->VerticalFov()/2.0),tan(Camera()->VerticalFov()/2.0), 1.0,Camera()->FarClip());
//	glFrustum(-tan(Camera()->HorizontalFov()/2.0)*2,tan(Camera()->HorizontalFov()/2.0)*2,-tan(Camera()->VerticalFov()/2.0)*2,tan(Camera()->VerticalFov()/2.0)*2, 1.0,far_z_clip + 1000.0 );
	glMatrixMode( GL_MODELVIEW );

	glLoadIdentity();

	glRotatef(90.0,0.0,0.0,1.0);
	glRotatef(90.0,0.0,1.0,0.0);

	glRotatef(-Camera()->Direction().x * 180.0 / 3.1415927,1.0,0.0,0.0); // tilt
	glRotatef(-Camera()->Direction().y * 180.0 / 3.1415927,0.0,1.0,0.0); // up-down
	glRotatef(-Camera()->Direction().z * 180.0 / 3.1415927,0.0,0.0,1.0); // direction

//	m_war_skydome->Draw();

	glTranslatef(-Camera()->Position().x,-Camera()->Position().y,-Camera()->Position().z); // position

// render the landscape
// ---------------------
	bool reset = false;
	if (wireframe)
	{
		glPolygonMode(GL_FRONT,GL_LINE);
		glDisable(GL_TEXTURE_2D);
		reset = true;
	}

	m_entity->Draw(Camera());

	if (reset)
	{
		glPolygonMode(GL_FRONT,GL_FILL);
		glEnable(GL_TEXTURE_2D);
	}

// if the selectionbox is active, then draw it
// ---------------------
	mouse_position = current_mouse_position;

	if (mouse_selection)
	{
		// initialize gl machine
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0.0,Width()+1,0.0,Height()+1,-10.0,1.0);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		// disable depthtest and texturing
		glDisable(GL_DEPTH_TEST);
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_FOG);

		// teh color of the selection box
		glColor3f(1.0,1.0,1.0);
		glDisable(GL_LIGHTING);
		// draw the selection box

		glBegin(GL_LINE_STRIP);
		glVertex2f(orig_mouse_position.x,orig_mouse_position.y);
		glVertex2f(orig_mouse_position.x,mouse_position.y);
		glVertex2f(mouse_position.x,mouse_position.y);
		glVertex2f(mouse_position.x,orig_mouse_position.y);
		glVertex2f(orig_mouse_position.x,orig_mouse_position.y);
		glEnd();
	}
// ---------------------
}
