#include "WarFace.h"

#include <math.h>
#include <stdio.h>

#ifndef abs
#define abs(x) (x<0?-x:x)
#endif

_EXPORT WarFace::WarFace(uint16 vertices)
{
	num_vertices = vertices;
	texture_coords = new BPoint[num_vertices];
	vex = new WarVertex*[num_vertices];
}

_EXPORT WarFace::~WarFace()
{
	delete []texture_coords;
	delete []vex;
}

_EXPORT bool WarFace::IsFront()
{
	int32 a= (int32)(vex[1]->x-vex[0]->x);
	int32 b= (int32)(vex[1]->y-vex[0]->y);

	if (abs(a) > abs(b))
	{
		if (a>0)	return ((vex[2]->x-vex[0]->x) * b / a + vex[0]->y > vex[2]->y);
		else		return ((vex[2]->x-vex[0]->x) * b / a + vex[0]->y < vex[2]->y);
		return true;
	}
	else
	{
		if (b>0)	return ((vex[2]->y-vex[0]->y) * a / b + vex[0]->x < vex[2]->x);
		else		return ((vex[2]->y-vex[0]->y) * a / b + vex[0]->x > vex[2]->x);
		return true;
	}
}

_EXPORT void WarFace::CalcNormal()
{
	if (VertexCount() < 3) return;

	float u1= vex[1]->x - vex[0]->x;
	float u2= vex[1]->y - vex[0]->y;
	float u3= vex[1]->z - vex[0]->z;
	float v1= vex[2]->x - vex[0]->x;
	float v2= vex[2]->y - vex[0]->y;
	float v3= vex[2]->z - vex[0]->z;

	normal.x = u2 * v3 - u3 * v2;
	normal.y = u3 * v1 - u1 * v3;
	normal.z = u1 * v2 - u2 * v1;

	float length = sqrt(normal.x*normal.x + normal.y*normal.y + normal.z*normal.z );

	normal.x /= length;
	normal.y /= length;
	normal.z /= length;
}
