#include <TranslationUtils.h>
#include <Bitmap.h>
#include <stdlib.h>
#include <gl.h>
#include <glu.h>
#include <string.h>
#include <File.h>
#include <Application.h>

#include "WarInputView.h"
#include "WarObject.h"
#include "WarGLView.h"

WarInputView::WarInputView(int32 left,int32 top,int32 width,int32 height,WarFont *font,WarList<char*> *command_list):WarView(left,top,width,height)
{
	m_war_font = font;
	m_word_list = command_list;

	*m_prompt = 0;
	m_command_line = new char[512];
	*m_command_line = 0;
	m_command_len = 0;
	m_scroll_offset = 0;
	m_cursor_position = 0;
	
	m_cursor_type = 2;
	
	for (uint16 i = 0; i < 256; i++) m_history[i] = NULL;
	m_history_position = 0;
	m_history_end = 0;

	m_speed = (1<<20)-1;

	strcpy(m_prompt,"");
	m_prompt_len = strlen(m_prompt);
	
	m_color.SetTo(1.0,1.0,1.0);
}

WarInputView::~WarInputView()
{
	delete []m_command_line;
	uint16 i;
	for (i=0;i<256;i++)
		if (m_history[i]) delete []m_history[i];
}

void WarInputView::Resize(int32 width,int32 height)
{
	// maybe the bounds have been greater so we don't need to scroll the commandline anymore.
	if ((m_prompt_len + m_command_len + 2) * m_war_font->Width() > Width())
		m_scroll_offset = ((m_prompt_len + m_command_len + 2) * m_war_font->Width() - Width())/m_war_font->Width();
	else
		m_scroll_offset = 0;

	WarView::Resize(width,height);
}

void WarInputView::Update()
{
// initialize
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho(0.0,Width()+1,0.0,Height()+1,0.0,100.0);
	glMatrixMode( GL_MODELVIEW );
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glDisable( GL_DEPTH_TEST );
	glColor3f(1.0,1.0,1.0);
	glLoadIdentity();

// draw the prompt
	DrawString(0,0,m_war_font,m_prompt,m_color);

// draw the command_line
	DrawString(m_prompt_len * m_war_font->Width(),0,m_war_font,m_command_line + m_scroll_offset,m_color);

	// should the cursor be drawn?
	if (m_blink && system_time() % m_speed < m_speed/2)
	{
		glDisable(GL_TEXTURE_2D);

		float x = (m_prompt_len + m_cursor_position - m_scroll_offset) * m_war_font->Width();
		float y = 0.0;

	// Draw the cursor
		glBegin(GL_QUADS);

		switch (m_cursor_type)
		{
			case 0: // BLOCK
			glVertex2f(x,y);
			glVertex2f(x+m_war_font->Width(),y);
			glVertex2f(x+m_war_font->Width(),y+m_war_font->Height());
			glVertex2f(x,y+m_war_font->Height());
			break;
			
			case 1: // UNDERSCORE
			glVertex2f(x,y);
			glVertex2f(x+m_war_font->Width(),y);
			glVertex2f(x+m_war_font->Width(),y+(m_war_font->Height()>>2));
			glVertex2f(x,y+(m_war_font->Height()>>2));
			break;

			case 2: // PIPE
			glVertex2f(x,y);
			glVertex2f(x+(m_war_font->Width()>>2),y);
			glVertex2f(x+(m_war_font->Width()>>2),y+m_war_font->Height());
			glVertex2f(x,y+m_war_font->Height());
			break;
		}

		glEnd();
	}
	glDisable(GL_BLEND);
}

void WarInputView::GetCommandLine(char *buffer,uint32 size)
{
	strncpy(buffer,m_command_line,size);
}

void WarInputView::GetPrompt(char *buffer,uint32 size)
{
	strncpy(buffer,m_prompt,size);
}

void WarInputView::SetPrompt(char *buffer)
{
	strncpy(m_prompt,buffer,30);
	m_prompt_len=strlen(m_prompt);
}

void WarInputView::LostFocus()
{
	SetCursorBlink(false);
	SetColor(WarVertex(0.5,0.5,0.5));
}

void WarInputView::GotFocus()
{
	SetCursorBlink(true);
	SetColor(WarVertex(1.0,1.0,1.0));
}

void WarInputView::Reset()
{
	if (m_command_len == 0) return;
	if (m_history[m_history_end]) delete []m_history[m_history_end];
	m_history[m_history_end]=new char[m_command_len + 1];

	strncpy(m_history[m_history_end],m_command_line,m_command_len + 1);
	m_history_end++;
	m_history_position = m_history_end;

	m_cursor_position = 0;
	m_command_len = 0;
	m_command_line[0] = 0;
	m_scroll_offset = 0;
}

void WarInputView::KeyDown(char key)
{
// *********** LEFT

	if (key == 28)
	{
		if (m_cursor_position > 0) m_cursor_position--;

		if ((m_prompt_len + m_cursor_position + 2) * m_war_font->Width() > Width())
			m_scroll_offset = ((m_prompt_len + m_cursor_position + 2) * m_war_font->Width() - Width())/m_war_font->Width();
		else
			m_scroll_offset = 0;
	}

// *********** RIGHT

	else if (key == 29)
	{
		if (m_cursor_position < m_command_len) m_cursor_position++;

		if ((m_prompt_len + m_cursor_position + 2) * m_war_font->Width() > Width())
			m_scroll_offset = ((m_prompt_len + m_cursor_position + 2) * m_war_font->Width() - Width())/m_war_font->Width();
		else
			m_scroll_offset = 0;
	}

// *********** UP

	else if (key == 30)
	{
		m_history_position--;
		if (!m_history[m_history_position])
		{
			m_history_position++;
			return;
		}
		strncpy(m_command_line,m_history[m_history_position],512);
		m_command_len=strlen(m_command_line);
		m_cursor_position = m_command_len;

		if ((m_prompt_len + m_cursor_position + 2) * m_war_font->Width() > Width())
			m_scroll_offset = ((m_prompt_len + m_cursor_position + 2) * m_war_font->Width() - Width())/m_war_font->Width();
		else
			m_scroll_offset = 0;
	}

// ************* DOWN

	else if (key == 31 && m_history[m_history_position])
	{
		m_history_position++;
		if (!m_history[m_history_position])
		{
			*m_command_line=0;
			m_command_len=0;
			m_cursor_position = 0;
			m_scroll_offset=0;
			return;
		}
		strncpy(m_command_line,m_history[m_history_position],512);
		m_command_len=strlen(m_command_line);
		m_cursor_position = m_command_len;

		if ((m_prompt_len + m_cursor_position + 2) * m_war_font->Width() > Width())
			m_scroll_offset = ((m_prompt_len + m_cursor_position + 2) * m_war_font->Width() - Width())/m_war_font->Width();
		else
			m_scroll_offset = 0;
	}

// ********* BACKSPACE

	else if (key == 8 && m_cursor_position > 0)
	{
		// remove the character left to the cursor
		for (int32 i = m_cursor_position - 1; i < m_command_len; i++)
			m_command_line[i] = m_command_line[i+1];

		// move the cursor and make the line shorter
		m_command_len--;
		m_cursor_position--;

		// check if we maybe should scroll the text a bit
		if ((m_prompt_len + m_cursor_position + 2) * m_war_font->Width() > Width())
			m_scroll_offset = ((m_prompt_len + m_cursor_position + 2) * m_war_font->Width() - Width())/m_war_font->Width();
		else
			m_scroll_offset = 0;
	}

// ********* DELETE

	else if (key == 127 && m_command_len > m_cursor_position)
	{
		// remove the character left to the cursor
		for (int32 i = m_cursor_position ; i < m_command_len; i++)
			m_command_line[i] = m_command_line[i+1];

		// make the line shorter
		m_command_len--;
	}

// ************ TAB

	else if (key == 9) // tab
	{
		if (m_command_len == 0 || m_command_line[m_command_len-1] == ' ') return;
		uint32 num_words=m_word_list->CountItems();
		uint16 *matched_words=new uint16[num_words];
		memset(matched_words,-1,num_words);
		
		uint16 len = strlen(m_command_line);
		int16 i=len;

		char *word;
		while (i>=0 && m_command_line[i]!=' ')
			i--;
		i++;
	
		word=m_command_line + i;
		len=strlen(word);

		int32 index = 0;

		uint16 k = 0;
		// defined wor is one of the potential matches
		char *defined_word;
		bool is_word_defined;
	
		while (is_word_defined = m_word_list->GetNextItem(&defined_word,&k))
			if (!strncmp(word,defined_word,len)) matched_words[index++] = k-1;
			
		if (index==1)
		{
			if (strlen(m_word_list->ItemAt(matched_words[0])) > len) strcat(m_command_line,m_word_list->ItemAt(matched_words[0]) + len);
			strcat(m_command_line," ");
			m_command_len = strlen(m_command_line);
		}
		else if (index>1)
		{
			// there where more than one match
		
			// the max len of the strings thats the same
			int32 max_matched_len = len;

			bool matched = true; // is true if every (max_matched_len):th character (in all the matched_words) matched
			do
			{
				char a = m_word_list->ItemAt(matched_words[0])[max_matched_len];
				for (i=1;i<index;i++)
					if (m_word_list->ItemAt(matched_words[i])[max_matched_len] != a)
					{
						matched = false;
						break;
					}
				max_matched_len++;
			}
			while (matched);
			
			max_matched_len--;
			
			if (max_matched_len>1)
			{
				strncat(m_command_line,m_word_list->ItemAt(matched_words[0]) + len,max_matched_len-len);
				m_command_len = strlen(m_command_line);
			}

// Building list of all matched words...
/*
			char tmp[256];
			tmp[0]=0;
			for (i=0;i<index;i++)
			{
				strcat(tmp,m_word_list->ItemAt(matched_words[i]));
				strcat(tmp," ");
			}
*/
		}

		delete matched_words;
		m_cursor_position = m_command_len;

		if ((m_prompt_len + m_command_len + 2) * m_war_font->Width() > Width())
			m_scroll_offset = ((m_prompt_len + m_command_len + 2) * m_war_font->Width() - Width())/m_war_font->Width();
		else
			m_scroll_offset = 0;

	}

// ********* ANY OTHER KEY

	else if (key > 31 && key < 127 && m_command_len < 511)
	{
		// insert a character
		for (int32 i = m_command_len; i >= m_cursor_position; i--)
			m_command_line[i+1] = m_command_line[i];

		// copy the character
		m_command_line[m_cursor_position] = key;
		
		// move the cursor and make the line longer
		m_command_len++;
		m_cursor_position++;

		// check if we maybe should scroll the text a bit
		if ((m_prompt_len + m_cursor_position + 2) * m_war_font->Width() > Width())
			m_scroll_offset = ((m_prompt_len + m_cursor_position+2) * m_war_font->Width() - Width())/m_war_font->Width();
		else
			m_scroll_offset = 0;
		
	}
}
