#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <Entry.h>
#include <Path.h>
#include <Directory.h>

#include "WarMap.h"
#include "WarPoint.h"
#include "WarBlockMap.h"
#include "Global.h"

_EXPORT WarMap::WarMap(uint8 default_size,BDirectory *env_dir,char *environment):BLocker()
{
	init(default_size,env_dir,environment);
}

_EXPORT WarMap::WarMap(BDirectory *map_dir,char *filename,BDirectory *env_dir):BLocker()
{
	m_blocks_per_control_point_x = 4;
	m_blocks_per_control_point_y = 4;

	char environment[20];

	BFile file(map_dir,filename,B_READ_ONLY);
	if (file.InitCheck()!=B_OK)
	{
		fprintf(stderr,"Couldn't open file \"%s\"\n",filename);
	}
	file.Read(&m_size,1); // read the map size
	file.Read(environment,20); // read the environent filename

//	fprintf(stderr,"filename: %s\n",filename);
//	fprintf(stderr,"environment: %s\n",environment);

	init(m_size,env_dir,environment);

	printf("loading terrain\n");
	for (int32 y=0;y < m_map_textures->Height();y++)
		for (int32 x=0;x < m_map_textures->Width();x++)
		{
			uint16 texture;
			file.Read(&texture,2);
			m_map_textures->SetValue(x,y,texture);
//			printf("%d\n",height);
		}

//	printf("loading %d bytes\n",(Width()>>3) * (Height()>>3) * 2);

//	BFile hfile(map_dir,"height_field",B_READ_ONLY);

	printf("loading heightmap\n");
	
	uint8 *data = new uint8[m_height_map->Width()*m_height_map->Height()];
	file.Read(data,m_height_map->Width()*m_height_map->Height());
	int32 i=0;
	for (int32 y=0;y < m_height_map->Height();y++)
		for (int32 x=0;x < m_height_map->Width();x++)
			m_height_map->SetValue(x,y,data[i++] / 4.0);
	delete []data;
}

void WarMap::init(uint8 default_size,BDirectory *env_dir,char *environment)
{
	m_size = default_size;
	m_width = m_height = 0;
	m_width_shift = m_height_shift = 0;
	m_map_flags = NULL;
	m_map_textures = NULL;

	m_blocks_per_control_point_x = 4;
	m_blocks_per_control_point_y = 4;

	// the available mapsizes;

	uint8 shift_table[]=
	{
		8,8,
		9,9,
		10,10
	};

	/* set the map size */
	m_width = 1<<shift_table[(m_size%10)<<1];
	m_height = 1<<shift_table[((m_size%10)<<1)+1];
	m_width_shift = shift_table[(m_size%10)<<1];
	m_height_shift = shift_table[((m_size%10)<<1)+1];
	m_block_count = (m_width<<m_height_shift)-1;

#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"WarMap:	%ld\n",sizeof(WarMap));
	fprintf(stderr,"  height_map:	%ld\n",m_width/m_blocks_per_control_point_x * m_height/m_blocks_per_control_point_y);
	fprintf(stderr,"  slope_map_x:	%ld\n",m_width/m_blocks_per_control_point_x * m_height/m_blocks_per_control_point_y);
	fprintf(stderr,"  slope_map_y:	%ld\n",m_width/m_blocks_per_control_point_x * m_height/m_blocks_per_control_point_y);
	fprintf(stderr,"  texture_map:	%ld\n",(m_width/TILE_SPAN) * (m_height/TILE_SPAN) * 2);
	fprintf(stderr,"  map_flags:	%ld\n",m_width<<(m_height_shift));
#endif

	/* allocate a new map */
	m_map_flags = new uint8[m_width<<(m_height_shift)];
	m_map_textures = new WarBlockMap<uint16>(WidthShift()-TILE_SHIFT,HeightShift()-TILE_SHIFT);

	m_box_height = new WarBlockMap<uint8>(WidthShift(),HeightShift());
	m_box_elevation = new WarBlockMap<float>(WidthShift(),HeightShift());

	m_height_map = new WarGrid<float>(m_width_shift,m_height_shift);
	m_map_slope = new WarGrid<WarPoint>(m_width_shift,m_height_shift);

	bump_map = new uint8[(m_width+1) * (m_height+1)];

	/* set the new allocated data to 0 */
	memset(bump_map,0,(m_width+1) * (m_height+1));
	memset(m_map_flags,0,m_width << m_height_shift);

/*
	BEntry entry;
	BPath path;
	env_dir->GetEntry(&entry);
	entry.GetPath(&path);
	printf("WarMap::init() path: %s\n",path.Path());
*/
	m_tiles = new WarTileset(env_dir,environment);
}

_EXPORT WarMap::~WarMap()
{
	if (m_tiles) delete m_tiles;

	if (m_height_map) delete m_height_map;
	if (m_map_slope) delete m_map_slope;

	delete m_box_elevation;
	delete m_box_height;

	if (m_map_flags) delete []m_map_flags;
	delete m_map_textures;

	if (bump_map) delete []bump_map;
}

_EXPORT uint8 WarMap::GetFlagsRect(int32 x,int32 y,uint16 width,uint16 height)
{
	uint8 flags=0;
	uint8 bl_x,bl_y;
	int16	left=x>>BLOCK_SHIFT;
	int16 right=(x+width-1)>>BLOCK_SHIFT;
	int16 top=y>>BLOCK_SHIFT;
	int16 bottom=(y+height-1)>>BLOCK_SHIFT;

	left=(left<0)?0:left;
	right=(right>Width())?Width():right;
	top=(top<0)?0:top;
	bottom=(bottom>Height())?Height():bottom;
	uint16 linefeed=Width()+left-right-1;

	uint32 offs=(top<<WidthShift())+left;
	for (bl_y=top;bl_y<=bottom;bl_y++)
	{
		for (bl_x=left;bl_x<=right;bl_x++)
			flags|=GetFlags(offs++);
		offs+=linefeed;
	}
	return flags;
}

_EXPORT void WarMap::GenerateMap()
{
/*
	for (int32 y=0;y<m_height_map->Height();y++)
		for (int32 x=0;x<m_height_map->Width();x++)
//			height_map->SetKey(x,y, ( sin(x*2*3.1415927 * blocks_per_control_point_x / 4.0) + sin(y*2*3.1415927/3.0 * blocks_per_control_point_x / 4.0) + sin(x*3.1415927/2.0 * blocks_per_control_point_x / 4.0) + sin(y*3.1415927 * blocks_per_control_point_x / 4.0) )*1.5 + 6.0);
//			height_map->SetKey(x,y, cos(x*2*3.1415927/height_map->Width() * 4)*5.0 + cos(y*2*3.1415927/height_map->Height() * 4)*5.0 + 10.0);
			m_height_map->SetValue(x,y, sin(y/8.0)*sin(x/8.0)*5.0 + 5.0);
//			height_map->SetKey(x,y,0.0);
*/

	printf("Calculating slope map\n");
	for (float y=0;y <= Height();y+=1.0)
		for (float x=0;x <= Width();x+=1.0)
		{
			WarPoint angles;
			angles.x = atan2( (GetHeight(x-0.1,y) - GetHeight(x+0.1,y)) ,0.2);
			angles.y = atan2( (GetHeight(x,y+0.1) - GetHeight(x,y-0.1)) ,0.2);
			m_map_slope->SetValue(x,y,angles);
		}

	printf("Calculating bump map\n");
	for (int32 y = TILE_SPAN>>1;y < Height();y += TILE_SPAN)
		for (int32 x = TILE_SPAN>>1;x < Width();x += TILE_SPAN)
			RecalcBump(x,y,false,TILE_SPAN>>1);

	printf("Calculating terrain bounding boxes\n");
	int32 i=0;
	for (int32 ty = 0;ty < Height();ty+=TILE_SPAN)
	{
		for (int32 tx = 0;tx < Width();tx+=TILE_SPAN)
		{
			float min = GetHeight(tx,ty);
			float max = min;
			for (int32 y = 0;y <= TILE_SPAN;y++)
			{
				for (int32 x = 0;x <= TILE_SPAN;x++)
				{
					float height = GetHeight(x + tx,y + ty);
					if (height > max) max = height;
					if (height < min) min = height;
				}
			}
			m_box_height->SetValue(tx>>TILE_SHIFT,ty>>TILE_SHIFT,max - min);
			m_box_elevation->SetValue(tx>>TILE_SHIFT,ty>>TILE_SHIFT,min + (max - min)*.5);
			i++;
		}
	}
}

_EXPORT void WarMap::Save(BDirectory *dir,char *filename)
{
	char *env=m_tiles->Name();
	BFile file(dir,filename,B_WRITE_ONLY|B_ERASE_FILE|B_CREATE_FILE);
	file.Write(&m_size,1);
	file.Write(env,20);
	file.Write(m_map_textures,m_block_count>>5);
//	file.Write(height_map,block_count);
}

// claclulates the maximum error introduced by
// dropping one level of detail on this node and
// all it's child nodes

_EXPORT void WarMap::RecalcBump(int32 x,int32 y,bool dir,int32 level)
{
	int32 sinus[]={0,1,1,1,0,-1,-1,-1};
	int32 cosin[]={1,1,0,-1,-1,-1,0,1};

/*
	dir is false:  dir is true:
	+-----+			+-----+
	|    /|			|\    |
	|   / |			| \   |
	|  *  |			|  *  |
	| /   |			|   \ |
	|/    |			|    \|
	+-----+			+-----+

height[]			

  3   2   1
	+--+--+
	|\ | /|
	| \|/ |
  4+––8––+ 0
	| /|\ |
	|/ | \|
	+--+--+
  5   6   7
*/	

	float height[9];
	float max = 0;
	for (int32 i = 0;i<8;i++) height[i] = GetHeight(x + level * cosin[i],y + level * sinus[i]);
	height[8] = GetHeight(x,y);

	for (int32 i = 0;i<4;i++)
	{
		float diff = height[2*i] - (height[(i-1)&7] + height[(i+1)&7])*.5;
		if (diff < 0) diff = -diff;
		if (diff > max) max = diff;
	}

	float diff;
	if (dir)
		diff = height[8] - (height[3] + height[7])*.5;
	else
		diff = height[8] - (height[1] + height[5])*.5;

	if (diff < 0) diff = -diff;
	if (diff > max) max = diff;
	bump_map[y * (Width()+1) + x] = (uint8)(max * 8.0);

	if (level > 1)
	{
		int32 half_level = level >> 1;
		RecalcBump(x + half_level,y + half_level,false,half_level);
		RecalcBump(x + half_level,y - half_level,true,half_level);
		RecalcBump(x - half_level,y - half_level,false,half_level);
		RecalcBump(x - half_level,y + half_level,true,half_level);
	}
}
