#include <TranslationUtils.h>
#include <Bitmap.h>
#include <stdlib.h>
#include <gl.h>
#include <glu.h>
#include <string.h>
#include <File.h>
#include <Application.h>
#include <String.h>

#include "WarGLView.h"
#include "WarMenu.h"
#include "WarObject.h"

WarMenu::WarMenu(int32 left,int32 top,int32 width,int32 height,WarFont *font,char *command_line):WarView(left,top,width,height)
{
	m_parent = NULL;
	m_war_font = font;

	m_max_width = 0;
	
	m_command_line = new BString(command_line);
	
	m_submenu = false;
	m_current_item = -1;
}

WarMenu::~WarMenu()
{

// remove all the items
	menu_item *item;
	uint16 cookie = 0;
	while (m_item_list.GetNextItem(&item,&cookie))
	{
		delete item->string;
		m_gl_view->Quit(item->sub_menu);
		delete item;
	}
}

void WarMenu::MouseDown(WarPoint point,uint8 button)
{
	if (!m_submenu)
	{
		MouseMoved(point);
		Invoke();
	}
}

void WarMenu::Invoke()
{
//	printf("WarMenu::Invoke()\n");
	char tmp[10];
	sprintf(tmp,"%ld",m_current_item);
//	printf("WarMenu::Invoke() parsing id#: %ld\n",m_current_item);
	m_gl_view->Parse(tmp);

	int32 len = m_command_line->CountChars() + 1;
	char *line = new char[len];
	m_command_line->MoveInto(line,0,len);

//	printf("WarMenu::Invoke() parsing command_line: %s\n",line);
	m_gl_view->Parse(line);
	delete []line;

	if (m_parent)
	{
//		printf("WarMenu::Invoke() Invoking parent\n");
		m_parent->Invoke();
	}
	else
	{
//		printf("WarMenu::Invoke() Quit\n");
		Quit();
	}
}

void WarMenu::MouseMoved(WarPoint point)
{
	m_mouse_position = point;

	int32 last_item = m_current_item;

	if (point.x >= 0.0 && point.x <= Width() && point.y >= 0.0 && point.y <= Height())
		m_current_item = (int32) ((point.y - 1) / (m_war_font->Height() + 1));
	else
		if (!m_submenu) m_current_item = -1;

	if (m_current_item != last_item)
	{
		menu_item *item;

		item = m_item_list.ItemAt(m_item_list.CountItems() - last_item - 1);
		if (item && item->sub_menu)
		{
			m_gl_view->RemoveChild(item->sub_menu);
			m_submenu = false;
		}

		item = m_item_list.ItemAt(m_item_list.CountItems() - m_current_item - 1);
		if (item && item->sub_menu)
		{
			item->sub_menu->SetParent(this);
			item->sub_menu->Move(Right(),m_current_item * (m_war_font->Height() + 1) + Top());

			m_gl_view->AddChild(item->sub_menu);
			m_gl_view->GiveFocus(item->sub_menu);
			m_submenu = true;
		}
	}
}

void WarMenu::AddItem(const char *string,WarMenu *sub_menu)
{
	menu_item *item = new menu_item;
	item->string = new BString(string);
	item->sub_menu = sub_menu;

	m_item_list.AddItem(item);

	int32 len = item->string->CountChars();
	if (len > m_max_width) m_max_width = len;

	Resize(m_max_width * m_war_font->Width() + 4,m_item_list.CountItems() * (m_war_font->Height() + 1));
}

void WarMenu::AddSeparator()
{
	menu_item *item = new menu_item;
	item->string = NULL;
	item->sub_menu = NULL;
	
	m_item_list.AddItem(item);

	Resize(m_max_width * m_war_font->Width() + 4,m_item_list.CountItems() * (m_war_font->Height() + 1));
}

void WarMenu::Update()
{
	glDisable(GL_BLEND);

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho(0.0,Width()+1,0.0,Height()+1,0.0,100.0);
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glDisable( GL_DEPTH_TEST );

	glDisable(GL_TEXTURE_2D);
	glColor3f(0.5,0.2,.8);
	glBegin(GL_QUADS);
	
	glVertex2f(0.0,0.0);
	glVertex2f(Width() + 1,0.0);
	glVertex2f(Width() + 1,Height() + 1);
	glVertex2f(0.0,Height() + 1);

	if (m_current_item != -1)
	{
		// draw the hilighted rect
		float y = m_current_item * (m_war_font->Height() + 1);
		
		glColor3f(.8,.4,1.0);
		
		glVertex2f(0.0,y);
		glVertex2f(Width() + 1,y);
		glVertex2f(Width() + 1,y + m_war_font->Height() + 2);
		glVertex2f(0.0,y + m_war_font->Height() + 2);
	}

	glEnd();

// draw the black border
	glBegin(GL_LINE_LOOP);

	glColor3f(0.0,0.0,0.0);
	glVertex2f(0.0,0.0);
	glVertex2f(Width(),0.0);
	glVertex2f(Width()+1,Height()+1);
	glVertex2f(0.0,Height());

	glEnd();


	glEnable(GL_TEXTURE_2D);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glLoadIdentity();

	glColor3f(1.0,1.0,1.0);

	float i = m_item_list.CountItems() * (m_war_font->Height() + 1) - 1;
	menu_item *item;
	uint16 cookie = 0;
	while (m_item_list.GetNextItem(&item,&cookie))
	{
		i -= m_war_font->Height() + 1;
		// draw the item name or a horizontal line, if it's a separator item
		if (item->string) DrawString(2.0,i + 2.0,m_war_font,item->string->String());
		else
		{
			glDisable(GL_TEXTURE_2D);
			glColor3f(0.0,0.0,0.0);
			
			glBegin(GL_LINES);
			glVertex2f(0.0,i + m_war_font->Height()/2 + 1);
			glVertex2f(Width(),i + m_war_font->Height()/2 + 1);
			glEnd();

			glColor3f(1.0,1.0,1.0);
			glEnable(GL_TEXTURE_2D);
		}
	}

	glEnable(GL_DEPTH_TEST);
	glDisable(GL_BLEND);
}
