#include "WarPlanet.h"
#include "WarGLView.h"
#include "WarMap.h"
#include "WarPolygon.h"
#include "WarBox.h"
#include "WarStack.h"
#include "WarSkydome.h"
#include "defs.h"

#include <TranslationUtils.h>
#include <List.h>
#include <stdlib.h>
#include <math.h>
#include <gl.h>
#include <glu.h>
#include <Autolock.h>

//#define WIREFRAME

_EXPORT WarPlanet::WarPlanet(WarEntity entity,float radius,WarMap *map,WarEntityList *elist,WarWeather *weather):
	WarEntity(entity),
	m_map(map),
	m_entity_list(elist),
	m_weather(weather),
	m_tiles(map->Tileset()),
	m_radius(radius)
{
// initialization
	m_triangulation = new WarTriangulation(m_map);
	m_triangulation->GenerateStatic();

	tmp_box = new WarBox();
}

_EXPORT WarPlanet::~WarPlanet()
{
	delete m_triangulation;
	delete tmp_box;
}

_EXPORT void WarPlanet::Update(WarCamera *camera)
{
	polygon_count = 0;
	trianglefan_count = 0;

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);

	WarVertex camera_position = camera->Position();
	WarVertex position = Position();
	
	double distance = sqrt(p(camera_position.x-position.x) + p(camera_position.y-position.y) + p(camera_position.z-position.z)) - m_radius;

	if (sqrt(p(camera_position.x-position.x) + p(camera_position.y-position.y) + p(camera_position.z-position.z)) - m_radius < 200)
	{
		// if we're near the planet, draw it as a plane

		// !!!! TEMPOARARY !!!!
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glEnableClientState(GL_COLOR_ARRAY);

		glVertexPointer(3,GL_FLOAT,0,vertex_array);
		glColorPointer(3,GL_FLOAT,0,color_array);
		glTexCoordPointer(2,GL_FLOAT,0,texcoord_array);
	
		glBindTexture(GL_TEXTURE_2D, m_tiles->Tile(0));
		// *****

	// Enable the GL fog
		glEnable(GL_FOG); 
		glFogfv(GL_FOG_COLOR, m_weather->FogColor());
		glFogi(GL_FOG_MODE, GL_LINEAR);
		glFogf(GL_FOG_DENSITY, 1.0);
		glFogf(GL_FOG_START, camera->FarClip() * m_weather->FogStart());
		glFogf(GL_FOG_END, camera->FarClip());
		glHint(GL_FOG_HINT, GL_FASTEST);

// now, we generate the map triangulation
//	m_triangulation->Regenerate(camera);

		render_map(m_map->Width()/2,m_map->Height()/2,m_map->Width()/2,camera/*,false*/);
		poly_count = polygon_count;
		fan_count = trianglefan_count;


// draw all the flying units
// ---------------------
		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);

		WarUnit *body;
		uint16 id,cookie=0;
		while(m_entity_list->GetNextFlyingEntity(&body,NULL,&id,&cookie))
		{
			// draws the unit, only if it's visible
			body->GetBoundingBox(tmp_box);
			if (camera->IsBoxVisible(tmp_box)) body->Draw(camera);
		}
		glDisable(GL_LIGHTING);

	// disable the GL fog
		glDisable(GL_FOG);
	}
	else
	{
		glBindTexture(GL_TEXTURE_2D, m_tiles->Tile(0));

		glEnable(GL_TEXTURE_2D);
		glColor3f(1.0,1.0,1.0);

		// otherwise, draw it as a spere
		GLUquadricObj *quadric = gluNewQuadric();
		gluQuadricNormals(quadric,GLU_NONE);
		gluQuadricDrawStyle(quadric,GLU_FILL);
		gluQuadricOrientation(quadric,GLU_OUTSIDE);
		gluQuadricTexture(quadric,GL_TRUE);
	
		int32 detail = 30;

		if (distance < 20) detail = 30;
		else if (distance > m_radius*20) detail = 3;
		else detail = ((1 - (distance-20)/(m_radius*20))*30 + (distance-20)/(m_radius*20))*3 * 0.5;
		
		printf("detail: %ld\n",detail);
	
		gluSphere(quadric,m_radius,detail,detail);

		gluDeleteQuadric(quadric);
	}
}

void WarPlanet::render_map(int32 x,int32 y,int32 level,WarCamera *camera/*,bool entire*/)
{
	if (level == TILE_SPAN>>1)
	{
		tmp_box->SetTo(TILE_SPAN,TILE_SPAN,m_map->BoxHeight(x>>TILE_SHIFT,y>>TILE_SHIFT));
		tmp_box->OffsetBy(WarVertex(x,y,m_map->BoxElevation(x>>TILE_SHIFT,y>>TILE_SHIFT)));
//		glDisable(GL_TEXTURE_2D);
//		tmp_box->Draw();
//		glEnable(GL_TEXTURE_2D);
	}
	else
	{
		tmp_box->SetTo(level<<1,level<<1,64.0);
		tmp_box->OffsetBy(WarVertex(x,y,32.0));
	}

	if (/*entire || */camera->IsBoxVisible(tmp_box/*,&entire*/))
	{
		WarTileset *m_tiles = m_map->Tileset();

		if (level == TILE_SPAN>>1)
		{
			uint16 tile_index = m_map->TileIndex(x>>TILE_SHIFT,y>>TILE_SHIFT);
			if (m_triangulation->Lookup(x,y) == 0)
			{

				float tex_offset_x = x & (TILE_SPAN-1);
				float tex_offset_y = y & (TILE_SPAN-1);
/*
				glBegin(GL_TRIANGLE_FAN);

					glColor3fv(*m_weather->Color(m_map->Slope(x + level,y + level)));
					glTexCoord2fv(m_tiles->ConvertUV(tile_index,tex_offset_x + level_span,tex_offset_y + level_span));
					glVertex3f(x + level,y + level,m_map->GetHeight(x + level,y + level));

					glColor3fv(*m_weather->Color(m_map->Slope(x - level,y + level)));
					glTexCoord2fv(m_tiles->ConvertUV(tile_index,tex_offset_x - level_span,tex_offset_y + level_span));
					glVertex3f(x - level,y + level,m_map->GetHeight(x - level,y + level));

					glColor3fv(*m_weather->Color(m_map->Slope(x - level,y - level)));
					glTexCoord2fv(m_tiles->ConvertUV(tile_index,tex_offset_x - level_span,tex_offset_y - level_span));
					glVertex3f(x - level,y - level,m_map->GetHeight(x - level,y - level));

					glColor3fv(*m_weather->Color(m_map->Slope(x + level,y - level)));
					glTexCoord2fv(m_tiles->ConvertUV(tile_index,tex_offset_x + level_span,tex_offset_y - level_span));
					glVertex3f(x + level,y - level,m_map->GetHeight(x + level,y - level));

				glEnd();
*/

				vertex_array[0].SetTo(x + level,y + level,m_map->GetHeight(x + level,y + level));
				vertex_array[1].SetTo(x - level,y + level,m_map->GetHeight(x - level,y + level));
				vertex_array[2].SetTo(x - level,y - level,m_map->GetHeight(x - level,y - level));
				vertex_array[3].SetTo(x + level,y - level,m_map->GetHeight(x + level,y - level));

				color_array[0] = m_weather->Color(m_map->Slope(x + level,y + level));
				color_array[1] = m_weather->Color(m_map->Slope(x - level,y + level));
				color_array[2] = m_weather->Color(m_map->Slope(x - level,y - level));
				color_array[3] = m_weather->Color(m_map->Slope(x + level,y - level));

				texcoord_array[0] = m_tiles->ConvertUV(tile_index,(tex_offset_x + level),(tex_offset_y + level));
				texcoord_array[1] = m_tiles->ConvertUV(tile_index,(tex_offset_x - level),(tex_offset_y + level));
				texcoord_array[2] = m_tiles->ConvertUV(tile_index,(tex_offset_x - level),(tex_offset_y - level));
				texcoord_array[3] = m_tiles->ConvertUV(tile_index,(tex_offset_x + level),(tex_offset_y - level));

				glDrawArrays(GL_TRIANGLE_FAN,0,4);

				trianglefan_count++;
				polygon_count += 2;
			}
			else render_tile(x,y,level,tile_index,false);
		}
		else
		{
			int32 half_level = level>>1;
			render_map(x+half_level,y+half_level,half_level,camera/*,entire*/);
			render_map(x+half_level,y-half_level,half_level,camera/*,entire*/);
			render_map(x-half_level,y-half_level,half_level,camera/*,entire*/);
			render_map(x-half_level,y+half_level,half_level,camera/*,entire*/);
		}
	}
}

void WarPlanet::render_tile(int32 x,int32 y,int8 level,uint16 tile_index,bool direction)
{
	int8 half_level = level>>1;
	int8 double_level = level<<1;

	static int32 sinus1[]={1,1,-1,-1};
	static int32 cosin1[]={1,-1,-1,1};
	static int32 sinus2[]={0,1,0,-1};
	static int32 cosin2[]={1,0,-1,0};

	bool children[4];
	if (level <= 1)
	{
		children[0] = false;
		children[1] = false;
		children[2] = false;
		children[3] = false;
	}
	else
	{
		children[0] = m_triangulation->Lookup(x + half_level,y + half_level);
		children[1] = m_triangulation->Lookup(x - half_level,y + half_level);
		children[2] = m_triangulation->Lookup(x - half_level,y - half_level);
		children[3] = m_triangulation->Lookup(x + half_level,y - half_level);
	}

	bool sibblings[4];
	sibblings[0] = m_triangulation->Lookup(x+double_level,y);
	sibblings[1] = m_triangulation->Lookup(x,y+double_level);
	sibblings[2] = m_triangulation->Lookup(x-double_level,y);
	sibblings[3] = m_triangulation->Lookup(x,y-double_level);

	bool draw[4];

	draw[0] = !sibblings[0] || !sibblings[1] || !children[0];
	draw[1] = !sibblings[1] || !sibblings[2] || !children[1];
	draw[2] = !sibblings[2] || !sibblings[3] || !children[2];
	draw[3] = !sibblings[3] || !sibblings[0] || !children[3];

	int32 j;

	bool fan_open = false;
	int32 tex_offset_x = x & (TILE_SPAN-1);
	int32 tex_offset_y = y & (TILE_SPAN-1);

	int32 counter = 0;
	for (int32 i=0;i<4;i++)
	{
		int32 x_add1 = level * cosin1[i];
		int32 y_add1 = level * sinus1[i];
		int32 x_add2 = level * cosin2[i];
		int32 y_add2 = level * sinus2[i];

		if (draw[i])
		{
			if (!fan_open)
			{
//				float height = blended_middle_height(x,y,direction,level);
				float height = m_map->GetHeight(x,y);

				vertex_array[counter].SetTo(x,y,height);
				color_array[counter] = m_weather->Color(m_map->Slope(x,y));
				texcoord_array[counter] = m_tiles->ConvertUV(tile_index,tex_offset_x,tex_offset_y);
				counter++;
				fan_open = true;
			}

			if (sibblings[i])
			{
//				float height = blended_leaf_height(x,y,i,level);
				float height = m_map->GetHeight(x + x_add2,y + y_add2);

				color_array[counter] = m_weather->Color(m_map->Slope(x + x_add2,y + y_add2));
				vertex_array[counter].SetTo(x + x_add2,y + y_add2,height);
				texcoord_array[counter] = m_tiles->ConvertUV(tile_index,tex_offset_x + x_add2,tex_offset_y + y_add2);
				counter++;
			}

			color_array[counter] = m_weather->Color(m_map->Slope(x + x_add1,y + y_add1));
			vertex_array[counter].SetTo(x + x_add1,y + y_add1,m_map->GetHeight(x + x_add1,y + y_add1));
			texcoord_array[counter] = m_tiles->ConvertUV(tile_index,tex_offset_x + x_add1,tex_offset_y + y_add1);
			counter++;

			j=(i+1)&3;
			if (!draw[j])
			{
//				float height = blended_leaf_height(x,y,j,level);
				float height = m_map->GetHeight(x + level * cosin2[j],y + x_add2);

				color_array[counter] = m_weather->Color(m_map->Slope(x + level * cosin2[j],y + x_add2));
				texcoord_array[counter] = m_tiles->ConvertUV(tile_index,tex_offset_x + (level * cosin2[j]),tex_offset_y + x_add2);
				vertex_array[counter].SetTo(x + level * cosin2[j],y + x_add2,height);
				counter++;

				glDrawArrays(GL_TRIANGLE_FAN,0,counter);
				trianglefan_count++;
				polygon_count += counter - 2;

				fan_open = false;
				counter = 0;
			}
		}
	}

	int32 x_add1 = level * cosin1[0];
	int32 y_add1 = level * sinus1[0];
	int32 x_add2 = level * cosin2[0];
	int32 y_add2 = level * sinus2[0];

	if (fan_open)
	{
		if (sibblings[0])
		{
//			float height = blended_leaf_height(x,y,0,level);
			float height = m_map->GetHeight(x + x_add2,y + y_add2);

			color_array[counter] = m_weather->Color(m_map->Slope(x + x_add2,y + y_add2));
			texcoord_array[counter] = m_tiles->ConvertUV(tile_index,tex_offset_x + x_add2,tex_offset_y + y_add2);
			vertex_array[counter].SetTo(x + x_add2,y + y_add2,height);
			counter++;
		}
		else
		{
			color_array[counter] = m_weather->Color(m_map->Slope(x + x_add1,y + y_add1));
			texcoord_array[counter] = m_tiles->ConvertUV(tile_index,tex_offset_x + x_add1,tex_offset_y + y_add1);
			vertex_array[counter].SetTo(x + x_add1,y + y_add1,m_map->GetHeight(x + x_add1,y + y_add1));
			counter++;
		}
		glDrawArrays(GL_TRIANGLE_FAN,0,counter);
		polygon_count += counter - 2;
		trianglefan_count++;
	}

/*
	int32 counter = 0;
	for (int32 i=0;i<4;i++)
	{
		int32 x_add1 = level * cosin1[i];
		int32 y_add1 = level * sinus1[i];
		int32 x_add2 = level * cosin2[i];
		int32 y_add2 = level * sinus2[i];
		float x_add3 = x_add2/(float)TILE_SPAN;
		float y_add3 = y_add2/(float)TILE_SPAN;
		float x_add4 = x_add1/(float)TILE_SPAN;
		float y_add4 = y_add1/(float)TILE_SPAN;

		if (draw[i])
		{
			if (!fan_open)
			{
				float height = blended_middle_height(x,y,direction,level);

				glBegin(GL_TRIANGLE_FAN);

				glColor3fv(m_weather->Color(m_map->Slope(x,y)));
				glTexCoord2fv(m_tiles->ConvertUV(tile_index,tex_offset_x,tex_offset_y));
				glVertex3f(x,y,height);
				counter++;
				fan_open = true;
			}

			if (sibblings[i])
			{
				float height = blended_leaf_height(x,y,i,level);

				glColor3fv(m_weather->Color(m_map->Slope(x + x_add2,y + y_add2)));
				glTexCoord2fv(m_tiles->ConvertUV(tile_index,tex_offset_x + x_add3,tex_offset_y + y_add3));
				glVertex3f(x + x_add2,y + y_add2,height);
				counter++;
			}

			glColor3fv(m_weather->Color(m_map->Slope(x + x_add1,y + y_add1)));
			glTexCoord2fv(m_tiles->ConvertUV(tile_index,tex_offset_x + x_add4,tex_offset_y + y_add4));
			glVertex3f(x + x_add1,y + y_add1,m_map->GetHeight(x + x_add1,y + y_add1));
			counter++;

			j=(i+1)&3;
			if (!draw[j])
			{
				float height = blended_leaf_height(x,y,j,level);

				glColor3fv(m_weather->Color(m_map->Slope(x + level * cosin2[j],y + level * sinus2[j])));
				glTexCoord2fv(m_tiles->ConvertUV(tile_index,tex_offset_x + (level * cosin2[j])/((float)TILE_SPAN),tex_offset_y + (level * sinus2[j])/(float)TILE_SPAN));
				glVertex3f(x + level * cosin2[j],y + level * sinus2[j],height);
				counter++;

				glEnd();
				trianglefan_count++;
				polygon_count += counter - 2;

				fan_open = false;
				counter = 0;
			}
		}
	}

	int32 x_add1 = level * cosin1[0];
	int32 y_add1 = level * sinus1[0];
	int32 x_add2 = level * cosin2[0];
	int32 y_add2 = level * sinus2[0];
	float x_add3 = x_add2/(float)TILE_SPAN;
	float y_add3 = y_add2/(float)TILE_SPAN;
	float x_add4 = x_add1/(float)TILE_SPAN;
	float y_add4 = y_add1/(float)TILE_SPAN;

	if (fan_open)
	{
		if (sibblings[0])
		{
			float height = blended_leaf_height(x,y,0,level);

			glColor3fv(m_weather->Color(m_map->Slope(x + x_add2,y + y_add2)));
			glTexCoord2fv(m_tiles->ConvertUV(tile_index,tex_offset_x + x_add3,tex_offset_y + y_add3));
			glVertex3f(x + x_add2,y + y_add2,height);
			counter++;
		}
		else
		{
			glColor3fv(m_weather->Color(m_map->Slope(x + x_add1,y + y_add1)));
			glTexCoord2fv(m_tiles->ConvertUV(tile_index,tex_offset_x + x_add4,tex_offset_y + y_add4));
			glVertex3f(x + x_add1,y + y_add1,m_map->GetHeight(x + x_add1,y + y_add1));
			counter++;
		}
		glEnd();
		polygon_count += counter - 2;
		trianglefan_count++;
	}
*/

	if (!draw[0]) render_tile(x+half_level,y+half_level,half_level,tile_index,0);
	if (!draw[1]) render_tile(x-half_level,y+half_level,half_level,tile_index,1);
	if (!draw[2]) render_tile(x-half_level,y-half_level,half_level,tile_index,0);
	if (!draw[3]) render_tile(x+half_level,y-half_level,half_level,tile_index,1);
}
