#include "WarPolygon.h"
#include "defs.h"

#include <stdio.h>
#include <Point.h>
#include <gl.h>

WarPolygon::WarPolygon()
{
	point_list = new BList();
}

WarPolygon::~WarPolygon()
{
	Empty();
	delete point_list;
}

void WarPolygon::Empty()
{
	BPoint *point;
	while ((point = (BPoint *)point_list->ItemAt(0)))
	{
		delete point;
		point_list->RemoveItem(0L);
	}
}

bool WarPolygon::AddPoint(BPoint point)
{
	BPoint *new_point = new BPoint(point);
	if (point_list->AddItem(new_point)) return true;
	delete new_point;
	return false;
}

bool WarPolygon::TestLine(BPoint l_p1,BPoint l_p2,BPoint t_p1,BPoint t_p2)
{
	if (l_p1.x == l_p2.x)
		return ((t_p1.x < l_p1.x) == (t_p2.x < l_p1.x));
	else
	{
		float k = (l_p2.y - l_p1.y)/(l_p2.x - l_p1.x);
		return (((t_p1.x - l_p1.x)*k+l_p1.y < t_p1.y) == ((t_p2.x - l_p1.x)*k+l_p1.y < t_p2.y));
	}
}

bool WarPolygon::Contains(BPoint point)
{
	uint16 j=0;
	for(uint16 i=0;i<point_list->CountItems();i++)
	{
		BPoint *line1=(BPoint *)point_list->ItemAt(j);
		j=IterateRight(j);
		BPoint *line2=(BPoint *)point_list->ItemAt(j);
		j=IterateRight(j);
		BPoint *line3=(BPoint *)point_list->ItemAt(j);
		j=IterateLeft(j);

		if (!TestLine(*line1,*line2,*line3,point)) return false;
	}
	return true;
}

// Adds a point to the polygon and makes sure the polygon still is
// convex. If the point is within the current polygon nothing happens
void WarPolygon::ExpandBy(BPoint point)
{
	if (CountPoints() < 3)
	{
		point_list->AddItem(new BPoint(point));
		return;
	}
	if (Contains(point)) return;
	
	int32 index1 = -1,index2 = -1;
	
	for (int32 i=0;i<CountPoints();i++)
	{
		if (TestLine(point,PointAt(i),PointAt(IterateLeft(i)),PointAt(IterateRight(i))))
		{
			if (index1 == -1) index1 = i;
			else index2 = i;
		}
	}
	
	if (index1 == -1) return;
	if (index2 == -1)
	{
//		printf("ahh!\n");
//		point_list->AddItem(new BPoint(point),index1 + 1);
		return;
	}

	bool case1;
	if ( index2 == IterateLeft(index1))
	{
		if (TestLine(PointAt(index1),PointAt(index2),point,PointAt(IterateRight(index1)))) case1 = false;
		else case1 = true;
	}
	else
	{
		if (TestLine(PointAt(index1),PointAt(index2),point,PointAt(IterateLeft(index1)))) case1 = true;
		else case1 = false;
	}

	if (case1)
	{
		if (index2 > index1)
		{
//			printf("1\n");
			uint16 num = CountPoints();
			for (uint16 i=index2 + 1;i<num;i++) delete point_list->RemoveItem(index2 + 1);
			for (uint16 i=0;i<index1;i++) delete point_list->RemoveItem(0L);
			point_list->AddItem(new BPoint(point));
		}
		else
		{
//			printf("2\n");
			for (uint16 i=index2 + 1;i<index1;i++) delete point_list->RemoveItem(index2 + 1);
			point_list->AddItem(new BPoint(point),index2 + 1);
		}
	}
	else
	{
		if (index2 < index1)
		{
//			printf("3\n");
			uint16 num = CountPoints();
			for (uint16 i=index1 + 1;i<num;i++) delete point_list->RemoveItem(index1 + 1);
			for (uint16 i=0;i<index2;i++) delete point_list->RemoveItem(0L);
			point_list->AddItem(new BPoint(point));
		}
		else
		{
//			printf("4\n");
			for (uint16 i=index1 + 1;i<index2;i++) delete point_list->RemoveItem(index1 + 1);
			point_list->AddItem(new BPoint(point),index1 + 1);
		}
	}
}

int32 WarPolygon::TopPointIndex()
{
	float min_y = PointAt(0).y;
	int32 index = 0;
	// find the top vertex
	for (uint16 i=1;i<CountPoints();i++)
	{
		if (PointAt(i).y < min_y)
		{
			min_y = PointAt(i).y;
			index = i;
		}
	}
	
	return index;
}

int32 WarPolygon::IterateLeft(int32 index)
{
	return (index + point_list->CountItems() - 1)%point_list->CountItems();
}

int32 WarPolygon::IterateRight(int32 index)
{
	return (index + 1)%point_list->CountItems();
}

void WarPolygon::Draw()
{
	if (CountPoints() < 2) return;

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glColor3f(1.0,1.0,1.0);

	glBegin(GL_LINE_STRIP);
	for (uint16 i=0;i<CountPoints();i++)
	{
		BPoint point = PointAt(i);
		glVertex3f(point.x,point.y,0.0);
	}
	BPoint point = PointAt(0);
	glVertex3f(point.x,point.y,0.0);
	glEnd();
}