#include <TranslationUtils.h>
#include <Bitmap.h>
#include <stdlib.h>
#include <gl.h>
#include <glu.h>
#include <string.h>
#include <File.h>
#include <Application.h>

#include "WarSelectionView.h"
#include "WarObject.h"

WarSelectionView::WarSelectionView(int32 left,int32 top,int32 width,int32 height,WarFont *font):WarView(left,top,width,height)
{
	war_font = font;
	num_classes = 0;

	for(int32 i=0;i<32;i++)
	{
		entity_list[i] = new WarList<WarUnit*>(20,10);
		war_class[i] = NULL;
	}
}

WarSelectionView::~WarSelectionView()
{
	for(int32 i=0;i<32;i++)
		delete entity_list[i];
}

void WarSelectionView::Update()
{
//	printf("WarSelectionView::Update()\n");

	if (num_classes == 0) return;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho(0.0,Width()+1,0.0,Height()+1,0.0,10.0);
	glMatrixMode( GL_MODELVIEW );
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glDisable( GL_DEPTH_TEST );
	glLoadIdentity();
	
	glColor3f(1.0,1.0,1.0);

	int32 j=Height() - war_font->Height() - 3;
	int32 k=Width() - war_font->Width() * 26;
	if (k < 0) k=0;
	for (int16 i=0;i < num_classes;i++)
	{
		DrawString(k,j,war_font,line[i]);
		j-=war_font->Height()+3; // step one line down
	}
	glEnable(GL_DEPTH_TEST);
	glDisable(GL_BLEND);
}

void WarSelectionView::RemoveAll()
{
	for (int32 i=0;i<num_classes;i++)
	{
		war_class[i] = NULL;
		entity_list[i]->Empty();
	}
	num_classes = 0;
}

void WarSelectionView::Add(WarUnit *entity)
{
//	printf("WarSelectioView::Add()\n");
	WarClass *entity_class = entity->Class();
	
	int32 i=0;
	for(i=0;i<num_classes;i++)
		if (war_class[i] == entity_class) break;

	if (i > 31) return;

	if (i == num_classes) num_classes++;
	
	war_class[i] = entity_class;
	entity_list[i]->AddItem(entity);
}

void WarSelectionView::Remove(WarUnit *entity)
{
	WarClass *entity_class = entity->Class();
	
	int32 i=0;
	for(i=0;i<num_classes;i++)
		if (war_class[i] == entity_class) break;

	if (i >= num_classes) return;
	
	entity_list[i]->RemoveItem(entity);
	if (entity_list[i]->CountItems() == 0)
	{
		WarList<WarUnit*> *tmp_list = entity_list[i];

		entity_list[i] = entity_list[num_classes-1];
		entity_list[num_classes-1] = tmp_list;

		war_class[i] = war_class[num_classes-1];
		war_class[num_classes-1] = NULL;

		num_classes--;
	}
}

void WarSelectionView::Regenerate()
{
//	printf("WarSelectionView::Regenerate() %d\n",num_classes);
	for(int32 i=0;i<num_classes;i++)
	{
		sprintf(line[i],"%4dx ",entity_list[i]->CountItems());
		strncpy(line[i] + 6,war_class[i]->Name(),19);
	}
}
