#include <stdlib.h>
#include <math.h>

#include "WarSpline.h"

WarSpline::WarSpline()
{
	m_start = 0;
	m_end = 0;

	m_key_list = new WarList<Key*>(10,10);

	m_loop=true;
}

WarSpline::~WarSpline()
{
	Key *key;
	uint16 cookie = 0;
	while (m_key_list->GetNextItem(&key,&cookie)) delete key;

	delete m_key_list;
}

void WarSpline::Loop(bool l)
{
	m_loop=l;
}

void WarSpline::AddKey(float val, float time)
{
	Key *key = new Key;
	key->val = val;
	key->time = time;

	m_key_list->AddItem(key);
}

void WarSpline::DelKey()
{
	if (m_key_list->CountItems())
		delete m_key_list->RemoveItemAt(m_key_list->CountItems()-1);
}

float WarSpline::Value(float time)
{
	if(m_loop) time = LoopedTime(time);
	
	// can't interpolate if only one key
	if(m_key_list->CountItems() == 1) return m_key_list->ItemAt(0)->val;
	else if(time < m_key_list->ItemAt(0)->time) return m_start;
	else if(time > m_key_list->ItemAt(m_key_list->CountItems()-1)->time) return m_key_list->ItemAt(m_key_list->CountItems()-1)->val;

	int i;
	float tq,t;
	float v0,v1,v2,v3;

	for(i=0;i < m_key_list->CountItems();i++)
	{
		if(m_key_list->ItemAt(i+1)->time > time)
			break;
	}

	if (i == 0) v0 = m_start;
	else v0 = m_key_list->ItemAt(i-1)->val;
	
	v1 = m_key_list->ItemAt(i)->val;
	v2 = m_key_list->ItemAt(i+1)->val;

	if (i == m_key_list->CountItems()-2) v3 = m_end;
	else v3 = m_key_list->ItemAt(i+2)->val;

	t=(time-m_key_list->ItemAt(i)->time)/( m_key_list->ItemAt(i+1)->time - m_key_list->ItemAt(i)->time);

	tq = t*t;
	return v1*((float)1-((float)3.0-(float)2*t)*tq) + v2*(((float)3.0-(float)2*t)*tq);
}

float WarSpline::LoopedTime(float time)
{
	return (float)fmod(time,m_key_list->ItemAt(m_key_list->CountItems()-1)->time);
}

