#include <stdlib.h>
#include <math.h>

#include "WarSplineGrid.h"
#include <memory.h>

WarSplineGrid::WarSplineGrid(uint16 w,uint16 h,float div)
{
	divider = div;
	width = w;
	height = h;
	if (width < 3) width = 3;
	if (height < 3) height = 3;

	grid = new int8[width*height];
	memset(grid,0,width*height);
}

WarSplineGrid::~WarSplineGrid()
{
	delete []grid;
}

void WarSplineGrid::SetKey(uint16 x,uint16 y,int8 value)
{
	if (x >= width || y >= height) return;

//	printf("SetKey(%d,%d,%f)\n",x,y,value);

	grid[y*width + x] = value;
}

float WarSplineGrid::Value(float x,float y)
{
	if (x<0 || y<0 || x >= width || y >= height) return 0.0f;
	
	int32 ix = (int32)x;
	int32 iy = (int32)y;

	float th = x - (float)ix;
	float tv = y - (float)iy;

/*
		   v1
		+---+-------+
		|   :       |
	 v4+...o.......+v2
	   |   :(th,tv)|
		|   :       |
		|   :       |
		+---+-------+
	(0,0)   v3
*/


//	printf("th: %f, tv: %f\n",th,tv);
	float v1,v2,v3,v4;

	uint32 offset = iy * width + ix;


	if (th == 0.0 && tv == 0.0) return grid[offset]/divider;
	else if (th == 1.0 && tv == 0.0) return grid[offset + 1]/divider;
	else if (th == 1.0 && tv == 1.0) return grid[offset + width + 1]/divider;
	else if (th == 0.0 && tv == 1.0) return grid[offset]/divider;
/*
	v1 = (grid[offset + width + 1]*th + grid[offset + width]*(1.0-th));
	v3 = (grid[offset + 1]*th + grid[offset]*(1.0-th));
	v2 = (grid[offset + 1 + width]*tv + grid[offset + 1]*(1.0-tv));
	v4 = (grid[offset + width]*tv + grid[offset]*(1.0-tv));

	return (v1*tv + v3*(1.0-tv) + v2*th + v4*(1.0-th))/2.0/divider;
*/

	if (th == 0.0)
	{
		v1 = grid[offset + width];
		v3 = grid[offset];
	}
	else if (th == 1.0)
	{
		v1 = grid[offset + width + 1];
		v3 = grid[offset + 1];
	}
	else
	{
		if (ix == 0)
		{
			v1 = spline(grid[offset + width],grid[offset + width],grid[offset + width + 1],grid[offset + width + 2],th);
			v3 = spline(grid[offset],        grid[offset],        grid[offset + 1],        grid[offset + 2],        th);
		}
		else if (ix == width-1)
		{
			v1 = spline(grid[offset + width - 1],grid[offset + width],grid[offset + width + 1],grid[offset + width + 1],th);
			v3 = spline(grid[offset - 1],        grid[offset],        grid[offset + 1],        grid[offset + 1],        th);
		}
		else
		{
			v1 = spline(grid[offset + width - 1],grid[offset + width],grid[offset + width + 1],grid[offset + width + 2],th);
			v3 = spline(grid[offset - 1],        grid[offset],        grid[offset + 1],        grid[offset + 2],        th);
		}
	}

	if (tv == 0.0)
	{
		v2 = grid[offset + 1];
		v4 = grid[offset + 1];
	}
	else if (tv == 1.0)
	{
		v2 = grid[offset + width + 1];
		v4 = grid[offset + width];
	}
	else
	{
		if (iy == 0)
		{
			v2 = spline(grid[offset + 1],grid[offset + 1],grid[offset + width + 1],grid[offset + 2*width + 1],tv);
			v4 = spline(grid[offset],    grid[offset],    grid[offset + width],    grid[offset + 2*width],    tv);
		}
		else if (iy == height-1)
		{
			v2 = spline(grid[offset - width + 1],grid[offset + 1],grid[offset + width + 1],grid[offset + width + 1],tv);
			v4 = spline(grid[offset - width],    grid[offset],    grid[offset + width],    grid[offset + width],    tv);
		}
		else
		{
			v2 = spline(grid[offset - width + 1],grid[offset + 1],grid[offset + width + 1],grid[offset + 2*width + 1],tv);
			v4 = spline(grid[offset - width],    grid[offset],    grid[offset + width],    grid[offset + 2*width],    tv);
		}
	}

	return (v1*tv + v3*(1.0-tv) + v2*th + v4*(1.0-th))/2.0/divider;
}

inline float WarSplineGrid::spline(int8 v0,int8 v1,int8 v2,int8 v3,float t)
{
	return (v1*(1.0-t) + v2*t);

// !!! FIXA SÅ ATT SPLINE FUNGERAR BRA!

	float tquad=t*t;

	float p1=(float)v1;
	float p2=(float)v2;
	float r1=(p2 - (float)v0)*1.5;
	float r2=((float)v3 - p1)*1.5;

	return p1*(1-(3.0-2*t)*tquad) + r1*(tquad-t)*(1.0-t) + p2*(  (3.0-2*t)*tquad) + r2*(1.0-t)*(tquad );
}
