#include "WarTileset.h"
#include "WarPoint.h"
#include "defs.h"

#include <TranslationUtils.h>
#include <Bitmap.h>
#include <SupportDefs.h>
#include <Directory.h>
#include <Path.h>

#include <stdlib.h>
#include <stdio.h>
#include <OS.h>
#include <gl.h>
#include <string.h>

/* ------------------------------------------------------------ */
// RedrawBuffer
_EXPORT WarTileset::WarTileset(BDirectory *dir,char *filename)
{
#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"WarTilset:	%ld\n",sizeof(WarTileset));
#endif

	textures=NULL;

	num_blocks = 16;

	strncpy(name,filename,21);

	BFile file(dir,filename,B_READ_ONLY);

	status_t err;
	if ((err = file.InitCheck()) != B_OK)
	{
		fprintf(stderr,"FAILED to load environmentfile \"%s\" , %lx\n",filename,err);
		return;
	}
	BBitmap *texture = BTranslationUtils::GetBitmap(&file);
	if (texture == NULL)
	{
		fprintf(stderr,"FAILED to load environmentfile \"%s\" , no translator\n",filename);
		return;
	}

#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"  texture_ids:	%d\n",num_blocks*4);
	fprintf(stderr,"  texture gfx: %d\n",num_blocks * BLOCK_WIDTH*BLOCK_WIDTH*3);
#endif

	textures = new GLuint[num_blocks];
	convertion_table = new WarPoint[(TILE_SPAN + 1) * (TILE_SPAN + 1) * 8];
	translation_table = new WarPoint[4 * 4];

	BuildConvertionTable();

// allocate names for all the tile textures
	glGenTextures(1,textures);

// convert to big-endian if needed
#ifdef __INTEL__
	int32 j=0;
	uint8 *buffer = (uint8*)texture->Bits();
	while (j<texture->BitsLength())
	{
		uint8 tmp = buffer[j];
		buffer[j] = buffer[j+2];
		buffer[j+2] = tmp;
		j+=4;
	}
#endif

// bind the graphics to every texture
	glBindTexture(GL_TEXTURE_2D, textures[0]);

// filers.
//	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
//	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

// and finally, load the graphics
	glTexImage2D(GL_TEXTURE_2D, 0, 4, 256 , 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, texture->Bits());
}

_EXPORT WarTileset::~WarTileset()
{
	delete []textures;
	delete []convertion_table;
	delete []translation_table;
}

_EXPORT uint16 WarTileset::TileCount()
{
	return num_blocks<<3;
}

_EXPORT void WarTileset::BuildConvertionTable()
{
	bool invert_x[]={false,true,true,false,true,true,false,false};
	bool invert_y[]={false,false,true,true,false,true,true,false};
	
	for (int32 i = 0;i<8;i++)
	{
		for (int32 y = 0;y<=TILE_SPAN;y++)
		{
			for (int32 x = 0;x<=TILE_SPAN;x++)
			{
				float nx = x;
				float ny = y;
			
				if (invert_x[i]) nx = TILE_SPAN - nx;
				if (invert_y[i]) ny = TILE_SPAN - ny;
				if (i&1) // swap axis
				{
					float tmp = nx;
					nx = ny;
					ny = tmp;
				}
				// !!! should be 8 instead of 4, in the future
				nx/= (float)4*TILE_SPAN;
				ny/= (float)4*TILE_SPAN;
				convertion_table[((TILE_SPAN + 1)*(TILE_SPAN + 1))*i + y * (TILE_SPAN + 1) + x].SetTo(nx,ny);
			}
		}
	}

	for (int32 y = 0;y<4;y++)
		for (int32 x = 0;x<4;x++)
			translation_table[y * 4 + x].SetTo(((float)x) * 1.0/4.0,((float)y) * 1.0/4.0);
}
