#include <stdio.h>
#include <Bitmap.h>
#include <TranslationUtils.h>
#include <GraphicsDefs.h>
#include <Application.h>
#include <Window.h>
#include <Directory.h>

#include "WarColor.h"
#include "GraphicsDefs.h"

#define pow(x) ((x)*(x))
	
class MyApplication:public BApplication
{
private:
	BEntry *entry;
public:
	MyApplication();
	~MyApplication();
};

MyApplication *app;
status_t err;
BBitmap *map;
BRect bounds;
uint16 rowbytes,width,height;
thread_id convert_thread;
BFile *outfile;
BTranslationUtils *utils;

static bool LoadFile(void *mem_ptr,const char *name,uint16 size)
{
	printf("loading: \"%s\"\n",name);
	BEntry *entry;
	BFile *file;
	bool success=TRUE;
	
	entry=new BEntry(name,TRUE);
	file=new BFile(entry,B_READ_ONLY);
	if (file->Read(mem_ptr,(size_t)size)==0) success=FALSE;
	delete file;
	delete entry;
	return success;
}

bool convert_bitmap(const char *file_name,int32 a_width,int32 a_height,uint16 num_frames,WarColor *table,BFile *outfile,uint8 *block_list)
{
	BBitmap *bitmap;
	uint16 frame;
	uint32 tot_write=0;
	WarColor tmp_block[a_width*a_height];
	uint16 blocks=0;
	uint16 index=0;
	uint16 px;
	uint16 py;
	uint16 b_x;
	uint16 b_y;
	int16 col;
	uint8 r,g,b,block_set;

	printf("loading \"%s\"\n",file_name);
	bitmap=utils->GetBitmap(file_name);
	if (!bitmap)
	{
		printf("No translator for \"%s\"!\n",file_name);
		return false;
	}
	rgb_color *ptr=(rgb_color*)bitmap->Bits();
	bounds=bitmap->Bounds();
	width=bounds.IntegerWidth()+1;
	height=bounds.IntegerHeight()+1;
	rowbytes=bitmap->BytesPerRow();

	b_x=0;
	b_y=0;

	uint16 tmp_red,tmp_green,tmp_blue;

	tot_write=0;

	printf("blocks: width:%d height:%d\n",a_width,a_height);
	printf("image: width:%d height:%d\n",width,height);

	frame=0;
	while (frame<num_frames)
	{
		tmp_red=0;
		tmp_green=0;
		tmp_blue=0;
		if ((b_x*a_width)>=width)
		{
			b_x=0;
			b_y++;
		}
		if (block_list[index++]!=0xff)
		{
			for (py=0;py<a_height;py++)
				for (px=0;px<a_width;px++)
				{
					tmp_block[px+(a_width*py)].red=ptr[((b_y*a_height)+py)*width+(b_x*a_width)+px].red;
					tmp_block[px+(a_width*py)].green=ptr[((b_y*a_height)+py)*width+(b_x*a_width)+px].green;
					tmp_block[px+(a_width*py)].blue=ptr[((b_y*a_height)+py)*width+(b_x*a_width)+px].blue;
					tmp_red+=tmp_block[px+(a_width*py)].red;
					tmp_green+=tmp_block[px+(a_width*py)].green;
					tmp_blue+=tmp_block[px+(a_width*py)].blue;
				}

			table[frame].red=tmp_red>>8;
			table[frame].green=tmp_green>>8;
			table[frame].blue=tmp_blue>>8;
/*
			for (py=0;py<a_height;py++)
				for (px=0;px<a_width;px++)
				{
					tmp_block[px+(a_width*py)]=table[frame];
				}
*/

//			printf("%c%c%c%c",table[frame].red,table[frame].green,table[frame].blue,table[frame].alpha);

			frame++;
			outfile->Write(tmp_block,a_width*a_height*sizeof(WarColor));
			tot_write+=a_width*a_height*sizeof(WarColor);
		}
		b_x++;
	}
	printf("\nwrote %d*%d=%d bytes\n",num_frames,a_width*a_height*sizeof(WarColor),tot_write);

	delete bitmap;
	return true;
};

static long convert(void *params)
{
	struct file_struct
	{
		uint16 num_blocks;
		char *image_filename;
		char *blocktype_filename;
		uint8 *block_list;
	};

	const num_files=1;
	file_struct files[]={{0,"masks.png","masks_types",0}};

	printf("converter thread launched.\n");

	BDirectory app_dir("/Blippo/home/sources/WarR4/create_environment/");
	BFile outfile(&app_dir,"environment_file",B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
	if (!(outfile.InitCheck()==B_NO_ERROR))
	{
		printf("Error creating output file\n");
		app->PostMessage(B_QUIT_REQUESTED);
		return NULL;

	}
	err=0;

	// BLOCK SETS
	uint8 file;
	for (file=0;file<num_files;file++)
	{
		BFile *bfile=new BFile(&app_dir,files[file].blocktype_filename,B_READ_ONLY);

		uint16 i;
		int64 size;
		uint8 byte;
		uint16 counter=0;

		bfile->GetSize(&size);
		files[file].block_list=new uint8[size];
		for(i=0;i<size;i++)
		{
			bfile->Read(&byte,1);
			if (byte!=10)
			{
				if (byte==' ') files[file].block_list[counter]=0xff; else files[file].block_list[counter]=byte-'0';
				if (files[file].block_list[counter]!=0xff) files[file].num_blocks++;
				counter++;
			}
		}
		delete bfile;
		printf("number of blocks in \"%s\": %d\n",files[file].image_filename,files[file].num_blocks);
	}

	uint16 tot_blocks=0;
	uint16 i;
	for (file=0;file<num_files;file++)
		tot_blocks+=files[file].num_blocks;
	outfile.Write(&tot_blocks,2);

	printf("total number of blocks: %d\n",tot_blocks);

	uint16 j=0;
	for (file=0;file<num_files;file++)
	{
		i=0;
		j=0;
		while(i<files[file].num_blocks)
		{
			if (files[file].block_list[j]!=0xff)
			{
				outfile.Write(&files[file].block_list[j],1);
				i++;
			}
			j++;
		}
	}

	WarColor *table=new WarColor[tot_blocks];

	uint16 block_nr=0;
	for (file=0;file<num_files;file++)
	{
		if (!convert_bitmap(files[file].image_filename,128,128,files[file].num_blocks,table+block_nr,&outfile,files[file].block_list))
		{
			printf("Error loading file \"%s\"!\n",files[file].image_filename);
			app->PostMessage(B_QUIT_REQUESTED);
			return NULL;
		}
		block_nr+=files[file].num_blocks;
	}

	if (!(outfile.Write(table,tot_blocks*sizeof(WarColor)))) err=1;
	delete []table;

	for (file=0;file<num_files;file++)
	{
		delete []files[file].block_list;
	}

	printf("Finished! exiting.\n");
	app->PostMessage(B_QUIT_REQUESTED);
	return NULL;
}

MyApplication::MyApplication():BApplication("application/x-vnd.Arvid Norberg-MakeEnvironment")
{
	convert_thread=spawn_thread(convert,"picture converter",B_NORMAL_PRIORITY,(void *)NULL);
	resume_thread(convert_thread);
}

MyApplication::~MyApplication()
{
	if (find_thread("convert_thread")!=B_NAME_NOT_FOUND) kill_thread(convert_thread);
}

int main()
{
	printf("Running application\n");
	app = new MyApplication();
	app->Run();
}
