#include <stdio.h>
#include <Bitmap.h>
#include <TranslationUtils.h>
#include <GraphicsDefs.h>
#include <Application.h>
#include <Window.h>

#include "WarColor.h"
#include "GraphicsDefs.h"

#define pow(x) ((x)*(x))
	
class MyApplication:public BApplication
{
private:
	BEntry *entry;
public:
	MyApplication();
	~MyApplication();
};

MyApplication *app;
status_t err;
BBitmap *map;
BRect bounds;
uint16 rowbytes,width,height;
thread_id convert_thread;
BFile *outfile;
BTranslationUtils *utils;

static bool LoadFile(void *mem_ptr,const char *name,uint16 size)
{
	printf("loading: \"%s\"\n",name);
	BEntry *entry;
	BFile *file;
	bool success=TRUE;
	
	entry=new BEntry(name,TRUE);
	file=new BFile(entry,B_READ_ONLY);
	if (file->Read(mem_ptr,(size_t)size)==0) success=FALSE;
	delete file;
	delete entry;
	return success;
}

bool convert_bitmap(const char *file_name,BFile *file)
{
	BBitmap *bitmap;
	rgb_color *ptr;
	uint16 blocks=0;
	uint16 index=0;

	printf("loading \"%s\"\n",file_name);
	bitmap=utils->GetBitmap(file_name);
	if (!bitmap)
	{
		printf("No translator for \"%s\"!\n",file_name);
		return false;
	}
	ptr=(rgb_color*)bitmap->Bits();
	bounds=bitmap->Bounds();
	width=bounds.IntegerWidth()+1;
	height=bounds.IntegerHeight()+1;

	printf("image: width:%d height:%d\n",width,height);

	for (int32 py=0;py<height;py++)
		for (int32 px=0;px<width;px++)
			file->Write(&ptr[py * width + px].red,1);

	delete bitmap;
	return true;
};

static long convert(void *params)
{
	struct file_struct
	{
		uint16 num_blocks;
		char *image_filename;
		char *blocktype_filename;
		uint8 *block_list;
	};

	printf("converter thread launched.\n");

	BEntry out_entry("height_field");
	BFile outfile(&out_entry,B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
	if (!(outfile.InitCheck()==B_NO_ERROR))
	{
		printf("Error creating output file\n");
		app->PostMessage(B_QUIT_REQUESTED);
		return NULL;

	}
	err=0;

	if (!convert_bitmap("modified.png",&outfile))
	{
		app->PostMessage(B_QUIT_REQUESTED);
		return NULL;
	}

	printf("Finished! exiting.\n");
	app->PostMessage(B_QUIT_REQUESTED);
	return NULL;
}

MyApplication::MyApplication():BApplication("application/x-vnd.Arvid Norberg-MakeEnvironment")
{
	convert_thread=spawn_thread(convert,"picture converter",B_NORMAL_PRIORITY,(void *)NULL);
	resume_thread(convert_thread);
}

MyApplication::~MyApplication()
{
	if (find_thread("convert_thread")!=B_NAME_NOT_FOUND) kill_thread(convert_thread);
}

int main()
{
	printf("Running application\n");
	app = new MyApplication();
	app->Run();
}
