#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <MenuBar.h>
#include <Bitmap.h>
#include <ScrollBar.h>
#include <Region.h>

#include "main.h"
#include "WarDisplay.h"
#include "Global.h"
#include "Clusters.h"

WarDisplay::WarDisplay(BRect frame,WarMap *map) : BView(frame, "MapView", B_FOLLOW_ALL_SIDES, B_WILL_DRAW|B_FRAME_EVENTS)
{
	rgb_color *buffer;
	war_map=map;
	printf("1\n");
	tiles=war_map->Tileset();
	printf("2 %d\n",tiles);
	gfx=new BBitmap*[tiles->TileCount()];
	printf("3\n");
	uint16 i;
	WarColor *bitmap;
	printf("4\n");
	for (i=0;i<tiles->TileCount();i++)
	{
		gfx[i]=new BBitmap(BRect(0,0,BLOCK_AND,BLOCK_AND),0,B_RGB32_BIG);
		bitmap=tiles->Tile(i);
		buffer=(rgb_color*)gfx[i]->Bits();
		for (uint16 j=0;j<BLOCK_WIDTH*BLOCK_WIDTH;j++)
		{
			buffer[j].red=bitmap[j].red;
			buffer[j].green=bitmap[j].blue;
			buffer[j].blue=bitmap[j].green;
		}
	}
}

WarDisplay::~WarDisplay()
{
//	for (uint16 i=0;i<tiles->TileCount();i++) delete gfx[i];
	delete []gfx;
}

void WarDisplay::AttachedToWindow(void)
{
	MakeFocus(true);
	FixupScrollbars();
	SetViewColor(B_TRANSPARENT_32_BIT);
}

void WarDisplay::KeyDown(const char *key,int32 numbytes)
{
	if (*key==30) current_terrain--;
	if (*key==31) current_terrain++;
	if (*key=='p')
	{
		for (uint16 y=0;y<5;y++)
		{
			for (uint16 x=0;x<5;x++)
				if (war_map->TileIndex(x,y)!=0) printf("%d,",war_map->TileIndex(x,y));
				else printf("-1,");
			printf("\n");
		}
		printf("\n\n");
	}
//	if (current_terrain>=num_clusters) current_terrain=num_clusters-1;
	if (current_terrain<0) current_terrain=0;
}

void WarDisplay::FrameResized(float width, float height)
{
	FixupScrollbars();
}

void WarDisplay::MouseDown(BPoint point)
{
	drawing=true;
//	SetBlock(point.x/BLOCK_WIDTH,point.y/BLOCK_WIDTH,current_block);
	Paint(point,current_terrain);
}

void WarDisplay::MouseMoved(BPoint point,uint32,const BMessage *)
{
//	if (drawing) SetBlock(point.x/BLOCK_WIDTH,point.y/BLOCK_WIDTH,current_block);
	if (drawing) Paint(point,current_terrain);
}

void WarDisplay::MouseUp(BPoint point)
{
	drawing=false;
	last_offs=-1;
}

void WarDisplay::FillMap(uint8 terrain)
{
	for(int32 y = 0;y<war_map->Height()/TILE_SPAN;y++)
		for(int32 x = 0;x<war_map->Width()/TILE_SPAN;x++)
			war_map->SetTileIndex(x * TILE_SPAN,y * TILE_SPAN,(terrain&3)+(rand()&3));
	Invalidate();
}

void WarDisplay::Paint(BPoint point,uint8 terrain)
{

	uint16 block=terrain*8+(rand()&7);
	uint16 x=point.x/BLOCK_WIDTH;
	uint16 y=point.y/BLOCK_WIDTH;
	int32 offs=(y<<war_map->WidthShift())+x;
	if (last_offs==offs) return;
	last_offs=offs;
	for (uint16 ty=0;ty<5;ty++)
		for (uint16 tx=0;tx<5;tx++)
//			if (cluster[terrain*25+ty*5+tx]!=-1) SetBlock(tx+x,ty+y,cluster[terrain*25+ty*5+tx]);
			SetBlock(x,y,terrain);
}

void WarDisplay::SetBlock(uint16 x,uint16 y,uint16 block)
{
//	printf("set block (%d,%d)\n",x,y);
	if (war_map->Tileset()->TileCount() <= block) return;
	war_map->SetTileIndex(x * TILE_SPAN,y * TILE_SPAN,block);
	Invalidate(BRect(x<<BLOCK_SHIFT,y<<BLOCK_SHIFT,(x<<BLOCK_SHIFT)+BLOCK_AND,(y<<BLOCK_SHIFT)+BLOCK_AND));
}

void WarDisplay::FixupScrollbars()
{
	BRect bounds;
	BScrollBar *sb;

	bounds = Bounds();
	float view_width = bounds.Width();
	float view_height = bounds.Height();
	float maxWidth = 1,maxHeight = 1;

	float propW=view_width/((war_map->Width()/TILE_SPAN)<<BLOCK_SHIFT);
	float propH=view_height/((war_map->Height()/TILE_SPAN)<<BLOCK_SHIFT);

	float rangeW=((war_map->Width()/TILE_SPAN)<<BLOCK_SHIFT) - view_width;
	float rangeH=((war_map->Height()/TILE_SPAN)<<BLOCK_SHIFT) - view_height;
	if (rangeW<0) rangeW=0;
	if (rangeH<0) rangeH=0;


	if ((sb=ScrollBar(B_HORIZONTAL))!=NULL)
	{
		sb->SetProportion(propW);
		sb->SetRange(0,rangeW);
		sb->SetSteps(BLOCK_WIDTH, view_width);
	} 

	if ((sb=ScrollBar(B_VERTICAL))!=NULL)
	{
		sb->SetProportion(propH);
		sb->SetRange(0,rangeH);
		sb->SetSteps(BLOCK_WIDTH, view_height);
	}
}

void WarDisplay::Draw(BRect)
{
	BRegion region;
	GetClippingRegion(&region);
	int32 i;
	for (i=0;i<region.CountRects();i++)
		DrawRect(region.RectAt(i));
}

void WarDisplay::DrawRect(BRect rect)
{
	uint16 left=rect.left/BLOCK_WIDTH;
	uint16 top=rect.top/BLOCK_WIDTH;
	uint16 right=rect.right/BLOCK_WIDTH + 1;
	uint16 bottom=rect.bottom/BLOCK_WIDTH + 1;

	uint16 x,y;
	for (y=top;y<bottom;y++)
		for (x=left;x<right;x++)
		{
			uint16 tile = war_map->TileIndex(x * TILE_SPAN,y * TILE_SPAN);
			DrawBitmapAsync(gfx[tile],BPoint(x<<BLOCK_SHIFT,y<<BLOCK_SHIFT));
		}
}