#include "WarTileset.h"
#include "defs.h"

#include <stdlib.h>
#include <stdio.h>
#include <OS.h>

/* ------------------------------------------------------------ */
// RedrawBuffer
_EXPORT WarTileset::WarTileset(BDirectory *dir,char *filename)
{
#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"WarTilset:	%d\n",sizeof(WarTileset));
#endif

	buffer=NULL;
	type=NULL;
	color=NULL;

	strcpy(name,filename);
	name[20]=0;
	fprintf(stderr,"WarTilset:	\"%s\"\n",name);
	BFile file(dir,filename,B_READ_ONLY);
	if (file.InitCheck()==B_OK)
	{
		file.Read(&num_blocks,2);
		if (num_blocks>512) num_blocks=512;

#ifdef _MEMORY_STATISTICS_
		fprintf(stderr,"  type_list:	%d\n",num_blocks);
		fprintf(stderr,"  buffer:	%d\n",(num_blocks<<(BLOCK_SHIFT+BLOCK_SHIFT))*4);
		fprintf(stderr,"  color:	%d\n",num_blocks*4);
#endif

		type=new uint8[num_blocks];
		buffer=new WarColor[num_blocks<<(BLOCK_SHIFT+BLOCK_SHIFT)];
		color=new WarColor[num_blocks];

		file.Read(type,num_blocks);
		file.Read(buffer,(num_blocks<<(BLOCK_SHIFT+BLOCK_SHIFT))*sizeof(WarColor));

		memset(color,-100,num_blocks*sizeof(WarColor));
		file.Read(color,num_blocks*sizeof(WarColor));
/*
		int32 j=0;
		while (j<BLOCK_WIDTH*BLOCK_WIDTH*num_blocks)
		{
			uint8 tmp = buffer[j].red;
			buffer[j].red = buffer[j].blue;
			buffer[j].blue = tmp;
			j++;
		}
*/
	}
	else fprintf(stderr,"FAILED to load environmentfile \"%s\"\n",filename);
}

_EXPORT WarTileset::~WarTileset()
{
	if (buffer) delete[] buffer;
	if (type) delete[] type;
	if (color) delete[] color;
}

_EXPORT uint16 WarTileset::TileCount()
{
	return num_blocks<<3;
}

_EXPORT WarColor *WarTileset::Tile(uint16 index)
{
	if (num_blocks==0)
	{
		fprintf(stderr,"num_block: %d\n",num_blocks);
		return buffer;
	}
	uint16 i=index&7;
	index>>=3;
	WarColor *ptr=buffer+((index%num_blocks)<<(BLOCK_SHIFT+BLOCK_SHIFT));
	switch (i)
	{
		case 0:
			return ptr;
		case 1:
		{
			uint16 j=0;
			for(int16 x=BLOCK_WIDTH-1;x>=0;x--)
				for(int16 y=0;y<BLOCK_WIDTH;y++)
					temp_block[y*BLOCK_WIDTH + x]=ptr[j++];
		} break;
		case 2:
		{
			uint16 j=0;
			for(int16 y=BLOCK_WIDTH-1;y>=0;y--)
				for(int16 x=BLOCK_WIDTH-1;x>=0;x--)
					temp_block[y*BLOCK_WIDTH + x]=ptr[j++];
		} break;
		case 3:
		{
			uint16 j=0;
			for(int16 x=0;x<BLOCK_WIDTH;x++)
				for(int16 y=BLOCK_WIDTH-1;y>=0;y--)
					temp_block[y*BLOCK_WIDTH + x]=ptr[j++];
		} break;
		case 4:
		{
			uint16 j=0;
			for(int16 y=0;y<BLOCK_WIDTH;y++)
				for(int16 x=BLOCK_WIDTH-1;x>=0;x--)
					temp_block[y*BLOCK_WIDTH + x]=ptr[j++];
		} break;
		case 5:
		{
			uint16 j=0;
			for(int16 x=BLOCK_WIDTH-1;x>=0;x--)
				for(int16 y=BLOCK_WIDTH-1;y>=0;y--)
					temp_block[y*BLOCK_WIDTH + x]=ptr[j++];
		} break;
		case 6:
		{
			uint16 j=0;
			for(int16 y=BLOCK_WIDTH-1;y>=0;y--)
				for(int16 x=0;x<BLOCK_WIDTH;x++)
					temp_block[y*BLOCK_WIDTH + x]=ptr[j++];
		} break;
		case 7:
		{
			uint16 j=0;
			for(int16 x=0;x<BLOCK_WIDTH;x++)
				for(int16 y=0;y<BLOCK_WIDTH;y++)
					temp_block[y*BLOCK_WIDTH + x]=ptr[j++];
		} break;
	}
	return temp_block;
}

_EXPORT uint8 WarTileset::TileType(uint16 index)
{
	return type[index%num_blocks];
}

_EXPORT WarColor WarTileset::TileColor(uint16 index)
{
	return color[index%num_blocks];
}
