#include <stdio.h>
#include <string.h>

#include <Screen.h>
#include <MenuItem.h>
#include <NodeInfo.h>
#include <TextView.h>
#include <ScrollView.h>
#include <TranslationUtils.h>
#include <FindDirectory.h>

#include "main.h"
#include "WarWindow.h"
#include "WarDisplay.h"
#include "Global.h"

WarWindow::WarWindow(BRect frame, const char *filename) : BWindow(frame, APP_NAME, B_DOCUMENT_WINDOW, B_WILL_ACCEPT_FIRST_CLICK + B_CURRENT_WORKSPACE)
{
	file_dir=new BDirectory();

	BPath path;
	find_directory(B_USER_DIRECTORY, &path);
	save = new BFilePanel(B_SAVE_PANEL,new BMessenger(this));
	save->SetPanelDirectory(path.Path());

// -------------------------

	BRect rect(0,0,0,0);
	BMenuItem *item;
	BMenu *sub;
	menu = new BMenuBar(rect, "MenuBar");
	sub = new BMenu("File");

	sub->AddItem(item=new BMenuItem("New", new BMessage(WAR_NEW_FILE),'N'));
	item->SetTarget(be_app);
	sub->AddItem(item=new BMenuItem("Open", new BMessage(WAR_OPEN_FILE),'O'));
	item->SetTarget(be_app);
	sub->AddItem(save_item=new BMenuItem("Save", new BMessage(WAR_SAVE_FILE),'S'));
	sub->AddItem(new BMenuItem((char*)"Save As", new BMessage(WAR_SAVE_AS),0));
	sub->AddSeparatorItem();
	sub->AddItem(new BMenuItem("Close", new BMessage(B_CLOSE_REQUESTED),'W'));
	sub->AddItem(item=new BMenuItem("About", new BMessage(B_ABOUT_REQUESTED),'A'));
	item->SetTarget(be_app);
	sub->AddItem(item=new BMenuItem("Quit", new BMessage(B_CLOSE_REQUESTED),'Q'));
	item->SetTarget(be_app);
	menu->AddItem(sub);

	sub=new BMenu("map");
	sub->AddItem(item = new BMenuItem("Fill", new BMessage(WAR_FILL_MAP),'F'));

	menu->AddItem(sub);
/*
	BMessage *msg;
	sub = new BMenu("Image");
	sub->AddItem(item = new BMenuItem(T_FITMENU, msg = new BMessage(T_TOFIT)));
	BMenu *ssub = new BMenu(T_SCALEMENU);
	ssub->SetRadioMode(true);
	ssub->AddItem(item = new BMenuItem("50%", msg = new BMessage(T_SCALE)));
	msg->AddFloat("scale",0.5);
	msg->AddInt32("scale_mode", SCALE_NORMAL);
	ssub->AddItem(item = new BMenuItem("100%", msg = new BMessage(T_SCALE)));
	msg->AddFloat("scale",1.0);
	msg->AddInt32("scale_mode", SCALE_NORMAL);
	ssub->AddItem(item = new BMenuItem("200%", msg = new BMessage(T_SCALE)));
	msg->AddFloat("scale",2.0);
	msg->AddInt32("scale_mode", SCALE_NORMAL);
	ssub->AddSeparatorItem();
	ssub->AddItem(item = new BMenuItem("Max", msg = new BMessage(T_SCALE)));
	msg->AddInt32("scale_mode", SCALE_MAX);
	item->SetMarked(true);
	sub->AddItem(ssub);
	menu->AddItem(sub);
*/
	AddChild(menu);

// --------------------

	rect = menu->Bounds();
	float hi = rect.bottom;
	rect = frame;
	rect.OffsetTo(0,0);
	rect.top += hi+ 1;
	rect.right -= B_V_SCROLL_BAR_WIDTH;
	rect.bottom -= B_H_SCROLL_BAR_HEIGHT;
	
	if (filename)
	{
		printf("path: %s\n",filename);
		BPath path(filename);
		BPath tmp;
		path.GetParent(&tmp);
		printf("directory: %s\n",tmp.Path());
		file_dir->SetTo(tmp.Path());
		strcpy(file_name,path.Leaf());
		printf("filename: %s\n",file_name);
		war_map=new WarMap(file_dir,file_name,new BDirectory("/boot/home/sources/WarR4/War/environments"));
		CurrentFile(file_name);
	}
	else
	{
		save_item->SetEnabled(false);
		war_map=new WarMap(0,new BDirectory("/boot/home/sources/WarR4/War/environments"),"default");
		CurrentFile("unnamed");
	}

	war_map->Correct();

	war_display = new WarDisplay(rect,war_map);
	AddChild(new BScrollView("MapView", war_display, B_FOLLOW_ALL_SIDES, B_WILL_DRAW, true, true));
	RedoSizes();
}

WarWindow::~WarWindow()
{
	delete war_map;
	delete save;
	delete file_dir;
}

void WarWindow::CurrentFile(char *filename)
{
	char title[B_FILE_NAME_LENGTH+12];
	strcpy(title,"WarEditor: ");
	strcat(title,filename);
	SetTitle(title);
}

void WarWindow::WindowActivated(bool active)
{
	if (!active) return;
	BMessage *msg=new BMessage(WAR_NEW_TARGET);
	msg->AddPointer("target",(void*)this);
	((WarApp*)be_app)->palette->PostMessage(msg);
}

void WarWindow::MessageReceived(BMessage *message)
{
	uint32 type; 
	int32 count;

	switch(message->what)
	{
	case B_SIMPLE_DATA:
		message->GetInfo("refs", &type, &count); 
		if ( count>0 && type == B_REF_TYPE )
		{ 
			message->what = B_REFS_RECEIVED;
			be_app->PostMessage(message);
		}
		break;
	case WAR_SAVE_AS:
		save->Show();
		break;
	case WAR_FILL_MAP:
		war_display->FillMap(war_display->current_terrain);
		break;
	case B_SAVE_REQUESTED:
	{
		char *filename;
		entry_ref ref;
		message->FindString("name",&filename);
		message->FindRef("directory",&ref);
		file_dir->SetTo(&ref);
		strcpy(file_name,filename);
		save_item->SetEnabled(true);
	}
	case WAR_SAVE_FILE:
		war_map->Save(file_dir,file_name);
		CurrentFile(file_name);
		break;
	default:
		BWindow::MessageReceived(message);
	}
}

void WarWindow::RedoSizes()
{
	BScreen *scr = new BScreen(this);
	BRect screenbounds = scr->Frame();
	delete scr;

	float sizeX,sizeY;
	sizeX=(war_map->Width()/TILE_SPAN)<<BLOCK_SHIFT;
	sizeY=(war_map->Height()/TILE_SPAN)<<BLOCK_SHIFT;
	
	float minX,maxX,minY,maxY;
		
	maxX = sizeX+B_V_SCROLL_BAR_WIDTH;
	maxY = sizeY+B_H_SCROLL_BAR_HEIGHT;
	maxX = min_c(maxX,floor(screenbounds.Width()+0.5));
	maxY = min_c(maxY,floor(screenbounds.Height()+0.5)) + menu->Bounds().bottom+1.;

	minX = min_c(maxX, B_V_SCROLL_BAR_WIDTH + 32);
	minY = min_c(maxY, B_H_SCROLL_BAR_HEIGHT + menu->Bounds().bottom+1 + 32);
	SetSizeLimits(minX,maxX,minY,maxY);
	SetZoomLimits(maxX, maxY);

	BRect cur = Bounds();
	if(cur.Width()+1. > maxX || cur.Height()+1. > maxY)
		ResizeTo(maxX,maxY);
}

void WarWindow::ScreenChanged(BRect frame, color_space mode)
{
	RedoSizes();
}

