#include <stdio.h>
#include <Application.h>
#include <FindDirectory.h>
#include <Message.h>
#include <Path.h>
#include <Alert.h>

#include "main.h"
#include "WarWindow.h"

// We don't want to count the Open Panel and the palette window
#define WINDOWS_TO_IGNORE 2

WarApp::WarApp():BApplication(APP_SIG)
{
	palette=NULL;
	SetPulseRate(1000000);
	BPath path;
	load = new BFilePanel(B_OPEN_PANEL);
	find_directory(B_USER_DIRECTORY, &path);
	load->SetPanelDirectory(path.Path());

	palette=new WarPaletteWindow(BRect(100,100,200,400));
}

bool WarApp::QuitRequested()
{
	palette->Lock();
	palette->Quit();
	return true;
}

WarApp::~WarApp()
{
	delete load;
}

void WarApp::AboutRequested()
{
	BAlert *alert=new BAlert("About","WarEditor written by Arvid Norberg 1999\n<arvid@iname.com>","ok");
	alert->Go();
}

void WarApp::ReadyToRun()
{
	BRect rect(50,50,250,250);
	WarWindow *win;
	win = new WarWindow(rect, NULL);
	win->Show();
	palette->Show();
}

void WarApp::Pulse()
{
	if(!IsLaunching() && CountWindows() <= WINDOWS_TO_IGNORE)
	{
		PostMessage(B_QUIT_REQUESTED);
	}
}

void WarApp::ArgvReceived(int32 argc, char **argv)
{
	BRect rect(50, 50, 250, 250);
	BWindow *win;
	if(argc <= 1)
	{
		win = new WarWindow(rect, NULL);
		win->Show();
	}
	else
		for(int i = 1; i < argc ; i++)
		{
			win = new WarWindow(rect, argv[i]);
			win->Show();
		}
}

void WarApp::MessageReceived(BMessage *message)
{
	switch(message->what){
	case WAR_OPEN_FILE:
		load->Show();
		break;
	case WAR_NEW_FILE:
	{
		WarWindow *win;
		win = new WarWindow(BRect(50,50,250,250),NULL);
		win->Show();
	}break;
	default:
		BApplication::MessageReceived(message);
	}
}

void WarApp::RefsReceived(BMessage *message)
{
	uint32 type; 
	int32 count;
	entry_ref ref;
	BPath path;

	message->GetInfo("refs", &type, &count); 
	if ( type != B_REF_TYPE ) return; 

	BRect rect(50, 50, 250, 250);
	BWindow *win;
	for ( long i = --count; i >= 0; i-- ) { 
		if ( message->FindRef("refs", i, &ref) == B_OK ) { 
			BEntry entry(&ref, true);
			if ( entry.IsFile() && entry.GetPath(&path)==B_OK){
				win = new WarWindow(rect, path.Path());
				win->Show();
			}
		} 
	} 
}

int main(int argc, char **argv)
{
	new WarApp();
	be_app->Run();
	delete be_app;
}
