#ifndef _WAR_CAMERA_
#define _WAR_CAMERA_

#include "WarMap.h"
#include "WarVertex.h"
#include "WarObject.h"
#include "WarEntityList.h"
#include "WarMatrix.h"

#include <gl.h>
#include <GraphicsCard.h>
#include <List.h>

class WarCamera : public WarMatrix
{
public:
	WarCamera();
	WarCamera(WarVertex position);
	virtual ~WarCamera();

	void Cycle(float frame_time,WarEntityList *war_entity_list,WarMap *map);

	void Scroll(float horiz,float vert);

	void AddMovementSpeed(WarVertex delta_pos);
	void AddRotationSpeed(WarVertex delta_angles);
	void AddScrollSpeed(float vert,float horiz);

// makes the camera attached to a nuit, and will show everything from
// his viewpoint
	void AttachToUnit(WarBody *body);
	void DetachFromUnit();

// makes the camera look on a unit and follow it
// only rotates the camera, don't change the tilt angle
	void LookAtUnit(WarBody *body);
	void StopLookAt();

// changes Field of vision
	void SetFov(float fov);
	float Fov();
	
private:

	WarVertex movement_speed;
	WarVertex rotation_speed;
	float vertical_move_speed;
	float horizontal_move_speed;

	float time_left; // the time left before the camera must have reached it's goal (camera_goal)
	WarVertex camera_goal; // the camera_goal, ignore it if time_left is 0

	float fov; // field of vision, in degrees

	WarBody *attached_unit; // is NULL if not attached
	// the id of the unit the camera is attached to, we neede it since we don't know
	// when the Body is deleted, we have to check if it still exists every cycle
	uint16 attached_unit_id;
	
	WarBody *look_at_unit; // is NULL if not looked at
	// the id of the unit the camera is attached to, we neede it since we don't know
	// when the Body is deleted, we have to check if it still exists every cycle
	uint16 look_at_unit_id;
};

// inline definitions

inline void WarCamera::AddScrollSpeed(float horiz,float vert)
{
	horizontal_move_speed += horiz;
	vertical_move_speed += vert;
}

inline void WarCamera::AddMovementSpeed(WarVertex speed)
{
	movement_speed += speed;
}

inline void WarCamera::AddRotationSpeed(WarVertex speed)
{
	rotation_speed += speed;
//	printf("rotation_speed: (%f %f %f)\n",rotation_speed.x,rotation_speed.y,rotation_speed.z);
}

inline void WarCamera::SetFov(float field_of_vision)
{
	if (field_of_vision > 179.0) fov=179.0f;
	else fov = field_of_vision;
}

inline float WarCamera::Fov()
{
	return fov;
}

#endif // #ifndef _WAR_CAMERA_