#ifndef _WAR_DISPLAY_
#define _WAR_DISPLAY_

#include "WarView.h"
#include "WarMap.h"
#include "WarVertex.h"
#include "WarObject.h"
#include "WarEntityList.h"
#include "WarCamera.h"
#include "WarPolygon.h"

#include <gl.h>

#include <Application.h>
#include <GraphicsCard.h>
#include <List.h>
#include <Point.h>

class WarDisplay: public WarView
{
public:
	WarDisplay(int32 left,int32 top,uint16 width,uint16 height,WarMap *map,WarEntityList *elist,WarCamera *camera);
	virtual ~WarDisplay();

	void ChangeCamera(WarCamera *new_eyes);

//	virtual void Resize(int32 left,int32 top,uint16 width,uint16 height);

	virtual void MouseDown(BPoint point,uint8 button);
	virtual void MouseUp(BPoint point,uint8 button);
	virtual void MouseMoved(BPoint point);
	virtual void Update();
/*
	virtual void KeyDown(char key);
	virtual void KeyUp(char key);
*/

//	void Cycle(float frame_time);

	void BeginMouseLook();
	void EndMouseLook();

	void BeginMouseSelection();
	void EndMouseSelection();

	void SetFarClip(float clip_dist);
	float FarClip();
	
protected:
private:
	float abs(float val);

	void ProjectViewVolume(WarPolygon *polygon);

	void Scanline(int line,int start,int end,WarMap *war_map,WarTileset *tiles,WarEntityList *war_entity_list);
	void ParallelTrapeze(float y1,float y2,float x1,float x2,float x3,float x4,WarMap *war_map,WarTileset *tiles,WarEntityList *war_entity_list);
	void DrawTriangle(BPoint point1,BPoint point2,BPoint point3,WarMap *war_map,WarTileset *tiles,WarEntityList *war_entity_list);
	void DrawPoly(WarPolygon *polygon,WarMap *war_map,WarTileset *tiles,WarEntityList *war_entity_list);

	void DrawMap(BPoint points[4],WarMap *war_map,WarTileset *tiles,WarEntityList *war_entity_list);

	WarCamera *camera;

	// desribes the scene to draw
	WarMap *war_map;
	WarEntityList *war_entity_list;

	bool mouse_selection; // is true while dragging a selectionbox
	bool mouse_look; // is true while mouselook is active
	
	BPoint mouse_position;
	BPoint orig_mouse_position;
	
	float far_z_clip;
};

inline void WarDisplay::ChangeCamera(WarCamera *new_eyes)
{
	camera = new_eyes;
}

inline void WarDisplay::BeginMouseLook()
{
//	printf("***********\nBeginMouseLook()\n********\n\n");
	be_app->HideCursor();
	orig_mouse_position = mouse_position;
	SetMousePosition(BPoint(Width()>>1,Height()>>1));
	mouse_look = true;
}

inline void WarDisplay::EndMouseLook()
{
	mouse_look = false;
	SetMousePosition(orig_mouse_position);
	be_app->ShowCursor();
}

inline void WarDisplay::BeginMouseSelection()
{
	camera->Lock();
	orig_mouse_position = mouse_position;
	mouse_selection = true;
}

inline void WarDisplay::EndMouseSelection()
{
	mouse_selection = false;
	camera->Unlock();
}

inline float WarDisplay::abs(float val)
{
	return (val<0.0)?-val:val;
}

inline void WarDisplay::SetFarClip(float clip_dist)
{
	if (clip_dist < 5.0) return;
	else far_z_clip = clip_dist;
}

inline float WarDisplay::FarClip()
{
	return far_z_clip;
}

#endif // #ifndef _WAR_DISPLAY_