#ifndef OpenGL_View_h
#define OpenGL_View_h

#include <opengl/GLView.h>

#include "WarView.h"
#include "WarDisplay.h"
#include "WarCli.h"
#include "WarTextOutput.h"
#include "WarList.h"

class WarGLView : public BGLView
{
public:
	WarGLView( BRect frame, const char *name, int32 mode, int32 flags );
	virtual ~WarGLView( void );

	// Override BView methods:
	virtual void AttachedToWindow();
	virtual void DetachedFromWindow();
	virtual void FrameResized( float width, float height );
	virtual void KeyDown( const char *bytes, int32 size );
	virtual void KeyUp( const char *bytes, int32 size );
	virtual void MouseDown(BPoint point);
	virtual void MouseUp(BPoint point);
	virtual void MouseMoved(BPoint point,uint32 transit,const BMessage *message);

	// Override BGLView method:
	virtual void ErrorCallback( GLenum err );

	void AddChild(WarView *view);

	void Render();
	void Cycle(float frame_time);

	void SetMousePosition(BPoint position);

	// takes a line, parses it and executes every command
	void Parse(char *line,bool quiet=false);
private:
	// executes one word (one command)
	void execute(char *word,bool quiet=false);

	// my own version of atoi, it supports negative numbers and returns
	// false if the string contains anything else than digits.
	bool atoi(int32 *val,char *string);
	
	// a support function to split a line in words, used by the parser
	char *GetWord(char *line,char **cookie);

	// the interface to the stack
	bool push(int32 val);
	bool pop(int32 *val);

	bool pop_string(char *string,uint16 len);

	static bool update_child(void *child);
//	static bool mouse_down(void *child);
//	static bool mouse_up(void *child);
	static long main_task(void *data);
	
	uint32 mouse_buttons; // the mouse buttons currently pressed

	bool running;
	thread_id main_thread;

	int32 *stack;
	uint16 stack_pointer;

	BDirectory *app_dir; // the applications directory
	BDirectory *script_dir; // the script directory
	BDirectory *map_dir; // the maps directory
	BDirectory *environment_dir; // the environment directory
	BDirectory *unit_dir; // the unit directory
	
	WarList *word_list;

	WarDisplay *war_display;
	WarCli *war_cli;
	WarTextOutput *war_text_output;

// the cameras
	WarCamera *current_camera;
	WarCamera *camera; 
	
	WarMap *war_map;
	WarEntityList *war_entity_list;

	uint8 team;

	BList *childs;

	bool keyboard_state[130];
	
	bool is_typing; // is tru while the user is typing text into a cli
	WarCli *current_focus; // points to the current focus
};


enum word_id
{
	WORD_PEEK_STACK=0,
	WORD_ADD,
	WORD_SUB,
	WORD_MUL,
	WORD_DIV,
	WORD_AND,
	WORD_OR,
	WORD_XOR,
	WORD_MODULO,
	WORD_POP_DEC,
	WORD_POP_TEXT,
	WORD_DROP,
	WORD_INC,
	WORD_DEC,
	WORD_DUP,
	WORD_DEPTH,
	WORD_EQUALS,
	WORD_LESS,
	WORD_GREATER,
	WORD_NOT_EQUAL,
	WORD_CLEAR,
	WORD_WORDS,
	WORD_EXEC,
	WORD_SET_CURSOR,
	WORD_CURSOR,

	WORD_COMPILE,
	WORD_FORGET,
	WORD_IF,
	WORD_ELSE,
	WORD_ENDIF,

// game control

	WORD_GAME_DESELECT_UNITS,
	WORD_GAME_ASSIGN_GROUP,
	WORD_GAME_SELECT_GROUP,
	WORD_GAME_DESELECT_GROUP,
	WORD_GAME_LEAVE_GROUPS,
// camera operations
	WORD_GAME_CAMERA_ATTACH,
	WORD_GAME_CAMERA_DETACH,
	WORD_GAME_CAMERA_LOOK_AT,
	WORD_GAME_CAMERA_STOP_LOOK_AT,
	WORD_GAME_CAMERA_ADD_MOVE_SPEED,
	WORD_GAME_CAMERA_ADD_ROTATION_SPEED,
	WORD_GAME_CAMERA_ADD_SCROLL_SPEED,
	WORD_GAME_CAMERA_ROTATE_BY,
	WORD_GAME_CAMERA_MOVE_BY,
	WORD_GAME_CAMERA_SET_FOV,
	WORD_GAME_CAMERA_FOV,
	WORD_GAME_CAMERA_MOVE,

	WORD_GAME_ACTIVATE_CONSOLE,

	WORD_GAME_SET_VIDEO_MODE,
	WORD_GAME_VIDEO_MODE,
	WORD_GAME_CONSOLE_HEIGHT,
	WORD_GAME_SET_CONSOLE_HEIGHT,
	WORD_GAME_SAY,
	WORD_GAME_QUIT,
	WORD_GAME_SET_MAP_MODE,
	WORD_GAME_MAP_MODE,
	// display operations
	WORD_GAME_DISPLAY_SWITCH_CAMERA,
	WORD_GAME_DISPLAY_BEGIN_MOUSE_SCROLL,
	WORD_GAME_DISPLAY_END_MOUSE_SCROLL,
	WORD_GAME_DISPLAY_BEGIN_MOUSE_LOOK,
	WORD_GAME_DISPLAY_END_MOUSE_LOOK,
	WORD_GAME_DISPLAY_BEGIN_MOUSE_SELECTION,
	WORD_GAME_DISPLAY_END_MOUSE_SELECTION,
	WORD_GAME_DISPLAY_SET_FAR_CLIP,
	WORD_GAME_DISPLAY_FAR_CLIP,
};

static char *words[]=
{
// primitives
	".s", // prints the stack content
	"+", // add
	"-", // sub
	"*", // multiplier
	"/", // divide
	"&", // bitwise AND
	"|", // bitwise OR
	"^", // bitwise XOR
	"%", // modulo
	".", // pops one entry from the stack and displays it
	".t", // displays every entry on the stack as an character, until NULL (prints a string from the stack)
	"drop", // removes the last entry on the stack
	"inc", // increases the last stack entry
	"dec", // decreases the last stack entry
	"dup", // duplicates the last stack entry
	"depth", // returns the depth of the stack, # stack entries
	"=", // equal to
	"<", // less than
	">", // greater than
	"!=", // not equal to
	"clear", // clears the stack
	"words", // lists the available words
	"exec",  // executes the given file
	"set_cursor", // sets the cursor type
	"cursor", // returns the cursor type

// these are never executed (except for endif, but it shouldn't do anyting)
	"compile", // define a new word
	"forget", // undefines a user defined word
	"if",
	"else",
	"endif",

// words to control the game

	"deselect_units", // deselects all units
	"group.assign", // assigns every selected unit to the given group
	"group.select", // selects only units that is a part of the give group
	"group.deselect", // selects only units that is a part of the give group
	"group.leave", // every selected unit will leave it's group

// camera operations
	"camera.attach", // attaches the camera to the given unit_id
	"camera.detach", // detaches the camera from the unit_id the camera is attached to
	"camera.look_at", // makes the camera look at a specified unit
	"camera.stop_look_at", // makes the camera stop looking at the unit the camera currently is locked on
	"camera.add_move_speed", // makes the camera move in the given direction in the given speed
	"camera.add_rotation_speed", // makes the camera rotate with the given speed
	"camera.add_scroll_speed", // scrolls the current display the given steps per second
	"camera.rotate_by", // makes the camera rotate
	"camera.move_by", // makes the camera rotate
	"camera.set_fov", // takes one parameter, sets the field of vision
	"camera.fov", // pushes the field of vision on the stack
	"camera.move", // takes x,y,z,time

	"activate_console", // throws up the console

	"set_video_mode", // sets the video mode
	"video_mode", // returns the current video mode
	"console_height", // returns the height of the console in pixels
	"set_console_height", // resizes the console
	"say", // sends the string from the stack to the server
	"quit", // quits the game
	"set_map_mode", // changes the minimap drawmode depending on the last stackentry
	"map_mode", // returns the curren map_mode

// display operations
	"display.switch_camera", // switches camera
	"display.begin_mouse_scroll",
	"display.end_mouse_scroll",
	"display.begin_mouse_look",
	"display.end_mouse_look",
	"display.begin_mouse_selection",
	"display.end_mouse_selection",
	"display.set_far_clip",
	"display.far_clip",
};

const uint16 num_words=sizeof(words)/4;
const uint16 stack_size=1024;

#endif
