#include "WarCamera.h"
#include "WarGLView.h"
#include "WarMap.h"
#include "defs.h"

#include <TranslationUtils.h>
#include <List.h>
#include <stdlib.h>
#include <math.h>
#include <gl.h>
#include <glu.h>

WarCamera::WarCamera():WarMatrix()
{
	attached_unit=NULL;
	look_at_unit=NULL;
	fov=90.0;
	time_left = 0.0;
	vertical_move_speed = 0.0;
	horizontal_move_speed = 0.0;
}

WarCamera::WarCamera(WarVertex camera_position):WarMatrix	(camera_position)
{
	attached_unit=NULL;
	look_at_unit=NULL;
	fov=90.0;
	time_left = 0.0;
	vertical_move_speed = 0.0;
	horizontal_move_speed = 0.0;
}

WarCamera::~WarCamera()
{
}

void WarCamera::Scroll(float horiz,float vertical)
{
	if (horiz)
	{
		position.x+=sin(direction.z)*horiz;
		position.y-=cos(direction.z)*horiz;
	}

	if (vertical)
	{
		position.x+=cos(direction.z)*vertical;
		position.y+=sin(direction.z)*vertical;
	}
}

void WarCamera::AttachToUnit(WarBody *unit)
{
	attached_unit = unit;

	if (attached_unit != NULL)
	{
		MoveTo(WarVertex(attached_unit->XPosition(),attached_unit->YPosition(),attached_unit->ZPosition()));
		RotateTo(WarVertex(attached_unit->XDirection(),attached_unit->YDirection(),attached_unit->ZDirection()));

		attached_unit_id = attached_unit->Id();
	}
}

void WarCamera::DetachFromUnit()
{
	attached_unit = NULL;
}

void WarCamera::LookAtUnit(WarBody *unit)
{
	look_at_unit = unit;

	if (look_at_unit != NULL)
	{
		float dist_x = Position().x - look_at_unit->XPosition();
		float dist_y = Position().y - look_at_unit->YPosition();
		float dist_z = Position().z - look_at_unit->ZPosition();

		RotateZTo(atan2(-dist_y,-dist_x));
		RotateYTo(atan2(dist_z,sqrt(dist_y*dist_y + dist_x*dist_x)));
		
		look_at_unit_id = look_at_unit->Id();
	}
}

void WarCamera::StopLookAt()
{
	look_at_unit = NULL;
}

void WarCamera::Cycle(float frame_time,WarEntityList *war_entity_list,WarMap *war_map)
{
	if (!IsLocked())
	{
		if (time_left)
		{
			if (time_left<frame_time)
			{
				position=camera_goal;
				time_left=0;
			}
			else
			{
				WarVertex tmp=(camera_goal-position)*((float)(frame_time/time_left));
				position += tmp;
				time_left-=frame_time;
			}
		}

		if (horizontal_move_speed || vertical_move_speed)
			Scroll(horizontal_move_speed * frame_time,vertical_move_speed * frame_time);

		if (movement_speed != WarVertex(0.0,0.0,0.0))
			MoveBy(movement_speed * frame_time);

		if (rotation_speed != WarVertex(0.0,0.0,0.0))
			RotateBy(rotation_speed * frame_time);

		if (attached_unit != NULL && war_entity_list->GetBodyByID(attached_unit_id) == attached_unit)
		{
			MoveTo(WarVertex(attached_unit->XPosition(),attached_unit->YPosition(),attached_unit->ZPosition()));
			RotateTo(WarVertex(attached_unit->XDirection(),attached_unit->YDirection(),attached_unit->ZDirection()));
		}

		if (look_at_unit != NULL && war_entity_list->GetBodyByID(look_at_unit_id) == look_at_unit)
		{
			float dist_x = look_at_unit->XPosition() - Position().x;
			float dist_y = look_at_unit->YPosition() - Position().y;
			float dist_z = look_at_unit->ZPosition() - Position().z;

			RotateZTo(atan2(dist_y,dist_x));
			RotateYTo(atan2(-dist_z,sqrt(dist_y*dist_y + dist_x*dist_x)));
			// dist_z have to be negative because the y-angles are positive downwards
		}
	}
}
