#include <TranslationUtils.h>
#include <Bitmap.h>
#include <stdlib.h>
#include <gl.h>
#include <glu.h>
#include <string.h>
#include <File.h>
#include <Application.h>

#include "WarCli.h"
#include "WarObject.h"

WarCli::WarCli(int32 left,int32 top,int32 width,int32 height,WarFont *font,WarList *command_list):WarView(left,top,width,height)
{
	war_font = font;
	word_list = command_list;

	*prompt = 0;
	command_line = new char[512];
	*command_line = 0;
	command_len = 0;
	scroll_offset = 0;
	cursor_position = 0;
	
	cursor_type = 2;
	
	for (uint16 i = 0; i < 256; i++) history[i] = NULL;
	history_position = 0;
	history_end = 0;

	speed = (1<<20)-1;

	strcpy(prompt,"0 > ");
	prompt_len=strlen(prompt);
}

WarCli::~WarCli()
{
	delete []command_line;
	uint16 i;
	for (i=0;i<256;i++)
		if (history[i]) delete []history[i];
}

void WarCli::Resize(int32 width,int32 height)
{
	// maybe the bounds have been greater so we don't need to scroll the commandline anymore.
	if ((prompt_len+command_len+2)*war_font->Width()>Width())
		scroll_offset = ((prompt_len+command_len+2)*war_font->Width() - Width())/war_font->Width();
	else
		scroll_offset = 0;

	WarView::Resize(width,height);
}

void WarCli::Update()
{
// initialize
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho(0.0,Width(),0.0,Height(),0.0,100.0);
	glMatrixMode( GL_MODELVIEW );
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glDisable( GL_DEPTH_TEST );
	glColor3f(1.0,1.0,1.0);
	glLoadIdentity();

// draw the prompt
	war_font->DrawString(0,0,prompt);

// draw the command_line
	war_font->DrawString(prompt_len*war_font->Width(),0,command_line+scroll_offset);

	// should the cursor be drawn?
	if (system_time() % speed < speed/2)
	{
		glDisable(GL_TEXTURE_2D);

		float x=(prompt_len+cursor_position-scroll_offset)*war_font->Width();
		float y=0.0;

	// Draw the cursor
		glBegin(GL_QUADS);

		switch (cursor_type)
		{
			case 0: // BLOCK
			glVertex2f(x,y);
			glVertex2f(x,y+war_font->Height());
			glVertex2f(x+war_font->Width(),y+war_font->Height());
			glVertex2f(x+war_font->Width(),y);
			break;
			
			case 1: // UNDERSCORE
			glVertex2f(x,y);
			glVertex2f(x,y+(war_font->Height()>>2));
			glVertex2f(x+war_font->Width(),y+(war_font->Height()>>2));
			glVertex2f(x+war_font->Width(),y);
			break;

			case 2: // PIPE
			glVertex2f(x,y);
			glVertex2f(x,y+war_font->Height());
			glVertex2f(x+(war_font->Width()>>2),y+war_font->Height());
			glVertex2f(x+(war_font->Width()>>2),y);
			break;
		}

		glEnd();
	}
	glDisable(GL_BLEND);
}

void WarCli::GetCommandLine(char *buffer,uint32 size)
{
	strncpy(buffer,command_line,size);
}

void WarCli::GetPrompt(char *buffer,uint32 size)
{
	strncpy(buffer,prompt,size);
}

void WarCli::SetPrompt(char *buffer)
{
	strncpy(prompt,buffer,30);
	prompt_len=strlen(prompt);
}

void WarCli::Reset()
{
	if (history[history_end]) delete []history[history_end];
	history[history_end]=new char[command_len+1];

	strncpy(history[history_end],command_line,command_len+1);
	history_end++;
	history_position=history_end;

	cursor_position = 0;
	command_len = 0;
	*command_line = 0;
	scroll_offset = 0;
}

void WarCli::KeyDown(char key)
{

//	printf("key: %d\n",key);

// *********** LEFT

	if (key==28)
	{
		if (cursor_position > 0) cursor_position--;

		if ((prompt_len+cursor_position+2)*war_font->Width()>Width())
			scroll_offset = ((prompt_len+cursor_position+2)*war_font->Width() - Width())/war_font->Width();
		else
			scroll_offset = 0;
	}

// *********** RIGHT

	else if (key==29)
	{
		if (cursor_position < command_len) cursor_position++;

		if ((prompt_len+cursor_position+2)*war_font->Width()>Width())
			scroll_offset = ((prompt_len+cursor_position+2)*war_font->Width() - Width())/war_font->Width();
		else
			scroll_offset = 0;
	}

// *********** UP

	else if (key==30)
	{
		history_position--;
		if (!history[history_position])
		{
			history_position++;
			return;
		}
		strncpy(command_line,history[history_position],512);
		command_len=strlen(command_line);
		cursor_position = command_len;

		if ((prompt_len+cursor_position+2)*war_font->Width()>Width())
			scroll_offset = ((prompt_len+cursor_position+2)*war_font->Width() - Width())/war_font->Width();
		else
			scroll_offset = 0;
	}

// ************* DOWN

	else if (key==31 && history[history_position])
	{
		history_position++;
		if (!history[history_position])
		{
			*command_line=0;
			command_len=0;
			cursor_position = 0;
			scroll_offset=0;
			return;
		}
		strncpy(command_line,history[history_position],512);
		command_len=strlen(command_line);
		cursor_position = command_len;

		if ((prompt_len+cursor_position+2)*war_font->Width()>Width())
			scroll_offset = ((prompt_len+cursor_position+2)*war_font->Width() - Width())/war_font->Width();
		else
			scroll_offset = 0;
	}

// ********* BACKSPACE

	else if (key==8 && cursor_position > 0)
	{
		// remove the character left to the cursor
		for (int32 i = cursor_position - 1; i < command_len; i++)
			command_line[i] = command_line[i+1];

		// move the cursor and make the line shorter
		command_len--;
		cursor_position--;

		// check if we maybe should scroll the text a bit
		if ((prompt_len+cursor_position+2)*war_font->Width()>Width())
			scroll_offset = ((prompt_len+cursor_position+2)*war_font->Width() - Width())/war_font->Width();
		else
			scroll_offset = 0;
	}

// ********* DELETE

	else if (key==127 && command_len > cursor_position)
	{
		// remove the character left to the cursor
		for (int32 i = cursor_position ; i < command_len; i++)
			command_line[i] = command_line[i+1];

		// make the line shorter
		command_len--;
	}

// ************ TAB

	else if (key==9) // tab
	{
		if (command_line[command_len-1] == ' ') return;
		uint32 num_words=word_list->CountItems();
		uint16 *matched_words=new uint16[num_words];
		uint16 index=0;
		memset(matched_words,-1,num_words);
		
		uint16 len=strlen(command_line);
		int16 i=len;

		char *word;
		while (i>=0 && command_line[i]!=' ')
			i--;
		i++;
	
		word=command_line+i;
		len=strlen(word);

		index=0;

		uint16 k=0;
		char *defined_word;
		bool is_word_defined;
	
		while (is_word_defined=word_list->GetNextItem((void**)&defined_word,&k))
			if (!strncmp(word,defined_word,len)) matched_words[index++]=k-1;
			
		if (index==1)
		{
			if (strlen((char*)word_list->ItemAt(matched_words[0]))>len) strcat(command_line,(char*)word_list->ItemAt(matched_words[0])+len);
			strcat(command_line," ");
			command_len=strlen(command_line);
		}
		else if (index>1)
		{
			uint16 min_matched_len = (uint16)-1;

			for (i=1;i<index;i++)
			{
				uint16 j=0;
				while (((char*)word_list->ItemAt(matched_words[i]))[j] && ((char*)word_list->ItemAt(matched_words[0]))[j] && ((char*)word_list->ItemAt(matched_words[0]))[j]==((char*)word_list->ItemAt(matched_words[i]))[j])
					j++;
				if (j<min_matched_len) min_matched_len=j;
			}
			
			if (min_matched_len>1)
			{
				strncat(command_line,(char*)word_list->ItemAt(matched_words[0])+len,min_matched_len-len);
				command_len=strlen(command_line);
			}

			char tmp[256];
			tmp[0]=0;
			for (i=0;i<index;i++)
			{
				strcat(tmp,(char*)word_list->ItemAt(matched_words[i]));
				strcat(tmp," ");
			}
		}
		delete matched_words;
		cursor_position = command_len;

		if ((prompt_len+command_len+2)*war_font->Width()>Width())
			scroll_offset = ((prompt_len+command_len+2)*war_font->Width() - Width())/war_font->Width();
		else
			scroll_offset = 0;

	}

// ********* ANY OTHER KEY

	else if (key > 31 && key < 127 && command_len < 511)
	{
		// insert a character
		for (int32 i = command_len; i >= cursor_position; i--)
			command_line[i+1] = command_line[i];

		// copy the character
		command_line[cursor_position]=key;
		
		// move the cursor and make the line longer
		command_len++;
		cursor_position++;

		// check if we maybe should scroll the text a bit
		if ((prompt_len+cursor_position+2)*war_font->Width()>Width())
			scroll_offset = ((prompt_len+cursor_position+2)*war_font->Width() - Width())/war_font->Width();
		else
			scroll_offset = 0;
	}
}
