#include "WarDisplay.h"
#include "WarGLView.h"
#include "WarMap.h"
#include "WarPolygon.h"
#include "defs.h"

#include <TranslationUtils.h>
#include <List.h>
#include <stdlib.h>
#include <math.h>
#include <gl.h>
#include <glu.h>

#define WIREFRAME

_EXPORT WarDisplay::WarDisplay(int32 left,int32 top,uint16 width,uint16 height,WarMap *map,WarEntityList *elist,WarCamera *eyes):
	WarView(left,top,width,height)
{
	war_map = map;
	camera = eyes;
	war_entity_list=elist;
	mouse_look = false;
	mouse_selection = false;
	
	far_z_clip = 40.0;

//	GLfloat LightDiffuse[]=         { 250.0f, 250.0f, 100.0f, 1.0f };
	GLfloat DirectionalDiffuse[]=        { .5f, .5f, .5f, .5f };

	// Define material properties of specular color and degree of
	// shininess.  Since this is only done once in this particular
	// example, it applies to all objects.  Material properties can
	// be set for individual objects, individual faces of the objects,
	// individual vertices of the faces, etc...
/*
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);
	glLightf(GL_LIGHT1, GL_QUADRATIC_ATTENUATION, 2.0);
	glLightf(GL_LIGHT1, GL_LINEAR_ATTENUATION, 50.0);
*/
	glLightfv(GL_LIGHT0, GL_DIFFUSE, DirectionalDiffuse);

	// Set the GL_AMBIENT_AND_DIFFUSE color state variable to be the 
	// one referred to by all following calls to glColor 
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE); 
	glEnable(GL_COLOR_MATERIAL);

	// Create a Directional Light Source 
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
//	glEnable(GL_LIGHT1);
}

_EXPORT WarDisplay::~WarDisplay()
{
}

/*
_EXPORT void WarDisplay::SetMousePosition(uint16 x,uint16 y)
{
	gl_view->SetMousePosition(x+Left(),y+Top());
}
*/
void WarDisplay::MouseDown(BPoint point,uint8 buttons)
{
	if (buttons&1) gl_view->Parse("+mouse1",true);
	if (buttons&2) gl_view->Parse("+mouse2",true);
	if (buttons&4) gl_view->Parse("+mouse3",true);
}

void WarDisplay::MouseUp(BPoint point,uint8 buttons)
{
	if (buttons&1) gl_view->Parse("-mouse1",true);
	if (buttons&2) gl_view->Parse("-mouse2",true);
	if (buttons&4) gl_view->Parse("-mouse3",true);
}

_EXPORT void WarDisplay::MouseMoved(BPoint point)
{
//	printf("mouse_position: (%f,%f)\n",point.x,point.y);
	if (mouse_look && point != BPoint(Width()>>1,Height()>>1))
	{
//		printf("middle: (%f,%f)\n",(float)(Width()>>1),(float)(Height()>>1));
		camera->RotateYBy(-(mouse_position.y - point.y)/500.0);
		camera->RotateZBy((mouse_position.x - point.x)/500.0);
		SetMousePosition(BPoint(Width()>>1,Height()>>1));
		mouse_position = BPoint(Width()>>1,Height()>>1);
	}
	else mouse_position = point;
}
/*
_EXPORT void WarDisplay::KeyDown(char key)
{
}

_EXPORT void WarDisplay::KeyUp(char key)
{
}
*/

_EXPORT void WarDisplay::Update()
{
// **************** WarPolygonTest *******************'
/*
	WarPolygon poly;
	
	poly.AddPoint(BPoint(10.0,15.0));
	poly.AddPoint(BPoint(40.0,10.0));
	poly.AddPoint(BPoint(45.0,40.0));
	poly.AddPoint(BPoint(25.0,50.0));

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho(0.0,Width(),0.0,Height(),0.0,100.0);
	glMatrixMode( GL_MODELVIEW );

	glDisable( GL_DEPTH_TEST );
	glLoadIdentity();

	glDisable(GL_TEXTURE_2D);

	if (poly.Contains(mouse_position)) glColor3f(1.0,0.0,0.0);
	else glColor3f(1.0,1.0,1.0);

	glBegin(GL_LINE_STRIP);
	for (uint16 i=0;i<poly.CountPoints();i++)
	{
		BPoint point = poly.PointAt(i);
		glVertex2f(point.x,point.y);
	}
	BPoint point = poly.PointAt(0);
	glVertex2f(point.x,point.y);
	glEnd();

	poly.ExpandBy(mouse_position);

	glColor3f(0.0,0.0,1.0);

	glBegin(GL_LINE_STRIP);
	for (uint16 i=0;i<poly.CountPoints();i++)
	{
		BPoint point = poly.PointAt(i);
		glVertex2f(point.x,point.y);
	}

	point = poly.PointAt(0);
	glVertex2f(point.x,point.y);
	glEnd();

	glBegin(GL_POINTS);
	glVertex2f(mouse_position.x,mouse_position.y);
	glEnd();
	
return;
*/
	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH);

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( camera->Fov(), (float)Width()/(float)Height(), 0.1, far_z_clip );
	glMatrixMode( GL_MODELVIEW );

	glLoadIdentity();

	glRotatef(90.0,0.0,0.0,1.0);
	glRotatef(90.0,0.0,1.0,0.0);

// sets the current matrix to the camera matrix
	glRotatef(-camera->Direction().x * 180.0 / 3.1415927,1.0,0.0,0.0); // tilt
	glRotatef(-camera->Direction().y * 180.0 / 3.1415927,0.0,1.0,0.0); // up-down
	glRotatef(-camera->Direction().z * 180.0 / 3.1415927,0.0,0.0,1.0); // direction

	glTranslatef(-camera->Position().x,-camera->Position().y,-camera->Position().z); // position

/*
	glRotatef(-90.0,0.0,1.0,0.0);
	glTranslatef(-10,-10,-40); // position
*/

//	printf("camera_direction: (%f,%f,%f)\n",camera->XDirection(),camera->YDirection(),camera->ZDirection());

	WarPolygon polygon;
	ProjectViewVolume(&polygon);

// sets the position of the blue light
//	GLfloat LightPosition[]={ cos(rtri)*10.0 + 20.0, sin(rtri)*10.0 + 20.0, 10.0f, 1.0f };
//	glLightfv(GL_LIGHT1, GL_POSITION, LightPosition);

// sets the normal of the directional light
	GLfloat DirectionalPosition[]=       { .5, .5, 1.0f, 0.0f };
	glLightfv(GL_LIGHT0, GL_POSITION, DirectionalPosition);


	WarTileset *tiles=war_map->Tileset();
/*
	map_points[0] = BPoint(camera->Position().x - 20.0,camera->Position().y - 20.0);
	map_points[1] = BPoint(camera->Position().x + 20.0,camera->Position().y - 20.0);
	map_points[2] = BPoint(camera->Position().x + 20.0,camera->Position().y + 20.0);
	map_points[3] = BPoint(camera->Position().x - 20.0,camera->Position().y + 20.0);
*/
// now, we render the map
	glEnable(GL_FOG); 

	GLfloat fog_color[4] = {0.0,0.0,0.0,1.0};
	glFogfv(GL_FOG_COLOR, fog_color);
	glFogi(GL_FOG_MODE, GL_LINEAR);
	glHint(GL_FOG_HINT, GL_FASTEST);

	glFogf(GL_FOG_DENSITY, 1.0);

	glFogf(GL_FOG_START, far_z_clip - 10.0);
	glFogf(GL_FOG_END, far_z_clip - 3.0);

	DrawPoly(&polygon,war_map,tiles,war_entity_list);

	// draw all the flying units
	WarBody *body;
	uint16 id,cookie=0;
	while(war_entity_list->GetNextFlyingEntity(&body,NULL,&id,&cookie))
	{
		body->Draw();
	}

	glDisable(GL_FOG);

// ***************************** DEBUG

//	glDisable(GL_DEPTH_TEST);
//	polygon.Draw();

// ****************************

	// if the selectionbox is active, then draw it
	if (mouse_selection)
	{
		glMatrixMode( GL_PROJECTION );
		glLoadIdentity();
		glOrtho(0.0,Width(),0.0,Height(),0.0,100.0);
		glMatrixMode( GL_MODELVIEW );
		glDisable( GL_DEPTH_TEST );
		glDisable( GL_TEXTURE_2D );
		glLoadIdentity();

		glColor3f(255.0,255.0,255.0);
		glBegin(GL_LINE_STRIP);
		glVertex2f(orig_mouse_position.x,Height()-orig_mouse_position.y);
		glVertex2f(orig_mouse_position.x,Height()-mouse_position.y);
		glVertex2f(mouse_position.x,Height()-mouse_position.y);
		glVertex2f(mouse_position.x,Height()-orig_mouse_position.y);
		glVertex2f(orig_mouse_position.x,Height()-orig_mouse_position.y);
		glEnd();
	}
}

void WarDisplay::ProjectViewVolume(WarPolygon *polygon)
{
	if (camera->ZPosition() > far_z_clip) return;

	float half_fov = camera->Fov()/2.0 /180.0 * 3.1415927;

	float h = atan2(Width(),Height() / tan(half_fov)),
			v = half_fov;

	WarVertex line[4];
	WarVertex line2[4];

	WarVertex direction[4];
	float d[4];

// set up the screen projection lines
	line[0].SetToAngle(v,-h);
	line[1].SetToAngle(-v,-h);
	line[2].SetToAngle(-v,h);
	line[3].SetToAngle(v,h);

	direction[0].SetTo(0.0,0.0,-1.0);
	direction[1].SetTo(0.0,-1.0,0.0);
	direction[2].SetTo(0.0,0.0,1.0);
	direction[3].SetTo(0.0,1.0,0.0);

//	printf("camera_pos: (%f,%f,%f)\n",camera->XPosition(),camera->YPosition(),camera->ZPosition());

//	printf("top_left: (%f,%f,%f) dist: %f\n",line[0].x,line[0].y,line[0].z,sqrt(line[0].x*line[0].x + line[0].y*line[0].y + line[0].z*line[0].z));

	bool hit_the_ground[4];

	for (int i=0;i<4;i++)
	{
		line2[i]=line[i];
		line2[i].RotateBy(camera->Direction());

		d[i] = -camera->Position().z / line2[i].z;

		if (d[i] > far_z_clip/line[i].x || line2[i].z>=0.0) // if the line doesn't hit the map-plane or if it hits it too far away
			hit_the_ground[i]=false;
		else
			hit_the_ground[i]=true;
	}

	for (uint16 i=0;i<4;i++)
	{
		if (hit_the_ground[i])
		{
			polygon->AddPoint(BPoint(camera->Position().x + line2[i].x * d[i],camera->Position().y + line2[i].y * d[i]));
		}
		else
		{
			WarVertex far_point(line[i]*(far_z_clip/line[i].x + 1.0f));
			far_point.RotateBy(camera->Direction());
			far_point += (WarVertex)camera->Position();
			WarVertex map_line;

			if (hit_the_ground[(i + 3)%4])
			{
				map_line = direction[(i + 1)%4];
				map_line.RotateBy(camera->Direction());
				d[i] = -far_point.z / map_line.z;
				polygon->AddPoint(BPoint(far_point.x + map_line.x * d[i],far_point.y + map_line.y * d[i]));
			}

			if (hit_the_ground[(i + 1)%4])
			{
				map_line = direction[i];
				map_line.RotateBy(camera->Direction());
				d[i] = -far_point.z / map_line.z;
				polygon->AddPoint(BPoint(far_point.x + map_line.x * d[i],far_point.y + map_line.y * d[i]));
			}

		}
	}
	
	if (hit_the_ground[0]==false && hit_the_ground[1]==false && hit_the_ground[2]==false && hit_the_ground[3]==false)
	{
		polygon->Empty();
		return;
	}
	
	polygon->ExpandBy(BPoint(camera->XPosition(),camera->YPosition()));
}

// Draws a line of the map
void WarDisplay::Scanline(int y,int x,int end,WarMap *war_map,WarTileset *tiles,WarEntityList *war_entity_list)
{
	if (x < 0) x = 0;
	if (end > war_map->Width() - 1) end = war_map->Width() - 1;

	while (x <= end)
	{
		WarVertex *normal;

#ifndef WIREFRAME
		float uv[8];

		glBindTexture(GL_TEXTURE_2D, tiles->Tile(war_map->GetBlock(x,y),uv));
		glBegin(GL_QUADS);

		normal=war_map->GetNormal(x,y);
		glNormal3f(normal->x,normal->y,normal->z);
		glTexCoord2f(uv[0], uv[1]);
		glVertex3f(x,y,war_map->GetFloatHeight(x,y));

		normal=war_map->GetNormal(x,y + 1.0);
		glNormal3f(normal->x,normal->y,normal->z);
		glTexCoord2f(uv[2], uv[3]);
		glVertex3f(x,y + 1.0,war_map->GetFloatHeight(x, y + 1.0));

		normal=war_map->GetNormal(x + 1,y+1.0);
		glNormal3f(normal->x,normal->y,normal->z);
		glTexCoord2f(uv[4], uv[5]);
		glVertex3f(x + 1.0,y + 1.0,war_map->GetFloatHeight(x + 1,y + 1));

		normal=war_map->GetNormal(x+1,y);
		glNormal3f(normal->x,normal->y,normal->z);
		glTexCoord2f(uv[6], uv[7]);
		glVertex3f(x + 1.0,y,war_map->GetFloatHeight(x + 1,y));

		glEnd();
#else
		glColor3f(1.0,1.0,1.0);
		glDisable(GL_TEXTURE_2D);
		glBegin(GL_LINE_STRIP);

		normal=war_map->GetNormal(x,y+1);
		glNormal3f(normal->x,normal->y,normal->z);
		glVertex3f(x,y+1.0,war_map->GetFloatHeight(x,y+1));

		normal=war_map->GetNormal(x,y);
		glNormal3f(normal->x,normal->y,normal->z);
		glVertex3f(x,y,war_map->GetFloatHeight(x,y));

		normal=war_map->GetNormal(x+1,y);
		glNormal3f(normal->x,normal->y,normal->z);
		glVertex3f(x+1.0,y,war_map->GetFloatHeight(x+1,y));

		normal=war_map->GetNormal(x+1,y+1);
		glNormal3f(normal->x,normal->y,normal->z);
		glVertex3f(x+1.0,y+1.0,war_map->GetFloatHeight(x+1,y+1));

		normal=war_map->GetNormal(x,y+1);
		glNormal3f(normal->x,normal->y,normal->z);
		glVertex3f(x,y+1.0,war_map->GetFloatHeight(x,y+1));

		glEnd();
#endif // WIREFRAME

//		if (war_map->GetState(x,y)&MAPSTATE_VISIBLE)
		{
			WarBody *body=war_entity_list->GetBodyByOffs((y<<war_map->WidthShift())+x);
			if (body) body->Draw();
		}

		glEnable(GL_TEXTURE_2D);

		x++;
	}
}

/*
              y1
       x1 /------\ x2
         /        \
        /          \
    x3 /------------\ x4
             y2
*/

void WarDisplay::ParallelTrapeze(float y1,float y2,float x1,float x2,float x3,float x4,WarMap *war_map,WarTileset *tiles,WarEntityList *war_entity_list)
{
	if (y2 == y1) return;
	float add_1 = (x3 - x1)/(y2 - y1);
	float add_2 = (x4 - x2)/(y2 - y1);

	glEnable(GL_LIGHTING);
	glColor3f(1.0,1.0,1.0);

	glEnable(GL_TEXTURE_2D);

	if (y1 < 0)
	{
		x1 += (0 - y1) * add_1;
		x2 += (0 - y1) * add_2;
		y1 = 0;
	}

	if (y2 > war_map->Height() - 1.0) y2 = war_map->Height() - 1.0;

	while ( y1 < y2 )
	{
		Scanline(y1,x1,x2,war_map,tiles,war_entity_list);
		x1+=add_1;
		x2+=add_2;
		y1+=1.0;
	}
}

void WarDisplay::DrawPoly(WarPolygon *polygon,WarMap *war_map,WarTileset *tiles,WarEntityList *war_entity_list)
{
	if (polygon->CountPoints() == 0) return;
	
// initializing GL
	glEnable(GL_LIGHTING);
	glColor3f(1.0,1.0,1.0);
	glEnable(GL_TEXTURE_2D);


// initializing vertices

	float line; // the line we're drawing
	int16 counter = polygon->CountPoints() - 1;
	uint16 prev_l,prev_r,next_l,next_r;

	prev_r = prev_l = polygon->TopPointIndex();
	line = (float)((int)polygon->PointAt(prev_l).y);

	float add_l;
	float add_r;
	
	float x_l;
	float x_r;

	next_l = polygon->IterateLeft(prev_l);
	next_r = polygon->IterateRight(prev_r);

	add_l = (polygon->PointAt(next_l).x - polygon->PointAt(prev_l).x)/(polygon->PointAt(next_l).y - polygon->PointAt(prev_l).y);
	add_r = (polygon->PointAt(next_r).x - polygon->PointAt(prev_r).x)/(polygon->PointAt(next_r).y - polygon->PointAt(prev_r).y);

	x_l = polygon->PointAt(next_l).x + (line + 1.0 - polygon->PointAt(next_l).y) * add_l;
	x_r = polygon->PointAt(next_r).x + (line + 1.0 - polygon->PointAt(next_r).y) * add_r;

	while (counter > 0 && line < war_map->Height())
	{

		if (line + 1.0 >= polygon->PointAt(next_l).y && counter > 0)
		{
			x_l = polygon->PointAt(next_l).x;

			while (line + 1.0 >= polygon->PointAt(next_l).y && counter > 0)
			{
/*
				float tmp_x = 999999999.0;
				if (points[prev_l].y < line) tmp_x = points[next_l].x + (line - points[next_l].y) * add_l;
*/
				counter--;

				if (polygon->PointAt(next_l).x < x_l) x_l = polygon->PointAt(next_l).x;

				prev_l = next_l;
				next_l = polygon->IterateLeft(prev_l);
				add_l = (polygon->PointAt(next_l).x - polygon->PointAt(prev_l).x)/(polygon->PointAt(next_l).y - polygon->PointAt(prev_l).y);
/*
				if (tmp_x < x_l) x_l = tmp_x;
				tmp_x = points[next_l].x + (line + 1.0 - points[next_l].y) * add_l;
				if (counter > 0 && points[next_l].y > line && tmp_x < x_l) x_l = tmp_x;
				x_l -= 1.0;
*/
			}
		}
		if (line + 1.0 >= polygon->PointAt(next_r).y && counter > 0)
		{
			x_r = polygon->PointAt(next_r).x;
			while (line + 1.0 >= polygon->PointAt(next_r).y && counter > 0)
			{
/*
				float tmp_x = 0.0;
				if (points[prev_r].y < line) tmp_x = points[next_r].x + (line - points[next_r].y) * add_r;
*/
				counter--;

				if (polygon->PointAt(next_r).x > x_r) x_r = polygon->PointAt(next_r).x;
			
				prev_r = next_r;
				next_r = polygon->IterateRight(prev_r);
				add_r = (polygon->PointAt(next_r).x - polygon->PointAt(prev_r).x)/(polygon->PointAt(next_r).y - polygon->PointAt(prev_r).y);
			
/*
				if (tmp_x > x_r) x_r = tmp_x;
				tmp_x = points[next_r].x + (line + 1.0 - points[next_r].y) * add_r;
				if (counter > 0 && points[next_r].y > line && tmp_x > x_r) x_r = tmp_x;
				x_r += 1.0;
*/
			}
		}

		if (line >=0.0) Scanline(line,x_l,x_r,war_map,tiles,war_entity_list);

//		Scanline(line,x_l,x_l,war_map,tiles,war_entity_list);
//		Scanline(line,x_r,x_r,war_map,tiles,war_entity_list);

		x_l += add_l;
		x_r += add_r;
		line += 1.0;
	}
}
/*
void WarDisplay::DrawMap(BPoint map_points[4],WarMap *war_map,WarTileset *tiles,WarEntityList *war_entity_list)
{
	float top_x=map_points[0].x,top_y=map_points[0].y;
	uint16 top_index=0;
	//sort by y hardcoded bub-sort
	for (uint16 i=1;i<4;i++)
	{
		if (map_points[i].y < top_y)
		{
			top_y = map_points[i].y;
			top_x = map_points[i].y;
			top_index = i;
		}
	}

	float m;

	BPoint points[6];
	points[0]=BPoint(map_points[top_index].x,map_points[top_index].y);
*/
/*
	    0
       /\
      /  \
     /    \
  1 /------/ 2
   /      /
3 /------/ 4
  \     /
   \   /
    \ /
     5
*/
/*
	if (map_points[index_1].y > map_points[index_2].y)
	{
		m = (map_points[index_1].x - map_points[top_index].x) / (map_points[index_1].y - map_points[top_index].y) * (map_points[index_2].y - map_points[top_index].y) + map_points[top_index].x;
		if (m < map_points[index_2].x)
		{
			points[1] = BPoint(m,map_points[index_2].y);
			points[2] = BPoint(map_points[index_2].x,map_points[index_2].y);
		}
		else
		{
			points[1] = BPoint(map_points[index_2].x,map_points[index_2].y);
			points[2] = BPoint(m,map_points[index_2].y);
		}


		if (map_points[index_1].y > map_points[index_3].y)
		{
			m = (map_points[index_1].x - map_points[top_index].x) / (map_points[index_1].y - map_points[top_index].y) * (map_points[index_3].y - map_points[top_index].y) + map_points[top_index].x;

			if (m < map_points[index_3].x)
			{
				points[3] = BPoint(m,map_points[index_3].y);
				points[4] = BPoint(map_points[index_3].x,map_points[index_3].y);
			}
			else
			{
				points[3] = BPoint(map_points[index_3].x,map_points[index_3].y);
				points[4] = BPoint(m,map_points[index_3].y);
			}
			points[5] = BPoint(map_points[index_1].x,map_points[index_1].y);
		}
		else
		{
			m = (map_points[index_3].x - map_points[index_2].x) / (map_points[index_3].y - map_points[index_2].y) * (map_points[index_1].y - map_points[index_2].y) + map_points[index_2].x;
			if (m < map_points[index_1].x)
			{
				points[3] = BPoint(m,map_points[index_1].y);
				points[4] = BPoint(map_points[index_1].x,map_points[index_1].y);
			}
			else
			{
				points[3] = BPoint(map_points[index_1].x,map_points[index_1].y);
				points[4] = BPoint(m,map_points[index_1].y);
			}
			points[5] = BPoint(map_points[index_3].x,map_points[index_3].y);
		}
		
	}
	else
	{
		m = (map_points[index_2].x - map_points[top_index].x) / (map_points[index_2].y - map_points[top_index].y) * (map_points[index_1].y - map_points[top_index].y) + map_points[top_index].x;

		if (m < map_points[index_1].x)
		{
			points[1] = BPoint(m,map_points[index_1].y);
			points[2] = BPoint(map_points[index_1].x,map_points[index_1].y);
		}
		else
		{
			points[1] = BPoint(map_points[index_1].x,map_points[index_1].y);
			points[2] = BPoint(m,map_points[index_1].y);
		}

		if (map_points[index_2].y > map_points[index_3].y)
		{
			m = (map_points[index_2].x - map_points[top_index].x) / (map_points[index_2].y - map_points[top_index].y) * (map_points[index_3].y - map_points[top_index].y) + map_points[top_index].x;

			if (m < map_points[index_3].x)
			{
				points[3] = BPoint(m,map_points[index_3].y);
				points[4] = BPoint(map_points[index_3].x,map_points[index_3].y);
			}
			else
			{
				points[3] = BPoint(map_points[index_3].x,map_points[index_3].y);
				points[4] = BPoint(m,map_points[index_3].y);
			}
			points[5] = BPoint(map_points[index_2].x,map_points[index_2].y);
		}
		else
		{
			m = (map_points[index_3].x - map_points[index_1].x) / (map_points[index_3].y - map_points[index_1].y) * (map_points[index_2].y - map_points[index_1].y) + map_points[index_1].x;
			if (m < map_points[index_2].x)
			{
				points[3] = BPoint(m,map_points[index_2].y);
				points[4] = BPoint(map_points[index_2].x,map_points[index_2].y);
			}
			else
			{
				points[3] = BPoint(map_points[index_2].x,map_points[index_2].y);
				points[4] = BPoint(m,map_points[index_2].y);
			}
			points[5] = BPoint(map_points[index_3].x,map_points[index_3].y);
		}
	}

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);

	ParallelTrapeze(points[0].y,points[1].y,points[0].x,points[0].x,points[1].x,points[2].x,war_map,tiles,war_entity_list);
	ParallelTrapeze(points[1].y,points[3].y,points[1].x,points[2].x,points[3].x,points[4].x,war_map,tiles,war_entity_list);
	ParallelTrapeze(points[3].y,points[5].y,points[3].x,points[4].x,points[5].x,points[5].x,war_map,tiles,war_entity_list);

	// now we have to draw the triangles
	DrawTriangle(map_points[0],map_points[1],BPoint(camera->Position().x,camera->Position().y),war_map,tiles,war_entity_list);
	DrawTriangle(map_points[1],map_points[2],BPoint(camera->Position().x,camera->Position().y),war_map,tiles,war_entity_list);
	DrawTriangle(map_points[3],map_points[0],BPoint(camera->Position().x,camera->Position().y),war_map,tiles,war_entity_list);
}
*/

void WarDisplay::DrawTriangle(BPoint point1,BPoint point2,BPoint point3,WarMap *war_map,WarTileset *tiles,WarEntityList *war_entity_list)
{
	BPoint tri_points[3];
	tri_points[0] = point1;
	tri_points[1] = point2;
	tri_points[2] = point3;

	BPoint temp;
	if (tri_points[0].y > tri_points[1].y) {temp=tri_points[0]; tri_points[0]=tri_points[1]; tri_points[1]=temp;};
	if (tri_points[1].y > tri_points[2].y) {temp=tri_points[1]; tri_points[1]=tri_points[2]; tri_points[2]=temp;};
	if (tri_points[0].y > tri_points[1].y) {temp=tri_points[0]; tri_points[0]=tri_points[1]; tri_points[1]=temp;};

	float middle=tri_points[0].x+(tri_points[2].x-tri_points[0].x)*(tri_points[1].y-tri_points[0].y)/(tri_points[2].y-tri_points[0].y);

	if (tri_points[1].x > middle)
	{
		ParallelTrapeze(tri_points[0].y,tri_points[1].y,tri_points[0].x,tri_points[0].x,middle,tri_points[1].x,war_map,tiles,war_entity_list);
		ParallelTrapeze(tri_points[1].y,tri_points[2].y,middle,tri_points[1].x,tri_points[2].x,tri_points[2].x,war_map,tiles,war_entity_list);
	}
	else
	{
		ParallelTrapeze(tri_points[0].y,tri_points[1].y,tri_points[0].x,tri_points[0].x,tri_points[1].x,middle,war_map,tiles,war_entity_list);
		ParallelTrapeze(tri_points[1].y,tri_points[2].y,tri_points[1].x,middle,tri_points[2].x,tri_points[2].x,war_map,tiles,war_entity_list);
	}
}
// **************************************************************************
