/*
The entries in the id_map that ther's no unit on is marked with
the id 0xFFFF. So, no unit can have that id#.
*/

#include <stdlib.h>
#include <stdio.h>
#include <Entry.h>
#include <Path.h>
#include <Directory.h>

#include "WarEntityList.h"
#include "WarPathfinder.h"
#include "WarMap.h"
#include "WarBody.h"
#include "WarBrain.h"
#include "WarMessage.h"
#include "WarClass.h"

//#include "WarUnitBody.h"

//#include "MyPathfinder.h"

class WarBrain;
class WarClass;

_EXPORT WarEntityList::WarEntityList(WarMap *map,BDirectory *ud,bool brain):BLocker()
{
	init=B_OK;
	brains=brain;
	unit_dir=ud;

	war_map=map;
	width_shift=war_map->WidthShift();

	class_list=new WarList(50,10);
	unit_list=new WarList(200,200);
	flying_list=new WarList(100,100); // contains the id-number+1, zero means empty entry, one means id zero.

	id_map=new uint16[war_map->BlockCount()];
	memset(id_map,-1,war_map->BlockCount()<<1);
	block_count=war_map->BlockCount()-1;
}

_EXPORT WarEntityList::~WarEntityList()
{
	uint16 i;
	unit_entry *u_entry;

	delete []id_map;
	delete flying_list;
	while (unit_list->GetNextItem((void **)&u_entry,&i))
	{
		unit_list->RemoveItem(i-1);
		delete u_entry->body;
		delete u_entry->brain;
		delete u_entry;
	}
	delete unit_list;
	
	class_entry *c_entry;
	while (class_list->GetNextItem((void **)&c_entry,&i))
	{
		class_list->RemoveItem(i-1);
		unload_add_on(c_entry->image);
		delete c_entry;
	}
	delete class_list;
}

_EXPORT bool WarEntityList::OccupyBlock(uint16 id,uint16 x,uint16 y)
{
	uint32 offs=((y<<war_map->WidthShift())+x)&block_count;
	if (id_map[offs]==0xFFFF)
	{
		id_map[offs]=id;
		return true;
	}
	else return false;
}

_EXPORT bool WarEntityList::OccupyBlock(uint16 id,uint32 offs)
{
	offs&=block_count;
	if (id_map[offs]==0xFFFF)
	{
		id_map[offs]=id;
		return true;
	}
	else return false;
}

_EXPORT bool WarEntityList::GetNextEntity(WarBody **body,WarBrain **brain,uint16 *id,uint16 *cookie)
{
	unit_entry *entry;
	if (!unit_list->GetNextItem((void **)&entry,cookie)) return false;
	*body=entry->body;
	if (brain) *brain=entry->brain;
	*id=(*cookie)-1;
	return true;
}

_EXPORT bool WarEntityList::GetNextFlyingEntity(WarBody **body,WarBrain **brain,uint16 *id,uint16 *cookie)
{
	uint32 flying_id;
	unit_entry *entry;
//	fprintf(stderr,"flying_list->GetNextItem()\n");
	if (!flying_list->GetNextItem((void **)&flying_id,cookie)) return false;
//	fprintf(stderr,"unit_list->ItemAt(%d)\n",flying_id-1);
	entry=(unit_entry*)unit_list->ItemAt(flying_id-1);
	*body=entry->body;
	if (brain) *brain=entry->brain;
	*id=flying_id-1;
//	fprintf(stderr,"returning\n");
	return true;
}

_EXPORT bool WarEntityList::GetNextSelectedEntity(WarBody **body,WarBrain **brain,uint16 *id,uint16 *cookie)
{
	unit_entry *entry;
	while (unit_list->GetNextItem((void **)&entry,cookie))
	{
		if (entry->body->IsSelected())
		{
			*body=entry->body;
			if (brain) *brain=entry->brain;
			*id=*cookie-1;
			return true;
		}
	}
	return false;
}

_EXPORT uint16 WarEntityList::IsClassLoaded(char *filename)
{
	class_entry *entry;
	uint16 i=0;
	// check if the unit is already loaded
	while (class_list->GetNextItem((void **)&entry,&i))
		if (!strcmp(filename,entry->filename)) return i-1;
	return 0xffff;
}

_EXPORT uint16 WarEntityList::LoadClass(char *filename)
{
	fprintf(stderr,"WarEntityList::LoadUnit(%s);\n",filename);
	class_entry *entry;

	uint16 i=IsClassLoaded(filename);
	if (i<0xffff)
	{
		entry=(class_entry*)class_list->ItemAt(i);
		entry->war_class->IncreaseLoads();
		return i;
	}
	
	entry=new class_entry;

	// try to find the file and the symbols in it
	BEntry file_entry;
	BPath path;

	if (unit_dir->FindEntry(filename,&file_entry) == B_OK)
	{
		file_entry.GetPath(&path); // getting the pathname for the add-on

		entry->image=load_add_on(path.Path());
		if (entry->image>0)
		{
			fprintf(stderr,"Loading add-on: %s\n",path.Path());
			
			WarClass *(*new_class)(BDirectory *unit_dir);

			// NewClass()
			if (get_image_symbol(entry->image,"instantiate_class__FP10BDirectory",B_SYMBOL_TYPE_TEXT,(void **)&new_class)==B_OK)
			{
				// the unit class need to know where it's unit is stored
				entry->war_class=new_class(unit_dir);
				// store the filename so we know it's loaded later on
				file_entry.GetName(entry->filename);
				entry->war_class->IncreaseLoads();
				uint16 id=class_list->AddItem(entry);
				fprintf(stderr,"unit_id: %d\n",id);
				return id;
			}
			else
			{
				unload_add_on(entry->image);
				// if we didn't find some of the symbols, unload it and return an error
				fprintf(stderr,"Couldn't find the right symbols in add-on \"%s\"\n",path.Path());
			}
		}
		else
		{
			fprintf(stderr,"Couldn't load add-on \"%s\"\n",path.Path());
		}
	}
	else
	{
		fprintf(stderr,"Couldn't find file\"%s\"\n",filename);
	}
	delete entry;
	return -1;
}

_EXPORT bool WarEntityList::UnloadClass(uint16 id)
{
	// check if the unit is loaded
	class_entry *entry;
	if ((entry=(class_entry*)class_list->ItemAt(id)))
	{
		if (entry->war_class->DecreaseLoads())
		{
			unload_add_on(entry->image);
			class_list->RemoveItem(entry);
			delete entry->war_class;
			delete entry;
		}
		return true;
	}
	return false;
}

_EXPORT bool WarEntityList::RemoveEntity(uint16 id)
{
	fprintf(stderr,"WarEntityList::RemoveUnit(%d)\n",id);
	unit_entry *entry;
	if ((entry=(unit_entry*)unit_list->ItemAt(id)))
	{
		if (entry->body->IsSelected()) selected_units[entry->body->Team()]--; // if the unit is selected
		if (entry->body) delete entry->body;
		if (brains) delete entry->brain;
		delete entry;
		unit_list->RemoveItem(id);
	}
	flying_list->RemoveItem((void*)(id+1));
	return entry;
}

_EXPORT uint16 WarEntityList::AddEntity(WarBuffer *buffer)
{
	uint16 type=buffer->PopUint16();

	class_entry *c_entry;
	if ((c_entry=(class_entry*)class_list->ItemAt(type)))
	{
		unit_entry *entry=new unit_entry;
//		entry->body=c_entry->war_class->InstantiateBody(buffer);
		uint16 id=entry->body->Id();

		if (!unit_list->AddItemAt(entry,id))
		{
			fprintf(stderr,"  unit added sucessfully.%p,%p\n",GetBodyByID(id),GetBrainByID(id));
			fprintf(stderr,"  Number of units:%d\n",unit_list->CountItems());
			return id;

		}
		else fprintf(stderr,"Trying to add a unit body @:(%f,%f) type:%d\n",entry->body->XPosition(),entry->body->YPosition(),type);
		delete entry;
	}
	else fprintf(stderr,"  Undefined unit type id, unit not added.\n");
	return -1;
}

_EXPORT uint16 WarEntityList::AddEntity(float x,float y,float z,uint16 type,uint8 team,uint16 angle)
{
	fprintf(stderr,"WarEntityList::AddUnit(%f,%f,%d,%d,%d...);\n",x,y,type,team,angle);

	class_entry *c_entry;
	if ((c_entry=(class_entry*)class_list->ItemAt(type)))
	{
		unit_entry *entry=new unit_entry;
		fprintf(stderr,"unit_list->AddItem(%p)\n",entry);
		uint16 id=unit_list->AddItem(entry);
		if (id<0xffff)
		{

			WarClass *war_class=c_entry->war_class;

			if (war_class->IsFlying())
			{
				if (flying_list->AddItem((void *)(id+1))==0xffff)
				{
					unit_list->RemoveItem(id);
					delete entry;
					fprintf(stderr,"  Block not occupied, unit not added.\n");
					return -1;
				}
			}
			else
			{
				// Occupy the block the unit stands on
				uint32 offs=(((int)y)<<width_shift)+((int)x)&block_count;
				if (!OccupyBlock(id,offs))
				{
					fprintf(stderr,"  The block is occupied, the unit cannot be added.\n");
					fprintf(stderr,"    Trying to remove item\n");
					unit_list->RemoveItem(id);
					fprintf(stderr,"    deleting entry\n");
					delete entry;
					fprintf(stderr,"    returning -1\n");
					return -1;
				}
				fprintf(stderr,"this block (%ld) should be occupied, and it IS%s!\n",offs,IsBlockOccupied(offs)?"":"'NT");
			}

			entry->body=war_class->InstantiateBody(x,y,z,angle,team,WarColor(255,0,0),war_map,id);
			if (!entry->body) // if the add-on didn't create a body object, then abort
			{
				unit_list->RemoveItem(id);
				delete entry;
				fprintf(stderr,"  the addon couldn't construct a body object, unit not added.\n");
				return -1;
			}
		
			if (brains) // if this unit list is running an the server, add a brain too
			{
				entry->brain=war_class->InstantiateBrain(war_map,this,team,id,x,y,entry->body,NULL);
				fprintf(stderr,"Brain created\n");
				if (!entry->brain) // if the add-on didn't create a brain object, then abort
				{
					unit_list->RemoveItem(id);
					delete entry->body;
					delete entry;
					fprintf(stderr,"  the addon couldn't construc a brain object, unit not added\n");
					return -1;
				}
			}
			fprintf(stderr,"  unit added sucessfully.\n");
			fprintf(stderr,"  Number of units:%d\n",unit_list->CountItems());
			return id;

		}
		else fprintf(stderr,"Trying to add a unit body @:(%f,%f) type:%d\n",x,y,type);
		delete entry;
	}
	else fprintf(stderr,"  Undefined unit type id, unit not added.\n");
	return -1;
}

_EXPORT bool WarEntityList::SelectEntity(WarBody *war_body,uint8 team)
{
	if (war_body->Team()==team)
	{
		if (!war_body->IsSelected())
		{
			selected_units[team]++;
			war_body->Select(true);
			return true;
		}
	}
	return false;
}

_EXPORT bool WarEntityList::DeselectEntity(WarBody *war_body,uint8 team)
{
	if (war_body->Team()==team)
	{
		if (war_body->IsSelected())
		{
			selected_units[team]--;
			war_body->Select(false);
			return true;
		}
	}
	return false;
}
