#include "WarGLView.h"
#include "WarWindow.h"
#include "WarDisplay.h"

#include <TranslationUtils.h>
#include <opengl/GL/gl.h>
#include <opengl/GL/glu.h>
#include <app/Application.h>
#include <interface/InterfaceDefs.h>
#include <Directory.h>
#include <Path.h>

#include <stdio.h>

#define _TARGET_x86_

long WarGLView::main_task(void *data)
{
	WarGLView *view=(WarGLView*)data;

	bigtime_t time=system_time();
	bigtime_t systime;
	float frame_time=0.01;

	// main loop
	while (view->running)
	{
		view->Cycle(frame_time);
		view->Render();
		systime = system_time();
		if (systime - time < 50000 ) snooze(5000 - systime + time);
		systime = system_time();
		frame_time = (float)(systime - time) / 1000000.0;
		time = systime;
	}
	return 0;
}


WarGLView::WarGLView( BRect frame, const char *name, int32 mode, int32 flags )
	: BGLView( frame, const_cast<char *>( name ), mode, flags, BGL_RGB | BGL_DOUBLE | BGL_DEPTH )
{
// temporary, will be set during connecting to a server
	team=0;

// we want keyboard events even when we're not in focus
//	SetEventMask(B_KEYBOARD_EVENTS);
	SetEventMask(B_POINTER_EVENTS,B_NO_POINTER_HISTORY);


	running = true;
	childs=new BList();

// create the word list
	word_list=new WarList(100,20);

// allocate the stack
	stack=new int32[stack_size];
	stack_pointer=0;

// copy the word list into the WarList
	for (uint16 i=0;i<num_words;i++)
	{
		char *word = new char[strlen(words[i])+1];
		strcpy(word,words[i]);
		word_list->AddItem(word);
	}

	app_info info;
	be_app->GetAppInfo(&info);
	BEntry app_entry(&info.ref);
	BPath app_path(&app_entry);
	app_path.GetParent(&app_path);

	app_dir = new BDirectory(app_path.Path());
	script_dir = new BDirectory(app_dir,"scripts/");
	environment_dir = new BDirectory(app_dir,"environments/");
	map_dir = new BDirectory(app_dir,"maps/");
	unit_dir = new BDirectory(app_dir,"units/");

// allocating the camera list
	camera = new WarCamera[16];
	current_camera = camera;

// initializing the keyboard_state array
	for (uint16 i=0;i<130;i++)
		keyboard_state[i]=false;
}

WarGLView::~WarGLView( void )
{
	delete war_entity_list;
	delete war_map;
	delete war_display;
	delete childs;

// delete all strings in the wordlist
	uint16 cookie = 0;
	char *word;
	while (word_list->GetNextItem((void**)&word,&cookie))
		delete []word;
	
	delete word_list;
	delete []stack;
	
	delete unit_dir;
	delete map_dir;
	delete environment_dir;
	delete script_dir;
	delete app_dir;
}

void WarGLView::AddChild(WarView *view)
{
	childs->AddItem((void*)view);
	view->gl_view=this;
}

void WarGLView::DetachedFromWindow( void )
{
	BGLView::DetachedFromWindow();

// Don't ask me why7 I have to do this,
// I just copied it from the GLTeapot sample
	long locks=0;
	while (Window()->IsLocked())
	{
		locks++;
		Window()->Unlock();
	}

	int32 ret;
	running = false;
	wait_for_thread(main_thread,&ret);

	while(locks--) Window()->Lock();
}

void WarGLView::SetMousePosition(BPoint position)
{
	ConvertToScreen(&position);
	set_mouse_position((int32)position.x,(int32)position.y);
}

void WarGLView::AttachedToWindow( void )
{
	// Make keyboard/mouse input go to this view when its window has
	// focus (that'll stay in effect until another view calls MakeFocus()
	// or the user puts the focus on a control... which won't happen in
	// this app because there are no other views and no controls).
	MakeFocus();

	// Initialize OpenGL; gInit() sets things up, gReshape() makes sure
	// OpenGL knows about the view's size/shape.
	LockGL();
	BGLView::AttachedToWindow();

	// Clear to black
	glClearColor( 0.0, 0.0, 0.0, 0.0 );
	
	// Set up the depth buffer
	glClearDepth( 1.0 );
	glDepthFunc( GL_LESS );
	glEnable( GL_DEPTH_TEST );

	BRect r = Bounds();

	war_map=new WarMap(map_dir,"default",environment_dir);
	war_map->Correct();
	war_map->GenerateMap();

	war_entity_list=new WarEntityList(war_map,unit_dir);

#ifdef _TARGET_x86_
	uint16 class_id = war_entity_list->LoadClass("tank1_x86");
#else
	uint16 class_id = war_entity_list->LoadClass("tank1_ppc");
#endif
	uint16 entity_id = war_entity_list->AddEntity(10.0,10.0,10.0,class_id,0,0);
	war_entity_list->TakeOff(entity_id);


	for (uint16 i=0;i<16;i++)
	{
		camera[i].MoveTo(WarVertex(10.0,10.0,10.0));
		camera[i].RotateYTo(3.1415927/2);
	}

	current_camera->MoveTo(WarVertex(10.0,10.0,10.0));
	current_camera->RotateYTo(3.1415927/2);

	war_display=new WarDisplay(0,0,r.IntegerWidth(),r.IntegerHeight(),war_map,war_entity_list,current_camera);
	AddChild((WarView*)war_display);

	WarFont *font=new WarFont(app_dir,"font");
	war_text_output = new WarTextOutput(0,r.IntegerHeight()>>1,r.IntegerWidth(),r.IntegerHeight()>>1,font);
	AddChild((WarView*)war_text_output);

	war_cli=new WarCli(0,0,r.IntegerWidth(),32,font,word_list);
	AddChild((WarView*)war_cli);

	war_cli->SetPrompt("0 > ");
	
	char string[16];
	strcpy(string,"\"startup\" exec");
	Parse(string);

	UnlockGL();

	main_thread=spawn_thread(main_task,"main_thread",B_DISPLAY_PRIORITY,(void*)this);
	resume_thread(main_thread);
}

void WarGLView::MouseMoved(BPoint point,uint32 transit,const BMessage *message)
{
	war_display->MouseMoved(point - BPoint(war_display->Left(),war_display->Top()));
}

void WarGLView::MouseDown(BPoint point)
{
	if (Bounds().Contains(point) && Window()->IsActive())
	{
		BMessage *message = Window()->CurrentMessage();
		int32 buttons;
		message->FindInt32("buttons",&buttons);
	
		war_display->MouseDown(point,buttons ^ mouse_buttons);

		mouse_buttons = buttons;
	}
}

void WarGLView::MouseUp(BPoint point)
{
	BMessage *message = Window()->CurrentMessage();
	uint32 buttons;
	message->FindInt32("buttons",(int32*)&buttons);
	if (buttons != mouse_buttons)
	{
		war_display->MouseUp(point,buttons ^ mouse_buttons);

		mouse_buttons = buttons;
	}
}

void WarGLView::FrameResized( float width, float height )
{
	// When the view is resized (it follows the window, since we created
	// it with B_FOLLOW_ALL_SIDES), we need to tell OpenGL that it's
	// rendering into a new size/shape.
	LockGL();
	BGLView::FrameResized( width, height );

	// The OpenGL view has been resized, so we have to reconfigure our
	// OpenGL context to reflect that.
	war_display->Resize(width,height);

//	war_display2->Resize(width/2,height);
//	war_display2->Move((int)width/2,0);

	war_cli->Resize(width,32);

	war_text_output->Move(0,height/2);
	war_text_output->Resize(width,height/2);

	UnlockGL();
	Render();
}

void WarGLView::KeyDown( const char *bytes, int32 size )
{
	if (!Window()->IsActive()) return;
	BMessage *message = Window()->CurrentMessage();

//	message->PrintToStream();

	int32 key;
	message->FindInt32("key",&key);


	if (!war_cli->IsHidden())
	{
		// the console is active, the key should go there
		if (bytes[0] == B_RETURN)
		{
			char command_line[512];
			
			war_cli->GetCommandLine(command_line,512);
			war_cli->Reset();

			if (*command_line) Parse(command_line);
			else war_cli->Hide(true);
		}
		else war_cli->KeyDown(*bytes);
	}
	else
	{
		if (keyboard_state[key]) return;
		keyboard_state[key]=true;

		if (key == 17)
			execute("+paragraph",true);
		else if (key == 1)
			execute("+escape",true);
		// now, we should execute the function assigned to the pressed key
		else if (size == 1)
		{
			char tmp[5];
			if (bytes[0] == B_UP_ARROW)
				execute("+up_arrow",true);
			else if (bytes[0] == B_DOWN_ARROW)
				execute("+down_arrow",true);
			else if (bytes[0] == B_LEFT_ARROW)
				execute("+left_arrow",true);
			else if (bytes[0] == B_RIGHT_ARROW)
				execute("+right_arrow",true);
			else if ((bytes[0] >= '0' && bytes[0] <= '9') || bytes[0] >= 'a' && bytes[0] <= 'z')
			{
				tmp[0]='+';
				tmp[1]=bytes[0];
				tmp[2]='\0';

				execute(tmp,true);
			}
			else if (bytes[0] == B_FUNCTION_KEY && key >= B_F1_KEY && key <= B_F12_KEY)
			{
				sprintf(tmp,"+f%ld",key - B_F1_KEY + 1);
				execute(tmp,true);
			}
		}
	}
	BGLView::KeyDown( bytes, size );
}

void WarGLView::KeyUp( const char *bytes, int32 size )
{
	BMessage *message = Window()->CurrentMessage();

//	message->PrintToStream();

	int32 key;
	message->FindInt32("key",&key);

//	printf("key: %ld\n",key);

	if (keyboard_state[key])
	{
		keyboard_state[key]=false;

		if (key == 17)
			execute("-paragraph",true);
		else if (key == 1)
			execute("-escape",true);
		// now, we should execute the function assigned to the released key
		else if (size == 1)
		{
			char tmp[5];
			if (bytes[0] == B_UP_ARROW)
				execute("-up_arrow",true);
			else if (bytes[0] == B_DOWN_ARROW)
				execute("-down_arrow",true);
			else if (bytes[0] == B_LEFT_ARROW)
				execute("-left_arrow",true);
			else if (bytes[0] == B_RIGHT_ARROW)
				execute("-right_arrow",true);
			else if ((bytes[0] >= '0' && bytes[0] <= '9') || bytes[0] >= 'a' && bytes[0] <= 'z')
			{
				tmp[0]='-';
				tmp[1]=bytes[0];
				tmp[2]='\0';

				execute(tmp,true);
			}
			else if (bytes[0] == B_FUNCTION_KEY && key >= B_F1_KEY && key <= B_F12_KEY)
			{
				sprintf(tmp,"-f%ld",key - B_F1_KEY + 1);
				execute(tmp,true);
			}
		}
	}
	BGLView::KeyDown( bytes, size );
}

void WarGLView::ErrorCallback( GLenum err )
{
	// The ErrorCallback() method is called whenever OpenGL has a problem;
	// you can override it to ignore specific known error messages and
	// display otheres, abort the program, etc... we'll just print them
	// out.  You'll have to run the application from a Terminal to see
	// these.
	fprintf( stderr, "Unexpected error occured (%ld):\n", err );
	fprintf( stderr, "\t%s\n", gluErrorString( err ) );
}

bool WarGLView::update_child(void *child)
{
	((WarView*)child)->Invalidate();
	return false;
}

void WarGLView::Cycle(float frame_time)
{
	WarBody *body;
	uint16 i,cookie=0;

	for (uint16 i=0;i<16;i++) camera[i].Cycle(frame_time,war_entity_list,war_map);

	while (war_entity_list->GetNextEntity(&body,NULL,&i,&cookie))
	{
		body->Cycle(NULL,frame_time);
//		if (body->Team()==team) body->Vision(body->X(),body->Y(),war_map,MAPSTATE_VISIBLE + MAPSTATE_DISCOVERED);
	}
}

void WarGLView::Render( void )
{
	// Render our OpenGL scene and swap the buffer so we can actually see
	// it.
	LockGL();
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	childs->DoForEach((bool (*)(void *))update_child);
	SwapBuffers();
	UnlockGL();
}

void WarGLView::Parse(char *line,bool quiet)
{
	if (*line == 0) return;
	if (line[0] == '#') return;

	// ********************** COMPILE ********************

	// if the user is trying to compile
	if (!strncmp(line,words[WORD_COMPILE],strlen(words[WORD_COMPILE])) && line[strlen(words[WORD_COMPILE])]==' ' && strlen(line)-strlen(words[WORD_COMPILE])>1)
	{
		char *string=new char[strlen(line)-strlen(words[WORD_COMPILE])+1];
		strcpy(string,line+strlen(words[WORD_COMPILE])+1);

		// insert the NULL between the word-name and the function definition (the command line to be compiled)
		uint16 i=0;
		while (string[i]!=' ' && string[i]!=0)
			i++;
		string[i]=0;

		// check if the word already is defined
		i=0;
		char *defined_word = NULL;
		while (word_list->GetNextItem((void**)&defined_word,&i))
		{
			if (!strcmp(string,defined_word))
			{
				char s[256];
				sprintf(s,"Can't compile, the word \"%s\" is already defined",string);
				war_text_output->PutLine(s);
				delete []string;
				return;
			}
		}

		word_list->AddItem(string);
		return;
	}

	// ********************** FORGET ********************
	
	// if the user is trying to forget
	if (!strncmp(line,words[WORD_FORGET],strlen(words[WORD_FORGET])) && line[strlen(words[WORD_FORGET])]==' ')
	{
		char *defined_word;
		char *unwanted_word=line+strlen(words[WORD_FORGET])+1;

		// prepare the parameter
		uint16 i=0;
		while (unwanted_word[i]!=' ' && unwanted_word[i]!=0)
			i++;
		unwanted_word[i]=0;

		//  look for the word to forget
		i=0;
		bool is_word_defined;
		while (is_word_defined=word_list->GetNextItem((void**)&defined_word,&i))
			if (!strcmp(unwanted_word,defined_word)) break;
		i--;

		if (!is_word_defined)
		{ // the wor does not exist in the word list, it is not defined. Tell the user and return
			char s[256];
			sprintf(s,"the word \"%s\" is not defined",unwanted_word);
			war_text_output->PutLine(s);
			return;
		}

		if (i<num_words)
		{ // the word is not user-defined and cannot be forgotten. Tell the user and return.
			char s[256];
			sprintf(s,"the word \"%s\" is not user defined and cannot be forgotten",unwanted_word);
			war_text_output->PutLine(s);
			return;
		}
		
		defined_word=(char*)word_list->RemoveItem(i);
		delete []defined_word;
		return;
	}
	

	// ************ STRING *********************

	char *word;
	char *cookie=NULL;
	word=GetWord(line,&cookie);
	do
	{
		if (!strcmp(word,words[WORD_IF])) // IF
		{
			int32 val;
			if (pop(&val) && !val)
				while ((word=GetWord(line,&cookie)))
				{
					if (!strcmp(word,words[WORD_ELSE])) break;
					if (!strcmp(word,words[WORD_ENDIF])) break;
				}
		}
		else if (!strcmp(word,words[WORD_ELSE])) // ELSE
		{
			while ((word=GetWord(line,&cookie)))
				if (!strcmp(word,words[WORD_ENDIF])) break;
		}
		else execute(word,quiet);
	word=GetWord(line,&cookie);
	}while (word);

	char prompt[10];
	sprintf(prompt,"%d > ",stack_pointer);
	war_cli->SetPrompt(prompt);
}

// the cookie should be initialized to NULL the first time the function call is made.
char *WarGLView::GetWord(char *command_line,char **cookie)
{
	char *line;
	if (*cookie==NULL) line=command_line;
	else line=*cookie;
	
	uint16 i=0;
	if (line[i]=='"')
	{
		i++;
		while (line[i]!=0 && line[i]!='"')
			i++;
		if (line[i]!='"') war_text_output->PutLine("ERROR: expected \"");
		i++;
	}
	else if (line[0]==0)
	{
		*cookie=NULL;
		return NULL;
	}
	else
		while (line[i]!=0 && line[i]!=' ')
			i++;
//	printf("line2: %s i:%d\n",line,i);

	if (line[i]!=0)
	{
		line[i]=0;
		i++;
	}
	*cookie=line+i;
	return line;
}

bool WarGLView::push(int32 val)
{
	if (stack_pointer<stack_size)
	{
		stack[stack_pointer]=val;
		stack_pointer++;
		return true;
	}
	else
	{
		war_text_output->PutLine("stack overflow!");
		return false;
	}
}

bool WarGLView::pop(int32 *val)
{
	if (stack_pointer>0)
	{
		stack_pointer--;
		*val=stack[stack_pointer];
		return true;
	}
	else
	{
		war_text_output->PutLine("stack underrun!");
		return false;
	}
}

bool WarGLView::pop_string(char *string,uint16 len)
{
	if (len==0)
	{
		*string=0; // terminate the string
		return false;
	}
	int32 val;
	uint16 i=0;

	pop(&val);
	do
	{
		string[i]=val;
		i++;
	} while (val && pop(&val) && i<len-1);
	if (val!=0) string[i]=0;
	return true;
}

void WarGLView::execute(char *word,bool quiet)
{

	uint16 i=0;
	char *defined_word;
	bool is_word_defined;

	// should probably replace this with some search tree
	while (is_word_defined=word_list->GetNextItem((void**)&defined_word,&i))
		if (!strcmp(word,defined_word)) break;
	i--;
	
	if (!is_word_defined) // the string we got is not a defined word
	{
		int32 val;
		if (atoi(&val,word)) // is it a value to push to the stack?
			push(val);
		else if (word[0]=='"') // is it a string to push to the stack?
		{
			uint16 len=strlen(word);
			word++;
			len-=2;
				if (push(0))
				while (len && push(word[len-1]))
					len--;
		}
		else if (!quiet) // or is it just another bad word
		{
			char tmp[256];
			sprintf(tmp,"bad-word \"%s\"",word);
			war_text_output->PutLine(tmp);
		}
		return;
	}

	if (i>=num_words) // the word is userdefined
	{
		char *command_line=new char[strlen(defined_word+strlen(defined_word)+1)+1];

		// the string contains the string to be executed after the word name
		strcpy(command_line,defined_word+strlen(defined_word)+1);
		// execute the userdefined command line
		Parse(command_line);
		delete []command_line;
		return;
	}

	// the word is not userdefined
	switch (i)
	{
		case WORD_PEEK_STACK: // .s
		{
			char tmp[640];
			char tmp2[20];
			uint16 i;

			tmp[0]=0;

			for (i=0;i<stack_pointer;i++)
			{
				sprintf(tmp2,"%ld ",stack[i]);
				strcat(tmp,tmp2);
			}
			war_text_output->PutLine(tmp);
		}break;
		case WORD_ADD: // +
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push(val1+val2);
		}break;
		case WORD_SUB: // -
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push(val2-val1);
		}break;
		case WORD_MUL: // *
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push(val1*val2);
		}break;
		case WORD_DIV: // /
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push(val2/val1);
		}break;
		case WORD_AND: // &
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push(val2&val1);
		}break;
		case WORD_OR: // |
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push(val2|val1);
		}break;
		case WORD_XOR: // ^
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push(val2^val1);
		}break;
		case WORD_MODULO: // %
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push(val2%val1);
		}break;
		case WORD_POP_DEC: // .d
		{
			int32 val;
			if (pop(&val))
			{
				char tmp[10];
				sprintf(tmp,"%ld",val);
				war_text_output->PutLine(tmp);
			}
		}break;
		case WORD_POP_TEXT: // .t
		{
			char tmp[256];
			pop_string(tmp,256);
			war_text_output->PutLine(tmp);
		}break;
		case WORD_DROP: // drop
		{
			int32 val;
			pop(&val);
		}break;
		case WORD_INC: // inc
		{
			int32 val;
			if (pop(&val)) push(val+1);
		}break;
		case WORD_DEC: // dec
		{
			int32 val;
			if (pop(&val)) push(val-1);
		}break;
		case WORD_GAME_SAY: // say
		{
			char tmp[256];
			pop_string(tmp,256);
//			war_game->Say(tmp);
		}break;
		case WORD_GAME_QUIT: // quit
		{
			be_app->PostMessage(B_QUIT_REQUESTED);
		}break;
		case WORD_EXEC: // execute
		{
			char tmp[B_FILE_NAME_LENGTH*2];
			pop_string(tmp,B_FILE_NAME_LENGTH);
			BFile file(script_dir,tmp,B_READ_ONLY);
			if (file.InitCheck()==B_OK)
			{
				uint16 i;
				uint16 len = 0;
				do
				{
					i=0;
					while (i<B_FILE_NAME_LENGTH && (len=file.Read(tmp+i,1)) == 1 && tmp[i]!=0x0a) i++;
					tmp[i]=0;
					Parse(tmp);
				} while (len==1);
			}
		}break;
		case WORD_SET_CURSOR: // set_cursor
		{
			int32 val;
			if (pop(&val)) war_cli->SetCursorType(val);
		} break;		
		case WORD_CURSOR: // cursor
		{
			push(war_cli->CursorType());
		} break;
/*
		case WORD_GAME_SET_MAP_MODE: // set_map_mode
		{
			int32 val;
			if (pop(&val))
				war_game->SetMapMode(val);
		}break;
		case WORD_GAME_MAP_MODE: // map_mode
		{
			push(war_game->MapMode());
		}break;
*/
		case WORD_DUP: // dup
		{
			int32 val;
			if (pop(&val))
			{
				push(val);
				push(val);
			}
		}break;
		case WORD_DEPTH: // depth
		{
			push(stack_pointer);
		}break;
		case WORD_EQUALS: // =
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push((val2==val1)?1:0);
		}break;
		case WORD_LESS: // <
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push((val2<val1)?1:0);
		}break;
		case WORD_GREATER: // >
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push((val2>val1)?1:0);
		}break;
		case WORD_NOT_EQUAL: // !=
		{
			int32 val1,val2;
			if (pop(&val1) && pop(&val2))
				push((val2!=val1)?1:0);
		}break;
		case WORD_CLEAR: // clear
		{
			stack_pointer=0;
		}break;
/*
		case WORD_GAME_DESELECT_UNITS: // deselect_units
		{
			war_game->DeselectAllUnits();
		}break;
		case WORD_GAME_ASSIGN_GROUP: // assign_group
		{
			int32 group_id;
			if (pop(&group_id))
			{
				if (group_id>254) Puts("value supposed to be in range 0-254");
				else war_game->AssignGroup(group_id);
			}
		}break;
		case WORD_GAME_SELECT_GROUP: // select_group
		{
			int32 group_id;
			if (pop(&group_id))
			{
				if (group_id>254) Puts("value supposed to be in range 0-254");
				else war_game->SelectGroup(group_id);
			}
		}break;
		case WORD_GAME_DESELECT_GROUP: // deselect_group
		{
			int32 group_id;
			if (pop(&group_id))
			{
				if (group_id>254) Puts("value supposed to be in range 0-254");
				else war_game->DeselectGroup(group_id);
			}
		}break;
		case WORD_GAME_LEAVE_GROUPS: // leave_groups
		{
			war_game->LeaveGroups();
		}break;
*/
		case WORD_GAME_CAMERA_ATTACH: // camera.attach
		{
			int32 val;
			if (pop(&val))
			{
				if (val<0 || val>0xfffe) war_text_output->PutLine("The unit id have to be in the range 0 to 65534");
				else
				{
					WarBody *body = war_entity_list->GetBodyByID(val);
					if (body != NULL) current_camera->AttachToUnit(body);
				}
			}
		}break;
		case WORD_GAME_CAMERA_DETACH: // camera.detach
		{
			current_camera->DetachFromUnit();
		}break;
		case WORD_GAME_CAMERA_LOOK_AT: // camera_look_at
		{
			int32 val;
			if (pop(&val))
			{
				if (val<0 || val>0xfffe) war_text_output->PutLine("The unit id have to be in the range 0 to 65534");
				else
				{
					WarBody *body = war_entity_list->GetBodyByID(val);
					if (body != NULL) current_camera->LookAtUnit(body);
				}
			}
		}break;
		case WORD_GAME_CAMERA_STOP_LOOK_AT: // camera_stop_look_at
		{
			current_camera->StopLookAt();
		}break;
		case WORD_GAME_ACTIVATE_CONSOLE: // activate_console
		{
			war_cli->Hide(false);
		}break;
		case WORD_GAME_CAMERA_ADD_MOVE_SPEED: // camera_add_move_speed
		{
			int32 x,y,z;
			if (pop(&z) && pop(&y) && pop(&x))
				current_camera->AddMovementSpeed(WarVertex(x,y,z));
		}break;
		case WORD_GAME_CAMERA_ADD_ROTATION_SPEED: // camera_add_rotation_speed
		{
			int32 val1,val2,val3;
			if (pop(&val3) && pop(&val2) && pop(&val1))
				current_camera->AddRotationSpeed(WarVertex(val1,val2,val3));
		}break;
		case WORD_GAME_CAMERA_ADD_SCROLL_SPEED: // camera_add_scroll_scroll_speed
		{
			int32 vert,horiz;
			if (pop(&vert) && pop(&horiz));
			current_camera->AddScrollSpeed(horiz,vert);
		}break;
		case WORD_GAME_CAMERA_ROTATE_BY: // camera_rotate_by
		{
			int32 val1,val2,val3;
			if (pop(&val3) && pop(&val2) && pop(&val1))
				current_camera->RotateBy(WarVertex(val1,val2,val3));
		}break;
		case WORD_GAME_CAMERA_MOVE_BY: // camera_move_by
		{
			int32 val1,val2,val3;
			if (pop(&val3) && pop(&val2) && pop(&val1))
				current_camera->MoveBy(WarVertex(val1,val2,val3));
		}break;
		case WORD_GAME_CAMERA_SET_FOV: // set fov
		{
			int32 fov;
			if (pop(&fov))
			{
				if (fov<1 || fov>189) war_text_output->PutLine("The field of vision must be a number from 1 to 189");
				else current_camera->SetFov(fov);
			}
		}break;
		case WORD_GAME_CAMERA_FOV: // fov
		{
			push(current_camera->Fov());
		}break;
		case WORD_GAME_CAMERA_MOVE: // camera_move
		{
			int32 x,y,z;
			if (pop(&z) && pop(&y) && pop(&x))
			{
				current_camera->MoveTo(WarVertex(x,y,z));
			}
		}break;

/*
		case WORD_GAME_SET_VIDEO_MODE: // set_video_mode
		{
			int32 val;
			if (pop(&val))
			{
				uint32 modes[]={B_32_BIT_640x480,B_32_BIT_800x600,B_32_BIT_1024x768};
				BMessage *msg=new BMessage(WAR_CHANGE_SPACE);
				msg->AddInt32("screen mode",(int32)modes[val%2]);
				be_app->PostMessage(msg);
			}
		}break;
		case WORD_GAME_VIDEO_MODE: // video_mode
		{

		}break;
		case WORD_GAME_CONSOLE_HEIGHT: // console_height
		{
			push(Height());
		}break;
		case WORD_GAME_SET_CONSOLE_HEIGHT: // set_console_height
		{
			int32 val;
			if (pop(&val)) war_game->ResizeConsoleTo(val);
		}break;
*/
		case WORD_WORDS: // words
		{
			uint16 i=0;
			char *defined_word;
			while (word_list->GetNextItem((void **)&defined_word,&i))
				war_text_output->PutLine(defined_word);
		}break;
		case WORD_COMPILE: // compile
		{
			war_text_output->PutLine("Compile can only be used at the beginning of a line");
		}break;
		case WORD_FORGET: // forget
		{
			war_text_output->PutLine("forget can only be used at the beginning of a line");
		}break;
		case WORD_IF: // if
		{
		}break;
		case WORD_ELSE: // else
		{
		}break;
		case WORD_ENDIF: // endif
		{
		}break;
		case WORD_GAME_DISPLAY_SWITCH_CAMERA: // display.switch_camera
		{
			int32 val;
			if (pop(&val))
			{
				if (val<0 || val>15) war_text_output->PutLine("The camera index is supposed to be in range 0 to 15");
				else
				{
					current_camera = camera + val;
					war_display->ChangeCamera(current_camera);
				}
			}
		}break;
		case WORD_GAME_DISPLAY_BEGIN_MOUSE_SCROLL: // begin mouse scroll
		{
//			war_display->BeginMouseScroll();
		}break;
		case WORD_GAME_DISPLAY_END_MOUSE_SCROLL: // begin mouse scroll
		{
//			war_display->EndMouseScroll();
		}break;
		case WORD_GAME_DISPLAY_BEGIN_MOUSE_LOOK: // begin mouse look
		{
			war_display->BeginMouseLook();
		}break;
		case WORD_GAME_DISPLAY_END_MOUSE_LOOK: // begin mouse look
		{
			war_display->EndMouseLook();
		}break;
		case WORD_GAME_DISPLAY_BEGIN_MOUSE_SELECTION: // begin mouse selection
		{
			war_display->BeginMouseSelection();
		}break;
		case WORD_GAME_DISPLAY_END_MOUSE_SELECTION: // end mouse selection
		{
			war_display->EndMouseSelection();
		}break;
		case WORD_GAME_DISPLAY_SET_FAR_CLIP: // set_far_clip
		{
			int32 val;
			if (pop(&val))
			{
				if (val<5) war_text_output->PutLine("The far z clip hat to be at least 5.0");
				war_display->SetFarClip(val);
			}
		}break;
		case WORD_GAME_DISPLAY_FAR_CLIP: // far_clip
		{
			push(war_display->FarClip());
		}break;
	}
}

// this is my own version of the function atoi() (ascii to integer)
// this can see if the given string really contains a number and
// it also handles negative numbers
bool WarGLView::atoi(int32 *val,char *string)
{
	uint16 len=strlen(string);
	bool negative=false;
	int32 tmp;

	*val=0;

	if (len==0) return false;

	if (string[0]=='-') // if negative
	{
		negative=true;
		len--;
		string++;
	} else if (string[0]=='+') // if positive
	{
		negative=false;
		len--;
		string++;
	}

	uint16 i=0;
	while (i<len)
	{
		if (string[i]<'0' || string[i]>'9') return false;

		tmp=1;
		uint16 j;
		for (j=0;j<len-i-1;j++) // powers ten to the position value
			tmp*=10;

		*val+=tmp*(string[i]-'0');
		i++;
	}
	if (negative) *val=-*val;
	return true;
}