#include "WarIFFFile.h"

#include <File.h>
#include <stdio.h>

WarIFFFile::WarIFFFile(BDirectory *dir,const char *path,int open_mode):
	BFile(dir,path,open_mode)
{
}

status_t WarIFFFile::FindChunk(uint32 chunk_name,uint32 *size)
{
// jumps to the beginning of the file
	uint32 chunk; // the chunk buffer
	uint32 format;

	off_t file_size; // the real file size
	GetSize(&file_size);
	
	Seek(0,SEEK_SET);
	ReadUint32(&chunk);
	ReadUint32(size);
	ReadUint32(&format);
	if (chunk != 'FORM' || *size+8 != file_size) return B_ERROR;

	while (ReadUint32(&chunk) == 4 && ReadUint32(size) == 4 && chunk != chunk_name)
		Seek(*size,SEEK_CUR);
	if (chunk == chunk_name) return B_OK;
	else return B_ERROR;
}

size_t WarIFFFile::ReadFloat(float *val)
{
	return ReadUint32((uint32*)(val));
}

size_t WarIFFFile::ReadUint8(uint8 *val)
{
	return Read(val,1);
}

size_t WarIFFFile::ReadUint16(uint16 *val)
{
	size_t stat;
	uint8 low,high;
	
	stat = ReadUint8(&high);
	stat += ReadUint8(&low);
	*val = high << 8;
	*val |= low;
	return stat;
}

size_t WarIFFFile::ReadUint32(uint32 *val)
{
	size_t stat;
	uint8 low,low_middle,high_middle,high;
	
	stat = ReadUint8(&high);
	stat += ReadUint8(&high_middle);
	stat += ReadUint8(&low_middle);
	stat += ReadUint8(&low);

	*val = high << 24;
	*val |= high_middle << 16;
	*val |= low_middle << 8;
	*val |= low;
	return stat;
}

