#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "WarList.h"

/* ------------------------------------------------------------ */
// List

_EXPORT WarList::WarList(uint16 size,uint16 step_size)
{
	allocated_items=size;
	list = new void*[allocated_items];
	memset(list,0,allocated_items*sizeof(void*));
	step=step_size;
	last_item=-1;
	num_items=0;
}

_EXPORT WarList::~WarList()
{
	delete []list;
}

_EXPORT bool WarList::GetNextItem(void **ptr,uint16 *index)
{
	if (*index>=allocated_items || *index>last_item)
	{
		*index=0;
		return false;
	}

	// skip the empty entries
	while(*index<allocated_items && list[*index]==NULL)
		*index+=1;

	// is this entry used? if not it must be the last entry
	if (*index<=last_item)
	{
		*ptr=list[*index];
		*index+=1;
		return true;
	}
	*index=0;
	return false;
}

_EXPORT uint16 WarList::AddItem(void *ptr)
{
	uint16 index=0;

	while(index<allocated_items && list[index] != NULL)
		index++;

	// the unit id number can't be greater than 65535
	if (index==0xFFFF) return -1;

	// if we add at the end of the list
	if (index>last_item)
	{
		last_item++;
		// check that we haven't ran out of memory for the list
		if (index==allocated_items-1) ExpandBy(step);
	}
	num_items++;
	list[index]=ptr;
	return index;
}

_EXPORT bool WarList::AddItemAt(void *ptr,uint16 index)
{
	// check that we haven't ran out of memory for the list
	if (index >= allocated_items)
		if (!ExpandBy(index - allocated_items)) return false;

	if (list[index] != NULL) return false;
	if (index>last_item) last_item=index;
	list[index]=ptr;

	num_items++;
	return true;
}

_EXPORT uint16 WarList::FirstEmpty()
{
	uint16 index=0;
	while(index<allocated_items && list[index])
		index++;
	return index;
}

_EXPORT bool WarList::RemoveItem(void *ptr)
{
	uint16 index=0;
	while(index<=last_item && list[index]!=ptr)
		index++;
	if (index>last_item) return false;
	
	if (index==last_item)
		last_item--;
	
	num_items--;
	list[index] = NULL;
	return true;
}

_EXPORT void *WarList::RemoveItem(uint16 index)
{
	void *ret;
	if (index>last_item) return NULL;
	ret=list[index];

	if (index==last_item)
	{
		if (index>0)
		{
			int32 index2=index-1;
			while (index2>=0 && list[index2]==NULL)
				index2--;
			last_item=index2;
		}
		else last_item=-1;
	}

	list[index]=NULL;
	if (ret) num_items--;
	return ret;
}

_EXPORT void *WarList::ItemAt(uint16 index)
{
	if (index>last_item) return NULL;
	return list[index];
}

_EXPORT bool WarList::ExpandBy(uint16 item_count)
{
	if (allocated_items + item_count >= 0xffff) return false;

	void **new_list = new void*[allocated_items + item_count];
	uint16 i=0;

	for (;i<allocated_items;i++)
		new_list[i] = list[i];
	
	for (;i<allocated_items + item_count;i++)
		new_list[i] = NULL;

	allocated_items += item_count;
	delete []list;
	list = new_list;

	return true;
}
