#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "WarMap.h"
#include "Global.h"

_EXPORT WarMap::WarMap(uint8 default_size,BDirectory *env_dir,char *environment):BLocker()
{
#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"WarMap:	%ld\n",sizeof(WarMap));
#endif

	size=default_size;
	width=height=0;
	width_shift=height_shift=0;
	map_data=NULL;

	// the available mapsizes;

	uint8 shift_table[]=
	{
		8,8,
		8,9,
		9,9,
		9,10,
		10,10
	};

	/* set the map size */
	width=1<<shift_table[(size%10)<<1];
	height=1<<shift_table[((size%10)<<1)+1];
	width_shift=shift_table[(size%10)<<1];
	height_shift=shift_table[((size%10)<<1)+1];
	block_count=(width<<height_shift)-1;

#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"  map_data:	%d\n",(width<<(height_shift))*2);
	fprintf(stderr,"  map_height:	%d\n",(width<<height_shift)+width+height+1);
	fprintf(stderr,"  normals:	%ld\n",sizeof(WarVertex)*(height+1)*(width+1));
#endif

	/* allocate a new map */
	map_data=new WarBlock[width<<(height_shift)];
	map_height=new uint8[(width<<height_shift)+width+height+1];
	normals=new WarVertex[(height+1)*(width+1)];

	/* set the new allocated data to 0 */
	memset(map_data,0,(width<<height_shift+1));
	memset(map_height,0,(width<<height_shift)+width+height+1);

	tiles=new WarTileset(env_dir,environment);
	fprintf(stderr,"tiles: %p\n",tiles);
}

_EXPORT WarMap::WarMap(BDirectory *map_dir,char *filename,BDirectory *env_dir):BLocker()
{
	char environment[20];

	BFile file(map_dir,filename,B_READ_ONLY);
	if (file.InitCheck()!=B_OK)
	{
		fprintf(stderr,"Couldn't open file \"%s\"\n",filename);
	}
	file.Read(&size,1); // read the map size
	file.Read(environment,20); // read the environent filename

	fprintf(stderr,"filename: %s\n",filename);
	fprintf(stderr,"environment: %s\n",environment);


//	WarMap(size,env_dir,environment); // construct the WarMap object
#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"WarMap:	%ld\n",sizeof(WarMap));
#endif
	width=height=0;
	width_shift=height_shift=0;
	map_data=NULL;
	// the available mapsizes;
	uint8 shift_table[]=
	{
		8,8,
		8,9,
		9,9,
		9,10,
		10,10
	};
	/* set the map size */
	width=1<<shift_table[(size%10)<<1];
	height=1<<shift_table[((size%10)<<1)+1];
	width_shift=shift_table[(size%10)<<1];
	height_shift=shift_table[((size%10)<<1)+1];
	block_count=(width<<height_shift)-1;
#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"  map_data:	%d\n",(width<<(height_shift))*2);
	fprintf(stderr,"  map_height:	%d\n",(width<<height_shift)+width+height+1);
	fprintf(stderr,"  normals:	%ld\n",sizeof(WarVertex)*(height+1)*(width+1));
#endif
	/* allocate a new map */
	map_data=new WarBlock[width<<(height_shift)];
	map_height=new uint8[(width<<height_shift)+width+height+1];
	normals=new WarVertex[(height+1)*(width+1)];
	/* set the new allocated data to 0 */
	memset(map_data,0,(width<<height_shift+1));
	memset(map_height,0,(width<<height_shift)+width+height+1);
	tiles=new WarTileset(env_dir,environment);
	fprintf(stderr,"tiles: %p\n",tiles);





	fprintf(stderr,"tiles: %p\n",tiles);

	file.Read(map_data,Width()*Height()*2); // read the map data *2 because it's uint16
/*
	uint8 tmp;
	uint8 *tmp_data=(uint8*)map_data;
	uint32 i=0;
	while (i<Width()*Height()*2)
	{
		tmp=tmp_data[i];
		tmp_data[i]=tmp_data[i+1];
		tmp_data[i+1]=tmp;
		i+=2;
	}
*/
	file.Read(map_height,Width()*Height()); // read the heights info
	fprintf(stderr,"tiles: %p\n",tiles);
}

_EXPORT bool WarMap::Correct()
{
	uint32 i;
	uint16 num_tiles=tiles->TileCount();
	uint16 block;
	bool ret=false;
	for (i=0;i<BlockCount();i++)
	{
		block=GetBlock(i);
		if (block>=num_tiles)
		{
			block=num_tiles-1;
			SetBlock(i,block);
			ret=true;
		}
	}
	return ret;
}

_EXPORT void WarMap::ClearStates(uint8 flags)
{
	uint32 cnt;
	for (cnt=0;cnt<=block_count;cnt++) map_data[cnt].flags&=~flags;
}

_EXPORT WarMap::~WarMap()
{
	if (tiles) delete tiles;
	if (normals) delete []normals; 
	if (map_height) delete []map_height;
	if (map_data) delete []map_data;
}

_EXPORT uint8 WarMap::StateRect(int32 x,int32 y,uint16 width,uint16 height)
{
	uint8 flags=0;
	uint8 bl_x,bl_y;
	int16	left=x>>BLOCK_SHIFT,
		right=(x+width-1)>>BLOCK_SHIFT,
		top=y>>BLOCK_SHIFT,
		bottom=(y+height-1)>>BLOCK_SHIFT;

	left=(left<0)?0:left;
	right=(right>Width())?Width():right;
	top=(top<0)?0:top;
	bottom=(bottom>Height())?Height():bottom;
	uint16 linefeed=Width()+left-right-1;

	uint32 offs=(top<<WidthShift())+left;
	for (bl_y=top;bl_y<=bottom;bl_y++)
	{
		for (bl_x=left;bl_x<=right;bl_x++)
			flags|=GetState(offs++);
		offs+=linefeed;
	}
	return flags;
}

_EXPORT void WarMap::GenerateMap()
{
	uint16 x,y;
	uint32 i=0;
	for (y=0;y<=Height();y++)
		for (x=0;x<=Width();x++)
			map_height[i++]= (uint8)((sin(x*2*3.1415927/2)*sin(y/3.0*2*3.1415927/2)+1) + ((sin(x * 3.1415927 / 16) * sin(y * 3.1415927 / 16)+1)*64));
//			map_height[i++]=0;
//			map_height[i++]=(sin(y*2*3.1415927/10)*50.0)+50;

/*
	i=0;
	for (y=0;y<Height();y++)
		for (x=0;x<Width();x++)
			map_data[i++]=rand()%tiles->TileCount();
*/
	for (y=0;y<height+1;y++)
		for (x=0;x<width+1;x++)
			CalcNormal(x,y);
}

_EXPORT float WarMap::GetHeight(float fx,float fy)
{
	uint16 x= (uint16)fx;
	uint16 y= (uint16)fy;
	float wt_x=fx-x;
	float wt_y=fy-y;

	float h[4];
	h[0]=GetFloatHeight(x,y);
	h[1]=GetFloatHeight(x,y+1);
	h[2]=GetFloatHeight(x+1,y+1);
	h[3]=GetFloatHeight(x+1,y);

	return (h[0]*(1-wt_x)*(1-wt_y) + h[1]*(1-wt_x)*(wt_y) + h[2]*(wt_x)*(wt_y) + h[3]*(wt_x)*(1-wt_y));
}

_EXPORT WarVertex WarMap::GetSmoothNormal(float fx,float fy)
{
	uint16 x= (uint16)fx;
	uint16 y= (uint16)fy;
/*
	WarVertex normal;
	normal=normals[(y<<WidthShift())+y+x];
	normal+=normals[((y+1)<<WidthShift())+y+1+x];
	normal+=normals[((y+1)<<WidthShift())+y+1+x+1];
	normal+=normals[(y<<WidthShift())+y+x+1];

	normal/=4;
	return normal;
*/

	float wt_x=fx-x;
	float wt_y=fy-y;

	WarVertex h[4];

	h[0] = normals[(y<<WidthShift())+y+x];
	h[1] = normals[((y+1)<<WidthShift())+y+1+x];
	h[2] = normals[((y+1)<<WidthShift())+y+1+x+1];
	h[3] = normals[(y<<WidthShift())+y+x+1];

	h[0] *= (1-wt_x)*(1-wt_y);
	h[1] *= (1-wt_x)*(wt_y);
	h[2] *= (wt_x)*(wt_y);
	h[3] *= (wt_x)*(1-wt_y);

	return (h[0] + h[1] + h[2] + h[3]);

//	return (h[0] * (1-wt_x)*(1-wt_y)) + (h[1] * (1-wt_x)*(wt_y)) + (h[2] * (wt_x)*(wt_y)) + (h[3] * (wt_x)*(1-wt_y));
}

WarVertex WarMap::CalcNormal(WarVertex *vex1,WarVertex *vex2,WarVertex *vex3)
{
	float u1= vex2->x - vex1->x;
	float u2= vex2->y - vex1->y;
	float u3= vex2->z - vex1->z;
	float v1= vex3->x - vex1->x;
	float v2= vex3->y - vex1->y;
	float v3= vex3->z - vex1->z;

	WarVertex normal(u2 * v3 - u3 * v2, u3 * v1 - u1 * v3, u1 * v2 - u2 * v1);
	normal /= normal.Abs();
	return normal;
}

_EXPORT void WarMap::CalcNormal(uint16 x,uint16 y)
{
	uint16 num_faces=0;
	WarVertex normal(0,0,0);
	WarVertex vex(0.0,0.0,GetFloatHeight(x,y));

	if (x<=width && y>0)
	{
		WarVertex vex2(0.0,-1.0,GetFloatHeight(x,y-1));
		WarVertex vex3(1.0,0.0,GetFloatHeight(x+1,y));
		normal += (WarVertex)CalcNormal(&vex,&vex2,&vex3);
		num_faces++;
	}
	if (x>0 && y>0)
	{
		WarVertex vex2(-1.0,0.0,GetFloatHeight(x-1,y));
		WarVertex vex3(0.0,-1.0,GetFloatHeight(x,y-1));
		normal += (WarVertex)CalcNormal(&vex,&vex2,&vex3);
		num_faces++;
	}
	if (x>0 && y<=height)
	{
		WarVertex vex3(0.0,1.0,GetFloatHeight(x,y+1));
		WarVertex vex2(-1.0,0.0,GetFloatHeight(x-1,y));
		normal += (WarVertex)CalcNormal(&vex,&vex2,&vex3);
		num_faces++;
	}
	if (x<=height && y<=height)
	{
		WarVertex vex2(1.0,0.0,GetFloatHeight(x+1,y));
		WarVertex vex3(0.0,1.0,GetFloatHeight(x,y+1));
		normal += (WarVertex)CalcNormal(&vex,&vex2,&vex3);
		num_faces++;
	}

	normal/=num_faces;
//	normals[(y<<WidthShift())+y+x]=normal;
	normals[y*(width+1)+x]=normal;

}

_EXPORT void WarMap::Save(BDirectory *dir,char *filename)
{
	char *env=tiles->Name();
	BFile file(dir,filename,B_WRITE_ONLY|B_ERASE_FILE|B_CREATE_FILE);
	file.Write(&size,1);
	file.Write(env,20);
	file.Write(map_data,block_count<<1);
	file.Write(map_height,block_count);
}
