#include <stdlib.h>
#include <math.h>

#include "WarSpline.h"

WarSpline::WarSpline()
{
	start = 0;
	end = 0;

	key_list = new WarList(10,10);

	loop=true;
}

WarSpline::~WarSpline()
{
	Key *key;
	uint16 cookie = 0;
	while (key_list->GetNextItem((void**)&key,&cookie)) delete key;

	delete key_list;
}

void WarSpline::Loop(bool l)
{
	loop=l;
}

void WarSpline::AddKey(float val, float time)
{
	Key *key = new Key;
	key->val = val;
	key->time = time;

	key_list->AddItem((void*)key);
}

void WarSpline::DelKey()
{
	if (key_list->CountItems())
		delete key_list->RemoveItem(key_list->CountItems()-1);
}

float WarSpline::Value(float time)
{
	if(loop) time=LoopedTime(time);
	
	// can't interpolate if only one key
	if(key_list->CountItems() == 1) return ((Key*)key_list->ItemAt(0))->val;
	else if(time < ((Key*)key_list->ItemAt(0))->time) return start;
	else if(time > ((Key*)key_list->ItemAt(key_list->CountItems()-1))->time) return ((Key*)key_list->ItemAt(key_list->CountItems()))->val;

	int i;
	float tq,t;
	float v0,v1,v2,v3;

	for(i=0;i<key_list->CountItems();i++)
	{
		if(((Key*)key_list->ItemAt(i+1))->time>time)
			break;
	}

	if (i == 0) v0 = start;
	else v0 = ((Key*)key_list->ItemAt(i-1))->val;
	
	v1 = ((Key*)key_list->ItemAt(i))->val;
	v2 = ((Key*)key_list->ItemAt(i+1))->val;

	if (i == key_list->CountItems()-2) v3 = end;
	else v3 = ((Key*)key_list->ItemAt(i+2))->val;

	t=(time-((Key*)key_list->ItemAt(i))->time)/( ((Key*)key_list->ItemAt(i+1))->time - ((Key*)key_list->ItemAt(i))->time);

	tq = t*t;
	return v1*((float)1-((float)3.0-(float)2*t)*tq) + v2*(((float)3.0-(float)2*t)*tq);
}

float WarSpline::LoopedTime(float time)
{
	return (float)fmod(time,((Key*)key_list->ItemAt(key_list->CountItems()-1))->time);
}

