#include <TranslationUtils.h>
#include <Bitmap.h>
#include <stdlib.h>
#include <gl.h>
#include <glu.h>
#include <string.h>
#include <File.h>
#include <Application.h>

#include "WarTextOutput.h"
#include "WarObject.h"

WarTextOutput::WarTextOutput(int32 left,int32 top,int32 width,int32 height,WarFont *font):WarView(left,top,width,height)
{
	war_font = font;
	visible_lines=0;
	line_position=0;

	for (uint16 i=0;i<16;i++)
		line[i]=NULL;
	
	for (uint16 i=0;i<16;i++)
		timeout[i]=0;
}

WarTextOutput::~WarTextOutput()
{
	for (uint16 i=0;i<16;i++)
		if (line[i]) delete []line[i];
}

void WarTextOutput::Update()
{
	if (visible_lines == 0) return;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho(0.0,Width(),0.0,Height(),0.0,100.0);
	glMatrixMode( GL_MODELVIEW );
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glDisable( GL_DEPTH_TEST );
	glLoadIdentity();

	if (timeout[(line_position - visible_lines + 1)&15] < system_time()) visible_lines--;

	uint32 j=Height() - visible_lines * (war_font->Height()+3);
	for (int16 i=line_position;i > line_position - visible_lines;i--)
	{
		war_font->DrawString(0,j,line[i&15]);
		j+=war_font->Height()+3; // step one line up
	}
	glEnable( GL_DEPTH_TEST );
	glDisable(GL_BLEND);
}

void WarTextOutput::PutLine(char *string)
{
	char tmp;
	int16 index = 0;

	if (string == NULL) return;

	if ((uint16)strlen(string) * war_font->Width() > Width())
	{
		// split the string into two lines
		index = Width()/war_font->Width() + 1;
		tmp = string[index];
//		string[index] = '\0';
	}

	uint8 next_pos = (line_position+1)&15;
	if (line[next_pos]) delete []line[next_pos];

	line[next_pos] = new char[strlen(string)+1];
	strcpy(line[next_pos],string);

	timeout[next_pos]=system_time() + 5000000;
	visible_lines++;
	if (visible_lines > Height()/(war_font->Height()+3)) visible_lines--;
	line_position=next_pos;
	
	if (index != 0)
	{
//		string[index] = tmp;
		PutLine(string + index);
	}
}
