#include "WarTileset.h"
#include "defs.h"

#include <TranslationUtils.h>
#include <Bitmap.h>
#include <stdlib.h>
#include <stdio.h>
#include <OS.h>
#include <gl.h>

static float trans_uv[]={	0.0,0.0, 0.0,1.0, 1.0,1.0, 1.0,0.0,
									0.0,1.0, 1.0,1.0, 1.0,0.0, 0.0,0.0,
									1.0,1.0, 1.0,0.0, 0.0,0.0, 0.0,1.0,
									1.0,0.0, 0.0,0.0, 0.0,1.0, 1.0,1.0,
	
									1.0,0.0, 1.0,1.0, 0.0,1.0, 0.0,0.0,
									1.0,1.0, 0.0,1.0, 0.0,0.0, 1.0,0.0,
									0.0,1.0, 0.0,0.0, 1.0,0.0, 1.0,1.0,
									0.0,0.0, 1.0,0.0, 1.0,1.0, 0.0,1.0};

/* ------------------------------------------------------------ */
// RedrawBuffer
_EXPORT WarTileset::WarTileset(BDirectory *dir,char *filename)
{
#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"WarTilset:	%ld\n",sizeof(WarTileset));
#endif

	textures=NULL;
	type=NULL;
	color=NULL;

	strcpy(name,filename);
	name[20]=0;
	fprintf(stderr,"WarTilset:	\"%s\"\n",name);
	BFile file(dir,filename,B_READ_ONLY);
	if (file.InitCheck()==B_OK)
	{
		file.Read(&((uint8*)&num_blocks)[0],1);
		file.Read(&((uint8*)&num_blocks)[1],1);
		if (num_blocks>4096) num_blocks=4096;

#ifdef _MEMORY_STATISTICS_
		fprintf(stderr,"  type_list:	%d\n",num_blocks);
		fprintf(stderr,"  buffer:	%d\n",(num_blocks<<(BLOCK_SHIFT+BLOCK_SHIFT))*4);
		fprintf(stderr,"  color:	%d\n",num_blocks*4);
#endif

		fprintf(stderr,"  num_blocks: %d\n",num_blocks);

		type=new uint8[num_blocks];
		buffer=new uint8[num_blocks*BLOCK_WIDTH*BLOCK_WIDTH*4];
		color=new WarColor[num_blocks];
		textures=new GLuint[num_blocks];

		file.Read(type,num_blocks);
		file.Read(buffer,num_blocks*BLOCK_WIDTH*BLOCK_WIDTH*4);

		uint8 tmp;
		uint8 *tmp_data=(uint8*)buffer;
		int32 i=0;
		while (i<num_blocks*BLOCK_WIDTH*BLOCK_WIDTH*4)
		{
			tmp=tmp_data[i];
			tmp_data[i]=tmp_data[i+3];
			tmp_data[i+3]=tmp;
			tmp=tmp_data[i+1];
			tmp_data[i+1]=tmp_data[i+2];
			tmp_data[i+2]=tmp;
			i+=4;
		}


// allocate names for all the tile textures
		glGenTextures(num_blocks,textures);

		for (uint16 i=0;i<num_blocks;i++)
		{
// bind the graphics to every texture
			glBindTexture(GL_TEXTURE_2D, textures[i]);
// no nice filers here yet. Just to gain some speed
//			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
//			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);

			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
// and finally, load the graphics
			glTexImage2D(GL_TEXTURE_2D, 0, 3, BLOCK_WIDTH, BLOCK_WIDTH, 0, GL_RGBA, GL_UNSIGNED_BYTE, &buffer[i*BLOCK_WIDTH*BLOCK_WIDTH*4]);
		}

		memset(color,-100,num_blocks*sizeof(WarColor));
		file.Read(color,num_blocks*sizeof(WarColor));
	}
	else fprintf(stderr,"FAILED to load environmentfile \"%s\"\n",filename);
}

_EXPORT WarTileset::~WarTileset()
{
	if (buffer) delete []buffer;
	if (textures) delete []textures;
	if (type) delete []type;
	if (color) delete []color;
}

_EXPORT uint16 WarTileset::TileCount()
{
	return num_blocks<<3;
}

_EXPORT uint8 WarTileset::TileType(uint16 index)
{
	return type[index%num_blocks];
}

_EXPORT WarColor *WarTileset::TileColor(uint16 index)
{
	return &color[index%num_blocks];
}

_EXPORT GLuint WarTileset::Tile(uint16 index,float *uv)
{
	uint8 i=(index&7)<<3;
	for (int8 j=0;j<8;j++)
		uv[j]=trans_uv[i+j];
	return textures[index>>3];
}
