#include "WarVertex.h"

#include <stdio.h>
#include <math.h>

_EXPORT WarVertex::WarVertex()
{
	x=0.0;
	y=0.0;
	z=0.0;
}

_EXPORT WarVertex::WarVertex(float x_,float y_,float z_)
{
	x=x_;
	y=y_;
	z=z_;
}

_EXPORT WarVertex WarVertex::operator=(WarVertex vec)
{
	x=vec.x;
	y=vec.y;
	z=vec.z;
	return vec;
}

_EXPORT WarVertex &WarVertex::operator=(WarVertex *vec)
{
	x=vec->x;
	y=vec->y;
	z=vec->z;
	return *this;
}

_EXPORT WarVertex &WarVertex::operator+=(WarVertex vec)
{
	x+=vec.x;
	y+=vec.y;
	z+=vec.z;
	return *this;
}

_EXPORT WarVertex &WarVertex::operator+=(WarVertex *vec)
{
	x+=vec->x;
	y+=vec->y;
	z+=vec->z;
	return *this;
}

_EXPORT WarVertex WarVertex::operator+(WarVertex vec)
{
	return WarVertex(x+vec.x,y+vec.y,z+vec.z);
}

_EXPORT WarVertex WarVertex::operator-(WarVertex vec)
{
	return WarVertex(x-vec.x,y-vec.y,z-vec.z);
}

_EXPORT WarVertex WarVertex::operator*(int32 val)
{
	return WarVertex(x*val,y*val,z*val);
}

_EXPORT WarVertex WarVertex::operator*(float val)
{
	return WarVertex(x*val,y*val,z*val);
}

_EXPORT void WarVertex::operator*=(float val)
{
	x*=val;
	y*=val;
	z*=val;
}

_EXPORT bool WarVertex::operator==(WarVertex vec)
{
	return x==vec.x && y==vec.y && z==vec.z;
}

_EXPORT bool WarVertex::operator!=(WarVertex vec)
{
	return x!=vec.x || y!=vec.y || z!=vec.z;
}

_EXPORT WarVertex &WarVertex::operator/=(int32 val)
{
	x/=val;
	y/=val;
	z/=val;
	return *this;
}

_EXPORT WarVertex WarVertex::operator-()
{
	return WarVertex(-x,-y,-z);
}

_EXPORT WarVertex WarVertex::operator/(int32 val)
{
	return WarVertex(x/val,y/val,z/val);
}

_EXPORT float WarVertex::Angle(WarVertex vertex)
{
	return acos(x*vertex.x + y*vertex.y + z*vertex.z);
}

_EXPORT void WarVertex::SetTo(float nx,float ny,float nz)
{
	x=nx;
	y=ny;
	z=nz;
}

_EXPORT void WarVertex::SetToAngle(float y_angle,float z_angle)
{
	x = 1.0;
	y = 0.0;
	z = 0.0;
	
	RotateBy(WarVertex(0.0,y_angle,z_angle));
}

_EXPORT void WarVertex::RotateBy(WarVertex vertex)
{
	float tx;
	float ty;
	float tz;

	ty = y * cos(vertex.x) - z * sin(vertex.x);
	tz = y * sin(vertex.x) + z * cos(vertex.x);
	y=ty;
	z=tz;

	tx = z * sin(vertex.y) + x * cos(vertex.y);
	tz = z * cos(vertex.y) - x * sin(vertex.y);
	x=tx;
	z=tz;

	tx = x * cos(vertex.z) - y * sin(vertex.z);
	ty = x * sin(vertex.z) + y * cos(vertex.z);
	x=tx;
	y=ty;

}
