
/* ++++++++++

   FILE:  AttributeView.cpp
   NAME:  joseph jones

   Copyright (c) 1996 by joseph jones.  All Rights Reserved.

+++++ */

#include <Application.h>
#include <Directory.h>
#include <Entry.h>
#include <Window.h>
#include <Rect.h>
#include <View.h>
#include <ScrollView.h>
#include <OutlineListView.h>
#include <Alert.h>
#include <AppDefs.h>
#include <Message.h>
#include <Errors.h>
#include <Path.h>
#include <image.h>
#include <KernelKit.h>
#include <TrackerAddOn.h>

#include <Debug.h>

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "mmgr.h"

#include "listimage.h"
#include "JLabelView.h"

extern char** environ;

class TStatusView : public BOutlineListView {
public:
				   TStatusView(BRect);

};

class TStatusWindow : public BWindow {
	BEntry*			pEntry;
	TStatusView*	m_pView;
	BScrollView*	m_psview;
	
public:
	TStatusWindow(BRect, const char*);
	
	~TStatusWindow();
	
	void SetEntry(BEntry* pentry);
	
private:

	void AddDataUnder(const char* path , BListItem* pItem);

	JLabelView* m_pLabel;
};

void process_refs(entry_ref dir_ref, BMessage *msg, void *)
{
	BAlert*		 alert;
	TStatusWindow*  wind = NULL;
	ulong		   type;
	entry_ref	   ref;
	int32		   result;
	long			count;
	long			i;

	// this add-on requires a selection
	msg->GetInfo("refs", &type, &count);
	
	if (count == 0) 
	{
		alert = new BAlert("", "You must have a selection to make a tar archive.", "Cancel");
		alert->Go();
		return;
	}


	BRect win_rect(100, 100, 400, 300);
	
	//Get Entry Name for Tab Title
	//ptentry->GetName(name);
	
	wind = new TStatusWindow(win_rect, "Image Information Browser");

	//printf("Is this it?\n");

   	wind->Show();

	for (i = 0; i < count; i++) 
	{

		if (msg->FindRef("refs", i, &ref) == B_NO_ERROR)
		{
			/*
				Create a new window for each entry
				and pass the new window the specific entry
				we get here.
			*/
			
			BEntry*		ptentry = NULL;
			
			ptentry = new BEntry(&ref);
			
			if (wind)
			{
				wind->SetEntry(ptentry);
			}

		   	delete ptentry;
		}			
		else 
		{
			alert = new BAlert("", "There was an error finding one of the items you chose to archive.", "Cancel");
			alert->Go();
			count = i;
			return;
		}
	}

	thread_id 	tid;
	long		t_exit_val;
		
	tid = wind->Thread();
	
	wait_for_thread(tid, &t_exit_val);

}

int main(/*char *argv[], int argc*/)
{
	char *argv[] = {"ImageBrowser", "/boot/home/sources/DwarfEngine/Engine2/bin/openglDevice.so"};
	int argc = 2;

	if (argc < 2)
	{
		fprintf(stderr, "USAGE: %s elf-file ...\n");
		return 1;
	}

	BApplication app("application/x-vnd.joseph_jones-ImageBrowser");

	BAlert*		 alert;
	TStatusWindow*  wind = NULL;
	ulong		   type;
	int32		   result;
	long			count;
	long			i;


	BRect win_rect(100, 100, 400, 300);
	wind = new TStatusWindow(win_rect, "Image Information Browser");

	wind->Show();

	for (i = 0; i < argc - 1; i++) 
	{
		BEntry ptentry(argv[i + 1]);

		if (wind) wind->SetEntry(&ptentry);
	}			

	thread_id 	tid;
	long		t_exit_val;
		
	tid = wind->Thread();
	
	wait_for_thread(tid, &t_exit_val);
}

// ---------------------------------------------------------------

TStatusView::TStatusView(BRect r)
			:BOutlineListView(	r, 
						"", 
						B_SINGLE_SELECTION_LIST, 
						B_FOLLOW_ALL, 
						B_FRAME_EVENTS | B_WILL_DRAW | B_NAVIGABLE)
{
		   	printf("Window Constructed\n");
}

// ---------------------------------------------------------------


TStatusWindow::TStatusWindow(BRect r, const char *title)
			  :pEntry(NULL),
			  m_pLabel(NULL),
			  BWindow(r, title, B_DOCUMENT_WINDOW, 0)
{
	//Create Label
	BRect label_rect(0, 0, 300, 19);
	
	label_rect.OffsetTo(B_ORIGIN);
	
	m_pLabel = new JLabelView(label_rect, title, (B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP), B_WILL_DRAW | B_FRAME_EVENTS);
		
//	pLabel->SetText(path.Path());
	m_pLabel->SetViewColor(200, 200, 200);
	
	AddChild(m_pLabel);

	//Create Attribute List View
	BRect   status_wind_rect(1, 21, 299, 199);
	
	status_wind_rect.right -= B_V_SCROLL_BAR_WIDTH;
	status_wind_rect.bottom -= B_H_SCROLL_BAR_HEIGHT;
	
	//status_wind_rect.OffsetTo(B_ORIGIN);
	m_pView = new TStatusView(status_wind_rect);
	
	//m_pView->SetViewColor(200, 200, 200);

	m_psview = new BScrollView("", m_pView, B_FOLLOW_ALL, B_WILL_DRAW, true, true);
	
	m_pView->TargetedByScrollView(m_psview);
	AddChild(m_psview);

}

TStatusWindow::~TStatusWindow()
{
	if (pEntry)
	{
		delete pEntry;
		pEntry = NULL;
	}
}

void TStatusWindow::SetEntry(BEntry* ptentry)
{
	if (ptentry)
	{
		
		
		//Add file path to ListView
		BPath	path;
		
		ptentry->GetPath(&path);

		//Loop through attributes
		char 				EntryName[100];
		char				type[5];
		char				listLine[300];
		
		struct attr_info	info;
		
		BNode node(ptentry);		
		
		listLine[0] = '\0';

		ptentry->GetName(listLine);
		
		BStringItem* pFileItem = new BStringItem(listLine);
		
		sprintf(EntryName, "Processing File %s", listLine);
		
		Lock();
		m_pLabel->SetText(EntryName);
		m_pView->AddItem(pFileItem);
		Unlock();
		
		AddDataUnder(path.Path(), pFileItem);
		
		Lock();
		m_pLabel->SetText("");
		Unlock();
	}
	
}

void TStatusWindow::AddDataUnder(const char* path , BListItem* pItem)
{
	BStringItem* pSubItem = NULL;
	BStringItem* pSubSubItem = NULL;

	image_id addon_image;
	char buf[255];
	
	//load addon image
	addon_image = load_add_on(path);
	
	if (0 > addon_image)
	{
		pSubItem = new BStringItem("Error: Add On could not be loaded!");
		Lock();
		m_pView->AddUnder(pSubItem, pItem);
		Unlock();
		return;
	}
	
	image_info ii;
	int result = get_image_info(addon_image, &ii);
	if (result < 0) 
	{
		sprintf(buf, "Error: Can't get image info: %d (%s)\n", result, strerror(result));
		pSubItem = new BStringItem(buf);
		Lock();
		m_pView->AddUnder(pSubItem, pItem);
		Unlock();
		return;
	}
	
	{
		pSubItem = new BStringItem("Exports");
		
		Lock();
		m_pView->AddUnder(pSubItem, pItem);
		Unlock();
		
		//cycle through image symbols
		int32 i = 0;
		char sym_name[1024];
		char tbuf[1024];
		int32 sym_name_size = sizeof(sym_name);
		int32 sclass;
		void *ptr;
		char *names[] = {"CODE", "DATA", "TEXT", "TOC ", "GLUE"};

		while (get_nth_image_symbol(addon_image, i, sym_name, &sym_name_size, &sclass, &ptr) == B_NO_ERROR) 
		{

			buf[0] = '\0';
			
			Lock();
			sprintf(buf, "Export Name: %s", demangle(sclass, sym_name, tbuf));
			pSubSubItem = new BStringItem(buf);
			m_pView->AddUnder(pSubSubItem, pSubItem);

			sprintf(buf, "Code Block:    %s", names[sclass]);
			BStringItem* pSSSubItem = new BStringItem(buf);
			m_pView->AddUnder(pSSSubItem, pSubSubItem);

			sprintf(buf, "Export Mangled Name:  %s", sym_name);
			pSSSubItem = new BStringItem(buf);
			m_pView->AddUnder(pSSSubItem, pSubSubItem);

			sym_name_size = sizeof(sym_name);

			m_pView->Collapse(pSubSubItem);

			Unlock();

			i++;
		}
	}
	
	Lock();
	m_pView->Collapse(pSubItem);
	Unlock();
	
	// dump image info
	char *typenames[] = {"UNKNOWN", "Application", "Library", "Add-on", "System"};
	if ((ii.type < 1) || (ii.type > 4)) ii.type = (image_type)0;
	
	{
		pSubItem = new BStringItem("Image Information");
		
		Lock();
		
		m_pView->AddUnder(pSubItem, pItem);
		
		sprintf(buf, "Name: %s", ii.name);
		pSubSubItem = new BStringItem(buf);
		m_pView->AddUnder(pSubSubItem, pSubItem);
		
		sprintf(buf, "Type: %s", typenames[ii.type]);
		pSubSubItem = new BStringItem(buf);
		m_pView->AddUnder(pSubSubItem, pSubItem);

		sprintf(buf, "Text Segment: %08x  Size: %08x (%d)", ii.text, ii.text_size, ii.text_size);
		pSubSubItem = new BStringItem(buf);
		m_pView->AddUnder(pSubSubItem, pSubItem);

		sprintf(buf, "Data Segment: %08x  Size: %08x (%d)", ii.data, ii.data_size, ii.data_size);
		pSubSubItem = new BStringItem(buf);
		m_pView->AddUnder(pSubSubItem, pSubItem);
		
		m_pView->Collapse(pSubItem);
		Unlock();
	}

	//unload add on
	unload_add_on(addon_image);
}

