#include "listimage.h"

#include "mmgr.h"

#define __STDC__ 1
#include <getopt.h>
#include <errno.h>
#include <stdio.h>

/*

	Code to process the mangled names exported from a shared library.
	
	NOTE: Portions of the code below have been privided by Trey Boudreau
	and Ficus Kirkpatric. If you use this code, please be sure to give 
	credit where credit is due. 
*/

extern "C" void getcwd(char *, int);
//extern "C" int getopt_long_only(int, char **, char *, struct option *, int*);

static const struct optable
{
  const char *code;
  const char *oper;
} optable[] = {
  { "nw",   " new"    },
  { "dl",   " delete" },
  { "as",   "="       },
  { "ne",   "!="      },
  { "eq",   "=="      },
  { "ge",   ">="      },
  { "gt",   ">"       },
  { "le",   "<="      },
  { "lt",   "<"       },
  { "pl",   "+"       },
  { "apl",  "+="      },
  { "mi",   "-"       },
  { "ami",  "-="      },
  { "ml",   "*"       },
  { "amu",  "*="      },
  { "aml",  "*="      },
  { "md",   "%"       },
  { "amd",  "%="      },
  { "dv",   "/"       },
  { "adv",  "/="      },
  { "aa",   "&&"      },
  { "oo",   "||"      },
  { "nt",   "!"       },
  { "pp",   "++"      },
  { "mm",   "--"      },
  { "or",   "|"       },
  { "aor",  "|="      },
  { "er",   "^"       },
  { "aer",  "^="      },
  { "ad",   "&"       },
  { "aad",  "&="      },
  { "co",   "~"       },
  { "cl",   "()"      },
  { "ls",   "<<"      },
  { "als",  "<<="     },
  { "rs",   ">>"      },
  { "ars",  ">>="     },
  { "pt",   "->"      },
  { "rf",   "->"      },
  { "vc",   "[]"      },
  { "cm",   ", "      },
  { "cn",   "?:"      },
  { "mx",   ">?"      },
  { "mn",   "<?"      },
  { "rm",   "->*"     }
};

char *strrnstr(const char *str, const char *pat, int len) {
	int patlen = strlen(pat);
	char *m = (char *)str + len - patlen;
	while ((m >= str) && (strncmp(m, pat, patlen) != 0)) m--;
	if (m < str) m = NULL;
	return m;
}

int getOperator(char *opcode, char *opbuf)
{
  for (int i = 0; i < sizeof(optable) / sizeof(optable[0]); i++) {
    if (!strncmp(opcode, "__", 2) && !strcmp(opcode + 2, optable[i].code)) {
      strcpy(opbuf, optable[i].oper);
      return(1);
    }
  }

  return(0);
}

char *resolveType(char *input, char *output, char *type) {

	char result[1024];
	char *r = result;
	int needPointer = 0;
	int sign = 0;
	bool needRef = false;
	bool needConst = false;
	bool cFunc = false;
	char *typeName = NULL;
	char *s = input;
	*type = 's'; // default to simple type, override where req'd
	while (!typeName) {
		switch (*s) {
			case 'A':
				// array of something, ignore for now
				while (*++s != '_')
					;
				s++;
				break;
			case 'C':
				needConst = true;
				s++;
				break;
			case 'F':
				{
					r = result;
					if (needPointer) {
						*r++ = '(';
						while (needPointer) { *r++ = '*'; needPointer--; }
						*r++ = ')';
					}
					*r++ = '(';
					s++;
					bool needComa = false;
					char buffer[1024];
					while (*s && (*s != '_')) {
						s = resolveType(s, buffer, type);
						sprintf(r, "%s%s", needComa ? ", " : "", buffer);
						r += strlen(r);
						needComa = true;
					}
					if (*s == '_') s++;
					s--;
					*r++ = ')';
					*r = '\0';
					if (needConst) {
						strcat(r, " const");
						needConst = false;
					}
					*type = 'F';
					typeName = result;
				}
				break;
			case '_':
				typeName = "";
				*type = *s;
				break;
			case 'P':
				needPointer++;
				s++;
				break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
			case 'Q':
				{
					int count = 1;
					char namebuf[256];
					memset(namebuf, 0, sizeof(namebuf));
					char *r = result;
					if (*s == 'Q') {
						s++; // skip past Q
						count = *s++ - '0';	// get count and skip
					}
					bool needScope = false;
					int classlen;
					// nested name space
					while (count--) {
						classlen = 0;
						while (strchr("0123456789", *s)) classlen = (classlen * 10) + *s++ - '0';
						if (classlen) {
							// check for templates
							char *tplate = strchr(s, '<');
							if (tplate && (tplate - s < classlen)) {
								// copy base name
								strncpy(namebuf, s, tplate - s + 1);
								// get template parts
								s = tplate + 1;
								do {
									s = resolveType(s, namebuf + strlen(namebuf), type);
								} while (*type != '>');
							} else {
								if (classlen < 256)
								{
									strncpy(namebuf, s, classlen);
									s += classlen;
								} else *namebuf = 0;
							}
						}
						//namebuf[classlen] = '\0';
						sprintf(r, "%s%s", needScope ? "::" : "", namebuf);
						r += strlen(r);
						needScope = true;
					}
					*type = 'Q';
					s--;
					typeName = result;
				}
				break;
			case '>':
				typeName = ">";
				*type = '>';
				break;
			case ',':
				typeName = ",";
				break;
			case 'R':
				needRef = true;
				s++;
				break;
			case 'S': // specifically signed
				sign = 1;
				s++;
				break;
			case 'U': // specifically unsigned
				sign = 2;
				s++;
				break;
			case 'b':
				// boolean
				typeName = "bool";
				break;
			case 'c':
				typeName = "char";
				break;
			case 'd':
				typeName = "double";
				break;
			case 'e':
				typeName = "...";
				break;
			case 'f':
				typeName = "float";
				break;
			case 'i': // just int
				typeName = "int";
				break;
			case 's': // short
				typeName = "int16";
				break;
			case 'l': // long
				typeName = "int32";
				break;
			case 'x': // long long
				typeName = "int64";
				break;
			case 'v': // void
				typeName = "void";
				break;
			default:
				sprintf(result, "@@@>%c<@@@", *s);
				typeName = result;
				break;
		}
	}
	// build up the type from parts
	sprintf(output, "%s%s%s%s",
		needConst ? "const " : "",
		needRef ? "&" : "",
		sign == 0 ? "" : sign == 1 ? "signed " : "unsigned ",
		typeName);
	while (needPointer) { strcat(output, "*"); needPointer--; }
	// return next place to start
	return ++s;
}

char *demangle(int32 type, char *name, char *output) {
	// bail if we don't know what it is
	if (type != B_SYMBOL_TYPE_TEXT) return name;
	
	// bail if it's not a C++ style mangled name
	char *s = strrnstr(name, "__", strlen(name));
	if (!s) return name;
	if (s == name) return name;

	while (!strchr("QFC0123456789", *(s+2))) {
		s = strrnstr(name, "__", s - name);
		if (s == name) return name;
		if (!s) return name;
	}
	// get function name
	char funcname[128];
	char *p = funcname;
	//*p++ = ':';	*p++ = ':';
	while (name < s) *p++ = *name++;
	*p = '\0';
	s += 2; // skip past underbars 

	//<-- Insert __op patch here! -->
  	if (strstr(funcname, "__op")) 
  	{
  		char ttype;
  		char op[1024];
  		char* ts;
  		
  		ts = resolveType((funcname+4), op, &ttype);
  		
  		sprintf(funcname, "operator %s", op);
  	}
	//<-- End __op patch here! -->
	
	// get function parms or class name
	// expecting type to be either 'F' or 'Q'
	char nameType;
	char demangled[1024];
	s = resolveType(s, demangled, &nameType);
	p = output;

	char oper[32];
	if (getOperator(funcname, oper)) {
	  sprintf(funcname, "operator%s", oper);
	}

	if (nameType == 'Q') {
	  if (strcmp(funcname, "__ct") == 0) strcpy(funcname, demangled);
	  else if (strcmp(funcname, "__dt") == 0) sprintf(funcname, "~%s", demangled);
	  strcpy(p, demangled);
	  p += strlen(p);
	  s = resolveType(s, demangled, &nameType);
	}

	sprintf(p, "::%s%s", funcname, demangled);
	return output;
}

bool DumpAddon(const char *addonpath)
{
	image_id addon_image;
	
	//load addon image
	addon_image = load_add_on(addonpath);
	
	if (0 > addon_image)
	{
		printf("Error: Add On could not be loaded!\n");
		return false;
	}
	
	image_info ii;
	int result = get_image_info(addon_image, &ii);
	if (result < 0) {
		printf("Error: Can't get image info: %d (%s)\n", result, strerror(result));
		fflush(stderr);
		return false;
	}
	
	// dump image info
	char *typenames[] = {"UNKNOWN", "Application", "Library", "Add-on", "System"};
	if ((ii.type < 1) || (ii.type > 4)) ii.type = (image_type)0;
	printf("\nName: %s\n", ii.name);
	printf("Type: %s\n", typenames[ii.type]);
	printf("Text Segment: %08x  Size: %08x (%d)\n", ii.text, ii.text_size, ii.text_size);
	printf("Data Segment: %08x  Size: %08x (%d)\n", ii.data, ii.data_size, ii.data_size);

	//cycle through image symbols
	int32 i = 0;
	char sym_name[1024];
	char buf[1024];
	int32 sym_name_size = sizeof(sym_name);
	int32 sclass;
	void *ptr;
	char *names[] = {"CODE", "DATA", "TEXT", "TOC ", "GLUE"};
	while (get_nth_image_symbol(addon_image, i, sym_name, &sym_name_size, &sclass, &ptr) == B_NO_ERROR) {
		printf("%s\n", names[sclass]);
		printf("\t%s\n", demangle(sclass, sym_name, buf));
		printf("\t%s\n", sym_name);
		printf("\n");
		//fflush(stdout);
		sym_name_size = sizeof(sym_name);
		i++;
	}
	fflush(stdout);
	
	//unload add on
	unload_add_on(addon_image);
}


