#ifndef _WAR_OBJECT_H_
#define _WAR_OBJECT_H_

#include "WarFace.h"
#include "WarVertex.h"
#include "WarLine.h"

struct vertex_struct
{
	float x;
	float y;
	float z;
};

struct face_struct
{
	int32 vex[3];
};

class WarObject
{
public:
	WarObject(float x,float y,float z);
	virtual ~WarObject();
	bool Load(char *filename);

	void New(uint32 num_vectors,vertex_struct *vertex_list,uint32 num_faces,face_struct *face_list);

	void PrecalcPointNorms();
	void CalcBoundingBox();
	
//	void Save(char *filename);
	virtual void Project(float *camera,WarVertex light);
	virtual void Project();

	void RotateTo(float xa,float ya,float za);
	void RotateXTo(float xa);
	void RotateYTo(float ya);
	void RotateZTo(float za);

	void RotateTo(WarVertex *normal);
	
	void RotateBy(float xa,float ya,float za);
	void RotateXBy(float xa);
	void RotateYBy(float ya);
	void RotateZBy(float za);

	void MoveTo(float x,float y,float z);
	void MoveXTo(float x);
	void MoveYTo(float y);
	void MoveZTo(float z);

	float X();
	float Y();
	float Z();

	void GetBoundingBox(float &x1,float &x2,float &y1,float &y2,float &z1,float &z2);
	void ProjectedBoundingBox(WarVertex *vex);

	uint32 line_count; // # of lines
	WarLine *line; // the lines

	uint32 face_count; // # of faces
	WarFace *face; // the faces

	WarMaterial *materials;
	uint16 material_count;

	uint32 vertex_count; // # of vertices
	WarVertex *vertices; // the vertices
	WarVertex *rotated_vertices;

	WarVertex *normals; // the vertices normals, used for envmapping and gouraud shading
	WarVertex *rotated_normals; // the rotated vertices normals, used for envmapping and gouraud shading
private:
	void rebuild(float xa,float ya,float za,float x,float y,float z);
	float t[16];
	float prod[16];
	float xa,ya,za;
	float xpos,ypos,zpos;

// describes the bounding box, is used when drawing the select cube around the unit
	float max_x,min_x;
	float max_y,min_y;
	float max_z,min_z;
};

// inline definitions

inline void WarObject::GetBoundingBox(float &x1,float &x2,float &y1,float &y2,float &z1,float &z2)
{
	x1=min_x;
	x2=max_x;
	y1=min_y;
	y2=max_y;
	z1=min_z;
	z2=max_z;
}

inline float WarObject::X()
{
	return t[3];
}

inline float WarObject::Y()
{
	return t[7];
}

inline float WarObject::Z()
{
	return t[11];
}

inline void WarObject::RotateBy(float x,float y,float z)
{
	RotateTo(xa+x,ya+y,za+z);
}

inline void WarObject::RotateXBy(float x)
{
	RotateXTo(xa+x);
}

inline void WarObject::RotateYBy(float y)
{
	RotateYTo(ya+y);
}

inline void WarObject::RotateZBy(float z)
{
	RotateZTo(za+z);
}

#endif // _WAR_OBJECT_H_
