#include "WarBitmap.h"
#include "WarFace.h"
#include "WarMaterial.h"

#include <TranslationUtils.h>
#include <Bitmap.h>

#include <List.h>
#include <stdlib.h>

_EXPORT WarBitmap::WarBitmap(uint16 width,uint16 height)
{
	frame_width=width;
	frame_height=height;
	frame_buffer = new WarColor[frame_width*frame_height];

	clp_left=0;
	clp_right=width-1;
	clp_top=0;
	clp_bottom=height-1;
}

_EXPORT WarBitmap::WarBitmap(const char *filename)
{
	BBitmap *image=BTranslationUtils::GetBitmapFile(filename);
	if (image==NULL) return;
	frame_buffer=NULL;

	frame_width=image->Bounds().Width()+1;
	frame_height=image->Bounds().Height()+1;

	frame_buffer = new WarColor[frame_width*frame_height];

	clp_left=0;
	clp_right=frame_width-1;
	clp_top=0;
	clp_bottom=frame_height-1;

	WarColor *bitmap=(WarColor *)image->Bits();
	for(uint32 i=0;i<frame_width*frame_height;i++)
	{
		frame_buffer[i]=bitmap[i];
	}
}

_EXPORT WarBitmap::~WarBitmap()
{
	if (frame_buffer) delete []frame_buffer;
}

void WarBitmap::Empty()
{
	uint32 *buffer=(uint32*)frame_buffer;
	for (uint16 i=0;i<(frame_height*frame_width)>>4;i++)
	{
		buffer[0]=0;
		buffer[1]=0;
		buffer[2]=0;
		buffer[3]=0;
		buffer[4]=0;
		buffer[5]=0;
		buffer[6]=0;
		buffer[7]=0;
		buffer[8]=0;
		buffer[9]=0;
		buffer[10]=0;
		buffer[11]=0;
		buffer[12]=0;
		buffer[13]=0;
		buffer[14]=0;
		buffer[15]=0;
		buffer+=16;
	}
}

_EXPORT void WarBitmap::DrawBitmap(int16 left,int16 top,WarBitmap *bitmap)
{
	// calculate how much will be clipped on each side
	uint16	margin_left=(left<clp_left)?clp_left-left:0,
				margin_top=(top<clp_top)?clp_top-top:0,
				margin_right=(left+bitmap->Width()-1>clp_right)?left+bitmap->Width()-clp_right-1:0,
				margin_bottom=(top+bitmap->Height()-1>clp_bottom)?top+bitmap->Height()-clp_bottom-1:0;

	if (left>clp_right || left+bitmap->Width()-1<clp_left || top>clp_bottom || top+bitmap->Height()-1<clp_top) return;

	// calculate the drawn width and height, after clipping that is
	uint16 dwidth=bitmap->Width()-margin_left-margin_right-1;
	uint16 dheight=bitmap->Height()-margin_top-margin_bottom-1;

	uint16 s_linefeed=Width()-dwidth-1;
	uint16 b_linefeed=bitmap->Width()-dwidth-1;

	// calculate the buffer offset for the the top left pixel
	WarColor *buffer=bitmap->Bits() + (margin_top)*bitmap->Width()+(margin_left);

	WarColor *frame=frame_buffer+(top+margin_top)*Width()+(left+margin_left);
	for(top=0;top<dheight;top++)
	{
		for(left=0;left<dwidth + 1;left++)
		{
			frame[0]=buffer[0];

			frame++;
			buffer++;
		}
		frame+=s_linefeed;
		buffer+=b_linefeed;
	}
}
