#include "WarFile.h"

#include <string.h>
#include <stdlib.h>

_EXPORT WarFile::WarFile(char *filename, bool create)
{
	bin = false;
	error=B_OK;
	if (create)
		f=fopen(filename,"wb");
	else
		f=fopen(filename,"rb");

	if (f==0){
		fprintf(stderr,"%s: no such file\n",filename);
		error=B_ERROR;
	}
}

_EXPORT WarFile::~WarFile()
{
	fclose(f);
}

_EXPORT status_t WarFile::InitCheck()
{
	return error;
}

_EXPORT void WarFile::IsBin(bool b)
{
	bin = b;
}

_EXPORT void WarFile::jntok(char *b, char *m)
{
	bool cont=true;
	int mc=strlen(m);
	int c;
	while (*b!=0 && cont){
		for (c=0;c<mc;c++)
			if (*b == m[c]){
				*b=0;
				cont=false;
			}
		b++;
	}	
}

_EXPORT void WarFile::Comment(int val)
{
	if (!bin) fprintf(f,"%x",val);
}

_EXPORT void WarFile::Comment(char *txt)
{
	if (!bin) fputs(txt,f);
}

_EXPORT void WarFile::Val(int val, int numbytes)
{
	if (bin){
		if (numbytes == 1)
			fputc(val,f);
		else
			for (int c=numbytes; c>0; c--)
				fputc((val >> ((c-1) << 3)) & 255,f);
	}else{
		fprintf(f,"%d\n",val);
	}
}

_EXPORT float WarFile::NextFloat()
{
	int n = NextVal(4);
	float *f = (float*)&n;
	return (*f);	
}

_EXPORT int WarFile::NextVal(int numbytes)
{	
	if (bin){
		// binary file
		if (numbytes == 1)	return fgetc(f);

		int32 tmp=0;
		for (int c=numbytes;c>0;c--)
			tmp |= fgetc(f) << (c-1 << 3);
		return tmp;

	}else{
		// text file
		bool ok=true;
		do {
			if (0==fgets(buf,BUFSIZE,f)) ok=false;
			if (ok) jntok(buf,"/\n");		
		} while ( ok && ((*buf)==0));
		return atoi(buf);
	}
}
