#include "WarObject.h"
#include "WarFile.h"
#include "WarMaterial.h"
#include <math.h>

_EXPORT bool WarObject::Load(char *filename)
{
	WarObject **vexes=new WarObject*[10];

	materials=new WarMaterial[1];
	materials[0].type = MATERIAL_ENVMAPPED;
//	materials[0].type = MATERIAL_TEXTURED;
//	materials[0].type = MATERIAL_ADD_NO_SHADE;

	WarFile tf(filename);
	tf.IsBin();
	
	fprintf(stderr,"Loading %s\n",filename);
	
	tf.NextVal(4); tf.NextVal(4);	tf.NextVal(4); tf.NextVal(4);
	tf.NextVal(4); tf.NextVal(4);	tf.NextVal(4); tf.NextVal(4);
//	tf.NextVal(4); tf.NextVal(4);

//	************* VECTORS

	int size = tf.NextVal(4);
	vertex_count = size/12;
	
	fprintf(stderr,"vertex_count: %d\n",vertex_count);

	vertices = new WarVertex[vertex_count];
	rotated_vertices = new WarVertex[vertex_count];
	normals = new WarVertex[vertex_count];
	rotated_normals = new WarVertex[vertex_count];

	for (int c=0;c<vertex_count;c++)
	{
		vertices[c].x = tf.NextFloat();
		vertices[c].y = tf.NextFloat();
		vertices[c].z = tf.NextFloat();
		
		rotated_vertices[c].x=vertices[c].x;
		rotated_vertices[c].y=vertices[c].y;
		rotated_vertices[c].z=vertices[c].z;
	}

//	************** POLYGONS
	fprintf(stderr,"looking for polys\n");

	tf.NextVal(4);
	size = tf.NextVal(4);

	fprintf(stderr,"size: %d\n",size);
//	fprintf(stderr,"0x%x size == %d faces\n",size,size/10);

	face = new WarFace[size/10];
	line = new WarLine[size/8];

	uint16 num_vertices;
	uint32 byte_counter=0;
	face_count=0;
	line_count=0;

	while(byte_counter<size)
	{
		num_vertices = tf.NextVal(2);

		if (num_vertices == 2)
		{
			line[line_count].material=&materials[0];
			line[line_count].vex[0] = rotated_vertices + tf.NextVal(2);
			line[line_count].vex[1] = rotated_vertices + tf.NextVal(2);
			line_count++;
			byte_counter+=8;
		}
		else if (num_vertices == 3)
		{
			face[face_count].material=&materials[0];
			face[face_count].vex[0] = rotated_vertices + tf.NextVal(2);
			face[face_count].vex[1] = rotated_vertices + tf.NextVal(2);
			face[face_count].vex[2] = rotated_vertices + tf.NextVal(2);
			face_count++;
			byte_counter+=10;
		}
		else
		{
			fprintf(stderr,"WARNING!!\nThis loader does not support polygons with %d vertices.\n",num_vertices);
			while (num_vertices)
			{
				tf.NextVal(2);
				byte_counter+=2;
			}
			byte_counter+=4;
		}
		tf.NextVal(2); // materials not supported yet
	}

//	set normals
	for (int c=0;c<face_count;c++)
		face[c].CalcNormal();

	PrecalcPointNorms();
	CalcBoundingBox();
	return true;
}

_EXPORT void WarObject::CalcBoundingBox()
{
	max_x=max_y=max_z=-10000.0;
	min_x=min_y=min_z=10000.0;
	for (uint16 i=0;i<vertex_count;i++)
	{
		if (vertices[i].x<min_x) min_x=vertices[i].x;
		if (vertices[i].x>max_x) max_x=vertices[i].x;
		if (vertices[i].y<min_y) min_y=vertices[i].y;
		if (vertices[i].y>max_y) max_y=vertices[i].y;
		if (vertices[i].z<min_z) min_z=vertices[i].z;
		if (vertices[i].z>max_z) max_z=vertices[i].z;
	}
}

_EXPORT void WarObject::ProjectedBoundingBox(WarVertex *vex)
{

	vex[0].SetTo(max_x,max_y,max_z);
	vex[1].SetTo(min_x,max_y,max_z);
	vex[2].SetTo(min_x,min_y,max_z);
	vex[3].SetTo(max_x,min_y,max_z);

	vex[4].SetTo(max_x,max_y,min_z);
	vex[5].SetTo(min_x,max_y,min_z);
	vex[6].SetTo(min_x,min_y,min_z);
	vex[7].SetTo(max_x,min_y,min_z);

	for (uint8 i=0;i<8;i++)
	{
		vex[i].Rotate(prod,&vex[i]);
		vex[i].Project();
	}
}

_EXPORT void WarObject::New(uint32 num_vertices,vertex_struct *vertex_list,uint32 num_faces,struct face_struct *face_list)
{//	************* VECTORS

#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"WarObject:	%d\n",sizeof(WarObject));
#endif

	vertex_count = num_vertices;
	vertices = new WarVertex[vertex_count];
	rotated_vertices = new WarVertex[vertex_count];
	normals = new WarVertex[vertex_count];


#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"  vertices:	%d\n",sizeof(WarVertex)*vertex_count);
	fprintf(stderr,"  rotated_vertices:	%d\n",sizeof(WarVertex)*vertex_count);
	fprintf(stderr,"  normals:	%d\n",sizeof(WarVertex)*vertex_count);
	fprintf(stderr,"  shades:	%d\n",sizeof(WarVertex)*vertex_count);
#endif


	for (int c=0;c<vertex_count;c++)
	{
		vertices[c].x = vertex_list[c].x;
		vertices[c].y = vertex_list[c].y;
		vertices[c].z = vertex_list[c].z;

		rotated_vertices[c].x=vertices[c].x;
		rotated_vertices[c].y=vertices[c].y;
		rotated_vertices[c].z=vertices[c].z;
	}

//	************** POLYGONS
	face_count = num_faces;

#ifdef _MEMORY_STATISTICS_
	fprintf(stderr,"  faces:	%d\n",sizeof(WarFace)*face_count);
#endif

	face=new WarFace[face_count];

	for (int c=0;c<face_count;c++)
	{
		face[c].vex[0] = rotated_vertices + face_list[c].vex[0];
		face[c].vex[1] = rotated_vertices + face_list[c].vex[1];
		face[c].vex[2] = rotated_vertices + face_list[c].vex[2];
	}
//	set normals
	for (int c=0;c<face_count;c++)
		face[c].CalcNormal();
	
	PrecalcPointNorms();
}

_EXPORT WarObject::WarObject(float x,float y,float z)
{
	rebuild(0,0,0,x,y,z);
	normals=NULL;
	rotated_normals=NULL;
	vertices=NULL;
	rotated_vertices=NULL;
	face_count = 0;
}

_EXPORT WarObject::~WarObject()
{
	if (normals) delete []normals;
	if (rotated_normals) delete []rotated_normals;
	if (vertices) delete []vertices;
	if (rotated_vertices) delete []rotated_vertices;
	if (face) delete []face;
}

_EXPORT void WarObject::RotateTo(WarVertex *normal)
{
	float x=normal->y*sin(-za) - normal->x*cos(-za);
	float y=normal->y*cos(-za) + normal->x*sin(-za);

	rebuild(-atan(y/normal->z),-atan(x/normal->z),za,xpos,ypos,zpos); // z: 0 grader
}

void WarObject::rebuild(float xang,float yang,float zang,float x,float y,float z)
{
	xa=xang;
	ya=yang;
	za=zang;
	
	xpos=x;
	ypos=y;
	zpos=z;

	// round x-axis
	float x_t[16] ={1,	0,			0,			0,
					0,	cos(xa),	-sin(xa),	0,
					0,	sin(xa),	cos(xa),	0,
					0,	0,			0,			1};

	// round y-axis
	float y_t[16] ={cos(ya),	0,	sin(ya),	0,
					0,			1,	0,			0,
					-sin(ya),	0,	cos(ya),	0,
					0,			0,	0,			1};

	// round z-axis
	float z_t[16] ={cos(za),	-sin(za),	0,	0,
					sin(za),	cos(za),	0,	0,
					0,			0,			1,	0,
					0,			0,			0,	1};

	// the camera
	float c_t[16] ={1,	0,	0,	xpos,
					0,	1,	0,	ypos,
					0,	0,	1,	zpos,
					0,	0,	0,	1};

	// invert the x-axis
	float i_t[16] ={-1,	0,	0,	0,
					0,	1,	0,	0,
					0,	0,	1,	0,
					0,	0,	0,	1};


	float t1[16];
	float t2[16];

	uint8 a,b;


	for (int b=0;b<4;b++)
		for (int a=0;a<4;a++)
			t1[b*4+a] = z_t[b*4+0]*x_t[0*4+a] +	z_t[b*4+1]*x_t[1*4+a] + z_t[b*4+2]*x_t[2*4+a] + z_t[b*4+3]*x_t[3*4+a];

/*
	for (int b=0;b<4;b++)
		for (int a=0;a<4;a++)
			t1[b*4+a] = x_t[b*4+0]*y_t[0*4+a] +	x_t[b*4+1]*y_t[1*4+a] + x_t[b*4+2]*y_t[2*4+a] + x_t[b*4+3]*y_t[3*4+a];
*/



/*
	for (int b=0;b<4;b++)
		for (int a=0;a<4;a++)
			t2[b*4+a] = z_t[b*4+0]*t1[0*4+a] + z_t[b*4+1]*t1[1*4+a] + z_t[b*4+2]*t1[2*4+a] + z_t[b*4+3]*t1[3*4+a];
*/
/*
	for (int b=0;b<4;b++)
		for (int a=0;a<4;a++)
			t2[b*4+a] = y_t[b*4+0]*t1[0*4+a] + y_t[b*4+1]*t1[1*4+a] + y_t[b*4+2]*t1[2*4+a] + y_t[b*4+3]*t1[3*4+a];
*/
	for (int b=0;b<4;b++)
		for (int a=0;a<4;a++)
			t2[b*4+a] = t1[b*4+0]*y_t[0*4+a] + t1[b*4+1]*y_t[1*4+a] + t1[b*4+2]*y_t[2*4+a] + t1[b*4+3]*y_t[3*4+a];


	for (int b=0;b<4;b++)
		for (int a=0;a<4;a++)
			t1[b*4+a] = c_t[b*4+0]*t2[0*4+a] + c_t[b*4+1]*t2[1*4+a] + c_t[b*4+2]*t2[2*4+a] + c_t[b*4+3]*t2[3*4+a];

/*
	for (int b=0;b<4;b++)
		for (int a=0;a<4;a++)
			t2[b*4+a] = t1[b*4+0]*c_t[0*4+a] + t1[b*4+1]*c_t[1*4+a] + t1[b*4+2]*c_t[2*4+a] + t1[b*4+3]*c_t[3*4+a];
*/

	for (int b=0;b<4;b++)
		for (int a=0;a<4;a++)
			t[b*4+a] = t1[b*4+0]*i_t[0*4+a] + t1[b*4+1]*i_t[1*4+a] + t1[b*4+2]*i_t[2*4+a] + t1[b*4+3]*i_t[3*4+a];

}

_EXPORT void WarObject::MoveTo(float x,float y,float z)
{
	rebuild(0,0,0,x,y,z);
}

_EXPORT void WarObject::MoveXTo(float x)
{
	rebuild(0,0,0,x,ypos,zpos);
}

_EXPORT void WarObject::MoveYTo(float y)
{
	rebuild(0,0,0,xpos,y,zpos);
}

_EXPORT void WarObject::MoveZTo(float z)
{
	rebuild(0,0,0,xpos,ypos,z);
}

_EXPORT void WarObject::RotateTo(float x,float y,float z)
{
	rebuild(x,y,z,xpos,ypos,zpos);
};

_EXPORT void WarObject::RotateXTo(float x)
{
	rebuild(x,ya,za,xpos,ypos,zpos);
};

_EXPORT void WarObject::RotateYTo(float y)
{
	rebuild(xa,y,za,xpos,ypos,zpos);
};

_EXPORT void WarObject::RotateZTo(float z)
{
	rebuild(xa,ya,z,xpos,ypos,zpos);
};

_EXPORT void WarObject::Project(float *camera,WarVertex light)
{
	WarVertex normal;
	for (int y=0;y<4;y++)
		for (int x=0;x<4;x++)
		prod[y*4+x] = camera[y*4+0]*t[0*4+x] + camera[y*4+1]*t[1*4+x] + camera[y*4+2]*t[2*4+x] + camera[y*4+3]*t[3*4+x];


	for (int c=0;c<vertex_count;c++)
	{
		vertices[c].Rotate(prod,&rotated_vertices[c]); // rotate
		normals[c].RotateSimple(t,&rotated_normals[c]);
		rotated_vertices[c].Project(); // project
	}
}

_EXPORT void WarObject::Project()
{
	WarVertex normal;
	for (int y=0;y<4;y++)
		for (int x=0;x<4;x++)
			prod[y*4+x] = t[y*4+x];


	for (int c=0;c<vertex_count;c++)
	{
		vertices[c].Rotate(prod,&rotated_vertices[c]); // rotate
		normals[c].RotateSimple(t,&rotated_normals[c]);
		rotated_vertices[c].Project(); // project
	}
}

_EXPORT void WarObject::PrecalcPointNorms()
{
	WarVertex normal;
	uint16 num_faces;

	for (int i=0;i<vertex_count;i++)
	{
		num_faces=0;
		normal.SetTo(0,0,0);
		for (int c=0;c<face_count;c++)
			if (face[c].vex[0]==&rotated_vertices[i] || face[c].vex[1]==&rotated_vertices[i] || face[c].vex[2]==&rotated_vertices[i])
			{
				normal+=face[c].normal;
				num_faces++;
			}
		if (num_faces) normal/=num_faces;
		normals[i]=normal;
	}
}
