#include "WarVertex.h"

#include <stdio.h>
#include <math.h>

_EXPORT WarVertex::WarVertex()
{
	x=0.0;
	y=0.0;
	z=0.0;
}

_EXPORT WarVertex::WarVertex(float x_,float y_,float z_)
{
	x=x_;
	y=y_;
	z=z_;
}

_EXPORT WarVertex &WarVertex::operator=(WarVertex &vec)
{
	x=vec.x;
	y=vec.y;
	z=vec.z;
	return vec;
}

_EXPORT WarVertex &WarVertex::operator+=(WarVertex &vec)
{
	x+=vec.x;
	y+=vec.y;
	z+=vec.z;
	return *this;
}

_EXPORT WarVertex WarVertex::operator+(WarVertex &vec)
{
	return WarVertex(x+vec.x,y+vec.y,z+vec.z);
}

_EXPORT WarVertex WarVertex::operator-(WarVertex &vec)
{
	return WarVertex(x-vec.x,y-vec.y,z-vec.z);
}

_EXPORT WarVertex WarVertex::operator*(int32 val)
{
	return WarVertex(x*val,y*val,z*val);
}

_EXPORT WarVertex WarVertex::operator*(float val)
{
	return WarVertex(x*val,y*val,z*val);
}

_EXPORT bool WarVertex::operator==(WarVertex &vec)
{
	return x==vec.x && y==vec.y && z==vec.z;
}

_EXPORT WarVertex &WarVertex::operator/=(int32 val)
{
	x/=val;
	y/=val;
	z/=val;
	return *this;
}

_EXPORT WarVertex WarVertex::operator/(int32 val)
{
	return WarVertex(x/val,y/val,z/val);
}

_EXPORT float WarVertex::Angle(WarVertex vertex)
{
	return acos(x*vertex.x + y*vertex.y + z*vertex.z);
}

_EXPORT void WarVertex::SetTo(float nx,float ny,float nz)
{
	x=nx;
	y=ny;
	z=nz;
}

_EXPORT void WarVertex::Rotate(float *t,WarVertex *vec)
{
	float tx = x;
	float ty = y;
	float tz = z;
	vec->x = t[0] * tx + t[1] * ty + t[2] * tz + t[3];
	vec->y = t[4] * tx + t[5] * ty + t[6] * tz + t[7];
	vec->z = t[8] * tx + t[9] * ty + t[10] * tz + t[11];
}

_EXPORT void WarVertex::RotateSimple(float *t,WarVertex *vec)
{
	float tx = x;
	float ty = y;
	float tz = z;
	vec->x = t[0] * tx + t[1] * ty + t[2] * tz;
	vec->y = t[4] * tx + t[5] * ty + t[6] * tz;
	vec->z = t[8] * tx + t[9] * ty + t[10] * tz;
}

_EXPORT void WarVertex::Rotate3x3(float *t,WarVertex *vec)
{
	float tx = x;
	float ty = y;
	float tz = z;
	vec->x = t[0] * tx + t[1] * ty + t[2] * tz;
	vec->y = t[3] * tx + t[4] * ty + t[5] * tz;
	vec->z = t[6] * tx + t[7] * ty + t[8] * tz;
}

_EXPORT uint16 WarVertex::CalcZoom(uint8 fov,uint16 screen_x,uint16 screen_y,float *vert_angle,float *horiz_angle)
{
	if (fov>179) fov=179;
	*horiz_angle=fov*3.1415927/360;
	uint16 zoom=(uint16)(screen_x*cos(*horiz_angle)/sin(*horiz_angle));
	*vert_angle=atan(((float)screen_y)/((float)zoom));

	return zoom;
}

_EXPORT void WarVertex::Project()
{
//	x = scr_x + (x * zoom / z);
//	y = scr_y + (y * zoom / z);

	x = 320 + (x * 300 / z);
	y = 160 + (y * 300 / z);
}
