/******************************************************************************/
/* SawteethPlayer Player Interface.                                               */
/*                                                                            */
/* Written by Thomas Neumann.                                                 */
/******************************************************************************/
/* This is part of the APlayer Programming Package (APPP).                    */
/* Copyright (C) 1998-2000 by The APlayer-Team.                               */
/* All rights reserved.                                                       */
/*                                                                            */
/* This source, or parts thereof, may only be used in APlayer related         */
/* software. If you want to use it elsewhere, please contact the author for a */
/* permission.                                                                */
/******************************************************************************/


// PolyKit headers
#include "PString.h"
#include "PFile.h"

// Player headers
#include "SawteethPlayer.h"

#include "txt.h"
#include "song.h"
#include "ins_part_channel.h"

/******************************************************************************/
/* Version                                                                    */
/******************************************************************************/
#define PlayerVersion		1.0


/******************************************************************************/
/* Constructor                                                                */
/******************************************************************************/
SawteethPlayer::SawteethPlayer(APGlobalData *global) : APAddOnPlayer(global)
{
	printf("SawteethPlayer(): global = 0x%08X\n", (int)global);

	// Fill out the version we have compiled under
	addOnVersion  = AP_ADDON_VERSION;
	playerVersion = AP_PLAYER_VERSION;
	agentVersion  = AP_AGENT_VERSION;
	
	m_song = 0;
}



/******************************************************************************/
/* Destructor                                                                 */
/******************************************************************************/
SawteethPlayer::~SawteethPlayer(void)
{
	printf("~SawteethPlayer()\n");
	if (m_song) delete m_song;
}



/******************************************************************************/
/* GetVersion() returns the version of the add-on.                            */
/*                                                                            */
/* Output: The add-on version.                                                */
/******************************************************************************/
float SawteethPlayer::GetVersion(void)
{
	printf("GetVersion()\n");
	return Song::LoaderVersion();
}



/******************************************************************************/
/* GetCount() returns the number of add-ons in the add-on.                    */
/*                                                                            */
/* Output: Is the number of the add-ons.                                      */
/******************************************************************************/
uint32 SawteethPlayer::GetCount(void)
{
	printf("GetCount()\n");
	return 1;
}



/******************************************************************************/
/* GetName() returns the name of the current add-on.                          */
/*                                                                            */
/* Input:  "index" is the add-on index starting from 0.                       */
/*                                                                            */
/* Output: The add-on name.                                                   */
/******************************************************************************/
PString SawteethPlayer::GetName(uint32 index)
{
	printf("GetName(): index = %ld\n", index);
	return "SawteethPlayer";
}



/******************************************************************************/
/* GetDescription() returns the description of the current add-on.            */
/*                                                                            */
/* Input:  "index" is the add-on index starting from 0.                       */
/*                                                                            */
/* Output: The add-on description.                                            */
/******************************************************************************/
PString SawteethPlayer::GetDescription(uint32 index)
{
	printf("GetDescription(): index = %ld\n", index);
	return "Plays Sawteeth songs\nSawteeth is written by Jonas Norberg\nAPlayer plugin is written by Arvid Norberg\n\nhttp://www.acc.umu.se/~bedev/\nsoftware/sawteeth/";
}


/******************************************************************************/
/* GetSupportFlags() returns some flags telling what the add-on supports.     */
/*                                                                            */
/* Output: Is the flags.                                                      */
/******************************************************************************/
uint32 SawteethPlayer::GetSupportFlags(void)
{
	printf("GetSupportFlags()\n");
	return appSetPosition | appSamplePlayer | appUseRingBuffer;
}



/******************************************************************************/
/* GetModTypeString() returns the module type string for the current player.  */
/*                                                                            */
/* Input:  "index" is the player index number.                                */
/*                                                                            */
/* Output: The module type string.                                            */
/******************************************************************************/
PString SawteethPlayer::GetModTypeString(uint32 index)
{
	printf("GetModTypeString(): index = %ld\n", index);
	return "audio/x-sawteeth";
}



/******************************************************************************/
/* ModuleCheck() tests the module to see if it's a Test module.               */
/*                                                                            */
/* Input:  "index" is the player index number.                                */
/*         "file" is a pointer to a APFile object with the file to check.     */
/*                                                                            */
/* Output: An APlayer result code.                                            */
/******************************************************************************/
ap_result SawteethPlayer::ModuleCheck(uint32 index, PFile *file)
{

	printf("ModuleCheck(): index = %ld - file = 0x%08X\n", index, (int)file);

	file->Seek(0,PFile::pSeekBegin);
	
	int32 len = file->GetLength();
	char *buffer = new char[len];
	
	file->Read(buffer,len);
	txt sfile(buffer,len);
	Song *song = new Song(sfile);
	bool ok = (JNG_OK == song->InitCheck());

	delete buffer;
	delete song;
	
	printf("out of ModuleCheck()\n");
	
	if (ok)
		return AP_OK;
	else
		return AP_UNKNOWN;
}



/******************************************************************************/
/* LoadModule() will load the module into the memory.                         */
/*                                                                            */
/* Input:  "index" is the player index number.                                */
/*         "file" is a pointer to a file object with the file to check.       */
/*                                                                            */
/* Output: An APlayer result code.                                            */
/******************************************************************************/
ap_result SawteethPlayer::LoadModule(uint32 index, PFile *file)
{
	if (m_song) delete m_song;

	file->Seek(0,PFile::pSeekBegin);
	int32 len = file->GetLength();
	char *buffer = new char[len];

	file->Read(buffer,len);
	txt sfile(buffer,len);
	m_song = new Song(sfile);
	delete buffer;

	if (m_song->InitCheck() != JNG_OK)
		return AP_UNKNOWN;
	else
		return AP_OK;
}

/******************************************************************************/
/* ExtraLoad() will load all extra files needed.                              */
/*                                                                            */
/* Input:  "fileName" is the filename of the module with full path.           */
/*                                                                            */
/* Output: An APlayer result code.                                            */
/******************************************************************************/
ap_result SawteethPlayer::ExtraLoad(PString fileName)
{
	return AP_OK;
}

/******************************************************************************/
/* InitPlayer() initialize the player.                                        */
/*                                                                            */
/* Output: True for success, false for an error.                              */
/******************************************************************************/
bool SawteethPlayer::InitPlayer(void)
{
	return true;
}

/******************************************************************************/
/* EndPlayer() ends the use of the player.                                    */
/******************************************************************************/
void SawteethPlayer::EndPlayer(void)
{
}

/******************************************************************************/
/* InitSound() initialize the module to play.                                 */
/*                                                                            */
/* Input:  "songNum" is the subsong to play.                                  */
/******************************************************************************/
void SawteethPlayer::InitSound(uint16 songNum)
{
	m_current_buffer = 0;

	virtChannels[0]->SetBuffer(left_buffer[m_current_buffer], 512);
	virtChannels[0]->SetPanning(APPAN_LEFT);
	virtChannels[0]->SetVolume(255);

	virtChannels[1]->SetBuffer(right_buffer[m_current_buffer], 512);
	virtChannels[1]->SetPanning(APPAN_RIGHT);
	virtChannels[1]->SetVolume(255);
}


/******************************************************************************/
/* EndSound() ends the current song.                                          */
/******************************************************************************/
void SawteethPlayer::EndSound(void)
{
}

/******************************************************************************/
/* Play() is the main player function.                                        */
/******************************************************************************/
void SawteethPlayer::Play(void)
{
	/*endReached = */m_song->Play(main_buffer, 512);

	// swap buffer
	m_current_buffer+=1;
	m_current_buffer&=1;

	int16 *left = left_buffer[m_current_buffer];
	int16 *right = right_buffer[m_current_buffer];
	
	float *main = main_buffer;
	while (main < main_buffer + 1024)
	{
		left[0] = (int16)((main[0])*32700.0);
		right[0] = (int16)((main[1])*32700.0);
		main+=2;
		left+=1;
		right+=1;
 	}

	virtChannels[0]->SetBuffer(left_buffer[m_current_buffer], 512);
	virtChannels[1]->SetBuffer(right_buffer[m_current_buffer], 512);
}

/******************************************************************************/
/* GetSamplePlayerInfo() will fill out the sample info structure given.       */
/*                                                                            */
/* Input:  "sampInfo" is a pointer to the sample info structure to fill.      */
/******************************************************************************/
void SawteethPlayer::GetSamplePlayerInfo(SamplePlayerInfo *sampInfo)
{
	sampInfo->bitSize = 16;
	sampInfo->frequency = 44100;
}

/******************************************************************************/
/* GetModuleName() returns the name of the module.                            */
/*                                                                            */
/* Output: Is the module name.                                                */
/******************************************************************************/
PString SawteethPlayer::GetModuleName(void)
{
	return m_song->Name();
}

/******************************************************************************/
/* GetAuthor() returns the name of the author.                                */
/*                                                                            */
/* Output: Is the author.                                                     */
/******************************************************************************/
PString SawteethPlayer::GetAuthor(void)
{
	return m_song->Author();
}

/******************************************************************************/
/* GetVirtualChannels() returns the number of channels the module use.        */
/*                                                                            */
/* Output: Is the number of channels.                                         */
/******************************************************************************/
uint16 SawteethPlayer::GetVirtualChannels(void)
{
	return 2;
}

/******************************************************************************/
/* GetModuleChannels() returns the number of channels the module want to use. */
/*                                                                            */
/* Output: Is the number of required channels.                                */
/******************************************************************************/
uint16 SawteethPlayer::GetModuleChannels(void)
{
	return GetVirtualChannels();
}

/******************************************************************************/
/* GetSubSongs() returns the number of sub songs the module have.             */
/*                                                                            */
/* Output: Is a pointer to a subsong array.                                   */
/******************************************************************************/
uint16 *SawteethPlayer::GetSubSongs(void)
{
	subsongs[0] = 1;
	subsongs[1] = 0;
	return subsongs;
}

/******************************************************************************/
/* GetSongLength() returns the length of the song.                            */
/*                                                                            */
/* Output: The song length.                                                   */
/******************************************************************************/
int16 SawteethPlayer::GetSongLength(void)
{
	return m_song->SimpLen();
}

/******************************************************************************/
/* GetSongPosition() returns the current position in the song.                */
/*                                                                            */
/* Output: The song position.                                                 */
/******************************************************************************/
int16 SawteethPlayer::GetSongPosition(void)
{
	return m_song->GetPos();
}

/******************************************************************************/
/* SetSongPosition() sets a new position in the song.                         */
/*                                                                            */
/* Input:  "pos" is the new song position.                                    */
/******************************************************************************/
void SawteethPlayer::SetSongPosition(int16 pos)
{
	m_song->SetPos(pos);
}

/******************************************************************************/
/* GetInfoCount() returns the number of module info lines the player have.    */
/*                                                                            */
/* Output: The number of extra module info lines.                             */
/******************************************************************************/
uint32 SawteethPlayer::GetInfoCount(void)
{
	return 4;
}

/******************************************************************************/
/* GetInfoString() returns the description or value string on the line given. */
/*                                                                            */
/* Input:  "line" is the line starting from 0.                                */
/*         "type" is the type of string you want.                             */
/*                                                                            */
/* Output: The string you want.                                               */
/******************************************************************************/
PString SawteethPlayer::GetInfoString(uint32 line, ap_infoType type)
{
	PString retStr = "";

	if (type == apDescription)
	{
		// Description strings
		switch (line)
		{
			case 0:
				retStr = "Song Version";
				break;
			case 1:
				retStr = "Channel count";
				break;
			case 2:
				retStr = "Instrument count";
				break;
			case 3:
				retStr = "Part count";
				break;
		}
	}
	else
	{
		// Value strings
		switch (line)
		{
			case 0:
				retStr.Format("%.2f",m_song->Version());
				break;
			case 1:
				retStr.Format("%d",m_song->ChannelCount());
				break;
			case 2:
				retStr.Format("%d",m_song->InsCount());
				break;
			case 3:
				retStr.Format("%d",m_song->PartCount());
				break;
		}
	}

	return retStr;
}

uint32 SawteethPlayer::GetInstrumentCount(void)
{
	return m_song->InsCount()-1;
}

void SawteethPlayer::GetInstrumentInfo(uint32 num, APInstInfo *info)
{
	Ins *ins = m_song->GetIns(num+1);
	
	info->name = ins->name;
	info->flags = 0;
	for (int32 i = 0; i< 10*12; i++) info->notes[i] = -1;
}
