#include "stInstrumentWindow.h"
#include "stCurveView.h"
#include "stTrackerView.h"
#include "stTrackerEntry.h"
#include "stMainWindow.h"
#include "song.h"
#include "stdefs.h"

#include <stdio.h>
#include <Application.h>
#include <Message.h>
#include <Point.h>
#include <CheckBox.h>

#include "layout.h"
#include "MWindow.h"
#include "MBorder.h"
#include "VGroup.h"
#include "HGroup.h"
#include "MButton.h"
#include "MPopup.h"
#include "Space.h"

stInstrumentWindow::stInstrumentWindow(uint8 index, stMainWindow *main_win,BPoint point,Ins *instrument):
	MWindow(BRect(point,point+BPoint(1,1)),instrument->name, B_FLOATING_WINDOW_LOOK, B_FLOATING_SUBSET_WINDOW_FEEL ,B_NOT_RESIZABLE | B_NOT_ZOOMABLE),
	m_index(index),
	m_instrument(instrument),
	m_main_win(main_win)
{

	UpdateTitle();

	field_type type_list[] = {ST_TYPE_INDEX, ST_TYPE_SEPARATOR,ST_TYPE_NOTE, ST_TYPE_SPACE,ST_TYPE_BOOL,ST_TYPE_NIBBLE};

	MPopup *clipping_popup;
	MPopup *boost_popup;
	MPopup *filter_popup;

// ********* set up the loop textcontrol ***************
	char loop_string[10];
	sprintf(loop_string,"%d",m_instrument->loop);

	MBorder *topview = new MBorder
	( M_NO_BORDER,3,0,
		new VGroup
		(
			new HGroup
			(
				new VGroup
				(
					new MTextControl("Name:",m_instrument->name,new BMessage(ST_INS_NAME)),
					new MBorder
					( M_ETCHED_BORDER, 5,0,
						m_amplitude_envelope = new stCurveView("Amplitude", new BMessage(ST_MOVE_AMP_KNOB))
					),
					new HGroup
					(
						new MButton("Add point", new BMessage(ST_ADD_AMP_KNOB)),
						new MButton("Delete point", new BMessage(ST_DELETE_AMP_KNOB)),
						clipping_popup = new MPopup("", new BMessage(ST_CLIPPING), 0, "no clipping", "hard clipping", "sinus clipping", 0),
						boost_popup = new MPopup("", new BMessage(ST_BOOST), 0, "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", 0),
						0
					),
					new Space(minimax(5,5,10,10,1.0)),
					new MBorder
					( M_ETCHED_BORDER, 5,0,
						m_filter_envelope = new stCurveView("Filter", new BMessage(ST_MOVE_FILTER_KNOB))
					),
					new HGroup
					(
						new MButton("Add point", new BMessage(ST_ADD_FILTER_KNOB)),
						new MButton("Delete point", new BMessage(ST_DELETE_FILTER_KNOB)),
						filter_popup = new MPopup("", new BMessage(ST_FILTER), 0, "no filter", "low-pass(no res)", "low-pass", "low-pass2", "high-pass", "band-pass", "band-stop", 0),
						0
					),
					0
				),
				new Space(minimax(5,5,10,10,1.0)),
				new VGroup
				(
					m_tracker_control = new stTrackerControl(m_instrument->len,m_instrument->sps,type_list,sizeof(type_list)>>2,new BMessage(43)),
					m_loop = new MTextControl("loop",loop_string,new BMessage(45)),
					0
				),
				0
			),
			m_resonance = new MSlider("resonance", 0, 255, 1, new BMessage(46), 0, B_TRIANGLE_THUMB),
			m_vibrato_speed = new MSlider("vibrator speed", 0, 255, 1, new BMessage(47), 0, B_TRIANGLE_THUMB),
			m_vibrato_depth = new MSlider("vibrato depth", 0, 255, 1, new BMessage(48), 0, B_TRIANGLE_THUMB),
			m_pwm_speed = new MSlider("pwm speed", 0, 255, 1, new BMessage(49), 0, B_TRIANGLE_THUMB),
			m_pwm_depth = new MSlider("pwm depth", 0, 255, 1, new BMessage(50), 0, B_TRIANGLE_THUMB),
			0
		)
	);

	AddChild(topview);

// ********* set up the Amplitude envelope ***************
	int32 time = m_instrument->amp[0].time;
	m_amplitude_envelope->SetKnobPosition(BPoint((float)time, ((float)m_instrument->amp[0].lev)/2.0),0);
	for (int32 i=1; i < m_instrument->amppoints;i++)
	{
		time += m_instrument->amp[i].time;
		m_amplitude_envelope->AddKnob();
		m_amplitude_envelope->SetKnobPosition(BPoint((float)time, ((float)m_instrument->amp[i].lev)/2.0),i);
	}

// ********* set up the Clipping popup ***************
	clipping_popup->SetActive(m_instrument->clipmode % 3, false);

// ********* set up the Boost popup ***************
	boost_popup->SetActive(m_instrument->boost & 15, false);

// ********* set up the Filter envelope ***************

	time = m_instrument->filter[0].time;
	m_filter_envelope->SetKnobPosition(BPoint((float)time,(float)m_instrument->filter[0].lev/2.0),0);
	for (int32 i=1; i < m_instrument->filterpoints;i++)
	{
		time += m_instrument->filter[i].time;
		m_filter_envelope->AddKnob();
		m_filter_envelope->SetKnobPosition(BPoint((float)time,(float)m_instrument->filter[i].lev/2.0),i);
	}
	
// ********* set up the Filter popup ***************
	filter_popup->SetActive(m_instrument->filtermode % 7, false);

// ********* set up the Tracker ***************
	for (int32 i=0; i< m_instrument->len; i++)
	{
		m_tracker_control->SetFieldContent(i,2, m_instrument->steps[i].note);
		m_tracker_control->SetFieldContent(i,4, m_instrument->steps[i].relative);
		m_tracker_control->SetFieldContent(i,5, m_instrument->steps[i].wform);
	}

// ********* set up the resonance slider ***************

	rgb_color col = {136,136,234};
	m_resonance->UseFillColor(true, &col);
	m_resonance->SetHashMarks(B_HASH_MARKS_BOTTOM);
	m_resonance->SetHashMarkCount(25);
	m_resonance->SetValue(m_instrument->res);

// ********* set up the vibrato speed slider ***************

	m_vibrato_speed->UseFillColor(true, &col);
	m_vibrato_speed->SetHashMarks(B_HASH_MARKS_BOTTOM);
	m_vibrato_speed->SetHashMarkCount(25);
	m_vibrato_speed->SetValue(m_instrument->vibs);

// ********* set up the vibrato depth slider ***************
	
	m_vibrato_depth->UseFillColor(true, &col);
	m_vibrato_depth->SetHashMarks(B_HASH_MARKS_BOTTOM);
	m_vibrato_depth->SetHashMarkCount(25);
	m_vibrato_depth->SetValue(m_instrument->vibd);

// ********* set up the pwm speed slider ***************
	m_pwm_speed->UseFillColor(true, &col);
	m_pwm_speed->SetHashMarks(B_HASH_MARKS_BOTTOM);
	m_pwm_speed->SetHashMarkCount(25);
	m_pwm_speed->SetValue(m_instrument->pwms);

// ********* set up the pwm depth slider ***************

	m_pwm_depth->UseFillColor(true, &col);
	m_pwm_depth->SetHashMarks(B_HASH_MARKS_BOTTOM);
	m_pwm_depth->SetHashMarkCount(25);
	m_pwm_depth->SetValue(m_instrument->pwmd);

	m_zoom = 1;
}

stInstrumentWindow::~stInstrumentWindow()
{
}

bool stInstrumentWindow::QuitRequested()
{
	m_main_win->CloseInsWin(m_instrument);
	return true;
}

void stInstrumentWindow::UpdateTitle()
{
	char * tmp = new char[strlen(m_instrument->name ) + 10];
	sprintf(tmp,"%x.%s", m_index, m_instrument->name);
	SetTitle(tmp);
	delete tmp;
}

void stInstrumentWindow::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
	case ST_ADD_AMP_KNOB:
		{
		if (m_instrument->amppoints == INSPOINTS) return;
		m_amplitude_envelope->AddKnob();
		m_instrument->amppoints++;

			BPoint prev_pos(0.0, 0.0);
			BPoint pos;
			
			for (int32 i=0; i < m_amplitude_envelope->KnobCount(); i++)
			{
				pos = m_amplitude_envelope->KnobPosition(i);
				if (pos.x - prev_pos.x > 255)
				{
					pos = BPoint(prev_pos.x + 255.0,pos.y);
					m_amplitude_envelope->SetKnobPosition(pos,i);
					m_instrument->amp[i].time = 255;
					m_instrument->amp[i].lev = (uint8)(pos.y*2.0);
				}
				else
				{
					m_instrument->amp[i].time = (uint8)(pos.x - prev_pos.x);
					m_instrument->amp[i].lev = (uint8)(pos.y * 2.0);
				}
				prev_pos = pos;
			}

			}break;
		case ST_DELETE_AMP_KNOB:
			{
			if (m_instrument->amppoints == 1) return;
			m_amplitude_envelope->DeleteKnob();
			m_instrument->amppoints--;
			
			BPoint prev_pos(0.0, 0.0);
			BPoint pos;
			
			for (int32 i=0; i < m_amplitude_envelope->KnobCount(); i++)
			{
				pos = m_amplitude_envelope->KnobPosition(i);
				if (pos.x - prev_pos.x > 255)
				{
					pos = BPoint(prev_pos.x + 255.0,pos.y);
					m_amplitude_envelope->SetKnobPosition(pos,i);
					m_instrument->amp[i].time = 255;
					m_instrument->amp[i].lev = (uint8)(pos.y*2.0);
				}
				else
				{
					m_instrument->amp[i].time = (uint8)(pos.x - prev_pos.x);
					m_instrument->amp[i].lev = (uint8)(pos.y * 2.0);
				}
				prev_pos = pos;
			}
			

			}break;
		case ST_MOVE_AMP_KNOB:
			{
			int8 index;
			message->FindInt8("knob",&index);

//			printf("ST_MOVE_KNOB %d\n",index);

			BPoint pos = m_amplitude_envelope->KnobPosition(index);

			BPoint prev_pos;
			if (index > 0) prev_pos = m_amplitude_envelope->KnobPosition(index - 1);
			else prev_pos = BPoint(0.0, 0.0);

			int32 bias = (int32)((pos.x - prev_pos.x) - m_instrument->amp[index].time);

			m_instrument->amp[index].lev = (uint8)(pos.y * 2.0);
			m_instrument->amp[index].time += bias;

			BPoint next_pos = pos;
			while (bias && index < m_amplitude_envelope->KnobCount() - 1)
			{
				index++;
				pos = next_pos;
				next_pos = m_amplitude_envelope->KnobPosition(index);
				
				if (m_instrument->amp[index].time - bias > 255)
				{
					bias = m_instrument->amp[index].time - bias - 255;
					m_instrument->amp[index].time = 255;
					m_amplitude_envelope->SetKnobPosition(next_pos + BPoint(255,0),index);
				}
				else
				{
					m_instrument->amp[index].time -= bias;
					bias = 0;
				}
			}
		
//			printf("time: %d\n",m_instrument->amp[index].time);

			}break;
		case ST_ADD_FILTER_KNOB:
			{
			if (m_instrument->filterpoints == INSPOINTS) return;
			m_filter_envelope->AddKnob();
			m_instrument->filterpoints++;

			BPoint prev_pos(0.0, 0.0);
			BPoint pos;
			
			for (int32 i=0; i < m_filter_envelope->KnobCount(); i++)
			{
				pos = m_filter_envelope->KnobPosition(i);
				if (pos.x - prev_pos.x > 255)
				{
					pos = BPoint(prev_pos.x + 255.0,pos.y);
					m_filter_envelope->SetKnobPosition(pos,i);
					m_instrument->filter[i].time = 255;
					m_instrument->filter[i].lev = (uint8)(pos.y*2.0);
				}
				else
				{
					m_instrument->filter[i].time = (uint8)(pos.x - prev_pos.x);
					m_instrument->filter[i].lev = (uint8)(pos.y * 2.0);
				}
				prev_pos = pos;
			}


			}break;
		case ST_DELETE_FILTER_KNOB:
			{
			if (m_instrument->filterpoints == 1) return;
			m_filter_envelope->DeleteKnob();
			m_instrument->filterpoints--;
			
			BPoint prev_pos(0.0, 0.0);
			BPoint pos;
			
			for (int32 i=0; i < m_filter_envelope->KnobCount(); i++)
			{
				pos = m_filter_envelope->KnobPosition(i);
				if (pos.x - prev_pos.x > 255)
				{
					pos = BPoint(prev_pos.x + 255.0,pos.y);
					m_filter_envelope->SetKnobPosition(pos,i);
					m_instrument->filter[i].time = 255;
					m_instrument->filter[i].lev = (uint8)(pos.y*2.0);
				}
				else
				{
					m_instrument->filter[i].time = (uint8)(pos.x - prev_pos.x);
					m_instrument->filter[i].lev = (uint8)(pos.y * 2.0);
				}
				prev_pos = pos;
			}
			
			}break;
		case ST_MOVE_FILTER_KNOB:
			{
			int8 index;
			message->FindInt8("knob",&index);

//			printf("ST_MOVE_KNOB %d\n",index);

			BPoint pos = m_filter_envelope->KnobPosition(index);

			BPoint prev_pos;
			if (index > 0) prev_pos = m_filter_envelope->KnobPosition(index - 1);
			else prev_pos = BPoint(0.0, 0.0);

			int32 bias = (int32)((pos.x - prev_pos.x) - m_instrument->filter[index].time);

			m_instrument->filter[index].lev = (uint8)(pos.y * 2.0);
			m_instrument->filter[index].time += bias;

			BPoint next_pos = pos;
			while (bias && index < m_filter_envelope->KnobCount() - 1)
			{
				index++;
				pos = next_pos;
				next_pos = m_filter_envelope->KnobPosition(index);
				
				if (m_instrument->filter[index].time - bias > 255)
				{
					bias = m_instrument->filter[index].time - bias - 255;
					m_instrument->filter[index].time = 255;
					m_filter_envelope->SetKnobPosition(next_pos + BPoint(255,0),index);
				}
				else
				{
					m_instrument->filter[index].time -= bias;
					bias = 0;
				}
			}
		
//			printf("time: %d\n",m_instrument->filter[index].time);

			}break;
	case 43:
		{
			uint8 action;
			message->FindInt8("action",(int8*)&action);
			switch (action)
			{
			case ST_SET_LENGTH:
				{
					stTrackerControl *control;
					BTextControl *text;
					message->FindPointer("control",(void**)&control);
					message->FindPointer("source",(void**)&text);
					uint8 len = atoi(text->Text());
					if (len > INSSTEPS) len = INSSTEPS;
					control->SetLength(len);
					m_instrument->len = len;

					if (m_instrument->loop >= m_instrument->len){
						m_instrument->loop = m_instrument->len -1; 

						char temp[10];
						sprintf(temp,"%d",m_instrument->loop);
						m_loop->SetText(temp);
					}					
					
					for (int32 i=0; i< m_instrument->len; i++)
					{
						m_tracker_control->SetFieldContent(i,2, m_instrument->steps[i].note);
						m_tracker_control->SetFieldContent(i,4, m_instrument->steps[i].relative);
						m_tracker_control->SetFieldContent(i,5, m_instrument->steps[i].wform);
					}
					
				} break;
			case ST_SET_SPEED:
				{
					stTrackerControl *control;
					BTextControl *text;
					message->FindPointer("control",(void**)&control);
					message->FindPointer("source",(void**)&text);
					uint8 speed = atoi(text->Text());
					if (speed == 0) speed = 1;
					m_instrument->sps = speed;
				} break;
			}
		} break;
	case ST_ENTRY_CHANGED:
		{
			int8 row,field,val;
			message->FindInt8("field",&field);
			message->FindInt8("row",&row);
			message->FindInt8("value",&val);

//			printf("ENTRY_CHANGED: %d, %d, %d\n",field,row,val);
	
			switch(field)
			{
				case 2:
					m_instrument->steps[row].note = val;
				break;
				case 4:
					m_instrument->steps[row].relative = val;
				break;
				case 5:
					m_instrument->steps[row].wform = val;
				break;
			}
		}break;

	case ST_INS_NAME:{
		BTextControl *tc;
		message->FindPointer("source",(void**)&tc);
		m_instrument->SetName(tc->Text());
		m_main_win->PostMessage(ST_INS_CHANGED);
		UpdateTitle();
	} break;

	case ST_CLIPPING:{
		m_instrument->clipmode = 15 & message->FindInt32("index");
	} break;

	case ST_BOOST:{
		m_instrument->boost = 15 & message->FindInt32("index");
	} break;
	
	case 44:
		{
			int32 index;
			message->FindInt32("index", &index);
			m_instrument->filtermode = index;
		} break;
	case 45:
		{
		BTextControl *tc;
		message->FindPointer("source",(void**)&tc);
		uint8 value = atoi(tc->Text());
		
		if (value >= m_instrument->len) value = m_instrument->len -1; 

		m_instrument->loop = value;
		char tmp[10];
		sprintf(tmp,"%d",value);
		tc->SetText(tmp);
		
		} break;
	case 46:
		{
			m_instrument->res = m_resonance->Value();
		} break;
	case 47:
		{
			m_instrument->vibs = m_vibrato_speed->Value();
		} break;
	case 48:
		{
			m_instrument->vibd = m_vibrato_depth->Value();
		} break;
	case 49:
		{
			m_instrument->pwms = m_pwm_speed->Value();
		} break;
	case 50:
		{
			m_instrument->pwmd = m_pwm_depth->Value();
		} break;
	}
}
