#include <stdio.h>
#include <stdlib.h>

#include "stPartWindow.h"
#include "stCurveView.h"
#include "stTrackerView.h"
#include "stTrackerEntry.h"
#include "stApp.h"
#include "song.h"

#include "MWindow.h"
#include "layout.h"
#include "MBorder.h"
#include "VGroup.h"

stPartWindow::stPartWindow(BPoint point,Part *part_to_edit, Song *song, stMainWindow *win):
	MWindow(BRect(point,point+BPoint(30,300)),"", B_FLOATING_WINDOW_LOOK, B_FLOATING_SUBSET_WINDOW_FEEL , B_NOT_H_RESIZABLE)
{
	main_win = win;
	part=part_to_edit;

	s = song;

	BMessage *mess = new BMessage(ST_PPLAY_START);
	AddShortcut(' ',B_COMMAND_KEY, mess);


	field_type type_list[]={ST_TYPE_INDEX, ST_TYPE_SEPARATOR,ST_TYPE_SPACE,ST_TYPE_NOTE, ST_TYPE_SPACE,ST_TYPE_BLANK_NIBBLE,ST_TYPE_NIBBLE,ST_TYPE_SPACE,ST_TYPE_BLANK_NIBBLE,ST_TYPE_NIBBLE,ST_TYPE_SPACE};

	MBorder *topview =
	new MBorder(M_NO_BORDER, 1, 0,
		new VGroup
		(
			name_string = new MTextControl("Name:", part->name, new BMessage(ST_NEW_PART_NAME)),
			tracker_control=new stTrackerControl(part->len,part->sps,type_list,sizeof(type_list)>>2,new BMessage(MSG_FROM_TRACKER)),
			0
		)
	);

	for (uint16 i=0;i<part->len;i++)
	{
		tracker_control->SetFieldContent(i,3,part->steps[i].note);

		tracker_control->SetFieldContent(i,5,(part->steps[i].ins & 0xf0) >> 4);
		tracker_control->SetFieldContent(i,6,part->steps[i].ins & 0xf);

		tracker_control->SetFieldContent(i,8,(part->steps[i].eff & 0xf0) >> 4);
		tracker_control->SetFieldContent(i,9,part->steps[i].eff & 0xf);
	}

	AddChild(dynamic_cast<BView*>(topview));
	UpdateTitle();
}

stPartWindow::~stPartWindow()
{
}

bool stPartWindow::QuitRequested()
{
	main_win->ClosePartWin(part);
	return true;
}

void stPartWindow::UpdateTitle()
{
	char * tmp = new char[strlen(part->name) + 20];
	sprintf(tmp,"%ld.%s", part - s->GetPart(0), part->name);
	SetTitle(tmp);
	delete tmp;
}

void stPartWindow::MessageReceived(BMessage *message)
{

	switch(message->what)
	{
		case ST_PPLAY_START:
		{
			BMessage *mess = new BMessage(ST_PPLAY_START);
			int16 temp = part - s->GetPart(0);
			mess->AddInt16("part",temp);
			mess->AddInt16("step",tracker_control->TrackerView()->CurrentRow());

			main_win->PostMessage( mess );
		}
		case ST_NEW_PART_NAME:
		{
			part->SetName((char*)name_string->Text());
			UpdateTitle();
			main_win->PostMessage(ST_PART_CHANGED);
		}
		case MSG_FROM_TRACKER:
		{
			uint8 action;
			message->FindInt8("action",(int8*)&action);
			switch (action)
			{
				case ST_SET_LENGTH:
				{
					stTrackerControl *control;
					BTextControl *text;
					message->FindPointer("control",(void**)&control);
					message->FindPointer("source",(void**)&text);
					control->SetLength(atoi(text->Text()));

					char tmp[10];
					sprintf(tmp,"%d",atoi(text->Text()));
					text->SetText(tmp);

					part->len = atoi(text->Text());
					
					for (uint16 i=0;i<part->len;i++)
					{
						tracker_control->SetFieldContent(i,3,part->steps[i].note);

						tracker_control->SetFieldContent(i,5,(part->steps[i].ins & 0xf0) >> 4);
						tracker_control->SetFieldContent(i,6,part->steps[i].ins & 0xf);

						tracker_control->SetFieldContent(i,8,(part->steps[i].eff & 0xf0) >> 4);
						tracker_control->SetFieldContent(i,9,part->steps[i].eff & 0xf);
					}
					main_win->PostMessage(ST_PART_CHANGED);
				}break;
				case ST_SET_SPEED:
				{
					stTrackerControl *control;
					BTextControl *text;
					message->FindPointer("control",(void**)&control);
					message->FindPointer("source",(void**)&text);
					part->sps = atoi(text->Text());
					if (part->sps < 1) part->sps = 1;

					char tmp[10];
					sprintf(tmp,"%d",part->sps);
					text->SetText(tmp);

					main_win->PostMessage(ST_PART_CHANGED);
				} break;
			}
		}break;
		case ST_ENTRY_CHANGED:
			{
			int8 row,field,val;
			message->FindInt8("field",&field);
			message->FindInt8("row",&row);
			message->FindInt8("value",&val);
//			printf("ST_ENTRY_CHANGED (%d,%d)=%d\n",field,row,val);
			switch(field)
			{
				case 3: // note
				{
					part->steps[row].note = val;
					s->SetPartPos(row);
				}break;
				case 5:
				{
					part->steps[row].ins &= 0xf;
					part->steps[row].ins |= val << 4;
				 // protection
					if (part->steps[row].ins >= s->InsCount())
					{
						part->steps[row].ins = s->InsCount()-1;
						tracker_control->SetFieldContent(row,5,(part->steps[row].ins & 0xf0) >> 4);
						tracker_control->SetFieldContent(row,6,part->steps[row].ins & 0xf);
					}
				}break;
				case 6:
				{
					part->steps[row].ins &= 0xf0;
					part->steps[row].ins |= val;
				 // protection
					if (part->steps[row].ins >= s->InsCount())
					{
						part->steps[row].ins = s->InsCount()-1;
						tracker_control->SetFieldContent(row,5,(part->steps[row].ins & 0xf0) >> 4);
						tracker_control->SetFieldContent(row,6,part->steps[row].ins & 0xf);
					}
				}break;
				case 8:
				{
					part->steps[row].eff &= 0xf;
					part->steps[row].eff |= val << 4;
				}break;
				case 9:
				{
					part->steps[row].eff &= 0xf0;
					part->steps[row].eff |= val;
				}break;
			}
		} break;
		default:
/*
			DetachCurrentMessage();
			main_win->PostMessage(message);
*/
			MWindow::MessageReceived(message);
	}
}

