#define vic2_base ((unsigned char *)0xd000)

#define vic2_sprite_enable *((unsigned char *)0xd015)
#define vic2_sprite_msb *((unsigned char *)0xd010)
#define vic2_raster_position *((unsigned char *)0xd012)
#define vic2_sprite_expand_vertical *((unsigned char *)0xd017)
#define vic2_sprite_expand_horizontal *((unsigned char *)0xd01d)

#include "sprites.h"


/* a bitmask indicating the sprites that are allocated */
static unsigned char g_allocated_sprites = 0;

sprite_t alloc_sprite(void)
{
	char c;
	for (c = 0; c < 8; c++)
	{
		char mask = 1 << c;
		if ((g_allocated_sprites & mask) == 0)
		{
			g_allocated_sprites |= mask;
			hide_sprite(c);
			move_sprite(c, 0, 0);
			shrink_sprite_x(c);
			shrink_sprite_y(c);
			return c;
		}
	}
	return -1;
}

void free_sprite(sprite_t s)
{
	hide_sprite(s);
	g_allocated_sprites &= ~(1 << s);
}

void move_sprite_x(sprite_t s, unsigned int x)
{
	vic2_base[(s<<1)] = (unsigned char)x;

	/* set or clear the MSB of the X coordinate */
	if (x >> 8) vic2_sprite_msb |= (1 << s);
	else vic2_sprite_msb &= ~(1 << s);
}

void move_sprite_y(sprite_t s, unsigned char y)
{
	vic2_base[(s<<1)+1] = y;
}

void move_sprite(sprite_t s, unsigned int x, unsigned char y)
{
	vic2_base[(s<<1)] = (unsigned char)x;
	vic2_base[(s<<1)+1] = y;

	/* set or clear the MSB of the X coordinate */
	if (x >> 8) vic2_sprite_msb |= (1 << s);
	else vic2_sprite_msb &= ~(1 << s);
}

void expand_sprite(sprite_t s)
{
	register unsigned char a = (1 << s);
	vic2_sprite_expand_vertical |= a;
	vic2_sprite_expand_horizontal |= a;
}

void shrink_sprite(sprite_t s)
{
	register unsigned char a = ~(1 << s);
	vic2_sprite_expand_vertical &= a;
	vic2_sprite_expand_horizontal &= a;
}

void expand_sprite_x(sprite_t s)
{
	vic2_sprite_expand_horizontal |= (1 << s);
}

void expand_sprite_y(sprite_t s)
{
	vic2_sprite_expand_vertical |= (1 << s);
}

void shrink_sprite_x(sprite_t s)
{
	vic2_sprite_expand_horizontal &= ~(1 << s);
}

void shrink_sprite_y(sprite_t s)
{
	vic2_sprite_expand_vertical &= ~(1 << s);
}



/******************************/


unsigned char sprite_y(sprite_t s)
{
	return vic2_base[(s<<1)+1];
}

unsigned int sprite_x(sprite_t s)
{
	return (vic2_sprite_msb & (1 << s)) ? vic2_base[s<<1] + 256 : vic2_base[s<<1];
}

/* returns 0 if the sprite is hidden, another value if the sprite is visible */
char is_sprite_visible(sprite_t s)
{
	return (vic2_sprite_enable & (1 << s));
}

void hide_sprite(sprite_t s)
{
	vic2_sprite_enable &= ~(1 << s);
}

void show_sprite(sprite_t s)
{
	vic2_sprite_enable |= 1 << s;
}
