#ifndef _SPRITE_H_
#define _SPRITE_H_

typedef signed char sprite_t;

/*
	allocates a sprite, if there are no sprites available, this function returns -1
	if you allocate a sprite, you must free it to make it available to others
*/
extern sprite_t alloc_sprite(void);

/* free the sprite */
extern void free_sprite(sprite_t s);

/*
	moves the sprite only along the x-axis
	if you just want to change the x-coordinate
	of the sprite this function is slightly faster than
	move_sprite() because this doesn't touch the y
	coordinate.
*/
extern void move_sprite_x(sprite_t s, unsigned int x);

/*
	moves the sprite only along the y-axis
	if you just want to change the y-coordinate
	of the sprite this function is faster than
	move_sprite() because this doesn't touch
	the x-coordinate (and the x-coordinates
	needs some calculations in order to be set)
*/
extern void move_sprite_y(sprite_t s, unsigned char y);

/*
	moves the sprite to the given location
*/
extern void move_sprite(sprite_t s, unsigned int x, unsigned char y);

/*
	expands the sprite in both x and y directions
*/
extern void expand_sprite(sprite_t s);

/*
	shrinks the sprite in both x and y directions (to normal size)
*/
extern void shrink_sprite(sprite_t s);

/*
	expands the sprite x
*/
extern void expand_sprite_x(sprite_t s);

/*
	expandss the sprite y
*/
extern void expand_sprite_y(sprite_t s);

/*
	shrinks the sprite x (to normal size)
*/
extern void shrink_sprite_x(sprite_t s);

/*
	shrink the sprite y (to normal size)
*/
extern void shrink_sprite_y(sprite_t s);


extern void set_multicolor(sprite_t s);

extern void set_singlecolor(sprite_t s);


/******************************/


/*
	returns the y position of the sprite
*/
extern unsigned char sprite_y(sprite_t s);

/*
	returns the x position of the sprite
*/
extern unsigned int sprite_x(sprite_t s);

/*
	returns 0 if the sprite is hidden, -1 of it is visible
*/
extern char is_sprite_visible(sprite_t s);

/*
	hides the sprite
*/
extern void hide_sprite(sprite_t s);

/*
	shows the sprite
*/
extern void show_sprite(sprite_t s);



#endif // _SPRITE_H_
