#include <stdio.h>
#include <stdlib.h>

#include <Bitmap.h>
#include <TranslationUtils.h>
#include <GraphicsDefs.h>
#include <Application.h>
#include <Entry.h>
#include <File.h>

bool convert_bitmap(const char *file_name,uint16 a_width,uint16 a_height,uint16 num_frames,BFile *outfile,uint8 bytes_per_pixel, bool lines);

class MyApp:public BApplication
{
public:
	MyApp(int argc,char **argv);
};

int main(int argc,char **argv)
{
	MyApp *app=new MyApp(argc,argv);
}

MyApp::MyApp(int argc,char **argv):BApplication("application/x-converter-vnd.Arvid Norberg")
{
	if (argc!=8)
	{
		printf("USAGE: convert image-file block_width block_height num_blocks output-file bytes_per_pixel lines_or_columns\n");
		be_app->PostMessage(B_QUIT_REQUESTED);
		return;
	}

	BEntry entry(argv[5]);
	BFile outfile(&entry,B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
	if (!(outfile.InitCheck()==B_NO_ERROR))
	{
		printf("Error opening output file \"%d\"\n",argv[5]);
		be_app->PostMessage(B_QUIT_REQUESTED);
		return;
	}

	if (!convert_bitmap(argv[1],atoi(argv[2]),atoi(argv[3]),atoi(argv[4]),&outfile,atoi(argv[6]),(bool)atoi(argv[7])))
	{
		printf("Error converting file \"blocks\"!\n");
		be_app->PostMessage(B_QUIT_REQUESTED);
		return;
	}

	printf("Finished! exiting.\n");
	be_app->PostMessage(B_QUIT_REQUESTED);
}

bool convert_bitmap(const char *file_name,uint16 a_width,uint16 a_height,uint16 num_frames,BFile *outfile,uint8 bytes_per_pixel, bool lines)
{
	BBitmap *bitmap;
	BRect bounds;
	uint16 frame;
	uint16 width,height,rowbytes;
	uint32 tot_write=0;
	rgb_color *ptr;
	uint16 blocks=0;
	uint16 px;
	uint16 py;
	uint16 b_x;
	uint16 b_y;
	int16 col;
	uint8 r,g,b,block_set;

	printf("loading \"%s\"\n",file_name);
	bitmap=BTranslationUtils::GetBitmap(file_name);
	if (!bitmap)
	{
		printf("No translator for \"%s\"!\n",file_name);
		return false;
	}
	ptr=(rgb_color*)bitmap->Bits();
	bounds=bitmap->Bounds();
	width=bounds.IntegerWidth()+1;
	height=bounds.IntegerHeight()+1;
	rowbytes=bitmap->BytesPerRow();

	b_x=0;
	b_y=0;

	tot_write=0;

	printf("blocks: width:%d height:%d\n",a_width,a_height);
	printf("image: width:%d height:%d\n",width,height);

	rgb_color tmp_pixel;

	for (frame=0;frame<num_frames;frame++)
	{
		if (lines)
		{
			if ((b_x*a_width)>=width)
			{
				b_x=0;
				b_y++;
			}
			for (py=0;py<a_height;py++)
				for (px=0;px<a_width;px++)
				{
					tmp_pixel.blue=ptr[((b_y*a_height)+py)*width+(b_x*a_width)+px].red;
					tmp_pixel.green=ptr[((b_y*a_height)+py)*width+(b_x*a_width)+px].green;
					tmp_pixel.red=ptr[((b_y*a_height)+py)*width+(b_x*a_width)+px].blue;
//					tmp_pixel.alpha=(tmp_pixel.red+tmp_pixel.green+tmp_pixel.blue)/3;
					tmp_pixel.alpha=(tmp_pixel.red==0 && tmp_pixel.green==255 && tmp_pixel.blue==0)?0:255;

				//*****
//					tmp_pixel.red=(tmp_pixel.red+tmp_pixel.green+tmp_pixel.blue)/3;
				//*****
				
					outfile->Write(&tmp_pixel,bytes_per_pixel);
					tot_write+=bytes_per_pixel;
				}
			b_x++;
		}
		else
		{
			if ((b_y*a_height)>=height)
			{
				b_y=0;
				b_x++;
			}
			for (py=0;py<a_height;py++)
				for (px=0;px<a_width;px++)
				{
					tmp_pixel.blue=ptr[((b_y*a_height)+py)*width+(b_x*a_width)+px].red;
					tmp_pixel.green=ptr[((b_y*a_height)+py)*width+(b_x*a_width)+px].green;
					tmp_pixel.red=ptr[((b_y*a_height)+py)*width+(b_x*a_width)+px].blue;
//					tmp_pixel.alpha=(tmp_pixel.red+tmp_pixel.green+tmp_pixel.blue)/3;
					tmp_pixel.alpha=(tmp_pixel.red==0 && tmp_pixel.green==255 && tmp_pixel.blue==0)?0:255;

				//*****
//					tmp_pixel.red=(tmp_pixel.red+tmp_pixel.green+tmp_pixel.blue)/3;
				//*****
				
					outfile->Write(&tmp_pixel,bytes_per_pixel);
					tot_write+=bytes_per_pixel;
				}
			b_y++;
		}
	}
	printf("\nwrote %d*%d*%d*%d=%d bytes\n",num_frames,a_width,a_height,bytes_per_pixel,tot_write);
	delete bitmap;
	return true;
}
