#include "insply.h"
#include "song.h"
#include <stdio.h>

InsPly::InsPly(Song *s)
{
	song = s;
	vib=new Wave(0,3);
	pwm=new Wave(0,3);
	w=new Wave();
	ins=&song->ins[0];
	currins=ins;	
	steps=(currins->steps);

	insfreq = currpartfreq = 440.0;
	currpartamp = currpartco = amp = currf = cutoff = 1.0;
	curr = res = last = llast = fstep = 0.0;

	TrigADSR(0);
}

void InsPly::TrigADSR(uint8 i)
{
//	fprintf(stderr,"InsPly::TrigADSR(%d)\n",i);

	currins = ins+i;
	steps = (currins->steps);

	vib->SetFreq(ins[i].vibs * 100);
	vamp = ins[i].vibd / 1000.0;

	pwm->SetFreq(ins[i].pwms * 10);
	pamp = ins[i].pwmd / 255.1;

	res = 1- ins[i].res / 255.0;
	res = 1 - (res * res * res);

//	amp adsr
	adsr=0;
	nextadsr=0;

//	filter adsr
	fadsr=0;
	nextfadsr=0;

//	step
	stepc=0;
	nexts=0;
}


void InsPly::SetAmp(float a)
{
//	fprintf(stderr,"InsPly::SetAmp(%f)\n",a);
	currpartamp=a;
}

void InsPly::SetFreq(float f)
{
//	fprintf(stderr,"InsPly::SetFreq(%f)\n",f);
	currpartfreq=f;
}

void InsPly::SetCutOff(float co)
{
//	fprintf(stderr,"InsPly::SetCutOff(%f)\n",co);
	currpartco=co;
}

void InsPly::PAL()
{
//fprintf(stderr,"InsPly::PAL()\n");
// checkinsstep
	if (nexts < 1){

//fprintf(stderr,"0.steps=%p nexts=%d stepc=%d\n",steps,nexts,stepc);

		currstep = steps + stepc;

		nexts=currins->sps;

//fprintf(stderr,"1.currstep=%p\n",currstep);
		if (currstep->note){
//fprintf(stderr,"2.currstep->note=%d\n",currstep->note);
			if (currstep->mask & 0x10){
				insfreq = song->R2F[currstep->note];
			}else{
				insfreq = song->N2F[currstep->note];
			}
		}

		if (currstep->mask & 0xf){
			w->SetForm(currstep->mask & 0xf);
		}
		stepc++;

		if (stepc >= (127 & currins->len)){
			if ((128 & currins->len) !=0)
				stepc=0;
			else
				stepc--;
		}
	}
	
// checkadsr
	if (nextadsr < 1){
		if (adsr < currins->amppoints){
			nextadsr= currins->amp[adsr].time;
			if (0==nextadsr) nextadsr=1;	//protection
			ampstep= ((currins->amp[adsr].lev/257.0)-curramp) / nextadsr;
			adsr++;
		}
		else
			ampstep=0.0;
	}

// checkfadsr
	if (nextfadsr < 1){
		if (fadsr < currins->filterpoints){
			nextfadsr= currins->filter[fadsr].time;
			if (0==nextfadsr) nextfadsr=1;	//protection
			float target = currins->filter[fadsr].lev/257.0;
			target *= target * target;
			fstep= (target-currf) / nextfadsr;
			fadsr++;
		}
		else
			fstep=0.0;
	}


//	adsr
	nextadsr--;
	nextfadsr--;

//	step
	nexts--;

//	Freq
	if (currstep->mask & 0x10)
		w->SetFreq(currpartfreq * insfreq * (1 +  vamp * vib->Next()));		
	else
		w->SetFreq(insfreq * (1 + vamp * vib->Next()));		

//	PWM
	if (currins->pwmd)	w->SetPWM(pamp * pwm->Next());
	
//	Amp
	curramp+=ampstep;
	amp=curramp*currpartamp;
//	Filter
	currf+=fstep;
	cutoff=currf*currpartco;
//fprintf(stderr,"insply:\nca:%f as:%f cpa:%f amp:%f\ncf:%f fs:%f cpc:%f cut:%f\n",curramp,ampstep,currpartamp,amp,currf,	fstep,	currpartco,	cutoff);
}

float InsPly::Next()
{

	// filter
	curr = w->Next();
//	fprintf(stderr,"\n%p - %f, %f, %f, %f, %f, %f\n",this,curr,amp,cutoff,last,llast,res);

	curr = amp * cutoff * curr + (1-cutoff)*last;
	curr+= (curr-llast)*res;

	llast= last;
	last = curr;

	if (curr < -1.0) curr = -1.0;
	else if (curr > 1.0) curr = 1.0;
	
//	fprintf(stderr,"curr = %f\n",curr);
	return curr;

};

