#include <stdio.h>
#include "wave.h"

#define ABS(x) ((x)>0)?(x):(-x)
/////////////////////////////////////////
//	prototypes
unsigned long jng_rand();
/////////////////////////////////////////
//	own random-function
#define BIGMOD 0x7fffffff
#define W 127773
#define C 2836
long jng_seed=1;

unsigned long jng_rand()
{
	jng_seed = 16807*(jng_seed % W) - (jng_seed/W)*C;
	if (jng_seed < 0) jng_seed+=BIGMOD;
	return jng_seed;
}



Wave::Wave(float freq, uint8 waveform)
{
	curr = pwm = step = step2 = stepdet = currval = currval2 = currvaldet = 0.0;

	SetForm(waveform);
	SetFreq(freq);
}

Wave::Wave()
{
	curr = pwm = step = step2 = stepdet = currval = currval2 = currvaldet = 0.0;

	SetForm(SAW);
	SetFreq(1760);
}


void Wave::SetFreq(float freq)
{
	step = freq * .00002267573696145124;
	step2 = step * 1.999;
	stepdet = step * 0.999;
//	fprintf(stderr,"(Wave)freq = %f\n",freq);
//	fprintf(stderr,"      step = %f\n",step);
}

void Wave::SetPWM(float p)
{
//	fprintf(stderr,"(Wave)pwm = %f\n",p);
	pwm=p;
}

void Wave::SetForm(uint8 waveform){
//	fprintf(stderr,"(Wave)form = %d\n",waveform);
	if (currval > 2) currval = currval2 = currvaldet = 0.0;// limit
	form = waveform;
}

float Wave::Next()
{
	switch (form){
	case SAW:	curr= currval; // saw
				currval += step;
				if (currval>1.0) currval-=2.0;
				break;
				
	case SQR:	curr= (pwm < currval)?(-1):(1); // pulse
				currval += step;
				if (currval>1.0) currval-=2.0;
				break;
				
	case NOS:	currval += step;
				if (currval>1.0){
					currval-=2.0;
					curr= (jng_rand()/(64.0*256.0*256.0*256.0))-1;
				}
				break;

	case TRI:	curr=2*((currval> 0 )?(-currval):(currval))+1.0;
				currval += step;
				if (currval>1.0) currval-=2.0;
				break;

	case SAW2:	curr=(currval+currval2)/2;
				currval += step;
				if (curr>1.0) currval-=2.0;
				currval2 += step2;
				if (curr>1.0) currval2-=2.0;
				break;

	case DETSAW:curr=(currval+currvaldet)/2;
				currval += step;
				if (currval>1.0) currval-=2.0;
				currvaldet += stepdet;
				if (currvaldet>1.0) currvaldet-=2.0;
				break;

	case SQR2:	if (currval>pwm)
					curr = 0.6;
				else
					curr = -0.6;
				
				if (currval2<pwm)
					curr += 0.4;
				else
					curr += -0.4;
					
				currval += step;
				if (currval>1.0) currval-=2.0;
				currval2 += step2;
				if (currval2>1.0) currval2-=2.0;
				break;

	case DETSQR:if (currval>pwm)
					curr = 0.6;
				else
					curr = -0.6;
				
				if (currvaldet<pwm)
					curr += 0.4;
				else
					curr += -0.4;
					
				currval += step;
				if (currval>1.0) currval-=2.0;
				currvaldet += stepdet;
				if (currvaldet>1.0) currvaldet-=2.0;
				break;

	default:	return 0;
	}

//printf("%f\n",curr);
	return curr;
}

