#ifndef _dGraphic_
#define _dGraphic_

#include "dColor.h"
#include "dFace.h"
#include "dObject.h"
#include "dClut2.h"

#include <Debug.h>
#include <Directory.h>
#include <stdlib.h>

class dGraphic
{
public:
	dGraphic(int32 width,int32 height);
	dGraphic(BPositionIO *file);
	virtual ~dGraphic();

	int32 Width();
	int32 Height();

	dColor *Bits();

	void Empty();

	void swap(int32 &val1,int32 &val2);

	void Read(BFile *file);

// –––––––––––––––––––––––––––––––– 2D stuff

	void PutPixel(uint32 x,uint32 y,dColor color);
	void PutSubPixel(dVertex *dot,dClut2 *clut); 				// OBS uint32 is 16:16 fixed point
	void AddSubPixel(dVertex *dot);
	void DrawSubPixelPoints(dObject *obj,dClut2 *clut); 		// the object should be rotated & projected

	void AddLine(dVertex *start,dVertex *end);
	void LineClut(dVertex *start,dVertex *end,dClut2 *clut);

	void DrawBitmap(int32 x,int32 y,dGraphic *bitmap);
	void DrawBitmapAdd(int32 x,int32 y,dGraphic *bitmap);
	void DrawBitmap(int32 x,int32 y,dGraphic *bitmap,dGraphic *stencil_bitmap,uint8 value);
	void DrawBitmap(int32 x,int32 y,dGraphic *bitmap,dGraphic *stencil_bitmap,uint8 value,dColor border);

	void DrawFaceTextured(dFace *face,dGraphic *texture);
	void DrawFaceTexturedClut(dFace *face,dGraphic *texture,dClut2 *clut);
	void DrawFaceFlat(dFace *face,dColor col);
	void DrawFaceFlatClut2(dFace *face,dColor color,dClut2 *clut);

private:
// the width and the height of the bitmap
	int32 m_width,m_height;
// the clip-region
	int16 m_clp_left, m_clp_top, m_clp_right, m_clp_bottom;

// the actual bitmap
	dColor *m_buffer;
private:
	void HorizLineTextured(dColor *line_pointer,int32 linestart,int32 lineend,dColor *texture,int32 u,int32 v,int32 add_u,int32 add_v);
	void HorizLineTexturedClut(dColor *line_pointer,int32 linestart,int32 lineend,dColor *texture,int32 u,int32 v,int32 add_u,int32 add_v,dClut2 *clut);
	void HorizLineFlat(dColor *line_pointer,int32 linestart,int32 lineend,dColor color);
	void HorizLineFlatClut2(dColor *line_pointer,int32 linestart,int32 lineend,dColor color,dClut2 *clut);
};

inline int32 dGraphic::Width()
{
	ASSERT(this != NULL);

	return m_width;
}

inline int32 dGraphic::Height()
{
	ASSERT(this != NULL);

	return m_height;
}

inline dColor *dGraphic::Bits()
{
	ASSERT(this != NULL);

	return m_buffer;
}

inline void dGraphic::PutPixel(uint32 x,uint32 y,dColor color)
{
	ASSERT(this != NULL);
	ASSERT(x < Width());
	ASSERT(y < Height());
	
	m_buffer[y * Width() + x] = color;
}

inline void dGraphic::PutSubPixel(dVertex *point,dClut2 *clut)
{
	int32 x = point->x * 65535.0f;
	int32 y = point->y * 65535.0f;

	if ((x>>16) < 0) return;
	if ((y>>16) < 0) return;

	if ((x>>16) > Width()-2)	return;
	if ((y>>16) > Height()-2)	return;
	
	dColor *p00 = m_buffer + ( (y>>16) * Width() ) + (x>>16);
	dColor *p01 = p00 + 1;
	dColor *p10 = p00 + Width();
	dColor *p11 = p10 + 1;
	
	uint32 x1 = (x >> 8) & 255;
	uint32 x0 = 255 - x1;
	uint32 y1 = (y >> 8) & 255;
	uint32 y0 = 255 - y1;

	*p00 = p00->Blend(clut,dColor((x0 * y0) >> 8));
	*p01 = p01->Blend(clut,dColor((x1 * y0) >> 8));
	*p10 = p10->Blend(clut,dColor((x0 * y1) >> 8));
	*p11 = p11->Blend(clut,dColor((x1 * y1) >> 8));
}

inline void dGraphic::AddSubPixel(dVertex *point)
{
	uint32 x = point->x * 65535.0;
	uint32 y = point->y * 65535.0;

	if ((x>>16) > Width()-2)	return;
	if ((y>>16) > Height()-2)	return;

	dColor *p00 = m_buffer + ( (y>>16) * Width() ) + (x>>16);
	dColor *p01 = p00 + 1;
	dColor *p10 = p00 + Width();
	dColor *p11 = p10 + 1;
	
	if ( ((x >> 8) & 63) & ((y >> 8) & 63) == 0)
	{
		*p00 = dColor(255);
		*p01 = dColor(255);
		p01[1] = dColor(255);
		p01[640] = dColor(255);
		p01[641] = dColor(255);

		*p10 = dColor(255);
		*p11 = dColor(255);
		p11[1] = dColor(255);
		p11[640] = dColor(255);
		p11[641] = dColor(255);

		return;
	}
	
	uint32 x1 = (x >> 8) & 255;
	uint32 x0 = 255 - x1;
	uint32 y1 = (y >> 8) & 255;
	uint32 y0 = 255 - y1;

	*p00 = *p00 + dColor((x0 * y0) >> 8);
	*p01 = *p01 + dColor((x1 * y0) >> 8);
	*p10 = *p10 + dColor((x0 * y1) >> 8);
	*p11 = *p11 + dColor((x1 * y1) >> 8);

}

inline void dGraphic::DrawSubPixelPoints(dObject *obj,dClut2 *clut)
{
	for (int32 c = 0 ; c < obj->VertexCount() ; c++ )
		PutSubPixel(obj->ProjectedVertex(c),clut);
}

inline void dGraphic::Empty()
{
	double *frame = (double*)m_buffer;
	double *frame_stop = (double*)(m_buffer + Width() * Height());

	while(frame<frame_stop)
	{
		frame[0] = 0;
		frame[1] = 0;
		frame[2] = 0;
		frame[3] = 0;
		frame[4] = 0;
		frame[5] = 0;
		frame[6] = 0;
		frame[7] = 0;
		frame += 8;
	}
}

inline void dGraphic::swap(int32 &val1,int32 &val2)
{
	register int32 tmp=val1;
	val1=val2;
	val2=tmp;
}

/*
0									linestart								lineend
|										|											|
V										V											V
..............................*********************************........................
line_pointer
*/

inline void dGraphic::HorizLineTextured(dColor *line_pointer,int32 linestart,int32 lineend,dColor *texture,int32 u,int32 v,int32 add_u,int32 add_v)
{
	if (linestart < m_clp_left)
	{
		u += (m_clp_left - linestart) * add_u;
		v += (m_clp_left - linestart) * add_v;
		linestart = m_clp_left;
	}
	if (lineend > m_clp_right+1) lineend = m_clp_right+1;

	dColor *run_pointer = line_pointer + linestart;
	dColor *stop_pointer = line_pointer + lineend;

	while (run_pointer < stop_pointer)
	{
		*run_pointer = texture[(((v>>16)&511)<<9) + ((u>>16)&511)];
		run_pointer++;
		u += add_u;
		v += add_v;
	}
}

inline void dGraphic::HorizLineTexturedClut(dColor *line_pointer,int32 linestart,int32 lineend,dColor *texture,int32 u,int32 v,int32 add_u,int32 add_v,dClut2 *clut)
{
	if (linestart < m_clp_left)
	{
		u += (m_clp_left - linestart) * add_u;
		v += (m_clp_left - linestart) * add_v;
		linestart = m_clp_left;
	}
	if (lineend > m_clp_right+1) lineend = m_clp_right+1;

	dColor *run_pointer = line_pointer + linestart;
	dColor *stop_pointer = line_pointer + lineend;

	while (run_pointer < stop_pointer)
	{
		*run_pointer = run_pointer->Blend(clut,texture[(((v>>16)&511)<<9) + ((u>>16)&511)]);
		run_pointer++;
		u += add_u;
		v += add_v;
	}
}

inline void dGraphic::HorizLineFlat(dColor *line_pointer,int32 linestart,int32 lineend,dColor color)
{
	lineend++;
	if (linestart < m_clp_left) linestart = m_clp_left;
	if (lineend > m_clp_right+1) lineend = m_clp_right+1;
//	if (lineend > m_clp_right) lineend = m_clp_right;

//	fprintf(stderr,"start: %ld end: %ld\n",linestart,lineend);
	dColor *run_pointer = line_pointer + linestart;
	dColor *stop_pointer = line_pointer + lineend;

	int lim3 = stop_pointer - run_pointer - 1;
	
	if (lim3 > 0)
		while (lim3--)
		{
			*run_pointer = color;
			run_pointer++;
		}
}

inline void dGraphic::HorizLineFlatClut2(dColor *line_pointer,int32 linestart,int32 lineend,dColor color,dClut2 *clut)
{
	lineend++;
	if (linestart < m_clp_left) linestart = m_clp_left;
	if (lineend > m_clp_right+1) lineend = m_clp_right+1;
//	if (lineend > m_clp_right) lineend = m_clp_right;

//	fprintf(stderr,"start: %ld end: %ld\n",linestart,lineend);
	dColor *run_pointer = line_pointer + linestart;
	dColor *stop_pointer = line_pointer + lineend;

	int lim = stop_pointer - run_pointer - 1;
	
	if (lim > 0)
		while (lim--)
		{
			*run_pointer = run_pointer->Blend(clut,color);
			run_pointer++;
		}
}

#endif // _dGraphic_
