#ifndef _dSCREEN_
#define _dSCREEN_

#include <WindowScreen.h>
#include <OS.h>
#include <Screen.h>
#include "dColor.h"
#include "dGraphic.h"
#include "dPalette.h"

class dScreen: public BWindowScreen
{
public:
	dScreen(const char *title,status_t *error);
	virtual ~dScreen();


	/*!
		\brief Draws the given bitmap on the given position on the screen
		\param bitmap a bitmap to draw on screen
		\param x the x position
		\param y the y position
		\warning you can only place the bitmap on even 32 pixels coordinates (0, 32, 64, 48, ...)
		and the width of the bitmap must be aligned to an even 32 pixels width 
	*/
	void Draw(dGraphic *bitmap, int32 x, int32 y);
	void SetPalette(dPalette *p);

	int32 Width();
	int32 Height();

	bool RedrawAll();
	bool TimeToQuit();

	void WaitVbl();
	void Blit(int32 src_x, int32 src_y, int32 width, int32 height, int32 dest_x, int32 dest_y);

	virtual void Quit();
	virtual bool QuitRequested();
private:
	int32 m_width;
	int32 m_height;

// the clip-region
	int16 m_clp_left, m_clp_top, m_clp_right, m_clp_bottom;

	bool m_redraw_all;

	BScreen *m_screen; // the BScreen object, used for monitor beam sync

	dColor *m_screen_base;
	dColor m_clear_color;
	sem_id m_disconnected; // is locked when the screen is disconnected
	sem_id m_ready_to_quit; // is released when the screen can delete itself
	bool m_quitting; // is true when the screen wants to quit

	virtual void ScreenConnected(bool connected);
};

inline void dScreen::SetPalette(dPalette *p)
{
	if (!m_quitting) SetColorList(p->Bits(), 0, 255);
}

inline void dScreen::WaitVbl()
{
	m_screen->WaitForRetrace();
}

inline bool dScreen::RedrawAll()
{
	bool val = m_redraw_all;
	m_redraw_all = false;
	return val;
}

inline bool dScreen::TimeToQuit()
{
//	printf("quitting = %s\n",quitting?"true":"false");
	if (m_quitting)
	{
//		printf("Draw() quitting, releasing ready_to_quit sem\n");
		release_sem(m_ready_to_quit);
		return true;
	}

	while ( acquire_sem_etc(m_disconnected,1,B_RELATIVE_TIMEOUT,500000) != B_NO_ERROR)
	{
		if (m_quitting)
		{
//			printf("Draw() quitting, releasing ready_to_quit sem\n");
			release_sem(m_ready_to_quit);
			return true;
		}
	}
	release_sem(m_disconnected);


	return false;
}

inline int32 dScreen::Width()
{
	return m_width;
}

inline int32 dScreen::Height()
{
	return m_height;
}

inline void dScreen::Blit(int32 src_x, int32 src_y, int32 width, int32 height, int32 dest_x, int32 dest_y)
{

}

#endif // _dSCREEN_