#include "WarIFFChunk.h"

#include <File.h>
#include <DataIO.h>
#include <stdio.h>

WarIFFChunk::WarIFFChunk()
{
	start_position = 0;
	chunk = 0;
	m_size = 0;
}

WarIFFChunk::WarIFFChunk(BDirectory *dir,const char *path,int open_mode):
	m_file(dir,path,open_mode)
{
	m_file.Seek(0,SEEK_SET);
	ReadUint32(&chunk);
	ReadUint32(&m_size);

	if (chunk == 'FORM' || chunk == 'LIST' || chunk == 'PROP')
		ReadUint32(&format);
	
	start_position = m_file.Position();
}

void WarIFFChunk::SetTo(WarIFFChunk *file,off_t start)
{
	m_file = file->m_file;

	start_position = start;

	m_file.Seek(start_position,SEEK_SET);
	ReadUint32(&chunk);
	ReadUint32((uint32*)&m_size);

	if (chunk == 'FORM' || chunk == 'LIST' || chunk == 'PROP')
		ReadUint32(&format);
	
	start_position = m_file.Position();
}

status_t WarIFFChunk::GetSize(off_t *ret_size)
{
	*ret_size = m_size;
	return B_OK;
}

status_t WarIFFChunk::FindChunk(uint32 name,WarIFFChunk *war_chunk)
{
	war_chunk->m_file.Unset();

	m_file.Seek(start_position,SEEK_SET);
	uint32 chunk_size;
	uint32 chunk_name;
	
	//---- sök efter namnet
	if ( ReadUint32(&chunk_name) != 4 ) return B_ERROR;
	if ( ReadUint32(&chunk_size) != 4 ) return B_ERROR;
	
	while ( chunk_name != name )
	{
		m_file.Seek(chunk_size,SEEK_CUR);
		if ( ReadUint32(&chunk_name) != 4 ) return B_ERROR;
		if ( ReadUint32(&chunk_size) != 4 ) return B_ERROR;
	}
	//---- namnet ev. funnet
	
	if (name == chunk_name)
	{
		war_chunk->SetTo(this,m_file.Position() - 8);
		return B_OK;
	}
	else return B_ERROR;
}

size_t WarIFFChunk::ReadFloat(float *val)
{
	return ReadUint32((uint32*)(val));
}

size_t WarIFFChunk::ReadUint8(uint8 *val)
{
	size_t stat;
	uint8 tmp;
	stat = m_file.Read(&tmp,1);
	
//	printf("read:%d - %c\n",tmp,tmp);
	*val = tmp;
	return stat;
}

size_t WarIFFChunk::ReadUint16(uint16 *val)
{
	size_t stat;
	uint8 low,high;
	
	stat = ReadUint8(&high);
	stat += ReadUint8(&low);
	*val = high << 8;
	*val |= low;
	return stat;
}

size_t WarIFFChunk::ReadUint32(uint32 *val)
{
	size_t stat;
	uint8 low,low_middle,high_middle,high;
	
	stat = ReadUint8(&high);
	stat += ReadUint8(&high_middle);
	stat += ReadUint8(&low_middle);
	stat += ReadUint8(&low);

	*val = high << 24;
	*val |= high_middle << 16;
	*val |= low_middle << 8;
	*val |= low;
	return stat;
}

