#include <Application.h>
#include <Directory.h>
#include <Path.h>
#include <Alert.h>
#include <stdlib.h>
#include <StorageDefs.h>
#include <Roster.h>

#include "dPakHandler.h"
#include "dScreen.h"
#include "dContourObject.h"
#include "dPart3DScene.h"
#include "dPart3DScene2.h"
#include "dPart3DSceneSpin2.h"
#include "dPart3DSceneDots.h"
#include "dPart3DSceneTwist.h"
#include "dApp.h"
#include "dVertex.h"
#include "dGuideReader.h"

#define DEBUG

static BDirectory *app_root()
{
	app_info theInfo;
	be_app->GetAppInfo(&theInfo);
	BEntry tEnt(&theInfo.ref);
	static BDirectory root;
	tEnt.GetParent(&root);
	return &root;
}

int32 dApp::main_thread(void *parameter)
{
	dScreen *screen = ((dApp::parameter*)parameter)->screen;
	dApp *app = ((dApp::parameter*)parameter)->app;

// –––––––––––––––––––––––––––––––––

// music
/*
	char tmp[B_PATH_NAME_LENGTH];
	tmp[0] = 0;
	strcat(tmp,app_path.Path());
	strcat(tmp,"/data/Spruce-ComicFakery.NNN");
	app->Load(tmp);
*/

	screen->SetPalette(&app->m_palette);

// START!
//	app->Start();

	uint16 frames = 0;
	bigtime_t start_time = system_time();
	float time = 0.0;
	float part_start;
	float frame_time = 0.1;

	dBitmap *object_display = new dBitmap(128,128,1000); // 1000 polys in the polygon-list
	dBitmap *fps_display = new dBitmap(128, 16);	

	dVertex::SetWidth(object_display->Width());
	dVertex::SetHeight(object_display->Height());

	part_start = time;

	text_state state;
	state.strong = true;
	state.bold = true;
	state.italic = false;

	dVertex pos(5,10);

	dObject *object;
	
	dGuideReader *text_display1 = new dGuideReader(320, 600, &app->m_font, dPakHandler::Instance()->Get("article/3dengine"));
	dGuideReader *text_display2 = new dGuideReader(320, 600, &app->m_font, dPakHandler::Instance()->Get("article/endianness"));
//	text_display2->Next();

	dBitmap *test = new dBitmap(dPakHandler::Instance()->Get("gfx/cover.raw"));

	state.strong = false;
	state.bold = true;
	state.italic = false;

	do
	{

// determine which object to display
		float tmp_time = fmod(time,30.0);
		if ( tmp_time < 5.0 )
			object = &app->m_linefeed_5;
		else if ( tmp_time < 10.0 )
			object = &app->m_cubes;
		else if ( tmp_time < 15.0 )
			object = &app->m_poles;
		else if ( tmp_time < 20.0 )
			object = &app->m_poles_big;
		else if ( tmp_time < 25.0 )
			object = &app->m_nut;
		else
			object = &app->m_cigarrs;

// rotate, project and render the object	
		object->RotateTo(time, time * 1.5, time * 3);
		object->Project();
		object_display->AddObject(object);
		object_display->Empty();
		object_display->Render();

		screen->Draw(object_display, 0, 0);

// clear the fps display and draw the current fps on it
		fps_display->Empty();
		app->m_font.Printf(dVertex(0.0,0.0), fps_display, &state, "fps: %f\n",1.0/frame_time);

// if we need to redraw everything, redraw everything
		if (screen->RedrawAll())
		{
			screen->Draw(text_display1, 128, 0);
			screen->Draw(text_display2, 128 + text_display1->Width() + 32, 0);
			screen->Draw(test, 128,128);
//			screen->Draw(&app->m_flum, 0, 128);
		}

		screen->Draw(fps_display, 0, 128);

		float prev_time = time;
		time = ((float)(system_time() - start_time))/1000000.0;
		frame_time = time - prev_time;
		frames++;
	} while (!screen->TimeToQuit());

	delete object_display;
	delete fps_display;
	delete text_display1;
	delete text_display2;

	delete test;

	printf("fps: %f\n",frames/time);

	be_app->PostMessage(B_QUIT_REQUESTED);

	return 0;
}

void dApp::Break(uint32 cmd)
{
	if (cmd < 20) current_part = cmd;
}

dApp::dApp():
	BApplication("application/x-vnd.Linefeed-diskmag"),
// palettes
	m_palette(dPakHandler::Instance()->Get("gfx/palette")),
/*
// textures
	m_tex_generic(512,512,app_root(),"data/tex-generic.raw"),
	m_tex_credits(512,512,app_root(),"data/tex-credits.raw"),
	m_tex_greets(512,512,app_root(),"data/tex-greetz.raw"),
	m_tex_linefeed(512,512,app_root(),"data/tex-linefeed.raw"),
	m_envmap(512,512,app_root(),"data/env-3lights.raw"),
// backgrounds
	m_flum(640,400,app_root(),"data/bg-flum.raw"),
	m_eyes(640,400,app_root(),"data/bg-eyes1.raw"),
	m_eyes2(640,400,app_root(),"data/bg-eyes2.raw"),
	m_logo(640,400,app_root(),"data/bg-linefeed.raw"),
// wipes
	m_wipe_hole(640,400,app_root(),"data/wipe-hole.raw"),
	m_wipe_ripple(640,400,app_root(),"data/wipe-ripple.raw"),
	m_wipe_twirl(640,400,app_root(),"data/wipe-twirl.raw"),
	m_wipe_square(640,400,app_root(),"data/wipe-square.raw"),
*/
// cluts
	m_clut_mix50(dPakHandler::Instance()->Get("gfx/clut-mix50.raw")),

// objects
	m_linefeed_5(0.0,0.0,9.0),
	m_cubes(0.0,0.0,9.0),
	m_poles(0.0,0.0,9.0),
	m_poles_big(0.0,0.0,9.0),
	m_nut(0.0,0.0,9.0),
	m_cigarrs(0.0,0.0,9.0),

// fonts
	m_font(dPakHandler::Instance()->Get("font/classic"))
{
	
	
	m_linefeed_5.Load(app_root(),"data/obj-5.lwo");
	m_linefeed_5.materials->type = MATERIAL_CLUT_FLAT;
	m_linefeed_5.materials->color = dColor(195);
	m_linefeed_5.materials->clut = &m_clut_mix50;
	m_linefeed_5.MoveTo(0.0, 0.0, 9.0);

	m_cubes.Load(app_root(),"data/obj-cubes.lwo");
	m_cubes.materials->type = MATERIAL_CLUT_FLAT;
	m_cubes.materials->color = dColor(150);
	m_cubes.materials->clut = &m_clut_mix50;
	m_cubes.MoveTo(0.0, 0.0, 9.0);

	m_poles.Load(app_root(),"data/obj-poles.lwo");
	m_poles.materials->type = MATERIAL_CLUT_FLAT;
	m_poles.materials->color = dColor(150);
	m_poles.materials->clut = &m_clut_mix50;
	m_poles.MoveTo(0.0, 0.0, 9.0);

	m_poles_big.Load(app_root(),"data/obj-polesbig.lwo");
	m_poles_big.materials->type = MATERIAL_CLUT_FLAT;
	m_poles_big.materials->color = dColor(150);
	m_poles_big.materials->clut = &m_clut_mix50;
	m_poles_big.MoveTo(0.0, 0.0, 9.0);

	m_nut.Load(app_root(),"data/obj-nut.lwo");
	m_nut.materials->type = MATERIAL_CLUT_FLAT;
	m_nut.materials->color = dColor(150);
	m_nut.materials->clut = &m_clut_mix50;
	m_nut.MoveTo(0.0, 0.0, 4.0);

	m_cigarrs.Load(app_root(),"data/obj-cigarrs.lwo");
	m_cigarrs.materials->type = MATERIAL_CLUT_FLAT;
	m_cigarrs.materials->color = dColor(150);
	m_cigarrs.materials->clut = &m_clut_mix50;
	m_cigarrs.MoveTo(0.0, 0.0, 4.0);

	screen = new dScreen("diskmag",&error);
	if ( error != B_OK ) PostMessage(B_QUIT_REQUESTED);

	screen->SetPalette(&m_palette);
	screen->Show();

	parameter param;
	param.screen = screen;
	param.app = this;
	thread = spawn_thread(main_thread,"main_thread",B_NORMAL_PRIORITY,(void*)&param);
	resume_thread(thread);
}

dApp::~dApp()
{
//	Stop();
}

int main()
{
	dApp *app;
	app = new dApp(); 
	app->Run();
	delete app;
	return 0;	
};
