#include "dFont.h"
#include "dGraphic.h"

#include <stdlib.h>
#include <Directory.h>
#include <File.h>

dFont::dFont(BPositionIO *file):
	m_normal_color(200),
	m_strong_color(255),
// width & height
	m_width(CharWidth()),
	m_height(CharHeight()*95),
// the buffer
	m_buffer(new dColor[m_width * m_height])
{
	file->Read(m_buffer,m_width * m_height);
	
	for (int32 i=0;i<256; i++)
		m_char_width[i] = 0;
	
	m_char_width[9] = CharWidth() * 3;

	file->Read(m_char_width + 32, 95);
}

dFont::~dFont()
{
	delete []m_buffer;
}

dVertex dFont::Printf(const dVertex &pos, dGraphic *bitmap, text_state *state,const char *format, ...) const
{
	static char string[1024];
	string[0] = '\0';
	va_list vl;
	va_start(vl, format);
	vsprintf(string, format, vl);
	va_end(vl);

	return DrawString(pos, bitmap, state, string);
}

dVertex dFont::DrawString(const dVertex &position, dGraphic *bitmap, text_state *state, const char *str) const
{
	if (state->bold) Draw(position + dVertex(1,1), bitmap, state->italic, !state->strong ? m_strong_color : m_normal_color, str);
	return Draw(position, bitmap, state->italic, state->strong ? m_strong_color : m_normal_color, str);
}

dVertex dFont::Draw(const dVertex &position, dGraphic *bitmap, bool italic, dColor text_color, const char *str) const
{
	const char *string = str;
	dVertex point = position;

	while (*string != 0 && point.y + CharHeight() < bitmap->Height())
	{
		int32 ch = *string - ' ';

		if (*string == ' ')
		{
			point.x += m_char_width[*string];
			string++;
			continue;
		}

		if (*string == '\n')
		{
			point.x = 5;
			point.y += CharHeight() + 2;
			string++;
			continue;
		}
		
		if (ch >= 0 && ch < 95 && point.x + CharWidth(*string) < bitmap->Width() + 1)
		{
			if (italic) BlitItalic(0, ch * CharHeight(), m_char_width[*string],CharHeight(),bitmap,point.x, point.y, text_color);
			else Blit(0, ch * CharHeight(), m_char_width[*string],CharHeight(),bitmap,point.x, point.y, text_color);
		}
		point.x += m_char_width[*string];
		string++;
	}

	return point;
}

int32 dFont::StringWidth(const char *string) const
{
	int32 sum = 0;
	while (*string != 0)
	{
		sum += m_char_width[*string];
		string++;
	}
	return sum;
}

void dFont::Blit(int32 src_x, int32 src_y, int32 src_width, int32 src_height, dGraphic *dest_bitmap, int32 dest_x, int32 dest_y, dColor text_color) const
{
	if (dest_x>dest_bitmap->Width() || dest_x+src_width-1<0 || dest_y>dest_bitmap->Height() || dest_y+src_height-1<0) return;

	uint16 b_linefeed = CharWidth()-src_width;
	uint16 f_linefeed = dest_bitmap->Width()-src_width;

	// calculate the buffer offset for the the top left pixel
	dColor *buf = m_buffer + src_y*CharWidth() + src_x;
	dColor *frame = dest_bitmap->Bits() + dest_y*dest_bitmap->Width() + dest_x;

	int32 x,y;
	for(y=0;y<src_height;y++)
	{
		for(x=0;x<src_width;x++)
		{
			if ((*buf) != dColor(0)) *frame = text_color;

			frame++;
			buf++;
		}
		frame+=f_linefeed;
		buf+=b_linefeed;
	}
}

void dFont::BlitItalic(int32 src_x, int32 src_y, int32 src_width, int32 src_height, dGraphic *dest_bitmap, int32 dest_x, int32 dest_y, dColor text_color) const
{
	dest_x += CharWidth() >> 2;

	if (dest_x>dest_bitmap->Width() || dest_x+src_width-1<0 || dest_y>dest_bitmap->Height() || dest_y+src_height-1<0) return;

	uint16 b_linefeed = CharWidth()-src_width;
	uint16 f_linefeed = dest_bitmap->Width()-src_width;

	// calculate the buffer offset for the the top left pixel
	dColor *buf = m_buffer + src_y*CharWidth() + src_x;
	dColor *frame = dest_bitmap->Bits() + dest_y*dest_bitmap->Width() + dest_x;

	int32 x,y;
	for(y=0;y<src_height;y++)
	{
		for(x=0;x<src_width;x++)
		{
			if ((*buf) != dColor(0)) *frame = text_color;

			frame++;
			buf++;
		}
		frame+=f_linefeed;
		buf+=b_linefeed;
		if (y&1) frame--;
	}
}
