#include "dGraphic.h"

#include <stdlib.h>
#include <Directory.h>
#include <File.h>

dGraphic::dGraphic(BPositionIO *file):
// width & height
	m_width(0),
	m_height(0),
// the clip region
	m_clp_left(0),
	m_clp_right(0),
	m_clp_top(0),
	m_clp_bottom(0),
// the buffer
	m_buffer(0)
{
	file->Read(&m_width,4);
	file->Read(&m_height,4);

	#ifdef __INTEL__
	
	char *tmp = (char*)&m_width;
	char temp;
	temp = tmp[0];
	tmp[0] = tmp[3];
	tmp[3] = temp;
	temp = tmp[1];
	tmp[1] = tmp[2];
	tmp[2] = temp;

	tmp = (char*)&m_height;
	temp = tmp[0];
	tmp[0] = tmp[3];
	tmp[3] = temp;
	temp = tmp[1];
	tmp[1] = tmp[2];
	tmp[2] = temp;

	#endif

	if (Width() & 15) fprintf(stderr,"WARNING!, bitmap width %d, have to be even 32\n",Width());

	m_buffer = new dColor[Width() * Height()];
	
	m_clp_right = Width() - 1;
	m_clp_bottom = Height() - 1;

	file->Read(Bits(),Width() * Height());
}

dGraphic::dGraphic(int32 w,int32 h):
// width & height
	m_width(w),
	m_height(h),
// the clip region
	m_clp_left(0),
	m_clp_right(Width()-1),
	m_clp_top(0),
	m_clp_bottom(Height()-1),
// the buffer
	m_buffer(new dColor[Width() * Height()])
{
	if (Width() & 15) fprintf(stderr,"WARNING!, bitmap width %d, have to be even 32\n",Width());
}

dGraphic::~dGraphic()
{
	delete []m_buffer;
}

void dGraphic::DrawBitmap(int32 x,int32 y,dGraphic *bitmap)
{
	register dColor pixel;
	// calculate how much will be clipped on each side
	uint16	margin_left=(x<m_clp_left)?m_clp_left-x:0,
				margin_top=(y<m_clp_top)?m_clp_top-y:0,
				margin_right=(x+bitmap->Width()-1>m_clp_right)?x+bitmap->Width()-m_clp_right-1:0,
				margin_bottom=(y+bitmap->Height()-1>m_clp_bottom)?y+bitmap->Height()-m_clp_bottom-1:0;

	if (x>m_clp_right || x+bitmap->Width()-1<m_clp_left || y>m_clp_bottom || y+bitmap->Height()-1<m_clp_top) return;

	// calculate the drawn width and height, after clipping that is
	uint16 dwidth=bitmap->Width()-margin_left-margin_right-1;
	uint16 dheight=bitmap->Height()-margin_top-margin_bottom;

	uint16 s_linefeed=Width()-dwidth-1;
	uint16 b_linefeed=bitmap->Width()-dwidth-1;

	// calculate the buffer offset for the the top left pixel
	dColor *buf=bitmap->Bits() + (margin_top)*bitmap->Width()+(margin_left);

	dColor *frame=m_buffer + (y+margin_top)*Width()+(x+margin_left);
	for(y=0;y<dheight;y++)
	{
		for(x=0;x<dwidth + 1;x++)
		{
			*frame=*buf;

			frame++;
			buf++;
		}
		frame+=s_linefeed;
		buf+=b_linefeed;
	}
}

void dGraphic::DrawBitmapAdd(int32 x,int32 y,dGraphic *bitmap)
{
	register dColor pixel;
	// calculate how much will be clipped on each side
	uint16	margin_left=(x<m_clp_left)?m_clp_left-x:0,
				margin_top=(y<m_clp_top)?m_clp_top-y:0,
				margin_right=(x+bitmap->Width()-1>m_clp_right)?x+bitmap->Width()-m_clp_right-1:0,
				margin_bottom=(y+bitmap->Height()-1>m_clp_bottom)?y+bitmap->Height()-m_clp_bottom-1:0;

	if (x>m_clp_right || x+bitmap->Width()-1<m_clp_left || y>m_clp_bottom || y+bitmap->Height()-1<m_clp_top) return;

	// calculate the drawn width and height, after clipping that is
	uint16 dwidth=bitmap->Width()-margin_left-margin_right-1;
	uint16 dheight=bitmap->Height()-margin_top-margin_bottom;

	uint16 s_linefeed=Width()-dwidth-1;
	uint16 b_linefeed=bitmap->Width()-dwidth-1;

	// calculate the buffer offset for the the top left pixel
	dColor *buf=bitmap->Bits() + (margin_top)*bitmap->Width()+(margin_left);

	dColor *frame=m_buffer + (y+margin_top)*Width()+(x+margin_left);
	for(y=0;y<dheight;y++)
	{
		for(x=0;x<dwidth + 1;x++)
		{
			*frame+=*buf;

			frame++;
			buf++;
		}
		frame+=s_linefeed;
		buf+=b_linefeed;
	}

}

void dGraphic::DrawBitmap(int32 x,int32 y,dGraphic *bitmap,dGraphic *stencil_bitmap,uint8 value)
{
	if (stencil_bitmap->Width() == bitmap->Width() && stencil_bitmap->Height() == bitmap->Height())

	register dColor pixel;
	// calculate how much will be clipped on each side
	uint16	margin_left=(x<m_clp_left)?m_clp_left-x:0,
				margin_top=(y<m_clp_top)?m_clp_top-y:0,
				margin_right=(x+bitmap->Width()-1>m_clp_right)?x+bitmap->Width()-m_clp_right-1:0,
				margin_bottom=(y+bitmap->Height()-1>m_clp_bottom)?y+bitmap->Height()-m_clp_bottom-1:0;

	if (x>m_clp_right || x+bitmap->Width()-1<m_clp_left || y>m_clp_bottom || y+bitmap->Height()-1<m_clp_top) return;

	// calculate the drawn width and height, after clipping that is
	uint16 dwidth=bitmap->Width()-margin_left-margin_right-1;
	uint16 dheight=bitmap->Height()-margin_top-margin_bottom;

	uint16 s_linefeed=Width()-dwidth-1;
	uint16 b_linefeed=bitmap->Width()-dwidth-1;

	// calculate the buffer offset for the the top left pixel
	dColor *buf=bitmap->Bits() + (margin_top)*bitmap->Width()+(margin_left);
	dColor *stencil=stencil_bitmap->Bits() + (margin_top)*bitmap->Width()+(margin_left);
	dColor *frame=m_buffer + (y+margin_top)*Width()+(x+margin_left);
	for(y=0;y<dheight;y++)
	{
		for(x=0;x<dwidth + 1;x++)
		{
			if ( *((uint8*)stencil) < value ) *frame=*buf;

			frame++;
			stencil++;
			buf++;
		}
		frame+=s_linefeed;
		buf+=b_linefeed;
		stencil+=b_linefeed;
	}

}

void dGraphic::DrawBitmap(int32 x,int32 y,dGraphic *bitmap,dGraphic *stencil_bitmap,uint8 value,dColor border)
{
	if (stencil_bitmap->Width() == bitmap->Width() && stencil_bitmap->Height() == bitmap->Height())

	register dColor pixel;
	// calculate how much will be clipped on each side
	uint16	margin_left=(x<m_clp_left)?m_clp_left-x:0,
				margin_top=(y<m_clp_top)?m_clp_top-y:0,
				margin_right=(x+bitmap->Width()-1>m_clp_right)?x+bitmap->Width()-m_clp_right-1:0,
				margin_bottom=(y+bitmap->Height()-1>m_clp_bottom)?y+bitmap->Height()-m_clp_bottom-1:0;

	if (x>m_clp_right || x+bitmap->Width()-1<m_clp_left || y>m_clp_bottom || y+bitmap->Height()-1<m_clp_top) return;

	// calculate the drawn width and height, after clipping that is
	uint16 dwidth=bitmap->Width()-margin_left-margin_right-1;
	uint16 dheight=bitmap->Height()-margin_top-margin_bottom;

	uint16 s_linefeed=Width()-dwidth-1;
	uint16 b_linefeed=bitmap->Width()-dwidth-1;

	// calculate the buffer offset for the the top left pixel
	dColor *buf=bitmap->Bits() + (margin_top)*bitmap->Width()+(margin_left);
	dColor *stencil=stencil_bitmap->Bits() + (margin_top)*bitmap->Width()+(margin_left);
	dColor *frame=m_buffer + (y+margin_top)*Width()+(x+margin_left);
	for(y=0;y<dheight;y++)
	{
		for(x=0;x<dwidth + 1;x++)
		{
			uint8 stencil_val = *((uint8*)stencil);
			if ( stencil_val < value ) *frame=*buf;
			else if ( stencil_val == value ) *frame = border;
			frame++;
			stencil++;
			buf++;
		}
		frame+=s_linefeed;
		buf+=b_linefeed;
		stencil+=b_linefeed;
	}

}

void dGraphic::LineClut(dVertex *start,dVertex *end,dClut2 *clut)
{
//	fprintf(stderr,"LINE: frame_buffer: %x\n",m_buffer);

	int32 x1=start->x*65536.0;
	int32 y1=start->y*65536.0;
	int32 x2=end->x*65536.0;
	int32 y2=end->y*65536.0;

	int32 clip_left=m_clp_left<<16;
	int32 clip_right=(m_clp_right)<<16;
	int32 clip_top=m_clp_top<<16;
	int32 clip_bottom=(m_clp_bottom)<<16;

// ******* CLIPPING START *************
	if( (x1<clip_left+0x10000 && x2<clip_left+0x10000) || (x1>clip_right && x2>clip_right) || (y1<clip_top+0x10000 && y2<clip_top+0x10000) || (y1>clip_bottom && y2>clip_bottom) ){
		return;
	}

	float dxy,dyx,dzx,dzy;
	dxy=(float)(x2-x1)/(float)(y2-y1);
	dyx=(float)(y2-y1)/(float)(x2-x1);
	if(x1<x2)
	{
		if(x1<clip_left+0x10000)
		{
			y1+=(int)(dyx*(clip_left-x1));
			x1=clip_left+1;
		}
		if(x2>(clip_right))
		{
			y2+=(int)(dyx*(clip_right-x2));
			x2=clip_right;
		}
	}
	else
	{
		if(x2<clip_left+0x10000)
		{
			y2+=(int)(dyx*(clip_left-x2));
			x2=clip_left+0x10000;
		}
		if(x1>(clip_right))
		{
			y1+=(int)(dyx*(clip_right-x1));
			x1=clip_right;
		}
	}
	if(y1<y2)
	{
		if(y1<clip_top+0x10000)
		{
			x1+=(int)(dxy*(clip_top-y1));
			y1=clip_top+0x10000;
		}
		if(y2>(clip_bottom))
		{
			x2+=(int)(dxy*(clip_bottom-y2));
			y2=clip_bottom;
		}
	}
	else
	{
		if(y2<clip_top+1)
		{
			x2+=(int)(dxy*(clip_top-y2));
			y2=clip_top+0x10000;
		}
		if(y1>(clip_bottom))
		{
			x1+=(int)(dxy*(clip_bottom-y1));
			y1=clip_bottom;
		}
	}



//	}


	if (x1<clip_left+0x10000 || y1<clip_top+0x10000 || x1>clip_right || y1>clip_bottom) return;

// ******** CLIPPING END **************

// ************ HARD CODED FOR 640 WIDE SCREENS **************

	int add;
	int c, lim;
	int of;
	int tmp1,tmp2;

	if (abs(x2-x1) > abs(y2-y1))
	{
		// BIG X - pretty optimized (hardcoded 640 width)
		if (x1>x2)
		{
			swap(x1,x2);
			swap(y1,y2);
		}


		if ((x2 - x1)>>16 != 0)
		{
			add = (y2-y1)/((x2-x1)>>16);
		
			x1 >>= 16;
			x2 >>= 16;
			lim = 1+x2-x1;
			while (lim != 0)
			{
				c=(y1 & 0xffff) >> 8;
	
				tmp1 = y1 >> 16;
				of=(tmp1<<9)+(tmp1<<7)+x1;
	
				m_buffer[of] = m_buffer[of].Blend(clut,dColor(255-c));
				m_buffer[of+640] = m_buffer[of+640].Blend(clut,dColor(c));
	
				y1+=add;
				x1++;
				lim--;
			}
		}
	}
	else
	{
		// BIG Y - this part is pretty optimized
		if (y1>y2)
		{
			swap(x1,x2);
			swap(y1,y2);
		}

		if ((y2 - y1)>>16 != 0)
		{
			add = (x2-x1)/((y2-y1)>>16);
			
			y1>>=16;
			y2>>=16;
			lim = 1+y2-y1;
			y1*=640;
			while (lim != 0)
			{
				c=(x1 & 0xffff) >> 8;
				of=y1+(x1>>16);

				m_buffer[of] = m_buffer[of].Blend(clut,dColor(255-c));
				m_buffer[of+1] = m_buffer[of+1].Blend(clut,dColor(c));

				x1+=add;
				y1+=640;
				lim--;
			}
		}
	}
}

void dGraphic::DrawFaceTextured(dFace *face,dGraphic *texture)
{
	if (face->vex[0]->z<0 || face->vex[1]->z<0 || face->vex[2]->z<0) return;
/*

	uint16 m_clp_left=m_clp_left+ScreenLeft(),m_clp_right=m_clp_right+ScreenLeft();
	uint16 m_clp_top=m_clp_top+ScreenTop(),m_clp_bottom=m_clp_bottom+ScreenTop();
*/

	int32 vx[3];
	int32 vy[3];
	int32 u[3];
	int32 v[3];
	
	vx[0]=face->vex[0]->x*65536;
	vy[0]=face->vex[0]->y;
	u[0]=face->u[0];
	v[0]=face->v[0];

	vx[1]=face->vex[1]->x*65536;
	vy[1]=face->vex[1]->y;
	u[1]=face->u[1];
	v[1]=face->v[1];

	vx[2]=face->vex[2]->x*65536;
	vy[2]=face->vex[2]->y;
	u[2]=face->u[2];
	v[2]=face->v[2];

	//sort by y hardco(re)ded bub-sort
	if (vy[0] > vy[1]) {	swap(vy[0],vy[1]); swap(vx[0],vx[1]); swap(u[0],u[1]); swap(v[0],v[1]);}
	if (vy[1] > vy[2]) {	swap(vy[1],vy[2]); swap(vx[1],vx[2]); swap(u[1],u[2]); swap(v[1],v[2]);}
	if (vy[0] > vy[1]) {	swap(vy[0],vy[1]); swap(vx[0],vx[1]); swap(u[0],u[1]); swap(v[0],v[1]);}

	if (vy[0] == vy[2]) return;

	int32 linestart,lineend;
	int32 add_start,add_end;
	int32 line_u_start,line_v_start;
	int32 u_line_add,v_line_add;
	int32 add_u,add_v;

	dColor *line_pointer = m_buffer + vy[0]	* Width();
	line_u_start = u[0];
	line_v_start = v[0];

	int32 middle_x=vx[0]+((vx[2]-vx[0])/(vy[2]-vy[0])*(vy[1]-vy[0]));
	int32 middle_u=u[0]+((u[2]-u[0])/(vy[2]-vy[0])*(vy[1]-vy[0]));
	int32 middle_v=v[0]+((v[2]-v[0])/(vy[2]-vy[0])*(vy[1]-vy[0]));

	if (vx[1]>>16 == middle_x>>16) return;

	if (middle_x<vx[1])
	{
		add_u=(u[1]-middle_u)/(((vx[1]>>16)-(middle_x>>16)));
		add_v=(v[1]-middle_v)/(((vx[1]>>16)-(middle_x>>16)));
	}
	else
	{
		add_u=(middle_u-u[1])/(((middle_x>>16)-(vx[1]>>16)));
		add_v=(middle_v-v[1])/(((middle_x>>16)-(vx[1]>>16)));
	}

// ***** UPPER TRIANGLE *************
	
	if (vy[1] > vy[0])
	{
		linestart=lineend=vx[0];

		dColor *line_stop_pointer = m_buffer + vy[1] * Width();
	
		if (middle_x < vx[1])
		{
			add_start=(vx[2]-linestart)/(vy[2]-vy[0]);
			add_end=(vx[1]-lineend)/(vy[1]-vy[0]);

			u_line_add=(u[2]-u[0])/(vy[2]-vy[0]);
			v_line_add=(v[2]-v[0])/(vy[2]-vy[0]);
		}
		else
		{
			add_start=(vx[1]-linestart)/(vy[1]-vy[0]);
			add_end=(vx[2]-lineend)/(vy[2]-vy[0]);

			u_line_add=(u[1]-u[0])/(vy[1]-vy[0]);
			v_line_add=(v[1]-v[0])/(vy[1]-vy[0]);
		}
	
	// CLIP Y

		if (vy[0] < m_clp_top)
		{
			line_pointer += (m_clp_top - vy[0]) * Width();
			linestart	 += (m_clp_top - vy[0]) * add_start;
			lineend		 += (m_clp_top - vy[0]) * add_end;

			line_u_start += u_line_add * (m_clp_top-vy[0]);
			line_v_start += v_line_add * (m_clp_top-vy[0]);
		}

		if (vy[1]>m_clp_bottom + 1)
			line_stop_pointer = m_buffer + (m_clp_bottom + 1) * Width();

		while (line_pointer<line_stop_pointer)
		{
			HorizLineTextured(line_pointer,linestart>>16,lineend>>16,texture->Bits(),line_u_start,line_v_start,add_u,add_v);
			linestart += add_start;
			lineend	 += add_end;
			line_pointer += Width();
			
			line_u_start += u_line_add;
			line_v_start += v_line_add;
		}
	}

// ***** BOTTOM TRIANGLE *************

	if (vy[2] > vy[1])
	{
		dColor *line_stop_pointer	= m_buffer + vy[2] * Width();

		if (middle_x<vx[1])
		{
			linestart=middle_x;
			lineend=vx[1];

			line_u_start = middle_u;
			line_v_start = middle_v;

			add_start=(vx[2]-vx[0])/(vy[2]-vy[0]);
			add_end=(vx[2]-vx[1])/(vy[2]-vy[1]);

			u_line_add=(u[2]-u[0])/(vy[2]-vy[0]);
			v_line_add=(v[2]-v[0])/(vy[2]-vy[0]);
		}
		else
		{
			linestart=vx[1];
			lineend=middle_x;

			line_u_start = u[1];
			line_v_start = v[1];

			add_start=(vx[2]-vx[1])/(vy[2]-vy[1]);
			add_end=(vx[2]-vx[0])/(vy[2]-vy[0]);

			u_line_add=(u[2]-u[1])/(vy[2]-vy[1]);
			v_line_add=(v[2]-v[1])/(vy[2]-vy[1]);
		}
	
	// CLIP Y

		if (vy[1] < m_clp_top)
		{
			line_pointer = m_buffer + m_clp_top * Width();
			linestart	 += (m_clp_top - vy[1]) * add_start;
			lineend		 += (m_clp_top - vy[1]) * add_end;

			line_u_start += u_line_add * (m_clp_top-vy[1]);
			line_v_start += v_line_add * (m_clp_top-vy[1]);
		}

		if (vy[2]>m_clp_bottom + 1)
			line_stop_pointer = m_buffer + (m_clp_bottom + 1) * Width();

		while (line_pointer<line_stop_pointer)
		{
			HorizLineTextured(line_pointer,linestart>>16,lineend>>16,texture->Bits(),line_u_start,line_v_start,add_u,add_v);
			linestart += add_start;
			lineend	 += add_end;
			line_pointer += Width();
			
			line_u_start += u_line_add;
			line_v_start += v_line_add;
		}
	}	
}

void dGraphic::DrawFaceTexturedClut(dFace *face,dGraphic *texture,dClut2 *clut)
{
	if (face->vex[0]->z<0 || face->vex[1]->z<0 || face->vex[2]->z<0) return;
/*
	uint16 m_clp_left=m_clp_left+ScreenLeft(),m_clp_right=m_clp_right+ScreenLeft();
	uint16 m_clp_top=m_clp_top+ScreenTop(),m_clp_bottom=m_clp_bottom+ScreenTop();
*/

	int32 vx[3];
	int32 vy[3];
	int32 u[3];
	int32 v[3];
	
	vx[0]=face->vex[0]->x*65536;
	vy[0]=face->vex[0]->y;
	u[0]=face->u[0];
	v[0]=face->v[0];

	vx[1]=face->vex[1]->x*65536;
	vy[1]=face->vex[1]->y;
	u[1]=face->u[1];
	v[1]=face->v[1];

	vx[2]=face->vex[2]->x*65536;
	vy[2]=face->vex[2]->y;
	u[2]=face->u[2];
	v[2]=face->v[2];

	//sort by y hardco(re)ded bub-sort
	if (vy[0] > vy[1]) {	swap(vy[0],vy[1]); swap(vx[0],vx[1]); swap(u[0],u[1]); swap(v[0],v[1]);}
	if (vy[1] > vy[2]) {	swap(vy[1],vy[2]); swap(vx[1],vx[2]); swap(u[1],u[2]); swap(v[1],v[2]);}
	if (vy[0] > vy[1]) {	swap(vy[0],vy[1]); swap(vx[0],vx[1]); swap(u[0],u[1]); swap(v[0],v[1]);}

	if (vy[0] == vy[2]) return;

	int32 linestart,lineend;
	int32 add_start,add_end;
	int32 line_u_start,line_v_start;
	int32 u_line_add,v_line_add;
	int32 add_u,add_v;

	dColor *line_pointer = m_buffer + vy[0]	* Width();
	line_u_start = u[0];
	line_v_start = v[0];

	int32 middle_x=vx[0]+((vx[2]-vx[0])/(vy[2]-vy[0])*(vy[1]-vy[0]));
	int32 middle_u=u[0]+((u[2]-u[0])/(vy[2]-vy[0])*(vy[1]-vy[0]));
	int32 middle_v=v[0]+((v[2]-v[0])/(vy[2]-vy[0])*(vy[1]-vy[0]));

	if (vx[1]>>16 == middle_x>>16) return;

	if (middle_x<vx[1])
	{
		add_u=(u[1]-middle_u)/(((vx[1]>>16)-(middle_x>>16)));
		add_v=(v[1]-middle_v)/(((vx[1]>>16)-(middle_x>>16)));
	}
	else
	{
		add_u=(middle_u-u[1])/(((middle_x>>16)-(vx[1]>>16)));
		add_v=(middle_v-v[1])/(((middle_x>>16)-(vx[1]>>16)));
	}

// ***** UPPER TRIANGLE *************
	
	if (vy[1] > vy[0])
	{
		linestart=lineend=vx[0];

		dColor *line_stop_pointer = m_buffer + vy[1] * Width();
	
		if (middle_x < vx[1])
		{
			add_start=(vx[2]-linestart)/(vy[2]-vy[0]);
			add_end=(vx[1]-lineend)/(vy[1]-vy[0]);

			u_line_add=(u[2]-u[0])/(vy[2]-vy[0]);
			v_line_add=(v[2]-v[0])/(vy[2]-vy[0]);
		}
		else
		{
			add_start=(vx[1]-linestart)/(vy[1]-vy[0]);
			add_end=(vx[2]-lineend)/(vy[2]-vy[0]);

			u_line_add=(u[1]-u[0])/(vy[1]-vy[0]);
			v_line_add=(v[1]-v[0])/(vy[1]-vy[0]);
		}
	
	// CLIP Y

		if (vy[0] < m_clp_top)
		{
			line_pointer += (m_clp_top - vy[0]) * Width();
			linestart	 += (m_clp_top - vy[0]) * add_start;
			lineend		 += (m_clp_top - vy[0]) * add_end;

			line_u_start += u_line_add * (m_clp_top-vy[0]);
			line_v_start += v_line_add * (m_clp_top-vy[0]);
		}

		if (vy[1]>m_clp_bottom + 1)
			line_stop_pointer = m_buffer + (m_clp_bottom + 1) * Width();

		while (line_pointer<line_stop_pointer)
		{
			HorizLineTexturedClut(line_pointer,linestart>>16,lineend>>16,texture->Bits(),line_u_start,line_v_start,add_u,add_v,clut);
			linestart += add_start;
			lineend	 += add_end;
			line_pointer += Width();
			
			line_u_start += u_line_add;
			line_v_start += v_line_add;
		}
	}

// ***** BOTTOM TRIANGLE *************

	if (vy[2] > vy[1])
	{
		dColor *line_stop_pointer	= m_buffer + vy[2] * Width();

		if (middle_x<vx[1])
		{
			linestart=middle_x;
			lineend=vx[1];

			line_u_start = middle_u;
			line_v_start = middle_v;

			add_start=(vx[2]-vx[0])/(vy[2]-vy[0]);
			add_end=(vx[2]-vx[1])/(vy[2]-vy[1]);

			u_line_add=(u[2]-u[0])/(vy[2]-vy[0]);
			v_line_add=(v[2]-v[0])/(vy[2]-vy[0]);
		}
		else
		{
			linestart=vx[1];
			lineend=middle_x;

			line_u_start = u[1];
			line_v_start = v[1];

			add_start=(vx[2]-vx[1])/(vy[2]-vy[1]);
			add_end=(vx[2]-vx[0])/(vy[2]-vy[0]);

			u_line_add=(u[2]-u[1])/(vy[2]-vy[1]);
			v_line_add=(v[2]-v[1])/(vy[2]-vy[1]);
		}
	
	// CLIP Y

		if (vy[1] < m_clp_top)
		{
			line_pointer = m_buffer + m_clp_top * Width();
			linestart	 += (m_clp_top - vy[1]) * add_start;
			lineend		 += (m_clp_top - vy[1]) * add_end;

			line_u_start += u_line_add * (m_clp_top-vy[1]);
			line_v_start += v_line_add * (m_clp_top-vy[1]);
		}

		if (vy[2]>m_clp_bottom + 1)
			line_stop_pointer = m_buffer + (m_clp_bottom + 1) * Width();

		while (line_pointer<line_stop_pointer)
		{
			HorizLineTexturedClut(line_pointer,linestart>>16,lineend>>16,texture->Bits(),line_u_start,line_v_start,add_u,add_v,clut);
			linestart += add_start;
			lineend	 += add_end;
			line_pointer += Width();
			
			line_u_start += u_line_add;
			line_v_start += v_line_add;
		}
	}	
}

void dGraphic::DrawFaceFlat(dFace *face,dColor col)
{
	if (face->vex[0]->z<0 || face->vex[1]->z<0 || face->vex[2]->z<0) return;
/*

	uint16 m_clp_left=m_clp_left+ScreenLeft(),m_clp_right=m_clp_right+ScreenLeft();
	uint16 m_clp_top=m_clp_top+ScreenTop(),m_clp_bottom=m_clp_bottom+ScreenTop();
*/
	int32 vx[3];
	int32 vy[3];
	
	vx[0]=face->vex[0]->x*65536;
	vy[0]=face->vex[0]->y;

	vx[1]=face->vex[1]->x*65536;
	vy[1]=face->vex[1]->y;

	vx[2]=face->vex[2]->x*65536;
	vy[2]=face->vex[2]->y;

	//sort by y hardco(re)ded bub-sort
	if (vy[0] > vy[1]) {	swap(vy[0],vy[1]); swap(vx[0],vx[1]); }
	if (vy[1] > vy[2]) {	swap(vy[1],vy[2]); swap(vx[1],vx[2]); }
	if (vy[0] > vy[1]) {	swap(vy[0],vy[1]); swap(vx[0],vx[1]); }

	if (vy[0] == vy[2]) return;

	int32 linestart,lineend;
	int32 add_start,add_end;

	dColor *line_pointer = m_buffer + vy[0]	* Width();

	int32 middle_x=vx[0]+((vx[2]-vx[0])/(vy[2]-vy[0])*(vy[1]-vy[0]));

	if (vx[1]>>16 == middle_x>>16) return;

// ***** UPPER TRIANGLE *************
	
	if (vy[1] > vy[0])
	{
		linestart=lineend=vx[0];

		dColor *line_stop_pointer = m_buffer + vy[1] * Width();
	
		if (middle_x < vx[1])
		{
			add_start=(vx[2]-linestart)/(vy[2]-vy[0]);
			add_end=(vx[1]-lineend)/(vy[1]-vy[0]);
		}
		else
		{
			add_start=(vx[1]-linestart)/(vy[1]-vy[0]);
			add_end=(vx[2]-lineend)/(vy[2]-vy[0]);
		}
	
	// CLIP Y

		if (vy[0] < m_clp_top)
		{
			line_pointer += (m_clp_top - vy[0]) * Width();
			linestart	 += (m_clp_top - vy[0]) * add_start;
			lineend		 += (m_clp_top - vy[0]) * add_end;
		}

		if (vy[1]>m_clp_bottom + 1)
			line_stop_pointer = m_buffer + (m_clp_bottom + 1) * Width();

		while (line_pointer<line_stop_pointer)
		{
			HorizLineFlat(line_pointer,linestart>>16,lineend>>16,col);
			linestart += add_start;
			lineend	 += add_end;
			line_pointer += Width();
		}
	}

// ***** BOTTOM TRIANGLE *************

	if (vy[2] > vy[1])
	{
		dColor *line_stop_pointer	= m_buffer + vy[2] * Width();

		if (middle_x<vx[1])
		{
			linestart=middle_x;
			lineend=vx[1];

			add_start=(vx[2]-vx[0])/(vy[2]-vy[0]);
			add_end=(vx[2]-vx[1])/(vy[2]-vy[1]);
		}
		else
		{
			linestart=vx[1];
			lineend=middle_x;

			add_start=(vx[2]-vx[1])/(vy[2]-vy[1]);
			add_end=(vx[2]-vx[0])/(vy[2]-vy[0]);
		}
	
	// CLIP Y

		if (vy[1] < m_clp_top)
		{
			line_pointer = m_buffer + m_clp_top * Width();
			linestart	 += (m_clp_top - vy[1]) * add_start;
			lineend		 += (m_clp_top - vy[1]) * add_end;
		}

		if (vy[2]>m_clp_bottom + 1)
			line_stop_pointer = m_buffer + (m_clp_bottom + 1) * Width();

		while (line_pointer<line_stop_pointer)
		{
			HorizLineFlat(line_pointer,linestart>>16,lineend>>16,col);
			linestart += add_start;
			lineend	 += add_end;
			line_pointer += Width();
		}
	}	
}

void dGraphic::DrawFaceFlatClut2(dFace *face,dColor color,dClut2 *clut)
{
	if (face->vex[0]->z<0 || face->vex[1]->z<0 || face->vex[2]->z<0) return;
/*

	uint16 m_clp_left=m_clp_left+ScreenLeft(),m_clp_right=m_clp_right+ScreenLeft();
	uint16 m_clp_top=m_clp_top+ScreenTop(),m_clp_bottom=m_clp_bottom+ScreenTop();
*/
	int32 vx[3];
	int32 vy[3];
	
	vx[0]=face->vex[0]->x*65536;
	vy[0]=face->vex[0]->y;

	vx[1]=face->vex[1]->x*65536;
	vy[1]=face->vex[1]->y;

	vx[2]=face->vex[2]->x*65536;
	vy[2]=face->vex[2]->y;

	//sort by y hardco(re)ded bub-sort
	if (vy[0] > vy[1]) {	swap(vy[0],vy[1]); swap(vx[0],vx[1]); }
	if (vy[1] > vy[2]) {	swap(vy[1],vy[2]); swap(vx[1],vx[2]); }
	if (vy[0] > vy[1]) {	swap(vy[0],vy[1]); swap(vx[0],vx[1]); }

	if (vy[0] == vy[2]) return;

	int32 linestart,lineend;
	int32 add_start,add_end;

	dColor *line_pointer = m_buffer + vy[0]	* Width();

	int32 middle_x=vx[0]+((vx[2]-vx[0])/(vy[2]-vy[0])*(vy[1]-vy[0]));

	if (vx[1]>>16 == middle_x>>16) return;

// ***** UPPER TRIANGLE *************
	
	if (vy[1] > vy[0])
	{
		linestart=lineend=vx[0];

		dColor *line_stop_pointer = m_buffer + vy[1] * Width();
	
		if (middle_x < vx[1])
		{
			add_start=(vx[2]-linestart)/(vy[2]-vy[0]);
			add_end=(vx[1]-lineend)/(vy[1]-vy[0]);
		}
		else
		{
			add_start=(vx[1]-linestart)/(vy[1]-vy[0]);
			add_end=(vx[2]-lineend)/(vy[2]-vy[0]);
		}
	
	// CLIP Y

		if (vy[0] < m_clp_top)
		{
			line_pointer += (m_clp_top - vy[0]) * Width();
			linestart	 += (m_clp_top - vy[0]) * add_start;
			lineend		 += (m_clp_top - vy[0]) * add_end;
		}

		if (vy[1]>m_clp_bottom + 1)
			line_stop_pointer = m_buffer + (m_clp_bottom + 1) * Width();

		while (line_pointer<line_stop_pointer)
		{
			HorizLineFlatClut2(line_pointer,linestart>>16,lineend>>16,color,clut);
			linestart += add_start;
			lineend	 += add_end;
			line_pointer += Width();
		}
	}

// ***** BOTTOM TRIANGLE *************

	if (vy[2] > vy[1])
	{
		dColor *line_stop_pointer	= m_buffer + vy[2] * Width();

		if (middle_x<vx[1])
		{
			linestart=middle_x;
			lineend=vx[1];

			add_start=(vx[2]-vx[0])/(vy[2]-vy[0]);
			add_end=(vx[2]-vx[1])/(vy[2]-vy[1]);
		}
		else
		{
			linestart=vx[1];
			lineend=middle_x;

			add_start=(vx[2]-vx[1])/(vy[2]-vy[1]);
			add_end=(vx[2]-vx[0])/(vy[2]-vy[0]);
		}
	
	// CLIP Y

		if (vy[1] < m_clp_top)
		{
			line_pointer = m_buffer + m_clp_top * Width();
			linestart	 += (m_clp_top - vy[1]) * add_start;
			lineend		 += (m_clp_top - vy[1]) * add_end;
		}

		if (vy[2]>m_clp_bottom + 1)
			line_stop_pointer = m_buffer + (m_clp_bottom + 1) * Width();

		while (line_pointer<line_stop_pointer)
		{
			HorizLineFlatClut2(line_pointer,linestart>>16,lineend>>16,color,clut);
			linestart += add_start;
			lineend	 += add_end;
			line_pointer += Width();
		}
	}	
}

