#include "dGuideReader.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <Directory.h>
#include <MemoryIO.h>
#include <file.h>

dGuideReader::dGuideReader(int32 w,int32 h, const dFont *font, BPositionIO *file):
	dGraphic(w,h),
	m_page_number(0),
	m_last_page(false),
	m_num_lines(0),
	m_font(font)
{
	m_page_state[0].strong = false;
	m_page_state[0].bold = false;
	m_page_state[0].italic = false;

	m_page_start[0] = 0;

	char *data; // the file
	int32 file_length;

	file->Seek(0, SEEK_END);
	file_length = file->Position();
	file->Seek(0, SEEK_SET);

	data = new char[file_length + 1];
	file->Read(data,file_length);
	
	data[file_length] = 0;

	int32 file_position = 0;
	int32 line_position = 0;
	int32 prev_space = 1;

	int32 sum_width = 0;
	bool command_mode = false;

	while (file_position < file_length)
	{
		while (sum_width < Width() - 10)
		{
			temp[line_position] = data[file_position];
			if (!command_mode) sum_width += m_font->CharWidth(temp[line_position]);
			if (temp[line_position] == '}' && command_mode) command_mode = false;
			if (temp[line_position] == '@' && data[file_position+1] == '{')
			{
				command_mode = true;
				sum_width -= m_font->CharWidth(temp[line_position]);
			}
			if (temp[line_position] == ' ') prev_space = 0;
			if (temp[line_position] == '\n')
			{
				prev_space = 0;
				sum_width = Width();
			}

			file_position++;
			line_position++;
			prev_space++;
			if (file_position > file_length) break;
		}

		if (prev_space <= line_position)
		{
			file_position -= prev_space - 1;
			line_position -= prev_space;
		}
		temp[line_position] = '\n';
		temp[line_position + 1] = 0;

		m_lines[m_num_lines] = strdup(temp);
		m_num_lines++;

		prev_space = 1;
		line_position = 0;
		sum_width = 0;
		command_mode = false;
	}

	printf("num_lines: %d\n",m_num_lines);

	delete []data;
	Generate();
}

dGuideReader::~dGuideReader()
{
	for (int32 i=0; i < m_num_lines; i++)
		free(m_lines[i]);
}

void dGuideReader::Next()
{
	if (LastPage()) return;
	m_page_number++;
	Generate();
}

void dGuideReader::Prev()
{
	if (Page() == 0) return;
	m_page_number--;
	Generate();
}

void dGuideReader::Generate()
{
	Empty();
	dVertex pos(5, 5);

	int32 line = m_page_start[Page()];
	text_state state = m_page_state[Page()];
	
	while (pos.y + m_font->CharHeight() < Height() - 10 && line < m_num_lines && line < m_num_lines)
	{
		temp[0] = 0;
		int32 line_position = 0;
		int32 string_position = 0;
		
		while (m_lines[line][line_position] != 0)
		{
			temp[string_position] = m_lines[line][line_position];

			if (temp[string_position] == '@' && m_lines[line][line_position + 1] == '{')
			{
				line_position++;
				line_position++;
				
				temp[string_position] = 0;
				if (line_position > 0) pos = m_font->DrawString(pos, this, &state, temp);
				string_position = 0;

				while (m_lines[line][line_position] != '}' && m_lines[line][line_position] != 0)
				{
					temp[string_position] = m_lines[line][line_position];
					line_position++;
					string_position++;
				}
				if (temp[string_position-1] == '}') temp[string_position-1] = 0;
				else temp[string_position] = 0;

				if (strcmp("s",temp) == 0) state.strong = true;
				else if (strcmp("b",temp) == 0) state.bold = true;
				else if (strcmp("i",temp) == 0) state.italic = true;
				else if (strcmp("us",temp) == 0) state.strong = false;
				else if (strcmp("ub",temp) == 0) state.bold = false;
				else if (strcmp("ui",temp) == 0) state.italic = false;

				if (m_lines[line][line_position] == 0) line_position--;
				string_position = -1;
				temp[0] = 0;
			}

			string_position++;
			line_position++;
		}
		temp[string_position] = 0;
		pos = m_font->DrawString(pos, this, &state, temp);
		line++;
	}
	
	if (line == m_num_lines) m_last_page = true;
	
	m_page_state[Page() + 1] = state;
	m_page_start[Page() + 1] = line;
}
