#include <Application.h>
#include <SupportDefs.h>

#define min(a,b) (((a)<(b))?(a):(b))

#include "dScreen.h"

dScreen::dScreen(const char *title,status_t *error):
	BWindowScreen(	title, B_8_BIT_800x600, error),
	m_width(800),
	m_height(600),
// the clip region
	m_clp_left(0),
	m_clp_right(Width() - 1 - 5),
	m_clp_top(0),
	m_clp_bottom(Height()-1),
	m_redraw_all(false),
	m_screen(new BScreen(this)),
	m_disconnected(create_sem(0,"drawing_sem")),
	m_ready_to_quit(create_sem(0,"sync_sem")),
	m_screen_base(0),
	m_clear_color(0),
	m_quitting(false)
{
};	

dScreen::~dScreen()
{
	ASSERT(this != 0);

	delete m_screen;
	delete_sem(m_disconnected);
}

void dScreen::Quit()
{
//	printf("Quit() quitting = true\n");
	m_quitting = true;
//	printf("trying to acquire ready_to_quit sem\n");
	acquire_sem(m_ready_to_quit);
//	printf("Quit() acquired ready_to_quit sem\n");
	snooze(100000);
	BWindowScreen::Quit();
}

bool dScreen::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

void dScreen::ScreenConnected(bool connected)
{
	ASSERT(this != 0);

	if (connected)
	{
		m_screen_base = (dColor*)CardInfo()->frame_buffer;

		dColor *base = m_screen_base;
		dColor *stop = m_screen_base + Width() * Height();

		while (base < stop)
		{
			*base = m_clear_color;
			base++;
		}
		
		release_sem(m_disconnected);
		m_redraw_all = true;
	}
	else
	{
//		printf("disconnected\n");
		acquire_sem(m_disconnected);
	}
}

void dScreen::Draw(dGraphic *bitmap, int32 x, int32 y)
{
//	printf("Draw()\n");
	ASSERT(this != 0);
	ASSERT(bitmap != 0);

//	printf("Draw() trying to acquireing sem disconnected\n");
	if( acquire_sem_etc(m_disconnected,1,B_RELATIVE_TIMEOUT,0) == B_WOULD_BLOCK)
		return;

	// calculate how much will be clipped on each side
	uint16	margin_left=(x<m_clp_left)?m_clp_left-x:0,
				margin_top=(y<m_clp_top)?m_clp_top-y:0,
				margin_right=(x+bitmap->Width()-1>m_clp_right)?x+bitmap->Width()-m_clp_right-1:0,
				margin_bottom=(y+bitmap->Height()-1>m_clp_bottom)?y+bitmap->Height()-m_clp_bottom-1:0;

	if (x>m_clp_right || x+bitmap->Width()-1<m_clp_left || y>m_clp_bottom || y+bitmap->Height()-1<m_clp_top) return;

	// calculate the drawn width and height, after clipping that is
	uint16 dwidth=bitmap->Width()-margin_left-margin_right-1;
	uint16 dheight=bitmap->Height()-margin_top-margin_bottom;

	uint16 s_linefeed = (Width()-dwidth-1) >> 3;
	uint16 b_linefeed = (bitmap->Width()-dwidth-1) >> 3;

	// calculate the buffer offset for the the top left pixel
	double *buf = (double*)(bitmap->Bits() + (margin_top)*bitmap->Width()+(margin_left));

	double *frame = (double*)(m_screen_base + (y+margin_top)*Width()+(x+margin_left));
	double *frame_stop = (double*)(m_screen_base + (y+margin_top)*Width()+(x+margin_left) + dwidth);
	for(y=0;y<dheight;y++)
	{
		while(frame < frame_stop)
		{
			frame[0]=buf[0];
			frame[1]=buf[1];
			frame[2]=buf[2];
			frame[3]=buf[3];

			frame+=4;
			buf+=4;
		}
		frame += s_linefeed;
		frame_stop += Width() >> 3;
		buf += b_linefeed;
	}

	release_sem(m_disconnected);
}

