/*
	Written by Arvid Norberg 1999 <arvid@iname.com>
*/

#include "Mp3Attr.h"
#include <File.h>
#include <stdio.h>
#include <string.h>
#include <Alert.h>
#include <Beep.h>

Mp3Attr::Mp3Attr(entry_ref *file_ref, bool correct_mime)
{
	file=new BFile(file_ref,B_READ_WRITE);
	if (file->InitCheck()!=B_OK)
	{
		init = file->InitCheck();
		return;
	}

	char mime[B_MIME_TYPE_LENGTH];
	file->ReadAttr("BEOS:TYPE",B_STRING_TYPE,0,mime,B_MIME_TYPE_LENGTH);
	if (strcmp(mime,"audio/x-mpeg"))
	{
		if (correct_mime)
			file->WriteAttr("BEOS:TYPE",B_STRING_TYPE,0,"audio/x-mpeg",B_MIME_TYPE_LENGTH);
		else
		{
			init = WRONG_MIME_TYPE;
			return;
		}
	}

	*title=*album=*artist=*genre=0;
	year=rating=track=0;

	file->ReadAttr(TITLE,B_STRING_TYPE,0,title,B_FILE_NAME_LENGTH);
	file->ReadAttr(ALBUM,B_STRING_TYPE,0,album,B_FILE_NAME_LENGTH);
	file->ReadAttr(ARTIST,B_STRING_TYPE,0,artist,B_FILE_NAME_LENGTH);
	file->ReadAttr(COMMENT,B_STRING_TYPE,0,comment,B_FILE_NAME_LENGTH);
	file->ReadAttr(GENRE,B_STRING_TYPE,0,genre,B_FILE_NAME_LENGTH);
	file->ReadAttr(YEAR,B_INT32_TYPE,0,&year,4);
	file->ReadAttr(RATING,B_INT32_TYPE,0,&rating,4);
	file->ReadAttr(TRACK,B_INT32_TYPE,0,&track,4);

	init=B_OK;
}

Mp3Attr::~Mp3Attr()
{
	delete file;
}

Mp3Attr& Mp3Attr::operator=(Mp3Attr& attr)
{
	track=attr.track;
	year=attr.year;
	rating=attr.rating;
	strncpy(title,attr.title,sizeof(title));
	strncpy(artist,attr.artist,sizeof(artist));
	strncpy(album,attr.album,sizeof(album));
	strncpy(comment,attr.comment,sizeof(comment));
	strncpy(genre,attr.genre,sizeof(genre));
	return attr;
}

uint32 Mp3Attr::InitCheck()
{
	return init;
}

void Mp3Attr::WriteToDisk()
{
	bool success=true;
	if (file->WriteAttr(TITLE,B_STRING_TYPE,0,title,strlen(title)+1) != strlen(title)+1) success=false;
	if (file->WriteAttr(ALBUM,B_STRING_TYPE,0,album,strlen(album)+1) != strlen(album)+1) success=false;
	if (file->WriteAttr(ARTIST,B_STRING_TYPE,0,artist,strlen(artist)+1) != strlen(artist)+1) success=false;
	if (file->WriteAttr(COMMENT,B_STRING_TYPE,0,comment,strlen(comment)+1) != strlen(comment)+1) success=false;
	if (file->WriteAttr(GENRE,B_STRING_TYPE,0,genre,strlen(genre)+1) != strlen(genre)+1) success=false;
	if (file->WriteAttr(YEAR,B_INT32_TYPE,0,&year,4) != 4) success=false;
	if (file->WriteAttr(RATING,B_INT32_TYPE,0,&rating,4) != 4) success=false;
	if (file->WriteAttr(TRACK,B_INT32_TYPE,0,&track,4) != 4) success=false;

	if (!success) beep();
}
