/*
	Written by Arvid Norberg 1999 <arvid@iname.com>
*/

#ifndef _MP3_ATTR_H
#define _MP3_ATTR_H

#include <OS.h>
#include <string.h>
#include <TypeConstants.h>

// attribute names
#define ARTIST  "Audio:Artist"
#define ALBUM   "Audio:Album"
#define TITLE   "Audio:Title"
#define GENRE   "Audio:Genre"
#define COMMENT "Audio:Comment"
#define YEAR    "Audio:Year"
#define RATING  "Audio:Rating"
#define TRACK   "Audio:Track"

const uint32 WRONG_MIME_TYPE='mime';

class Mp3Attr
{
public:
	Mp3Attr(entry_ref *file_ref, bool correct_mime = false);
	~Mp3Attr();

	uint32 InitCheck();

	char *Artist();
	char *Album();
	char *Title();
	char *Comment();
	char *Genre();
	int32 Year();
	int32 Track();
	int32 Rating();

	void SetArtist(char *artist);
	void SetAlbum(char *artist);
	void SetComment(char *comment);
	void SetTitle(char *artist);
	void SetGenre(char *genre);
	void SetYear(int32 year);
	void SetTrack(int32 track);
	void SetRating(int32 rating);

	Mp3Attr& operator=(Mp3Attr& attr); // copies data only, not file pointer

	void WriteToDisk();
private:
	uint32 init;
	BFile *file;

	int32 track;
	int32 year;
	int32 rating;
	char title[B_FILE_NAME_LENGTH];
	char artist[B_FILE_NAME_LENGTH];
	char album[B_FILE_NAME_LENGTH];	
	char comment[B_FILE_NAME_LENGTH];
	char genre[B_FILE_NAME_LENGTH];
};

// inline definitions

inline char *Mp3Attr::Title()
{
	return title;
}

inline char *Mp3Attr::Artist()
{
	return artist;
}

inline char *Mp3Attr::Album()
{
	return album;
}

inline int32 Mp3Attr::Year()
{
	return year;
}

inline int32 Mp3Attr::Rating()
{
	return rating;
}

inline int32 Mp3Attr::Track()
{
	return track;
}

inline char *Mp3Attr::Comment()
{
	return comment;
}

inline char *Mp3Attr::Genre()
{
	return genre;
}

inline void Mp3Attr::SetGenre(char *string)
{
	strncpy(genre,string,sizeof(genre));
}

inline void Mp3Attr::SetTitle(char *string)
{
	strncpy(title,string,sizeof(title));
}

inline void Mp3Attr::SetArtist(char *string)
{
	strncpy(artist,string,sizeof(artist));
}

inline void Mp3Attr::SetAlbum(char *string)
{
	strncpy(album,string,sizeof(album));
}

inline void Mp3Attr::SetYear(int32 val)
{
	year=val;
}

inline void Mp3Attr::SetRating(int32 val)
{
	rating=val;
}

inline void Mp3Attr::SetTrack(int32 val)
{
	track=val;
}

inline void Mp3Attr::SetComment(char *string)
{
	strncpy(comment,string,sizeof(comment));
}

#endif // _MP3_ATTR_H