/*
	Written by Arvid Norberg 1999 <arvid@iname.com>
*/

#include "VGroup.h"
#include "HGroup.h"
#include "MButton.h"
#include "MBorder.h"
#include "MCheckBox.h"
#include "MStringView.h"
#include "MTextControl.h"
#include "MWindow.h"
#include "layout.h"
#include "MPopup.h"

#include "MultiRequesterWindow.h"

#include <string.h>
#include <stdio.h>

MultiRequesterWindow::MultiRequesterWindow(char *pattern,uint32 size,char *genre,char *title,char *artist,char *album,char *comment,int32 *year_val,int32 *rating_val,int32 *track_val, bool * action)
	: MWindow(BRect(100,100,180,150),"mp3 attribute [multiple files]",B_TITLED_WINDOW,B_NOT_RESIZABLE)
{
	pattern_string=pattern;
	title_string=title;
	artist_string=artist;
	album_string=album;
	comment_string=comment;
	genre_string=genre;
	rating=rating_val;
	track=track_val;
	year=year_val;

	len=size;

	topview=
	new VGroup
	(
		new MBorder(M_LABELED_BORDER,10,"pattern",
			new VGroup
			(
				new MStringView("Enter the patter by which your mp3 files are named."),
				new MStringView("/a = Artist name"),
				new MStringView("/t = song Title"),
				new MStringView("/n = album Name"),
				new MStringView("/c = comment"),
				new MStringView("/y = year"),
				new MStringView("/r = rating"),
				new MStringView("/k = track"),
				new MStringView("/g = genre"),
				new MStringView("/* = wildcard, can be anything"),
				pattern_check=new MCheckBox("use pattern matching",new BMessage(PATTERN_CHECK)),
				pattern_gadget=new MTextControl("pattern:","/a - /t.mp3"),
				0
			)
		),
		new MBorder(M_LABELED_BORDER,10,"set every file's attribute to:",
			new HGroup
			(
				new VGroup
				(
					minimax(-1,-1,-1,-1),
					title_check=new MCheckBox("Title:",new BMessage(TITLE_CHECK)),
					artist_check=new MCheckBox("Artist:",new BMessage(ARTIST_CHECK)),
					album_check=new MCheckBox("Album:",new BMessage(ALBUM_CHECK)),
					comment_check=new MCheckBox("Comment:",new BMessage(COMMENT_CHECK)),
					year_check=new MCheckBox("Year:",new BMessage(YEAR_CHECK)),
					genre_check=new MCheckBox("Genre:",new BMessage(GENRE_CHECK)),
					rating_check=new MCheckBox("Rating:",new BMessage(RATING_CHECK)),
					track_check=new MCheckBox("Track:",new BMessage(TRACK_CHECK)),
					0
				),
				new VGroup
				(
					title_gadget=new MTextControl("",""),
					artist_gadget=new MTextControl("",""),
					album_gadget=new MTextControl("",""),
					comment_gadget=new MTextControl("",""),
					year_gadget=new MTextControl("",""),
					genre_gadget=new MTextControl("",""),
					rating_gadget=new MTextControl("",""),
					track_gadget=new MTextControl("",""),
					0
				),
				0
			)
		),
		new HGroup
		(
			new MButton("Cancel",new BMessage(CANCEL_BUTTON)),
			ok_button=new MButton("Run",new BMessage(OK_BUTTON)),
			0
		),
		0
	);
	AddChild(dynamic_cast<BView*>(topview));
	SetDefaultButton(ok_button);
	year_gadget->SetEnabled(false);
	title_gadget->SetEnabled(false);
	artist_gadget->SetEnabled(false);
	album_gadget->SetEnabled(false);
	comment_gadget->SetEnabled(false);
	pattern_gadget->SetEnabled(false);
	genre_gadget->SetEnabled(false);
	track_gadget->SetEnabled(false);
	rating_gadget->SetEnabled(false);
	
	do_it = action;
}

bool MultiRequesterWindow::QuitRequested()
{
	return true;
}

void MultiRequesterWindow::MessageReceived(BMessage *message)
{
	int32 val;
	switch (message->what)
	{
		case OK_BUTTON:
			*do_it = true;
			// copy the data
			if (pattern_check->Value())
				strncpy(pattern_string,pattern_gadget->Text(),len);
			if (genre_check->Value())
				strncpy(genre_string,genre_gadget->Text(),B_FILE_NAME_LENGTH);
			if (title_check->Value())
				strncpy(title_string,title_gadget->Text(),B_FILE_NAME_LENGTH);
			if (artist_check->Value())
				strncpy(artist_string,artist_gadget->Text(),B_FILE_NAME_LENGTH);
			if (album_check->Value())
				strncpy(album_string,album_gadget->Text(),B_FILE_NAME_LENGTH);
			if (comment_check->Value())
				strncpy(comment_string,comment_gadget->Text(),B_FILE_NAME_LENGTH);
			if (year_check->Value())
				*year=atoi(year_gadget->Text());
			if (rating_check->Value())
				*rating=atoi(rating_gadget->Text());
			if (track_check->Value())
				*track=atoi(track_gadget->Text());

			Quit();
			break;
		case CANCEL_BUTTON:
			Quit();
			break;
		case YEAR_CHECK:
			message->FindInt32("be:value",&val);
			year_gadget->SetEnabled(val);
			break;
		case TITLE_CHECK:
			message->FindInt32("be:value",&val);
			title_gadget->SetEnabled(val);
			break;
		case ARTIST_CHECK:
			message->FindInt32("be:value",&val);
			artist_gadget->SetEnabled(val);
			break;
		case ALBUM_CHECK:
			message->FindInt32("be:value",&val);
			album_gadget->SetEnabled(val);
			break;
		case COMMENT_CHECK:
			message->FindInt32("be:value",&val);
			comment_gadget->SetEnabled(val);
			break;
		case GENRE_CHECK:
			message->FindInt32("be:value",&val);
			genre_gadget->SetEnabled(val);
			break;
		case RATING_CHECK:
			message->FindInt32("be:value",&val);
			rating_gadget->SetEnabled(val);
			break;
		case TRACK_CHECK:
			message->FindInt32("be:value",&val);
			track_gadget->SetEnabled(val);
			break;
		case PATTERN_CHECK:
			message->FindInt32("be:value",&val);
			pattern_gadget->SetEnabled(val);
			break;
		case '!pop':
    	uint32 index;
    	if (B_NO_ERROR==message->FindInt32("index",(int32*)&index))
    		tmp_genre=index;
			break;
		default:
			MWindow::MessageReceived(message);
	}
}
