/*
	Written by Arvid Norberg 1999 <arvid@iname.com>
*/

#include "RequesterWindow.h"
#include "MultiRequesterWindow.h"
#include "Mp3Attr.h"

#include <string.h>
#include <stdio.h>
#include <Message.h>
#include <Alert.h>
#include <Entry.h>
#include <Path.h>
#include <TrackerAddOn.h>

_EXPORT void process_refs(entry_ref,BMessage *msg,void *);

bool match(char *name,uint8 name_offs,char *pattern,uint8 pattern_offs,char *title,char *artist,char *album,char *year,char *comment,char *genre,char *track,char *rating);

// recursive pattern matcher
bool match(char *name,uint8 name_offs,char *pattern,uint8 pattern_offs,char *title,char *artist,char *album,char *year,char *comment,char *genre,char *track,char *rating)
{
	char dummy;
	char *target;
	uint16 max_len; // the length of the target string
	bool result=false;
	while(pattern_offs<strlen(pattern) && name_offs<strlen(name))
	{
		if (pattern[pattern_offs]=='/')
		{
			uint8 target_offs=0;
			target=0;
			switch(pattern[++pattern_offs])
			{
			case 'n':
				target=album;
				max_len=B_FILE_NAME_LENGTH;
				break;
			case 'a':
				target=artist;
				max_len=B_FILE_NAME_LENGTH;
				break;
			case 't':
				target=title;
				max_len=B_FILE_NAME_LENGTH;
				break;
			case 'y':
				target=year;
				max_len=B_FILE_NAME_LENGTH;
				break;
			case 'c':
				target=comment;
				max_len=B_FILE_NAME_LENGTH;
				break;
			case 'r':
				target=rating;
				max_len=B_FILE_NAME_LENGTH;
				break;
			case 'k':
				target=track;
				max_len=B_FILE_NAME_LENGTH;
				break;
			case 'g':
				target=genre;
				max_len=B_FILE_NAME_LENGTH;
				break;
			case '*':
				target=&dummy;
				max_len=0;
				break;
			default:
				return false;
				break;
			}
			pattern_offs++;
			while (name_offs<strlen(name) && !(result=match(name,name_offs,pattern,pattern_offs,title,artist,album,year,comment,genre,track,rating)) )
			{
				if (target_offs<max_len) target[target_offs++]=name[name_offs];
				name_offs++;
			}
			target[target_offs]=0;
			return (result || (pattern[pattern_offs]==0 && name[name_offs]==0));
		}
		if (name[name_offs]!=pattern[pattern_offs]) break;
		pattern_offs++;
		name_offs++;
	}
	return (pattern[pattern_offs]==0 && name[name_offs]==0);
}


void process_refs(entry_ref,BMessage *msg,void *)
{
	bool	do_it = false;

	BPath path;
	entry_ref file_ref;

	int32 track;
	int32 year;
	int32 rating;
	char title[B_FILE_NAME_LENGTH];
	char artist[B_FILE_NAME_LENGTH];
	char album[B_FILE_NAME_LENGTH];	
	char comment[B_FILE_NAME_LENGTH];
	char genre[B_FILE_NAME_LENGTH];

	int32 file_count;
	type_code num_types;

	msg->GetInfo("refs",&num_types,&file_count);
	if (file_count>1) // ************************ MULTIFILE *********************
	{
		char filename[B_FILE_NAME_LENGTH];
		uint32 cnt=0;
		char pattern[200];
		*pattern=-1;

		char rating_tmp[B_FILE_NAME_LENGTH];
		char year_tmp[B_FILE_NAME_LENGTH];
		char track_tmp[B_FILE_NAME_LENGTH];
		char title_tmp[B_FILE_NAME_LENGTH];
		char artist_tmp[B_FILE_NAME_LENGTH];
		char album_tmp[B_FILE_NAME_LENGTH];
		char comment_tmp[B_FILE_NAME_LENGTH];
		char genre_tmp[B_FILE_NAME_LENGTH];

		year=-1;
		rating=-1;
		track=-1;
		*genre=-1;
		*artist=-1;
		*album=-1;
		*title=-1;
		*comment=-1;

		status_t t;
		MultiRequesterWindow *window=new MultiRequesterWindow(pattern,200,genre,title,artist,album,comment,&year,&rating,&track, &do_it);
		window->Show();
 		wait_for_thread(window->Thread(), &t );		
		if (! do_it)
			return;

		while (B_NO_ERROR==msg->FindRef("refs",cnt++,&file_ref))
		{
			Mp3Attr file(&file_ref,true);
			uint32 error_code;
			if (B_OK!=(error_code=file.InitCheck()))
			{
				char error_string[120+B_FILE_NAME_LENGTH];
				char name[B_FILE_NAME_LENGTH];
				
				BEntry entry(&file_ref);
				entry.GetName(name);
				
				switch (error_code)
				{
				case B_NO_MEMORY:
					sprintf(error_string,"Out of memory when opening \"%s\"",name);
					break;
				case B_PERMISSION_DENIED:
					sprintf(error_string,"\"%s\" Permission denied",name);
					break;
				case B_ENTRY_NOT_FOUND:
					sprintf(error_string,"\"%s\" not found",name);
					break;
				default:
					sprintf(error_string,"Some error on the file \"%s\"",name);
					break;
				}
				BAlert *alert=new BAlert("Error opening file",error_string,"ok");
				alert->Go();
				continue;
			}

			if (*pattern!=-1)
			{
				BEntry entry(&file_ref);
				entry.GetName(filename);

				*year_tmp=-1;
				*rating_tmp=-1;
				*track_tmp=-1;
				*genre_tmp=-1;
				*artist_tmp=-1;
				*album_tmp=-1;
				*title_tmp=-1;
				*comment_tmp=-1;

				// pattern matching
				if (match(filename,0,pattern,0,title_tmp,artist_tmp,album_tmp,year_tmp,comment_tmp,genre_tmp,track_tmp,rating_tmp))
				{
					// look for fields the patter matcher has changed
					if (*artist_tmp!=-1) file.SetArtist(artist_tmp);
					if (*album_tmp!=-1) file.SetAlbum(album_tmp);
					if (*title_tmp!=-1) file.SetTitle(title_tmp);
					if (*year_tmp!=-1) file.SetYear(atoi(year_tmp));
					if (*comment_tmp!=-1) file.SetComment(comment_tmp);
					if (*genre_tmp!=-1) file.SetGenre(genre_tmp);
					if (*track_tmp!=-1) file.SetTrack(atoi(track_tmp));
					if (*rating_tmp!=-1) file.SetRating(atoi(rating_tmp));
				}
				else
				{
					char error_string[60+B_FILE_NAME_LENGTH];
					sprintf(error_string,"\"%s\" did not match the pattern and is left untouched",filename);
					BAlert *alert=new BAlert("Pattern",error_string,"ok");
					alert->Go();
				}
			}

			if (*genre!=-1) file.SetGenre(genre);
			if (*artist!=-1) file.SetArtist(artist);
			if (*album!=-1) file.SetAlbum(album);
			if (*title!=-1) file.SetTitle(title);
			if (*comment!=-1) file.SetComment(comment);
			if (year!=-1) file.SetYear(year);
			if (rating!=-1) file.SetRating(rating);
			if (track!=-1) file.SetTrack(track);
			file.WriteToDisk();
		}
	}

	else // ************************ SINGLEFILE *********************
	{
		if (B_NO_ERROR!=msg->FindRef("refs",&file_ref)) return;

		Mp3Attr file(&file_ref,true);
		char *error_string;
		uint32 error_code;
		if (B_OK!=(error_code=file.InitCheck()))
		{
			switch (error_code)
			{
			case B_NO_MEMORY:
				error_string="Out of memory";
				break;
			case B_PERMISSION_DENIED:
				error_string="File permission denied";
				break;
			case B_ENTRY_NOT_FOUND:
				error_string="File not found";
				break;
			default:
				error_string="error";
				break;
			}
			BAlert *alert=new BAlert("Error opening file",error_string,"ok");
			alert->Go();
			return;
		}

		char filename[B_FILE_NAME_LENGTH];
		BEntry entry(&file_ref);
		entry.GetName(filename);

		year=file.Year();
		track=file.Track();
		rating=file.Rating();
		strcpy(title,file.Title());
		strcpy(artist,file.Artist());
		strcpy(album,file.Album());
		strcpy(comment,file.Comment());
		strcpy(genre,file.Genre());

		RequesterWindow *window=new RequesterWindow(filename,genre,title,artist,album,comment,&year,&track,&rating, &do_it);
		window->Show();
		status_t t;
		wait_for_thread(window->Thread(), &t );		
		if (do_it)
		{
			file.SetTitle(title);
			file.SetArtist(artist);
			file.SetAlbum(album);
			file.SetYear(year);
			file.SetComment(comment);
			file.SetGenre(genre);
			file.SetTrack(track);
			file.SetRating(rating);
			file.WriteToDisk();
		}
	}
}

int main()
{
	// 
}
