/*
	Written by Arvid Norberg 1999 <arvid@iname.com>
*/

#include "RequesterWindow.h"
#include "Mp3Attr.h"
#include <string.h>
#include <stdio.h>
#include <Message.h>
#include <Alert.h>
#include <Entry.h>
#include <Path.h>
#include <TrackerAddOn.h>

//_EXPORT void process_refs(entry_ref,BMessage *msg,void *);

void process_refs(entry_ref,BMessage *msg,void *)
{
	bool do_it = false;
	char the_pattern[200];

	BPath path;
	entry_ref file_ref;

	RequesterWindow *window = new RequesterWindow(the_pattern,200,&do_it);
	window->Show();
	
	int32 t;
	wait_for_thread(window->Thread(), &t );		
	if (!do_it)	return;

	int32 cnt=0;

	while (B_NO_ERROR == msg->FindRef("refs",cnt++,&file_ref))
	{
		char *pattern = the_pattern;

		Mp3Attr file(&file_ref,true);
		BEntry entry(&file_ref);

		uint32 error_code;
		if (B_OK != (error_code=file.InitCheck()))
		{
			char error_string[120+B_FILE_NAME_LENGTH];
			char name[B_FILE_NAME_LENGTH];
				
			entry.GetName(name);
				
			switch (error_code)
			{
				case B_NO_MEMORY:
					sprintf(error_string,"Out of memory when opening \"%s\"",name);
					break;
				case B_PERMISSION_DENIED:
					sprintf(error_string,"\"%s\" Permission denied",name);
					break;
				case B_ENTRY_NOT_FOUND:
					sprintf(error_string,"\"%s\" not found",name);
					break;
				case WRONG_MIME_TYPE:
					sprintf(error_string,"\"%s\" has the wrong MIME-type, an MPEG-file should have the MIME-Type \"audio/x-mpeg\". This file is left untouched",name);
					break;
				default:
					sprintf(error_string,"Some error on the file \"%s\"",name);
					break;
			}
			BAlert *alert=new BAlert("Error opening file",error_string,"ok");
			alert->Go();
			continue;
		}
		
		
		// if there where no errors while finding the file, extract the attributes and rename it

		int32 track;
		int32 year;
		int32 rating;
		char title[B_FILE_NAME_LENGTH];
		char artist[B_FILE_NAME_LENGTH];
		char album[B_FILE_NAME_LENGTH];	
		char comment[B_FILE_NAME_LENGTH];
		char genre[B_FILE_NAME_LENGTH];

		year = file.Year();
		track = file.Track();
		rating = file.Rating();
		strcpy(title, file.Title());
		strcpy(artist, file.Artist());
		strcpy(album, file.Album());
		strcpy(comment, file.Comment());
		strcpy(genre, file.Genre());

		char buffer[B_FILE_NAME_LENGTH];
		memset(buffer,0,B_FILE_NAME_LENGTH);
		int32 i = 0;
		while (*pattern)
		{
			while (*pattern && *pattern != '/')
			{
				buffer[i++] = *pattern;
				pattern++;
			}
			if (*pattern == '/')
			{
				char tmp[10];
				pattern++;
				switch(*pattern)
				{
					case 'a':
						strcat(buffer,artist);
						i += strlen(artist);
					break;
					case 't':
						strcat(buffer,title);
						i += strlen(title);
					break;
					case 'n':
						strcat(buffer,album);
						i += strlen(album);
					break;
					case 'c':
						strcat(buffer,comment);
						i += strlen(comment);
					break;
					case 'g':
						strcat(buffer,genre);
						i += strlen(genre);
					break;
					case 'k':
						sprintf(tmp,"%02ld",track);
						strcat(buffer,tmp);
						i += strlen(tmp);
					break;
					case 'r':
						sprintf(tmp,"%02ld",rating);
						strcat(buffer,tmp);
						i += strlen(tmp);
					break;
					case 'y':
						sprintf(tmp,"%02ld",year);
						strcat(buffer,tmp);
						i += strlen(tmp);
					break;
				}
				if (*pattern) pattern++;
			}
		}

		// replace illegal characters
		for (int32 i=0; i< strlen(buffer); i++)
			if (buffer[i] == '/') buffer[i]='\\';
		
		entry.Rename(buffer);
	}
}

int main()
{
	// 
}
