/* :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	*	B2K CLIENT
	*
	*/
	
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <OS.h>
#include <InterfaceDefs.h>

#include "B2Kdefs.h"
#include "B2KClient.h"
#include "B2KDrawStruct.h"
#include "TextFile.h"

#include "ThingyBodyAddOn.h"

/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	C O N S T R U C T O R
*
*/
B2KClient::B2KClient(tcp_sock *tcp, BDirectory *root,char* world, status_t *ok,
					uint32 *offscreen_buf, uint32 *vis_onscreen,
					uint32 words_per_row, sem_id sem)
{
	EnemyCount=0;
	BubbleCount=0;
	RoomCount=0;
	b2kRoot=root;
	vis=vis_onscreen;
	offscreen=offscreen_buf;
	offscreen_bg=offscreen_buf+(640*480);	
	rowwords=words_per_row;
	sema=sem;

	sock=tcp;
	
	if (LoadWorld(world)) {
		*ok=B_OK;		
	}else{
		*ok=B_ERROR;
		return;
	}

//	Receiver
	bool ok2;
	r=new Receiver((char *)draw_this,THINGYSMAX*sizeof(DrawStruct),UDP_PORT,&ok2);
	if (!ok2){
		fputs("Receiver isn't ok\n",stderr);
		*ok=B_ERROR;
		return;
	}
}

/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	D E S T R U C T O R
*
*/
#define FREE(x) if (x) {free(x); x=0; }
B2KClient::~B2KClient()
{
	puts("B2KClient::~B2KClient()");
	/*	-------------------------------------------------------------
		delete addons
	*/
	while (EnemyCount>0)
		delete Enemy[--EnemyCount];
	
	while (BubbleCount>0)
		delete Bubble[--BubbleCount];
	
	FREE(Enemy);
	FREE(Bubble);

	FREE(blocks);
	FREE(Images);

	FREE(BgBlocks);
	
	EnemyCount=0;
	BubbleCount=0;

	delete r;
}

inline void B2KClient::PlayerDraw(DrawStruct *ds, EraseStruct *es)
{
	if (ds->index>PlayerCount) return;
	Player[ds->index]->Body->Draw(ds,es);
}

inline void B2KClient::EnemyDraw(DrawStruct *ds, EraseStruct *es)
{
	if (ds->index>EnemyCount) return;
	Enemy[ds->index]->Body->Draw(ds,es);
}

inline void B2KClient::BubbleDraw(DrawStruct *ds, EraseStruct *es)
{
	if (ds->index>BubbleCount) return;
	Bubble[ds->index]->Body->Draw(ds,es);
}

inline void B2KClient::NoCollDraw(DrawStruct *ds, EraseStruct *es)
{
	if (ds->index > NoCollCount) return;
	NoColl[ds->index]->Body->Draw(ds,es);
}



/*	:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	B G - > D R A W
*
*	! Unoptimized !	32bit safe
*/
void B2KClient::bg2draw(uint32 x,uint32 y,uint32 w,uint32 h)
{
//puts("trying to bg->draw");
//printf("y=%d x=%d w=%d h=%d\n",x,y,w,h);
//assert(offscreen != 0);
//assert(offscreen_bg != 0);

	uint32 y2=y+h;
	uint32 x2=x+w;
	uint32 tx,ty;
	for (ty=y;ty<y2;ty++){
		for (tx=x;tx<x2;tx++)
			offscreen[ty*640+tx]=offscreen_bg[ty*640+tx];
		}
//puts("finished bg->drawing");
}

/*	:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	D R A W - > V I S
*
*	! Unoptimized !	32bit safe
*/
void B2KClient::draw2vis(uint32 x,uint32 y,uint32 w,uint32 h)
{
//puts("trying to draw->vis");
//printf("y=%d x=%d w=%d h=%d\n",x,y,w,h);
//assert(offscreen != 0);
//assert(vis != 0);
	uint32 y2=y+h;
	uint32 x2=x+w;
	uint32 tx,ty;
	
	for (ty=y;ty<y2;ty++)
		for (tx=x;tx<x2;tx++)
			vis[ty*rowwords+tx]=offscreen[ty*640+tx];

//puts("finished draw->vis:ing");
}


/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	P U T R O O M
*
*	Only use on offscreenbuffer!
*/
void B2KClient::PutRoom(uint index, uint32 *buf)
{
	if (index>RoomCount)	{fputs("(PutRoom)Skipped\n",stderr); return;}
	if (index>ImageCount)	{fputs("(PutRoom)(Image)Skipped\n",stderr); return;}
//	fputs("(PutRoom)Putting Room\n",stderr);

	uint32 *image=Images+640*480*index;
	uint8 *mbase;
	int x,y;
	int _x,_y;

// copy the picture
	for (x=0;x<(640*480);x++)
		buf[x]=image[x];


	int add=4*(640-16);
	uint8 a;
//	fprintf(stderr,"BadGuy x=%d y=%d s=%d\n",ds->x>>22,ds->y>>22,ds->state);

	uint8 *piece=blocks+(30*40*index); // första blocket som skall ritas ut;
	uint8 *gfx;
	for (y=0;y<30;y++)
		for (x=0;x<40;x++){
			//printf("%x",*piece);
			gfx=((uint8 *)BgBlocks)+((*piece)*4*256);
			mbase=((uint8 *)buf)+(y*640+x)*16*4;
			for (_y=0;_y<16;_y++){
				for (_x=0;_x<16;_x++){
					mbase++; //skip 'alpha'
					a=*(gfx++);
					*mbase=((*gfx)*a + (*mbase)*(255-a)) >>8;
					mbase++;
					gfx++;
					*mbase=((*gfx)*a + (*mbase)*(255-a)) >>8;
					mbase++;
					gfx++;
					*mbase=((*gfx)*a + (*mbase)*(255-a)) >>8;
					mbase++;
					gfx++;
				}
				mbase+=add;
			}
			piece++;
	}

//puts("finished putting room");

};


/*	:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	L O A D 1 6 x 1 6
*
*/
int	B2KClient::Load16x16(char *name, uint8 *data, int loaded)
{
	int ret; // bytes read

	BFile f(b2kRoot,name,B_READ_ONLY);
	if (B_NO_ERROR!=f.InitCheck()) return 0;
	ret=f.Read(data,(128-loaded)*4*16*16);	

	//printf("%s contains %d blocks read to %p\n",name,ret,data);	
	return ret>>10;
}


/* :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	L O A D W O R L D
*
*/
#define NEXTLN 	if (!(f.NextLine(buf,BUFSIZE))) { return false; }
bool B2KClient::LoadWorld(char *n)
{
	fprintf(stderr,"\n===== Loading world ==================\n");
	Enemy=0;
	Bubble=0;
	blocks=0;
		
	char buf[BUFSIZE];
	
	TextFile f(b2kRoot,n);
	
	if (B_NO_ERROR!=f.InitCheck()){
		fprintf(stderr,"couldn't open:%s\n",n);
		return false;
	}
			
	/*	Player		(class)
	*/
		
	NEXTLN;
	PlayerCount=atoi(buf);
	printf("PlayerCount = %d\n",PlayerCount);
		
	Player=(ThingyBodyAddOn **)malloc( PlayerCount*sizeof(ThingyBodyAddOn *));
		
	for(int count=0;count<PlayerCount;count++)
	{
		NEXTLN; // skip brain
		NEXTLN;
		printf("	%s\n",buf);
		Player[count] = new ThingyBodyAddOn(b2kRoot,buf, offscreen);
		if (!(Player[count]->IsLoaded())) { PlayerCount=count; return false; };
	}
		
		
	/*	Enemies		(class)
	*/
	
	NEXTLN;
	EnemyCount=atoi(buf);
	printf("EnemyCount = %d\n",EnemyCount);
		
	Enemy=(ThingyBodyAddOn **)malloc( EnemyCount*sizeof(ThingyBodyAddOn *));
		
	for(int count=0;count<EnemyCount;count++)
	{
		NEXTLN; // skip brain
		NEXTLN;
		printf("	%s\n",buf);
		Enemy[count] = new ThingyBodyAddOn(b2kRoot,buf, offscreen);
		if (!(Enemy[count]->IsLoaded())) { EnemyCount=count; return false; };
	}
		
		
	/*	Bubbles		(class)
	*/
		
	NEXTLN;
	BubbleCount=atoi(buf);
	printf("\nBubbleCount = %d\n",BubbleCount);
		
	Bubble=(ThingyBodyAddOn **)malloc( BubbleCount*sizeof(ThingyBodyAddOn *));
		
	for(int count=0;count<BubbleCount;count++)
	{
		NEXTLN; // skip brain
		NEXTLN;
		printf("	%s\n",buf);
		Bubble[count] = new ThingyBodyAddOn(b2kRoot,buf,offscreen);
		if (!(Bubble[count]->IsLoaded())){ BubbleCount=count; return false; };
	}

	/*	NoColls		(class)
	*/
		
	NEXTLN;
	NoCollCount=atoi(buf);
	printf("\nNoCollCount = %d\n",NoCollCount);
		
	NoColl=(ThingyBodyAddOn **)malloc( NoCollCount*sizeof(ThingyBodyAddOn *));
		
	for(int count=0;count<NoCollCount;count++)
	{
		NEXTLN; // skip brain
		NEXTLN;
		printf("	%s\n",buf);
		NoColl[count] = new ThingyBodyAddOn(b2kRoot,buf,offscreen);
		if (!(NoColl[count]->IsLoaded())){ NoCollCount=count; return false; };
	}
		
			
	/*	Images
	*/

	NEXTLN;
	ImageCount=atoi(buf);
	printf("\nImageCount = %d\n",ImageCount);
	Images = (uint32 *)malloc(ImageCount*(640*480)*sizeof(uint32));
		
	for(int count=0;count<ImageCount;count++)
	{
		NEXTLN;
		printf("	%s\n",buf);
		BBitmap *b=BTranslationUtils::GetBitmap(buf, NULL);

		if (b && (b->Bounds()==BRect(0,0,639,479))){
/*			printf("color-space: ");
			switch(b->ColorSpace()){
				case B_RGB32		:puts("xRGB 8:8:8:8 small endian");break;
				case B_RGBA32		:puts("ARGB 8:8:8:8 small endian");break;
				case B_RGB32_BIG	:puts("xRGB 8:8:8:8 big endian");break;
				case B_RGBA32_BIG	:puts("ARGB 8:8:8:8 big endian");break;
			default	:puts("wrong");
			}
*/			
			uint32 *v=(uint32 *)b->Bits();
			int yc,xc,wpr=b->BytesPerRow()/4;
			for (yc=0;yc<480;yc++)
				for (xc=0;xc<wpr;xc++){// reversing byteorder
					Images[(count*640*480*4)+yc*640+xc]=(((*v) & 0xff000000) >> 24 ) | (((*v) & 0xff0000) >>8) | (((*v) & 0xff00) << 8) | (((*v) & 0xff) << 24);
					v++;
				}
		}
		if (b) delete b;
	}
	

	/*	Blocks
	*/

	BgBlocks=(uint32 *)malloc(sizeof(uint32)*16*16*256);
	Blocks=BgBlocks+128*256;

		
	BlockCount=0;
	NEXTLN;
	int Count=atoi(buf);
	printf("\nBlockCount = %d\n",Count);
		
	for(int count=0;count<Count;count++)
	{
		NEXTLN;
		printf("	%s\n",buf);
		BlockCount+=Load16x16(buf,((uint8 *)Blocks)+BlockCount*256*4,BlockCount);
	}
			
	/*	BackgroundBlocks
	*/
		
	BgBlockCount=1;
	NEXTLN;
	Count=atoi(buf);
	printf("\nBgBlockCount = %d\n",Count);
		
	for(int count=0;count<Count;count++)
	{
		NEXTLN;
		BgBlockCount+=Load16x16(buf,((uint8 *)BgBlocks)+BgBlockCount*4*256,BgBlockCount);
		printf("	%s\n",buf);
	}
		
	/*	Rum
	*/
		
	NEXTLN;
	RoomCount=atoi(buf);
	printf("\nRoomCount = %d\n",RoomCount);
		
	blocks = (uint8 *)malloc( RoomCount*(30*40));
	imgindex = (uint8 *)malloc( RoomCount);
	
	for(int count=0;count<RoomCount;count++)
	{
		NEXTLN; // name
		printf("\t%s\n",buf);
		NEXTLN;
		imgindex[count]=atoi(buf);
		NEXTLN; // left
		NEXTLN; // up
		NEXTLN; // right
		NEXTLN; // down
		
		for (int c=0;c<30;c++){
			NEXTLN;
			if (!(f.Line2Data(blocks+(count*30*40)+(40*c),buf))) return false;
		}
	}
		
	// ignore startplaces				
	return true;
}

/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	N E X T F R A M E
*
*/
#undef TIME

void B2KClient::NextFrame(const bigtime_t delta)
{
/*
* 	blittar från bg till draw och lägger till i blit_this
*/
	//puts("\n---Erase");
	int bc, ec, nc, pc;
	EraseStruct *erase;
	erase=erase_this;

#ifdef TIME
Start();
#endif

//		get counters
/*
	pc=((HeaderStruct *)draw_this)->player;
	ec=((HeaderStruct *)draw_this)->enemy;
	bc=((HeaderStruct *)draw_this)->bubble;
	nc=((HeaderStruct *)draw_this)->nocoll;
*/

// flyttar data erase_this->blit_this
	memcpy((void *)blit_this,(void *)erase_this,(total)*sizeof(EraseStruct));

	int countdown=total;
	while (countdown--){
		bg2draw(erase->x,erase->y,erase->width,erase->height);
		erase++;
	}

/*
*		säger åt alla thingys att rita ut
*/
//	puts("\n---Draw");
	static DrawStruct *draw_s;
	draw_s=draw_this;
	erase=erase_this;
	
//		get counters
	pc=((HeaderStruct *)draw_this)->player;
	ec=((HeaderStruct *)draw_this)->enemy;
	bc=((HeaderStruct *)draw_this)->bubble;
	nc=((HeaderStruct *)draw_this)->nocoll;
	draw_s++;

	//puts("\nPlayers");
	while (pc--){
		//putchar('.');
		PlayerDraw(draw_s,erase);
		draw_s->x +=(draw_s->xspeed * delta);
		draw_s->y +=(draw_s->yspeed * delta);
		erase++;
		draw_s++;
	}

	//puts("\nEnemies");
	while (ec--){
		//putchar('.');
		EnemyDraw(draw_s,erase);
		draw_s->x +=(draw_s->xspeed * delta);
		draw_s->y +=(draw_s->yspeed * delta);
		erase++;
		draw_s++;
	}

	//puts("\nBubbles");
	while (bc--){
		//putchar('.');
		BubbleDraw(draw_s,erase);
		draw_s->x +=(draw_s->xspeed * delta);
		draw_s->y +=(draw_s->yspeed * delta);
		erase++;
		draw_s++;
	}

//	puts("\nNoColl");
	while (nc--){
		//putchar('.');
		NoCollDraw(draw_s,erase);
		draw_s->x +=(draw_s->xspeed * delta);
		draw_s->y +=(draw_s->yspeed * delta);
		erase++;
		draw_s++;
	}

/*	blittar till vis
*/
	//puts("\n---Blit");
	erase=erase_this;
	static EraseStruct *blit_s;
	blit_s=blit_this;
		
//		get counters
	pc=((HeaderStruct *)draw_this)->player;
	ec=((HeaderStruct *)draw_this)->enemy;
	bc=((HeaderStruct *)draw_this)->bubble;
	nc=((HeaderStruct *)draw_this)->nocoll;
	
	int countdown2=pc+ec+bc+nc;
	countdown=total;

	while (countdown--){
		//putchar('.');
		draw2vis( erase->x, erase->y, erase->width, erase->height );
		erase++;
	}

	while (countdown2--){
		draw2vis( blit_s->x, blit_s->y, blit_s->width, blit_s->height );
		blit_s++;
	}

	total=pc+ec+bc+nc;
	
	//putchar('\n');
#ifdef TIME
Stop();
#endif
};


/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	S E N D
*
*/
#define KEY(x)	(ki.key_states[(x)>>3] & (1 << (x & 0x7)))
void B2KClient::SendKeys()
{
	printf("\nKEYS\n\t");
	Start();
	if (get_key_info(&ki)==B_OK){
	Stop();
		for (int c=0;c<65;c++)
			printf("%2x-%2x\n",KEY(c));
		printf("\n");
	}

}

/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	D R A W L O O P
*
*/
void B2KClient::DrawLoop()
{
	HeaderStruct *hs=(HeaderStruct *)draw_this;
	int tst;
	int fps=60;

	int period=1000000/fps;
	
	bigtime_t time=system_time();

	bigtime_t nextf=time+period;

	bigtime_t delay=0;

	bigtime_t stoptime=time+TEMPTIME;
	
	PutRoom(0,offscreen_bg);
	bg2draw(0,0,640,480);
	draw2vis(0,0,640,480);

///
/// Här skall jag ändra

	while(1){
	//puts("(B2KClient::DrawLoop)(draw)");
		if (B_OK==acquire_sem(sema)){
			
			// send keys that are pressed
			SendKeys();
			
			// reveive with security
			tst=r->Receive();
			if ((tst>0)&&((hs->room>=RoomCount)||(tst!= sizeof(DrawStruct) *(1+hs->enemy+hs->bubble+hs->nocoll+hs->player)))){
				hs->enemy=0;
				hs->bubble=0;
				hs->nocoll=0;
				hs->player=0;
				hs->room=0;
			}
							
			if (delay>0)
				NextFrame(period);
			else
				NextFrame(period-delay);
			
			delay=nextf-system_time();
			if (delay>0) snooze(delay);
			time=system_time();
			nextf=time+period;

			release_sem(sema);
		}
	}
	
	puts("(B2KClient::DrawLoop)(exiting)");

}


//**************************************
// public thread function
bool B2KClient::Go()
{
	puts("(Go!)");
	if (0>(draw_thread = spawn_thread(thread_func, "B2K-DrawLoop", B_NORMAL_PRIORITY, this)))
	{
		puts("(Go!)(spawn thread? I don't think so)");
		return false;		
	}
	resume_thread(draw_thread);
	return true;
}

//**************************************
// thread function
int32 B2KClient::thread_func(void *arg)
{
	((B2KClient *)arg)->DrawLoop();
	return 0;
}

//**************************************
// timing functions
void B2KClient::Start()
{
	t1=system_time();
}
void B2KClient::Stop()
{
	t2=system_time();
	t3=t2-t1;
	fprintf(stderr,"%6d us means %6d/s\n",t3,1000000/t3);
}
