#include <TranslationUtils.h>
#include <File.h>
#include <WindowScreen.h>
#include <string.h>
#include <assert.h>
#include "B2KScreen.h"
#include "TextFile.h"

//**************************************
// Constructor
B2KScreen::B2KScreen(BDirectory *root, const char *title,status_t *e):
		BWindowScreen(title,B_32_BIT_640x480,e)
{
	b2kroot = root;
	
	screen_sem=create_sem(1,"screen semaphore");
	first=true;

/*
	if (NULL == LoadRead("Graphics/bg1",480,3*640,(uint8 *)buf)){
		*e=-1;
		fprintf(stderr,"needed background\n");
	}
*/
	if (NULL == LoadRead("Graphics/font.raw",256-STRTCHR,256,(uint8 *)font)){
		*e=-1;
		fprintf(stderr,"needed font\n");
	}

	int servc=0;
	TextFile f(root,"AdressBook");
	while( (servc < MAXSERVERS) && f.NextLine((char *)txtbuf1,SBUFSIZE)){
		f.NextLine((char *)txtbuf2,SBUFSIZE);
		f.NextLine((char *)txtbuf3,SBUFSIZE);
		Servers[servc]=new Server(txtbuf1,txtbuf2,txtbuf3);
		servc++;
	}

};


//**************************************
// PutChr
void B2KScreen::PutChr(int x_,int y_,char v, uint8 r, uint8 g, uint8 b)
{
	if (v<STRTCHR || (x_+16>639) || (y_+16>479)) return;

	uint8 *gfx=font+(256*(v-STRTCHR));
	uint8 *mbase=((uint8 *)(offscreen))+4*(y_*640+x_);

	int _x,_y,res;
	int add=4*(640-16);


	for (_y=0;_y<16;_y++){
		for (_x=0;_x<16;_x++){

			mbase++; //skip 'alpha'

			res=*mbase + (*gfx * r >> 8);
			if (res>255) res=255;
			*mbase=res;
			mbase++;

			res=*mbase + (*gfx * g >> 8);
			if (res>255) res=255;
			*mbase=res;
			mbase++;

			res=*mbase + (*gfx * b >> 8);
			if (res>255) res=255;
			*mbase=res;
			mbase++;

			gfx++;
		}
		mbase+=add;
	}

}

//**************************************
// PutStr
void B2KScreen::PutStr(int x_, int y_, const char *text)
{
	while (*text){
		PutChr(x_,y_,*text,127+(rand() & 0x7f),127+(rand() & 0x7f),127+(rand() & 0x7f));
		if ('/t'==*text) x_+=40;
		x_+=10;
		text++;
	}	
}

//**************************************
// LoadRead
int B2KScreen::LoadRead(const char*name, int no,int size, uint8 *buffer)
{
	assert(size>0);
	
	int ret; // bytes read

	BFile f(b2kroot,name,B_READ_ONLY);
	if (B_NO_ERROR!=f.InitCheck()){
		fprintf(stderr,"error reading:%s\n",name);
		return 0;
	}
	ret=f.Read(buffer,no*size);	
	return ret/size;
}



//**************************************
// Destructor
B2KScreen::~B2KScreen()
{
puts("B2KScreen::~B2KScreen");
}


//**************************************
// ScreenConnected
void B2KScreen::ScreenConnected(bool connected)
{
	status_t ok=B_OK;
	bool ok2;
printf("B2KScreen::Screen%sonnected()\n",(connected)?("C"):("Disc"));
	if (connected)
	{

		if (first)
		{
			rowbytes=CardInfo()->bytes_per_row;
			rowwords=rowbytes>>2;

			fprintf(stderr,"	rowbytes=%d\n",rowbytes);
			vis=(uint32 *)CardInfo()->frame_buffer;
			fputs("	rgba_orger=",stderr);
			putc(CardInfo()->rgba_order[0],stderr);
			putc(CardInfo()->rgba_order[1],stderr);
			putc(CardInfo()->rgba_order[2],stderr);
			putc(CardInfo()->rgba_order[3],stderr);
			putc('\n',stderr);
			
			sock = new tcp_sock("0.0.0.0",LIST_PORT,&ok2);
			ok=(ok2)?(B_OK):(B_ERROR);

			if (ok==B_OK){
				sock->Receive(sbuf,200);
				fprintf(stderr,"	(B2KScreen)(I received)%s\n",sbuf);

				sbuf[0]='j';
				sbuf[1]='o';
				sbuf[2]='n';
				sbuf[3]='a';
				sbuf[4]='x';
				sbuf[5]=0;
				sock->Send(sbuf,6);
				
				printf("	(B2KScreen)ofscrbf=%p\n",(uint8 *)offscreen);
				// load world
				client = new B2KClient(sock,b2kroot,"Worlds/Default",&ok,(uint32 *)offscreen,vis,rowwords,screen_sem);
				printf("\n======================================\n");
				if (B_OK==ok){
					printf("===== SUCCESSFULLY LOADED %s\n===== ","Worlds/TestWorld++");
					fflush(stdout);
					system("date");
				}else{
					printf("===== ERROR\n");
				}	
			}				
			if (ok!=B_OK){
				printf("B2KScreen::ScreenConnected()(ok!=B_OK)\n");
				be_app->PostMessage(B_QUIT_REQUESTED);
			}
			first=false;
		}

		release_sem(screen_sem);		
		if ((ok==B_OK)&& !client->Go()){
				be_app->PostMessage(B_QUIT_REQUESTED);
		}
	}
	else
	{
		acquire_sem(screen_sem);
	}
}

