/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	B2K SERVER
*
*/
#include <stdio.h>
#include <string.h>

#include "Room.h"
#include "B2Kdefs.h"
#include "B2KDrawStruct.h"
#include "B2KServer.h"
#include "tcp_listener.h"
#include "tcp_sock.h"

#include "PlayerAddOn.h"
#include "Player.h"

#include "ThingyBrain.h"
#include "ThingyAddOn.h"

#include "TerminatorAddOn.h"
#include "Terminator.h"

uint8 B2KServer::Char2Val(char c)
{
	switch(c){
		case '0': return 0;
		case '1': return 1;
		case '2': return 2;
		case '3': return 3;
		case '4': return 4;
		case '5': return 5;
		case '6': return 6;
		case '7': return 7;
		case '8': return 8;
		case '9': return 9;
		case 'A': return 10;
		case 'B': return 11;
		case 'C': return 12;
		case 'D': return 13;
		case 'E': return 14;
		case 'F': return 15;
		case 'a': return 10;
		case 'b': return 11;
		case 'c': return 12;
		case 'd': return 13;
		case 'e': return 14;
		case 'f': return 15;
		default : return 0;
	}	
}

bool B2KServer::Line2Data(uint8 *to, char *text)
{
	bool ok=true;
	if (strlen(text)!=80){
		fprintf(stderr,"wrong linelength was %d should be 40\n",strlen(text));
		return false;
	}
	
	while (*text != 0){
		*to=Char2Val(*text++)<<4;
		*to|=Char2Val(*text++);
		//printf("%02x",*to);
		to++;
	}
	//putchar('\n');		
	return ok;
}

void B2KServer::jntok(char *b, char *m)
{
	bool cont=true;
	int mc=strlen(m);
	int c;
	while (*b!=0 && cont){
		for (c=0;c<mc;c++)
			if (*b == m[c]){
				*b=0;
				cont=false;
			}
		b++;
	}	
}
		

bool B2KServer::NextLine(FILE *stream,char *buf)
{
	bool ok=true;
	do {
		if (0==fgets(buf,BUFSIZE,stream)) ok=false;
		if (ok) jntok(buf,"#\n\t");		
	} while ( ok && ((*buf)==0));

	return ok;	
}

#define NEXTLN 	if (!(NextLine(f,buf))) { fclose(f); return false; }

bool B2KServer::LoadWorld(char *filename)
{
	EnemyClass=0;
	BubbleClass=0;
	NoCollClass=0;
	Rooms=0;

	EnemyCreateCount=0;
	BubbleCreateCount=0;
	NoCollCreateCount=0;
	RoomCount=0;
		
	char buf[BUFSIZE];
		
	FILE *f=fopen(filename,"r");
	
	if (f==0){
		fprintf(stderr,"%s: no such file\n",filename);
		return false;
	}

	/*
	*	Players		(class)
	*
	*/
	NEXTLN;
	PlayerCreateCount=atoi(buf);
	printf("\nPlayerCreateCount = %d\n",PlayerCreateCount);
		
	PlayerClass=(PlayerAddOn **)malloc( PlayerCreateCount*sizeof(PlayerAddOn *));
		
	for(int count=0;count<PlayerCreateCount;count++)
	{
		NEXTLN;
		printf("	%s\n",buf);
		PlayerClass[count] = new PlayerAddOn((char *)b2kRoot,buf);
		if (!(PlayerClass[count]->IsLoaded())) { PlayerCreateCount=count;  fclose(f); return false; };
		NEXTLN; // skip body
	}
		
		
	/*
	*	Enemies		(class)
		*
	*/
	NEXTLN;
	EnemyCreateCount=atoi(buf);
	printf("\nEnemyCreateCount = %d\n",EnemyCreateCount);
		
	EnemyClass=(ThingyAddOn **)malloc( EnemyCreateCount*sizeof(ThingyAddOn *));
		
	for(int count=0;count<EnemyCreateCount;count++)
	{
		NEXTLN;
		printf("	%s\n",buf);
		EnemyClass[count] = new ThingyAddOn((char *)b2kRoot,buf);
		if (!(EnemyClass[count]->IsLoaded())) { EnemyCreateCount=count;  fclose(f); return false; };
		NEXTLN; // skip body
	}
		
		
	/*
	*	Bubbles		(class)
	*
	*/
		
	NEXTLN;
	BubbleCreateCount=atoi(buf);
	printf("\nBubbleCreateCount = %d\n",BubbleCreateCount);
		
	BubbleClass=(ThingyAddOn **)malloc( BubbleCreateCount*sizeof(ThingyAddOn*));
		
	for(int count=0;count<BubbleCreateCount;count++)
	{
		NEXTLN;
		printf("	%s\n",buf);
		BubbleClass[count] = new ThingyAddOn((char *)b2kRoot,buf);
		if (!(BubbleClass[count]->IsLoaded())){ BubbleCreateCount=count; fclose(f); return false; };
		NEXTLN; //skip body
	}

	/*
	*	NoColls		(class)
	*
	*/
		
	NEXTLN;
	NoCollCreateCount=atoi(buf);
	printf("\nNoCollsCreateCount = %d\n",NoCollCreateCount);
		
	NoCollClass=(ThingyAddOn **)malloc( NoCollCreateCount*sizeof(ThingyAddOn *));
		
	for(int count=0;count<NoCollCreateCount;count++)
	{
		NEXTLN;
		printf("	%s\n",buf);
		NoCollClass[count] = new ThingyAddOn((char *)b2kRoot,buf);
		if (!(NoCollClass[count]->IsLoaded())){ NoCollCreateCount=count; fclose(f); return false; };
		//printf("(%d)(%p)%s\n",count,NoCollClass[count]->Path()); 
		NEXTLN; //skip body
	}

	/*	Images (ignore)
	*/
	
	NEXTLN;
	int ImageCount=atoi(buf);
	//printf("\nImageCount = %d\n",ImageCount);
		
	for(int count=0;count<ImageCount;count++)
	{
		NEXTLN;
		//printf("	%s\n",buf);
	}
			
		
	/*
	*	Blocks (ignore)
	*
	*/
		
	uint8 BlockCount;
	NEXTLN;
	BlockCount=atoi(buf);
	//printf("\nBlockCount = %d\n",BlockCount);
		
	for(int count=0;count<BlockCount;count++)
	{
		NEXTLN;
		//printf("	%s\n",buf);
	}
		
		
	/*
	*	BackgroundBlocks (ignore)
	*
	*/
		
	NEXTLN;
	BlockCount=atoi(buf);
	//printf("\nBgBlockCount = %d\n",BlockCount);
		
	for(int count=0;count<BlockCount;count++)
	{
		NEXTLN;
		//printf("	%s\n",buf);
	}
		
		
	/*
	*	Rum
	*
	*/
		
	NEXTLN;
	RoomCount=atoi(buf);
	printf("\nRoomCount = %d\n",RoomCount);
		
	Rooms=(Room *)malloc( RoomCount*sizeof(Room));
		
	for(int count=0;count<RoomCount;count++)
	{
		NEXTLN;
			
		Rooms[count].name=strdup(buf);
		printf("	%s\n",buf);
		NEXTLN; // skip image
		uint8 temp8;
		NEXTLN;
		temp8=atoi(buf);
		Rooms[count].left	=&Rooms[temp8];
		NEXTLN;
		temp8=atoi(buf);
		Rooms[count].up		=&Rooms[temp8];
		NEXTLN;
		temp8=atoi(buf);
		Rooms[count].right	=&Rooms[temp8];
		NEXTLN;
		temp8=atoi(buf);
		Rooms[count].down	=&Rooms[temp8];
			
		Rooms[count].firstenemy=NULL;
		Rooms[count].firstbubble=NULL;
		Rooms[count].firstnocoll=NULL;
		
		for (int c=0;c<30;c++){
			NEXTLN;
			if (!(Line2Data(Rooms[count].blocks+(40*c),buf))){ fclose(f); return false; };
		}
	}
		
		
			
	/*
	*	Terminator (class)
	*
	*/
	NEXTLN;
	printf("	%s\n",buf);
	TerminatorClass = new TerminatorAddOn((char *)b2kRoot,buf);
	if (!(TerminatorClass->IsLoaded())) { fclose(f); return false; };
	
	NEXTLN;
	MyTerminator = TerminatorClass->TerminatorCreate(this, Rooms, RoomCount,buf);
	if (!MyTerminator) { fclose(f); return false; };
	
	/*
	*	Enemys
	*
	*/
	
		
	uint8 EnemyCount;
	NEXTLN;
	EnemyCount=atoi(buf);
	
	printf("\nEnemyCount = %d\n",EnemyCount);
		
	Thingy *TempEnemy;
	uint8 index;
	uint8 roomindex;
	uint32 x;
	uint32 y;
	int16 xspeed;
	int16 yspeed;
		
	for(int count=0;count<EnemyCount;count++)
	{
		NEXTLN;		index=atoi(buf);
		if (index>EnemyCreateCount-1){ fclose(f); return false; };
		NEXTLN;		roomindex=atoi(buf);
		NEXTLN;		x=atoi(buf)<<22;
		NEXTLN;		y=atoi(buf)<<22;
		NEXTLN;		xspeed=atoi(buf);
		NEXTLN;		yspeed=atoi(buf);
		NEXTLN;
		TempEnemy= NewEnemy(index, &Rooms[roomindex], x, y, xspeed, yspeed,buf);
		if (TempEnemy) printf("	%s by %s @(%d,%d)(%d,%d)\n",TempEnemy->Name(),TempEnemy->CreatorName(),x>>22,y>>22,xspeed,yspeed);
	}
		
	/*
	*	Bubbles
	*
	*/
		
	uint8 BubbleCount;
	NEXTLN;
	BubbleCount=atoi(buf);
	printf("\nBubbleCount = %d\n",BubbleCount);
		
	Thingy *TempBubble;
		
	for(int count=0;count<BubbleCount;count++)
	{
		NEXTLN;		index=atoi(buf);
		if (index>BubbleCreateCount-1){ fclose(f); return false; };
		NEXTLN;		roomindex=atoi(buf);
		NEXTLN;		x=atoi(buf)<<22;
		NEXTLN;		y=atoi(buf)<<22;
		NEXTLN;		xspeed=atoi(buf);
		NEXTLN;		yspeed=atoi(buf);
		NEXTLN;		
			
		TempBubble= NewBubble(index, &Rooms[roomindex], x, y, xspeed, yspeed,buf);
		if (TempBubble) printf("	%s by %s\n",TempBubble->Name(),TempBubble->CreatorName());
	}

	/*
	*	NoColls
	*
	*/
		
	uint8 NoCollCount;
	NEXTLN;
	NoCollCount=atoi(buf);
	printf("\nNoCollCount = %d\n",NoCollCount);
		
	Thingy *TempNoColl;
		
	for(int count=0;count<NoCollCount;count++)
	{
		NEXTLN;		index=atoi(buf);
		if (index>NoCollCreateCount-1){ fclose(f); return false; };
		NEXTLN;		roomindex=atoi(buf);
		NEXTLN;		x=atoi(buf)<<22;
		NEXTLN;		y=atoi(buf)<<22;
		NEXTLN;		xspeed=atoi(buf);
		NEXTLN;		yspeed=atoi(buf);
		NEXTLN;		
			
		TempNoColl= NewNoColl(index, &Rooms[roomindex], x, y, xspeed, yspeed,buf);
		if (TempNoColl) printf("	%s by %s\n",TempNoColl->Name(),TempNoColl->CreatorName());
	}
		
	fclose(f);
	return true;
}


/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	C O N S T R U C T O R
*
*/

B2KServer::B2KServer(char *root,char* world,bool *ok)
{
/*
	putchar('\n');
	puts(world);
	putchar('\n');
*/
	cyklethread=NULL;
	b2kRoot=root;
	currworld = strdup(world);
	
	printf("==========================================\n");
	printf("B2KRoot  : %s\n",b2kRoot);
	char * FileName=(char *)malloc(B_PATH_NAME_LENGTH);
	strcat(FileName,b2kRoot);
	strcat(FileName,"/");
	strcat(FileName,world);

	if (!LoadWorld(FileName)) {
		printf("==========================================\n");
		printf("===== ERROR %s\n",world);
		printf("==========================================\n");
		*ok=false;
		return;
	}

	printf("==========================================\n");
	printf("===== SUCCESSFULLY LOADED %s\n===== ",world);
	fflush(stdout);
	system("date");
	printf("==========================================\n");
	printf("This world will stop when %s\n",MyTerminator->Desc());
	
	if ((room_sem = create_sem(1, "room_sem")) < B_NO_ERROR){
		printf("No create room semaphore\n");
		*ok=false;
		return;
	}
}

/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	D E S T R U C T O R
*
*/

#define FREE(x) if (x) {free(x); x=0; }
B2KServer::~B2KServer()
{
	/*	-------------------------------------------------------------
		delete things in world
	*/
	Thingy *TempThingy;
	for (int count=0;count<RoomCount;count++) // för alla rum
	{
		
		// för alla enemys
		while (TempThingy=Rooms[count].firstenemy)
		{
			TempThingy->Remove();
			delete TempThingy;
		}
		
		// för alla bobbles
		while (TempThingy=Rooms[count].firstbubble)
		{
			TempThingy->Remove();
			delete TempThingy;
		}

		// för alla nocolls
		while (TempThingy=Rooms[count].firstnocoll)
		{
			TempThingy->Remove();
			delete TempThingy;
		}
	}

	/*	-------------------------------------------------------------
		delete addons
	*/
	while (EnemyCreateCount>0)
		delete &EnemyClass[--EnemyCreateCount];
	
	while (BubbleCreateCount>0)
		delete &BubbleClass[--BubbleCreateCount];
	
	FREE(EnemyClass);
	FREE(BubbleClass);
	FREE(Rooms);
	FREE(currworld);
	
}

/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	C R E A T E P A C K E T
*
*/
void B2KServer::CreatePacket()
{
// skall gaa igenom rummen och fixa paket där det finns players.

	for (int c=0;c<RoomCount;c++){
		if (Rooms[c].firstplayer){
//puts("(B2KServer)::(CreatePacket)");
			int cp,cb,ce,cn;
			((HeaderStruct *)draw_this)->room=c;
	
			DrawStruct *curr=draw_this+1;
		


			Thingy *thy=Rooms[c].firstplayer;		
			for (cp=0;(thy!=NULL && cp<THINGYSMAX);cp++)
			{
				thy->GetPos(curr);
				thy=thy->Next();
				curr++;
//		putchar('.');
			}
//puts("Enemies");
			((HeaderStruct *)draw_this)->player=cp;


			thy=Rooms[c].firstenemy;		
			for (ce=0;(thy!=NULL && (cp+ce)<THINGYSMAX);ce++)
			{
				thy->GetPos(curr);
				thy=thy->Next();
				curr++;
//		putchar('.');
			}
//puts("Enemies");
			((HeaderStruct *)draw_this)->enemy=ce;

	
			thy=Rooms[c].firstbubble;
			for (cb=0;(thy!=NULL && (cp+ce+cb)<THINGYSMAX);cb++){
				thy->GetPos(curr);
				thy=thy->Next();
				curr++;
//		putchar('.');
			}
//puts("Bubbles");
			((HeaderStruct *)draw_this)->bubble=cb;

	
			thy=Rooms[c].firstnocoll;
			for (cn=0;(thy!=NULL && (cp+ce+cb+cn)<THINGYSMAX);cn++){
				thy->GetPos(curr);
				thy=thy->Next();
				curr++;
//		putchar('.');
			}
//puts("NoColls");
			((HeaderStruct *)draw_this)->nocoll=cn;

			Player *ply=Rooms[c].firstplayer;
			while (ply)
			{
				ply->Send((char *)draw_this,(cp+ce+cb+cn+1)*sizeof(DrawStruct));
				ply=(Player *)ply->Next();
			}
		}
	}		
}


/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	L I S T E N E R
*
*/
void B2KServer::Listener()
{
	char buf[BUFSIZE];
	bool ok;
	int index;
	tcp_sock *n;
	Sender *s;
	tcp_listener *l= new tcp_listener(LIST_PORT,&ok);
	if (!ok) return;
	
	while ( 1 ){
		n = l->Accept();
		n->Send(currworld,strlen(currworld)+1);
		n->Receive((char *)buf,BUFSIZE);
		
		s=new Sender("192.168.0.1",UDP_PORT,&ok);

		if (!ok){
			delete s;
			continue;
		}
		
		if (B_OK==acquire_sem(room_sem)){
			NewPlayer(s, n, 0,Rooms, 100,100,0,0,NULL);
			release_sem(room_sem);
		}
		printf("(B2KServer::Listener)\n<%s>\n<%d>\n",buf,buf[2+strlen((char *)buf)]);
	}
}

/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	G O
*
*/
long ThreadFunc(void *s);
long ThreadFunc(void *s) { ((B2KServer *)s)->Listener(); return 0;}
bool B2KServer::Go()
{
	// fixa traad
	listen_thread = spawn_thread( ThreadFunc,"B2KListener",B_NORMAL_PRIORITY,(void *)this);

	if (listen_thread <= 0){ puts("list_err");  return false;}	
	resume_thread(listen_thread);
	CycleThread();
	return true;
}


/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	C Y C L E T H R E A D
*
*/
void B2KServer::CycleThread()
{
		printf("\n\nCycla lite...\n");
		
#define MAX_CYKLERATE 16667 // 60Hz
#define SKIP 5;	// var 6:e cykel
		

		uint skip=SKIP; 
		int count=0;
		int64 us =system_time();
		int64 oldus =us;
		int64 micsec =system_time();
//		int64 breakus = micsec+TEMPTIME;
		
		if (acquire_sem(room_sem) != B_NO_ERROR){ puts("sem_err"); return;}
		while ( MyTerminator->Cycle(us))
		{
			Cycle();
			if (0==skip--) {CreatePacket(); skip=SKIP; }		
			release_sem(room_sem);

			oldus=oldus+MAX_CYKLERATE;
			us =system_time();
			if ((us-oldus)<MAX_CYKLERATE) snooze(MAX_CYKLERATE-(us-oldus));
			count++;
		if (acquire_sem(room_sem) != B_NO_ERROR){ puts("sem_err"); return;}
		}
		CreatePacket();
		release_sem(room_sem);
		
		printf("\nmade %d cykles in %d us\nmedelcps: %f\n",count,TEMPTIME,(float)1000000*count/(float)(us-micsec));
		
}


/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*	C Y C L E
*
*/

void	B2KServer::Cycle()
{
//	printf("Cycle\n");
	
//	Stat();
	Thingy *TempTh;
	Thingy *NextTh;
	
	for (int count=0;count<RoomCount;count++) // för alla rum
	{
		// för alla enemys
		TempTh=Rooms[count].firstenemy;
		while (TempTh){
			if (TempTh->Cycle(system_time())){
				TempTh=TempTh->Next();
			}else{
				NextTh=TempTh->Next();
				TempTh->Remove();
				delete TempTh;
				
				TempTh=NextTh;
			}
		}
		
		// för alla bubbles
		TempTh=Rooms[count].firstbubble;
		while (TempTh){
			if (TempTh->Cycle(system_time())){
				TempTh=TempTh->Next();
			}else{
				NextTh=TempTh->Next();
				TempTh->Remove();
				delete TempTh;
				
				TempTh=NextTh;
			}
		}
		
		// för alla nocolls
		TempTh=Rooms[count].firstnocoll;
		while (TempTh){
			if (TempTh->Cycle(system_time())){
				TempTh=TempTh->Next();
			}else{
				NextTh=TempTh->Next();
				TempTh->Remove();
				delete TempTh;
				
				TempTh=NextTh;
			}
		}
	}	
}


/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*  skriver ut statistik
*/

void	B2KServer::Stat()
{
	int cp;
	Thingy *thy;
	
	for (int count=0;count<RoomCount;count++) // för alla rum
	{
		printf("Room %d is inhabited by:\n",count);
		
		thy=(Thingy *)Rooms[count].firstplayer;
		for (cp=0;thy!=NULL;cp++) thy=thy->Next();
		printf("	%d players\n",cp);

		thy=Rooms[count].firstenemy;
		for (cp=0;thy!=NULL;cp++) thy=thy->Next();
		printf("	%d enemies\n",cp);

		thy=Rooms[count].firstbubble;
		for (cp=0;thy!=NULL;cp++) thy=thy->Next();
		printf("	%d bubbles\n",cp);

		thy=Rooms[count].firstnocoll;
		for (cp=0;thy!=NULL;cp++) thy=thy->Next();
		printf("	%d nocolls\n",cp);
		
		putchar('\n');
	}
}

/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*  returnerar ett index mot ett pathname negativt fel
*/

int		B2KServer::EnemyPath2Index(char *path)
{
	int c=EnemyCreateCount-1;
	while (c>=0 && (strcmp(EnemyClass[c]->Path(),path)))
		c--;
	return c;
}

int		B2KServer::BubblePath2Index(char *path)
{
	int c=BubbleCreateCount-1;
	while (c>=0 && (strcmp(BubbleClass[c]->Path(),path)))
		c--;
	return c;
}

int		B2KServer::NoCollPath2Index(char *path)
{
	int c=NoCollCreateCount-1;
	while (c>=0 && (strcmp(NoCollClass[c]->Path(),path)))
	{
		c--;
	}
	return c;
}

/*  :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
*  lägger till i rätt lista
*/

Player	*B2KServer::NewPlayer(Sender *s, tcp_sock *tcp, uint32 index, Room *room, uint32 x, uint32 y, int32 xspeed, int32 yspeed,char *arg)
{
	if (index<PlayerCreateCount && PlayerClass[index]->IsLoaded() )
		return PlayerClass[index]->PlayerCreate(s,tcp,this,index,room,x,y,xspeed,yspeed,system_time(),arg);
	else
		return NULL;
}

Thingy	*B2KServer::NewEnemy(uint32 index, Room *room, uint32 x, uint32 y, int32 xspeed, int32 yspeed,char *arg)
{
	if (index<EnemyCreateCount && EnemyClass[index]->IsLoaded() )
		return EnemyClass[index]->ThingyCreate(this,index,room,x,y,xspeed,yspeed,system_time(),ENEMY,arg);
	else
		return NULL;
}

Thingy	*B2KServer::NewBubble(uint32 index, Room *room, uint32 x, uint32 y, int32 xspeed, int32 yspeed,char *arg)
{
	if (index<BubbleCreateCount	&& BubbleClass[index]->IsLoaded() )
		return BubbleClass[index]->ThingyCreate(this,index,room,x,y,xspeed,yspeed,system_time(),BUBBLE,arg);
	else
		return NULL;
}

Thingy	*B2KServer::NewNoColl(uint32 index, Room *room, uint32 x, uint32 y, int32 xspeed, int32 yspeed,char *arg)
{
	if (index<NoCollCreateCount	&& NoCollClass[index]->IsLoaded() )
		return NoCollClass[index]->ThingyCreate(this,index,room,x,y,xspeed,yspeed,system_time(),NOCOLL,arg);
	else
		return NULL;
}

