/*
*	B2K BadGuy
*
*/

#include "BadGuyBrain.h"
#include "ThingyBrain.h"
#include "Room.h"
#include "B2KServer.h"

BadGuy::BadGuy(B2KServer *serv, uint32 indexnr, Room *room,uint32 x, uint32 y, int32 xspeed, int32 yspeed, bigtime_t timenow,uint8 kind, char *arg):
Thingy(serv,indexnr, room, x, y, xspeed, yspeed, timenow, kind, arg)
{
	bubble_index = server->BubblePath2Index("Thingys/BadBubble.so");
	spark_index = server->NoCollPath2Index("Thingys/Spark.so");
	minispark_index = server->NoCollPath2Index("Thingys/MiniSpark.so");
/*	
	printf("bubbleindex: %d\n",bubble_index);
	printf("sparkindex: %d\n",spark_index);
	printf("minisparkindex: %d\n",minispark_index);
*/
	lives=((bubble_index>=0) && (spark_index>=0) && (minispark_index>=0));

	width=16<<22;
	height=16<<22;
	weight=100;
	health=1000;
	fire_rate=900000;

	next_shoot=system_time()+(rand()<<5);
}


/* returnerar om enemyn skall fortsätta leva eller tas bort
 *
 * 'I cycle, therefore I am'
 *			  Jonas Norberg
 */

bool	BadGuy::Cycle(bigtime_t timenow)
{
	bigtime_t delta_t=timenow-time;

	Thingy *TempBubble;
	// för alla bubbles
	TempBubble=room->firstbubble;
	while (TempBubble)
	{
		if (TempBubble->Intersects(x,y,x+width,y+height)){
			health-=TempBubble->GetDamage();
			TempBubble->HitBy(xspeed,yspeed,weight);

			server->NewNoColl(minispark_index,room,x+(8<<22),y+(8<<22),rand() % 0xff - 128,rand() % 0xff - 128,NULL);
			//fprintf(stderr,"GuyHIT! h:%d\n",health);
		}
		
		TempBubble = TempBubble->Next();
	}


	// kolla hälsan
	if (health<1) lives=false;

	if (timenow>next_shoot){
		if (xspeed < 0)
			server->NewBubble(bubble_index,room,x-(17<<22),y,xspeed-200,0,NULL);
		else
			server->NewBubble(bubble_index,room,x+(17<<22),y,xspeed+200,0,NULL);
		next_shoot=system_time()+fire_rate+(rand()<<5);
	}
		
	// fall
	yspeed+=(weight*delta_t/300000);

	// röra sig
	y+=(yspeed*delta_t);
	x+=(xspeed*delta_t);

	RoomCollision();
	
	// titta åt rätt håll
	if (xspeed>0) state=0; else state=2;
	if (yspeed<0) state++;
	
	//printf("guy: x=%d y=%d\n",x>>22,y>>22);

	if (!lives){	// burst
		server->NewNoColl(spark_index,room,x+(8<<22),y+(8<<22),xspeed+420,	yspeed-500,	NULL);
		server->NewNoColl(spark_index,room,x+(8<<22),y+(8<<22),xspeed+40,	yspeed+300,	NULL);
		server->NewNoColl(spark_index,room,x+(8<<22),y+(8<<22),xspeed-120,	yspeed+10,	NULL);
		server->NewNoColl(spark_index,room,x+(8<<22),y+(8<<22),xspeed-240,	yspeed-680,	NULL);
		server->NewNoColl(spark_index,room,x+(8<<22),y+(8<<22),xspeed-150,	yspeed-230,	NULL);
		server->NewNoColl(spark_index,room,x+(8<<22),y+(8<<22),xspeed-640,	yspeed+210,	NULL);
		server->NewNoColl(spark_index,room,x+(8<<22),y+(8<<22),xspeed+350,	yspeed+110,	NULL);
		server->NewNoColl(spark_index,room,x+(8<<22),y+(8<<22),xspeed+640,	yspeed+310,	NULL);
		server->NewNoColl(spark_index,room,x+(8<<22),y+(8<<22),xspeed-350,	yspeed+490,	NULL);
		server->NewNoColl(spark_index,room,x+(8<<22),y+(8<<22),xspeed+20,	yspeed-70,	NULL);
	}
	
	time=timenow;
	return lives;
}


void	BadGuy::HitBy(int32 xspeed, int32 yspeed, int32 weight)
{
}

// returnerar hur mycket skada en krock (med badguy) ger dig
int32	BadGuy::GetDamage()
{
	return 1;
}


void BadGuy::Room_ClearFloor()
{
	yspeed=-abs(yspeed);
	y+=(yspeed*2);
	server->NewNoColl(minispark_index,room,x+(8<<22),y+(8<<22),rand() % 0xff - 128,-(rand() % 0xff),NULL);
}

char	*BadGuy::Name()
{
	return "BadGuy";
}

char	*BadGuy::CreatorName()
{
	return "Jonas Norberg (c)1998";
}

