/*
*	B2K Thingy
*
*	(c) Jonas Norberg 1998-07-26
*/

#include "ThingyBrain.h"
#include "B2Kdefs.h"
#include "Room.h"

__declspec(dllexport) Thingy::Thingy(B2KServer *serv, uint32 indexnr, Room *startroom,uint32  xpos, uint32  ypos, int32 xstartspeed, int32 ystartspeed, bigtime_t timenow,uint8 kind, char *arg)
{
	width=0;
	height=0;
	
	mykind=kind;
	server=serv;
	index=indexnr;
	x=xpos;
	y=ypos;
	xspeed=xstartspeed;
	yspeed=ystartspeed;
	time=timenow;
	state=0;
	Add(startroom);
}

// returnerar om enemyn skall fortsätta leva eller tas bort
__declspec(dllexport) bool Thingy::Cycle(bigtime_t timenow)
{
	fprintf(stderr,"Cycle not implemented in %s!\n",Name());
	return false;
}

__declspec(dllexport) void Thingy::HitBy(int32 xspeed, int32 yspeed, int32 weight)
{
}

__declspec(dllexport) bool Thingy::Intersects(uint32 left, uint32 top, uint32 right, uint32 down)
{
	return( left<(x+width) && right>x && top<(y+height) && down>y );
}

// returnerar hur mycket skada en krock ger dig som krockar
__declspec(dllexport) int32 Thingy::GetDamage()
{
	return 0;
}

__declspec(dllexport) char *Thingy::Name()
{
	return "unnamed";
}

__declspec(dllexport) char *Thingy::CreatorName()
{
	return "unnamed";
}



// #######################################################
// HELP FUNCTIONS

// testerna om man kolliderar med
/*
	3        2
	 ########
	 ########
	 ########
	 ########
	1        0
*/

	#define TURNX	{xspeed=-xspeed; x+=(xspeed*2);}
	#define TURNY	{yspeed=-yspeed; y+=(yspeed*2);}
	#define TURN	{TURNX;TURNY;}

	#define GOUP	{yspeed=-abs(yspeed);	y+=(yspeed*2);}
	#define GODOWN	{yspeed=abs(yspeed);	y+=(yspeed*2);}
	#define GOLEFT	{xspeed=-abs(xspeed);	x+=(xspeed*2);}
	#define GORIGHT	{xspeed=abs(xspeed);	x+=(xspeed*2);}
	
__declspec(dllexport) void Thingy::Room_ClearFloor()		GOUP
__declspec(dllexport) void Thingy::Room_ClearLeftWall()		GORIGHT
__declspec(dllexport) void Thingy::Room_ClearRightWall()	GOLEFT
__declspec(dllexport) void Thingy::Room_ClearCeiling()		{}
__declspec(dllexport) void Thingy::Room_LeftUpFree()		{GOLEFT;	GOUP;}
__declspec(dllexport) void Thingy::Room_LeftDownFree()		{GOLEFT;	GODOWN;}
__declspec(dllexport) void Thingy::Room_RightUpFree()		{GORIGHT;	GOUP;}
__declspec(dllexport) void Thingy::Room_RightDownFree()		{GORIGHT;	GODOWN;}
__declspec(dllexport) void Thingy::Room_LeftUpCorner()		{if ( (x & (15<<22)) > (y & (15<<22)) ) GORIGHT;/*else GODOWN;*/}
__declspec(dllexport) void Thingy::Room_LeftDownCorner()	{if (((16<<22) - (x & (15<<22))) > ((y+height) & (15<<22)) ) GOUP else GORIGHT}
__declspec(dllexport) void Thingy::Room_RightUpCorner()		{if ( ((x+width) & (15<<22)) > ( (16<<22) - (y & (15<<22)))) {/*GODOWN*/} else GOLEFT;}
__declspec(dllexport) void Thingy::Room_RightDownCorner()	{if (((x+(width)) & (15<<22))> (y+width & (15<<22)) ) GOUP else GOLEFT}
__declspec(dllexport) void Thingy::Room_Trapped()			TURNX
__declspec(dllexport) void Thingy::Room_Clear()				{}
__declspec(dllexport) void Thingy::Room_LeftUpDiagonal()	TURN
__declspec(dllexport) void Thingy::Room_LeftDownDiagonal()	TURN


__declspec(dllexport) void Thingy::RoomCollision()
{
	bool t3,t2,t1,t0;	
	
	t3=(127 < room->blocks[(y>>26)*40+(x>>26)]);
	t2=(127 < room->blocks[(y>>26)*40+((x+width)>>26)]);
	t1=(127 < room->blocks[((y+height)>>26)*40+(x>>26)]);
	t0=(127 < room->blocks[((y+height)>>26)*40+((x+width)>>26)]);

	if (t0){
		if (t1){
			if (t2)	if (t3) Room_Trapped();			else Room_LeftUpFree();
			else	if (t3) Room_RightUpFree();		else Room_ClearFloor();
		}else{//t1 is 0
			if (t2)	if (t3)	Room_LeftDownFree();	else Room_ClearRightWall();
			else	if (t3)	Room_LeftUpDiagonal();	else Room_RightDownCorner();
		}
	}else{//t0 is 0
		if (t1){
			if (t2)	if (t3)	Room_RightDownFree();	else Room_LeftDownDiagonal();
			else	if (t3)	Room_ClearLeftWall();	else Room_LeftDownCorner();
		}else{//t1 is 0
			if (t2)	if (t3)	Room_ClearCeiling();	else Room_RightUpCorner();
			else	if (t3)	Room_LeftUpCorner();	else Room_Clear();
		}
	}
}


// "#######################################################"
//   "########### following implemented by me ###########"

__declspec(dllexport) void Thingy::GetPos(DrawStruct *ds)
{
	ds->index = index;
	ds->x = x;
	ds->y = y;
	ds->xspeed = xspeed;
	ds->yspeed = yspeed;
	ds->state = state;
}

__declspec(dllexport) int32 Thingy::Health()
{
	return health;
}

__declspec(dllexport) int32 Thingy::Weight()
{
	return weight;
}

/////////////////////////
// these for list things.
__declspec(dllexport) bool	Thingy::IsLast()
{
	return (next_Thingy==NULL);
}

__declspec(dllexport) bool	Thingy::IsFirst()
{
	return (prev_Thingy==NULL);
}

__declspec(dllexport) Thingy	*Thingy::Next()
{
	return next_Thingy;
}

__declspec(dllexport) Thingy	*Thingy::Prev()
{
	return prev_Thingy;
}

__declspec(dllexport) void	Thingy::SetNext(Thingy *next)
{
	next_Thingy=next;
}

__declspec(dllexport) void	Thingy::SetPrev(Thingy *prev)
{
	prev_Thingy=prev;
}

__declspec(dllexport) void	Thingy::Add(Room *newroom)	 				// kollar room->FirstX
{
	room=newroom;
	
	Thingy **first;

	//printf("firstenemy is %p\n",&room->firstenemy);
	//printf("firstbubble is %p\n",&room->firstbubble);

	switch (mykind)
	{
		case ENEMY	: first=&room->firstenemy;	break;
		case BUBBLE	: first=&room->firstbubble; break;
		case NOCOLL	: first=&room->firstnocoll; break;
		case PLAYER	: first=&room->firstplayer; break;
		default		: first=NULL;
	}

	//printf("mykind has the value of %d, ENEMY is %d and BUBBLE is %d\n",mykind,ENEMY,BUBBLE);
	//printf("first is %p\n",first);
	//printf("and points at %p\n",*first);
	
	if (first)
	{
		//printf("adding myself to the list between %p and %p\n",Prev(),Next());
		SetPrev(NULL);
		SetNext(first[0]);
		if (*first) (*first)->SetPrev(this);
		*first=this;
	} // else printf("(Brain)	couldn't add myself to the list\n");

}

__declspec(dllexport) void	Thingy::Remove() 				// kollar room->FirstX
{
	if (prev_Thingy!=NULL)
		Prev()->SetNext(Next());
	else
	{
		Thingy **first;
		switch (mykind)
		{
			case ENEMY	: first=&room->firstenemy;	break;
			case BUBBLE	: first=&room->firstbubble;	break;
			case NOCOLL	: first=&room->firstnocoll; break;
			case PLAYER	: first=&room->firstplayer; break;
			default		: first=NULL;
		}
		if (first) *first=Next();
	}
	if (next_Thingy!=NULL) Next()->SetPrev(Prev());
}

__declspec(dllexport) void	Thingy::MoveTo(Room *newroom)	// kollar room->X,
{										// ändrar den privata room.
	Remove();
	Add(newroom);
}
