#include "WorldView.h"

#include <stdio.h>
#include <string.h>
#include <View.h>

int	WorldView::Load16x16(char *filename, uint8 *data, uint8 loaded)
{
	int count;
	FILE *f2=fopen(filename,"rb");
	
	if (f2==0){
		fprintf(stderr,"%s: no such file (Load16x16)\n",filename);
		return 0;
	}
	
	count=fread(data,4*16*16,128-loaded,f2);
	fclose(f2);
	printf("%s contains %d blocks read to %p\n",filename,count,data);	
	return count;
}

void WorldView::WriteRoom(FILE *f, uint8 index)
{
	int x,y;
	uint8 *data=Rooms[index].blocks;

	putc('\n',f);
	fprintf(f,"%s\t# name of this room\n",Rooms[index].name);
	fprintf(f,"%d\t# up\n",		(Rooms[index].up 	-	Rooms));
	fprintf(f,"%d\t# right\n",	(Rooms[index].right - 	Rooms));
	fprintf(f,"%d\t# down\n",	(Rooms[index].down 	- 	Rooms));
	fprintf(f,"%d\t# left\n",	(Rooms[index].left 	- 	Rooms));
	
	
	for (y=0;y<30;y++){
		for (x=0;x<40;x++){
			fprintf(f,"%02X",*(data++));
		}
		putc('\n',f);
	}

}

uint8 WorldView::Char2Val(char c)
{
	switch(c){
		case '0': return 0;
		case '1': return 1;
		case '2': return 2;
		case '3': return 3;
		case '4': return 4;
		case '5': return 5;
		case '6': return 6;
		case '7': return 7;
		case '8': return 8;
		case '9': return 9;
		case 'A': return 10;
		case 'B': return 11;
		case 'C': return 12;
		case 'D': return 13;
		case 'E': return 14;
		case 'F': return 15;
		case 'a': return 10;
		case 'b': return 11;
		case 'c': return 12;
		case 'd': return 13;
		case 'e': return 14;
		case 'f': return 15;
		default : return 0;
	}	
}

bool WorldView::Line2Data(uint8 *to, char *text)
{
	bool ok=true;
	if (strlen(text)!=80){
		fprintf(stderr,"wrong linelength was %d should be 40\n",strlen(text));
		return false;
	}
	
	while (*text != 0){
		*to=Char2Val(*text++)<<4;
		*to|=Char2Val(*text++);
		//printf("%02x",*to);
		to++;
	}
	//putchar('\n');		
	return ok;
}

void WorldView::jntok(char *b, char *m)
{
	bool cont=true;
	int mc=strlen(m);
	int c;
	while (*b!=0 && cont){
		for (c=0;c<mc;c++)
			if (*b == m[c]){
				*b=0;
				cont=false;
			}
		b++;
	}	
}
		

bool WorldView::NextLine(FILE *stream,char *buf)
{
	bool ok=true;
	do {
		if (0==fgets(buf,BUFSIZE,stream)) ok=false;
		if (ok) jntok(buf,"#\n\t");		
	} while ( ok && ((*buf)==0));
	//puts(buf);
	return ok;	
}

bool WorldView::LoadWorld(char *filename)
{
	char buf[BUFSIZE];
	
	FILE *f=fopen(filename,"r");
	
	if (f==0){
		fprintf(stderr,"%s: no such file\n",filename);
		return false;
	}
			
		
	/*
	*	Enemies		(class)
	*/
	uint Count;

	NextLine(f,buf);
	Count=atoi(buf);		
	for(int count=0;count<Count;count++)
	{
		NextLine(f,buf); // brain		
		NextLine(f,buf); // body
	}
		
		
	/*
	*	Bubbles		(class)
	*/
		
	NextLine(f,buf);
	Count=atoi(buf);
	for(int count=0;count<Count;count++)
	{
		NextLine(f,buf); // brain		
		NextLine(f,buf); // body
	}

	/*
	*	NoColl		(class)
	*/
		
	NextLine(f,buf);
	Count=atoi(buf);	
	for(int count=0;count<Count;count++)
	{
		NextLine(f,buf); // brain		
		NextLine(f,buf); // body
	}

		
	/*	Images (skip for now)
	*/
	
	NextLine(f,buf);
	Count=atoi(buf);
		
	for(int count=0;count<Count;count++)
	{
		NextLine(f,buf);		
	}

	/*
	*	Blocks
	*
	*/
	BgBlocks=(uint32 *)malloc(4*16*16*256);
	Blocks=BgBlocks+128*256;

		
	BlockCount=0;
	NextLine(f,buf);
	Count=atoi(buf);
	printf("\nBlockCount = %d\n",Count);
		
	for(int count=0;count<Count;count++)
	{
		NextLine(f,buf);		
		BlockCount+=Load16x16(buf,(uint8 *)Blocks+4*BlockCount*256,BlockCount);
		//printf("	%s\n",buf);
	}
		
		
	/*
	*	BackgroundBlocks
	*
	*/
		
	BgBlockCount=1;
	NextLine(f,buf);
	Count=atoi(buf);
	printf("\nBgBlockCount = %d\n",Count);
		
	for(int count=0;count<Count;count++)
	{
		NextLine(f,buf);		
		BgBlockCount+=Load16x16(buf,(uint8 *)BgBlocks+4*BgBlockCount*256,BgBlockCount);
		//printf("	%s\n",buf);
	}
		
		
	/*
	*	Rum
	*
	*/
		
	NextLine(f,buf);
	RoomCount=atoi(buf);
	//printf("\nRoomCount = %d\n",RoomCount);
		
	Rooms=(Room *)malloc( RoomCount*sizeof(Room));
		
	for(int count=0;count<RoomCount;count++)
	{
		NextLine(f,buf);
			
		Rooms[count].name=strdup(buf);
		//printf("	%s\n",buf);
		NextLine(f,buf);//image
		uint8 temp8;
		NextLine(f,buf);
		temp8=atoi(buf);
		Rooms[count].left	=&Rooms[temp8];
		NextLine(f,buf);
		temp8=atoi(buf);
		Rooms[count].up		=&Rooms[temp8];
		NextLine(f,buf);
		temp8=atoi(buf);
		Rooms[count].right	=&Rooms[temp8];
		NextLine(f,buf);
		temp8=atoi(buf);
		Rooms[count].down	=&Rooms[temp8];
			
		Rooms[count].firstenemy=NULL;
		Rooms[count].firstbubble=NULL;
		
		for (int c=0;c<30;c++){
			NextLine(f,buf);
			Line2Data(Rooms[count].blocks+(40*c),buf);
		}
	}
				
	fclose(f);
	return true;
}

void WorldView::Update(uint8 _x, uint _y)
{
//	printf("%3d,%3d\n",_x,_y);
	int x,y;
	uint32 *pieceoffs=BgBlocks+(256*Rooms[room].blocks[_y*40+_x]);
//	printf("piece:%d\n",(pieceoffs-BgBlocks)/256);
	uint32 *p=(uint32 *)bitmap->Bits();
	p+=(_y*40*16*16)+(16*_x);
	for (y=0;y<16;y++)
		for (x=0;x<16;x++)
			p[y*(40*16)+x]=*(pieceoffs++);

	Draw(BRect(_x*16,_y*16,_x*16+15,_y*16+15));

}

void WorldView::UpdateAll()
{
	int x,y;
	
	for (y=0;y<30;y++)
		for (x=0;x<40;x++)
			Update(x,y);
}

void WorldView::MouseDown(BPoint p)
{
	if ((p.x<32*16) && (p.y>=30*16)){
		piece=( (int)(p.y-(30*16))/16)*32+(int)(p.x/16);
		printf("piece=%d\n",piece);
		return;
	}

	if ((p.x>32*16) && (p.y>=31*16)){
		WriteRoom(stderr,room);
		return;
	}
		
	if ((p.x>36*16) && (p.y>=30*16)){
		if (room<(RoomCount-1)) room++;
		printf("room=%d\n",room);
		UpdateAll();
		return;
	}

	if ((p.x>32*16) && (p.y>=30*16)){
		if (room>0) room--;
		printf("room=%d\n",room);
		UpdateAll();
		return;
	}

	uint32	buttons=0;
	int x0,y0;
	
	do {
		if (p.y<30*16){
			Rooms[room].blocks[(int)(p.y/16)*40+(int)(p.x/16)]=piece;
			Update(p.x/16,p.y/16);
			x0 = (int)(p.x/16)*16;
			y0 = (int)(p.y/16)*16;
			//printf("putting piece=%d at %d, %d\n",piece,x0/16,y0/16);
		}
	
		snooze(10000);
		GetMouse(&p, &buttons,true); 
	} while (buttons);
}

void WorldView::Draw(BRect rect)
{
	//puts("DRAW");
	DrawBitmap(bitmap,rect,rect);
	if (rect.bottom>=30*16){
		DrawBitmap(pbitmap,BPoint(0,30*16));
		if (rect.right>32*16){
			StrokeRect(BRect(32*16+1,30*16+1,36*16-2,31*16-2));
			StrokeRect(BRect(36*16+1,30*16+1,40*16-2,31*16-2));
			StrokeRect(BRect(32*16+1,31*16+1,40*16-2,38*16-2));
			DrawString("prev",BPoint(33*16,31*16-5),0);
			DrawString("next",BPoint(37*16,31*16-5),0);
			DrawString("print to stream",BPoint(33*16,32*16),0);
		}
	}
}

WorldView::WorldView(char *filename)
	:BView(BRect(0,0,40*16-1,38*16-1), "WorldView", B_FOLLOW_ALL, B_WILL_DRAW+ B_PULSE_NEEDED)

{
	room=0;
	LoadWorld(filename);
	
	// palette
	pbitmap = new BBitmap(BRect(0,0,16*32-1,16*8-1),B_RGBA32_BIG);
	uint32 *target = (uint32 *)pbitmap->Bits();
	
	int x,y,y0;
	for (y=0;y<8;y++)
		for (x=0;x<32;x++){
			for (y0=0;y0<16;y0++)
				memcpy(target+(y*16*16*32+x*16+y0*32*16),BgBlocks+((y*32+x)*256+y0*16),16*4);
		}
			

			
	// The on screen thing
	bitmap = new BBitmap(BRect(0,0,16*40-1,16*30-1),B_RGBA32_BIG);

	UpdateAll();
//memcpy(bitmap->Bits(),BgBlocks,256*256);
}


