#include "insply.h"
#include "song.h"

const float N2F[]={
#include "freq-table.h"	
};

const float R2F[]={
#include "freq-table(relative).h"	
};


InsPly::InsPly(Ins *s)
{
	w=new Wave();
	ins=s;
	currins=ins;	
	steps=(currins->steps);
}

void InsPly::TrigADSR(uint8 i)
{
	currins=ins+i;
	steps=(currins->steps);

//	inspattern amp, filter och freq
	insfreqstep = 0;
	insamp = 1.0;
	insampstep = 0;
	insf = 1.0;
	insfstep = 0;

//	amp adsr
	adsr=0;
	nextadsr=0;

//	filter adsr
	fadsr=0;
	nextfadsr=0;

//	step
	stepc=0;
	nexts=0;
}


void InsPly::SetFilt(float f)
{
	currpartfilt=f;
}

void InsPly::SetFreq(float f)
{
	currpartfreq=f;
}

void InsPly::SetAmp(float a)
{
	currpartamp=a;
}

float InsPly::Next()
{
// checkinsstep
	if (!nexts){
//	fprintf(stderr,"!!!stepc=%d\n",stepc);
		if (stepc >= currins->len) stepc=0;

		currstep=&(steps[stepc]);
		stepc++;
		nexts=currins->sps;

		if (currstep->note){
		// ett-staell amp och filter
			insamp=1.0;
			insf=1.0;
			insfstep=0;
			insfreqstep=0;
			insampstep=0;
			
			if (currstep->mask & 0x10)
				insfreq = R2F[currstep->note];
			else
				insfreq = N2F[currstep->note];
		}

		if (currstep->mask & 0xf) w->SetForm(currstep->mask & 0xf);

		if (currstep->eff){
			switch (currstep->eff & 0xf0){
			case 0x10 : insfreqstep=(currstep->eff & 0xf)/(100.0); break;
			case 0x20 : insfreqstep=-(currstep->eff & 0xf)/(100.0); break;

			case 0x50 : stepc = (currstep->eff & 0xf); break;

			case 0x70 : insfstep=(currstep->eff & 0xf)/(32.0*1000); break;
			case 0x80 : insfstep=-(currstep->eff & 0xf)/(32.0*1000); break;
			case 0x90 : insfstep=0; insf=((currstep->eff & 0xf) +1)/16.0; break;

			case 0xa0 : insampstep=(currstep->eff & 0xf)/(32.0*1000); break;
			case 0xb0 : insampstep=-(currstep->eff & 0xf)/(32.0*1000); break;
			case 0xc0 : insampstep=0; insamp=(currstep->eff & 0xf)/15.0; break;

			case 0xd0 : inspwmstep=(currstep->eff & 0xf)/(64.0*1000); break;
			case 0xe0 : inspwmstep=-(currstep->eff & 0xf)/(64.0*1000); break;
			case 0xf0 : inspwmstep=0; inspwm=(currstep->eff & 0xf)/15.0; break;
			}
		}
	}
	
// checkadsr
	if (!nextadsr){
		if (adsr < 4){
			nextadsr= 1+256*currins->adsr[adsr];
			ampstep= ((currins->adsrlev[adsr]/257.0)-curramp) / nextadsr;
//			fprintf(stderr,"adsr=%d\nampstep=%f\n",adsr,ampstep);
			adsr++;
		}
		else
			ampstep=0;
	}

// checkfadsr
	if (!nextfadsr){
		if (fadsr < 4){
			nextfadsr= 1+256*currins->fadsr[fadsr];
			float tmp=currins->fadsrlev[fadsr]/257.0;
			fstep= ((tmp*tmp*tmp)-currf) / nextfadsr;
//			fprintf(stderr,"fadsr=%d\nfstep=%f\n",fadsr,fstep);
			fadsr++;
		}
		else
			fstep=0;
	}


//	adsr
	nextadsr--;
	nextfadsr--;

//	step
	nexts--;

//	Freq
	insfreq+=insfreqstep;
	if (insfreq<0) insfreq=0;
	if (currstep->mask & 0x10)
		w->SetFreq(currpartfreq * insfreq);		
	else
		w->SetFreq(insfreq);		
//	PWM
	inspwm+=inspwmstep;
	if (inspwm>1) inspwm=1;
	if (inspwm<0) inspwm=0;
	w->SetPulseWidth(inspwm);

//	Amp
	insamp+=insampstep;
	if (insamp>1) insamp=1;
	if (insamp<0) insamp=0;
	curramp+=ampstep;
	w->SetAmp(curramp*insamp*currpartamp);

//	Filter
	insf+=insfstep;
	if (insf>1) insf=1;
	if (insf<0) insf=0;
	currf+=fstep;
	w->SetFilt(currf*insf*currpartfilt);

	return w->Next();
};
